@@ -1,40 +1,40 @@
 /****************************************************************************
 **
-** Copyright (C) 2009 Nokia Corporation and/or its subsidiary(-ies).
+** Copyright (C) 2011 Nokia Corporation and/or its subsidiary(-ies).
 ** All rights reserved.
 ** Contact: Nokia Corporation (qt-info@nokia.com)
 **
 ** This file is part of the qmake application of the Qt Toolkit.
 **
 ** $QT_BEGIN_LICENSE:LGPL$
-** Commercial Usage
-** Licensees holding valid Qt Commercial licenses may use this file in
-** accordance with the Qt Commercial License Agreement provided with the
-** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and Nokia.
-**
 ** GNU Lesser General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU Lesser
-** General Public License version 2.1 as published by the Free Software
-** Foundation and appearing in the file LICENSE.LGPL included in the
-** packaging of this file.  Please review the following information to
-** ensure the GNU Lesser General Public License version 2.1 requirements
-** will be met: http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
+** This file may be used under the terms of the GNU Lesser General Public
+** License version 2.1 as published by the Free Software Foundation and
+** appearing in the file LICENSE.LGPL included in the packaging of this
+** file. Please review the following information to ensure the GNU Lesser
+** General Public License version 2.1 requirements will be met:
+** http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
 **
 ** In addition, as a special exception, Nokia gives you certain additional
-** rights.  These rights are described in the Nokia Qt LGPL Exception
+** rights. These rights are described in the Nokia Qt LGPL Exception
 ** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
 **
 ** GNU General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU
-** General Public License version 3.0 as published by the Free Software
-** Foundation and appearing in the file LICENSE.GPL included in the
-** packaging of this file.  Please review the following information to
-** ensure the GNU General Public License version 3.0 requirements will be
-** met: http://www.gnu.org/copyleft/gpl.html.
+** Alternatively, this file may be used under the terms of the GNU General
+** Public License version 3.0 as published by the Free Software Foundation
+** and appearing in the file LICENSE.GPL included in the packaging of this
+** file. Please review the following information to ensure the GNU General
+** Public License version 3.0 requirements will be met:
+** http://www.gnu.org/copyleft/gpl.html.
+**
+** Other Usage
+** Alternatively, this file may be used in accordance with the terms and
+** conditions contained in a signed written agreement between you and Nokia.
+**
+**
+**
+**
 **
-** If you have questions regarding the use of this file, please contact
-** Nokia at qt-info@nokia.com.
 ** $QT_END_LICENSE$
 **
 ****************************************************************************/
@@ -397,7 +397,7 @@ bool QMakeSourceFileInfo::findDeps(SourceFile *file)
         buffer = getBuffer(fst.st_size);
         for(int have_read = 0;
             (have_read = QT_READ(fd, buffer + buffer_len, fst.st_size - buffer_len));
-            buffer_len += have_read);
+            buffer_len += have_read) ;
         QT_CLOSE(fd);
     }
     if(!buffer)
@@ -418,22 +418,22 @@ bool QMakeSourceFileInfo::findDeps(SourceFile *file)
                 ++x;
                 if(buffer_len >= x + 12 && !strncmp(buffer + x, "includehint", 11) &&
                    (*(buffer + x + 11) == ' ' || *(buffer + x + 11) == '>')) {
-                    for(x += 11; *(buffer + x) != '>'; ++x);
+                    for(x += 11; *(buffer + x) != '>'; ++x) ;
                     int inc_len = 0;
-                    for(x += 1 ; *(buffer + x + inc_len) != '<'; ++inc_len);
+                    for(x += 1 ; *(buffer + x + inc_len) != '<'; ++inc_len) ;
                     *(buffer + x + inc_len) = '\0';
                     inc = buffer + x;
                 } else if(buffer_len >= x + 13 && !strncmp(buffer + x, "customwidget", 12) &&
                           (*(buffer + x + 12) == ' ' || *(buffer + x + 12) == '>')) {
-                    for(x += 13; *(buffer + x) != '>'; ++x); //skip up to >
+                    for(x += 13; *(buffer + x) != '>'; ++x) ; //skip up to >
                     while(x < buffer_len) {
-                        for(x++; *(buffer + x) != '<'; ++x); //skip up to <
+                        for(x++; *(buffer + x) != '<'; ++x) ; //skip up to <
                         x++;
                         if(buffer_len >= x + 7 && !strncmp(buffer+x, "header", 6) &&
                            (*(buffer + x + 6) == ' ' || *(buffer + x + 6) == '>')) {
-                            for(x += 7; *(buffer + x) != '>'; ++x); //skip up to >
+                            for(x += 7; *(buffer + x) != '>'; ++x) ; //skip up to >
                             int inc_len = 0;
-                            for(x += 1 ; *(buffer + x + inc_len) != '<'; ++inc_len);
+                            for(x += 1 ; *(buffer + x + inc_len) != '<'; ++inc_len) ;
                             *(buffer + x + inc_len) = '\0';
                             inc = buffer + x;
                             break;
@@ -448,10 +448,10 @@ bool QMakeSourceFileInfo::findDeps(SourceFile *file)
                     for(x += 8; *(buffer + x) != '>'; ++x) {
                         if(buffer_len >= x + 9 && *(buffer + x) == 'i' &&
                            !strncmp(buffer + x, "impldecl", 8)) {
-                            for(x += 8; *(buffer + x) != '='; ++x);
+                            for(x += 8; *(buffer + x) != '='; ++x) ;
                             if(*(buffer + x) != '=')
                                 continue;
-                            for(++x; *(buffer+x) == '\t' || *(buffer+x) == ' '; ++x);
+                            for(++x; *(buffer+x) == '\t' || *(buffer+x) == ' '; ++x) ;
                             char quote = 0;
                             if(*(buffer+x) == '\'' || *(buffer+x) == '"') {
                                 quote = *(buffer + x);
@@ -475,13 +475,13 @@ bool QMakeSourceFileInfo::findDeps(SourceFile *file)
                         }
                     }
                     int inc_len = 0;
-                    for(x += 1 ; *(buffer + x + inc_len) != '<'; ++inc_len);
+                    for(x += 1 ; *(buffer + x + inc_len) != '<'; ++inc_len) ;
                     *(buffer + x + inc_len) = '\0';
                     inc = buffer + x;
                 }
             }
             //read past new line now..
-            for(; x < buffer_len && !qmake_endOfLine(*(buffer + x)); ++x);
+            for(; x < buffer_len && !qmake_endOfLine(*(buffer + x)); ++x) ;
             ++line_count;
         } else if(file->type == QMakeSourceFileInfo::TYPE_QRC) {
         } else if(file->type == QMakeSourceFileInfo::TYPE_C) {
@@ -494,12 +494,12 @@ bool QMakeSourceFileInfo::findDeps(SourceFile *file)
                         ++x;
                         if(buffer_len >= x) {
                             if(*(buffer+x) == '/') { //c++ style comment
-                                for(; x < buffer_len && !qmake_endOfLine(*(buffer + x)); ++x);
+                                for(; x < buffer_len && !qmake_endOfLine(*(buffer + x)); ++x) ;
                                 beginning = 1;
                             } else if(*(buffer+x) == '*') { //c style comment
                                 for(++x; x < buffer_len; ++x) {
                                     if(*(buffer+x) == '*') {
-                                        if(x < buffer_len-1 && *(buffer + (x+1)) == '/') {
+                                        if(x+1 < buffer_len && *(buffer + (x+1)) == '/') {
                                             ++x;
                                             break;
                                         }
@@ -558,7 +558,7 @@ bool QMakeSourceFileInfo::findDeps(SourceFile *file)
                    *(buffer+x+keyword_len) != '_') {
                     for(x+=keyword_len; //skip spaces after keyword
                         x < buffer_len && (*(buffer+x) == ' ' || *(buffer+x) == '\t');
-                        x++);
+                        x++) ;
                     break;
                 } else if(qmake_endOfLine(*(buffer+x+keyword_len))) {
                     x += keyword_len-1;
@@ -568,7 +568,8 @@ bool QMakeSourceFileInfo::findDeps(SourceFile *file)
                 keyword_len++;
             }
 
-            if(keyword_len == 7 && !strncmp(keyword, "include", keyword_len)) {
+            if((keyword_len == 7 && !strncmp(keyword, "include", 7)) // C & Obj-C
+               || (keyword_len == 6 && !strncmp(keyword, "import", 6))) { // Obj-C
                 char term = *(buffer + x);
                 if(term == '<') {
                     try_local = false;
@@ -579,7 +580,7 @@ bool QMakeSourceFileInfo::findDeps(SourceFile *file)
                 x++;
 
                 int inc_len;
-                for(inc_len = 0; *(buffer + x + inc_len) != term && !qmake_endOfLine(*(buffer + x + inc_len)); ++inc_len);
+                for(inc_len = 0; *(buffer + x + inc_len) != term && !qmake_endOfLine(*(buffer + x + inc_len)); ++inc_len) ;
                 *(buffer + x + inc_len) = '\0';
                 inc = buffer + x;
                 x += inc_len;
@@ -594,7 +595,7 @@ bool QMakeSourceFileInfo::findDeps(SourceFile *file)
 
                 int msg_len;
                 for(msg_len = 0; (term && *(buffer + x + msg_len) != term) &&
-                              !qmake_endOfLine(*(buffer + x + msg_len)); ++msg_len);
+                              !qmake_endOfLine(*(buffer + x + msg_len)); ++msg_len) ;
                 *(buffer + x + msg_len) = '\0';
                 debug_msg(0, "%s:%d %s -- %s", file->file.local().toLatin1().constData(), line_count, keyword, buffer+x);
                 x += msg_len;
@@ -706,7 +707,7 @@ bool QMakeSourceFileInfo::findMocs(SourceFile *file)
         buffer = getBuffer(fst.st_size);
         for(int have_read = buffer_len = 0;
             (have_read = QT_READ(fd, buffer + buffer_len, fst.st_size - buffer_len));
-            buffer_len += have_read);
+            buffer_len += have_read) ;
         QT_CLOSE(fd);
     }
 
@@ -720,7 +721,7 @@ bool QMakeSourceFileInfo::findMocs(SourceFile *file)
             ++x;
             if(buffer_len >= x) {
                 if(*(buffer + x) == '/') { //c++ style comment
-                    for(;x < buffer_len && !qmake_endOfLine(*(buffer + x)); ++x);
+                    for(;x < buffer_len && !qmake_endOfLine(*(buffer + x)); ++x) ;
                 } else if(*(buffer + x) == '*') { //c style comment
                     for(++x; x < buffer_len; ++x) {
                         if(*(buffer + x) == 't' || *(buffer + x) == 'q') { //ignore
