@@ -1,40 +1,40 @@
 /****************************************************************************
 **
-** Copyright (C) 2009 Nokia Corporation and/or its subsidiary(-ies).
+** Copyright (C) 2011 Nokia Corporation and/or its subsidiary(-ies).
 ** All rights reserved.
 ** Contact: Nokia Corporation (qt-info@nokia.com)
 **
 ** This file is part of the qmake application of the Qt Toolkit.
 **
 ** $QT_BEGIN_LICENSE:LGPL$
-** Commercial Usage
-** Licensees holding valid Qt Commercial licenses may use this file in
-** accordance with the Qt Commercial License Agreement provided with the
-** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and Nokia.
-**
 ** GNU Lesser General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU Lesser
-** General Public License version 2.1 as published by the Free Software
-** Foundation and appearing in the file LICENSE.LGPL included in the
-** packaging of this file.  Please review the following information to
-** ensure the GNU Lesser General Public License version 2.1 requirements
-** will be met: http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
+** This file may be used under the terms of the GNU Lesser General Public
+** License version 2.1 as published by the Free Software Foundation and
+** appearing in the file LICENSE.LGPL included in the packaging of this
+** file. Please review the following information to ensure the GNU Lesser
+** General Public License version 2.1 requirements will be met:
+** http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
 **
 ** In addition, as a special exception, Nokia gives you certain additional
-** rights.  These rights are described in the Nokia Qt LGPL Exception
+** rights. These rights are described in the Nokia Qt LGPL Exception
 ** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
 **
 ** GNU General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU
-** General Public License version 3.0 as published by the Free Software
-** Foundation and appearing in the file LICENSE.GPL included in the
-** packaging of this file.  Please review the following information to
-** ensure the GNU General Public License version 3.0 requirements will be
-** met: http://www.gnu.org/copyleft/gpl.html.
+** Alternatively, this file may be used under the terms of the GNU General
+** Public License version 3.0 as published by the Free Software Foundation
+** and appearing in the file LICENSE.GPL included in the packaging of this
+** file. Please review the following information to ensure the GNU General
+** Public License version 3.0 requirements will be met:
+** http://www.gnu.org/copyleft/gpl.html.
+**
+** Other Usage
+** Alternatively, this file may be used in accordance with the terms and
+** conditions contained in a signed written agreement between you and Nokia.
+**
+**
+**
+**
 **
-** If you have questions regarding the use of this file, please contact
-** Nokia at qt-info@nokia.com.
 ** $QT_END_LICENSE$
 **
 ****************************************************************************/
@@ -65,6 +65,10 @@ UnixMakefileGenerator::init()
         }
     }
 
+    if (project->isEmpty("QMAKE_PREFIX_SHLIB"))
+        // Prevent crash when using the empty variable.
+        project->values("QMAKE_PREFIX_SHLIB").append("");
+
     if(!project->isEmpty("QMAKE_FAILED_REQUIREMENTS")) /* no point */
         return;
 
@@ -88,7 +92,7 @@ UnixMakefileGenerator::init()
     if(project->isEmpty("QMAKE_LIBTOOL"))
         project->values("QMAKE_LIBTOOL").append("libtool --silent");
     if(project->isEmpty("QMAKE_SYMBOLIC_LINK"))
-        project->values("QMAKE_SYMBOLIC_LINK").append("ln -sf");
+        project->values("QMAKE_SYMBOLIC_LINK").append("ln -f -s");
 
     /* this should probably not be here, but I'm using it to wrap the .t files */
     if(project->first("TEMPLATE") == "app")
@@ -99,8 +103,6 @@ UnixMakefileGenerator::init()
         MakefileGenerator::init();
         if(project->isEmpty("MAKEFILE"))
             project->values("MAKEFILE").append("Makefile");
-        if(project->isEmpty("QMAKE_QMAKE"))
-            project->values("QMAKE_QMAKE").append("qmake");
         if(project->values("QMAKE_INTERNAL_QMAKE_DEPS").indexOf("qmake_all") == -1)
             project->values("QMAKE_INTERNAL_QMAKE_DEPS").append("qmake_all");
         return; /* subdirs is done */
@@ -146,6 +148,8 @@ UnixMakefileGenerator::init()
                 project->values("QMAKE_LFLAGS") += var("QMAKE_LFLAGS_RPATH") + libdirs[i];
             if (project->isActiveConfig("rvct_linker")) {
                 project->values("QMAKE_LIBDIR_FLAGS") += "--userlibpath " + escapeFilePath(libdirs[i]);
+            } else if (project->isActiveConfig("armcc_linker")) {
+                project->values("QMAKE_LIBDIR_FLAGS") += "-L--userlibpath=" + escapeFilePath(libdirs[i]);
             } else {
                 project->values("QMAKE_LIBDIR_FLAGS") += "-L" + escapeFilePath(libdirs[i]);
             }
@@ -204,6 +208,33 @@ UnixMakefileGenerator::init()
                 // icc style
                 pchFlags = pchFlags.replace("${QMAKE_PCH_OUTPUT}",
                                             pchBaseName + project->first("QMAKE_PCH_OUTPUT_EXT"));
+            } else {
+                // gcc style (including clang_pch_style)
+                QString headerPrefix = project->first("QMAKE_PRECOMP_PREFIX");
+                QString headerSuffix;
+                if (project->isActiveConfig("clang_pch_style"))
+                    headerSuffix = project->first("QMAKE_PCH_OUTPUT_EXT");
+                else
+                    pchBaseName += project->first("QMAKE_PCH_OUTPUT_EXT");
+
+                pchBaseName += Option::dir_sep;
+                QString pchOutputFile;
+
+                if(comps[i] == "C") {
+                    pchOutputFile = "c";
+                } else if(comps[i] == "CXX") {
+                    pchOutputFile = "c++";
+                } else if(project->isActiveConfig("objective_c")) {
+                    if(comps[i] == "OBJC")
+                        pchOutputFile = "objective-c";
+                    else if(comps[i] == "OBJCXX")
+                        pchOutputFile = "objective-c++";
+                }
+
+                if(!pchOutputFile.isEmpty()) {
+                    pchFlags = pchFlags.replace("${QMAKE_PCH_OUTPUT}",
+                            pchBaseName + pchOutputFile + headerSuffix);
+                }
             }
 
             if (!pchFlags.isEmpty())
@@ -377,7 +408,9 @@ QStringList
         QString header_prefix;
         if(!project->isEmpty("PRECOMPILED_DIR"))
             header_prefix = project->first("PRECOMPILED_DIR");
-        header_prefix += project->first("QMAKE_ORIG_TARGET") + project->first("QMAKE_PCH_OUTPUT_EXT");
+        header_prefix += project->first("QMAKE_ORIG_TARGET");
+        if (!project->isActiveConfig("clang_pch_style"))
+            header_prefix += project->first("QMAKE_PCH_OUTPUT_EXT");
         if (project->isActiveConfig("icc_pch_style")) {
             // icc style
             for(QStringList::Iterator it = Option::cpp_ext.begin(); it != Option::cpp_ext.end(); ++it) {
@@ -387,23 +420,25 @@ QStringList
                 }
             }
         } else {
-            // gcc style
+            // gcc style (including clang_pch_style)
+            QString header_suffix = project->isActiveConfig("clang_pch_style")
+                    ? project->first("QMAKE_PCH_OUTPUT_EXT") : "";
             header_prefix += Option::dir_sep + project->first("QMAKE_PRECOMP_PREFIX");
             for(QStringList::Iterator it = Option::c_ext.begin(); it != Option::c_ext.end(); ++it) {
                 if(file.endsWith(*it)) {
                     if(!project->isEmpty("QMAKE_CFLAGS_PRECOMPILE")) {
-                        QString precomp_c_h = header_prefix + "c";
+                        QString precomp_c_h = header_prefix + "c" + header_suffix;
                         if(!ret.contains(precomp_c_h))
                             ret += precomp_c_h;
                     }
                     if(project->isActiveConfig("objective_c")) {
                         if(!project->isEmpty("QMAKE_OBJCFLAGS_PRECOMPILE")) {
-                            QString precomp_objc_h = header_prefix + "objective-c";
+                            QString precomp_objc_h = header_prefix + "objective-c" + header_suffix;
                             if(!ret.contains(precomp_objc_h))
                                 ret += precomp_objc_h;
                         }
                         if(!project->isEmpty("QMAKE_OBJCXXFLAGS_PRECOMPILE")) {
-                            QString precomp_objcpp_h = header_prefix + "objective-c++";
+                            QString precomp_objcpp_h = header_prefix + "objective-c++" + header_suffix;
                             if(!ret.contains(precomp_objcpp_h))
                                 ret += precomp_objcpp_h;
                         }
@@ -414,13 +449,13 @@ QStringList
             for(QStringList::Iterator it = Option::cpp_ext.begin(); it != Option::cpp_ext.end(); ++it) {
                 if(file.endsWith(*it)) {
                     if(!project->isEmpty("QMAKE_CXXFLAGS_PRECOMPILE")) {
-                        QString precomp_cpp_h = header_prefix + "c++";
+                        QString precomp_cpp_h = header_prefix + "c++" + header_suffix;
                         if(!ret.contains(precomp_cpp_h))
                             ret += precomp_cpp_h;
                     }
                     if(project->isActiveConfig("objective_c")) {
                         if(!project->isEmpty("QMAKE_OBJCXXFLAGS_PRECOMPILE")) {
-                            QString precomp_objcpp_h = header_prefix + "objective-c++";
+                            QString precomp_objcpp_h = header_prefix + "objective-c++" + header_suffix;
                             if(!ret.contains(precomp_objcpp_h))
                                 ret += precomp_objcpp_h;
                         }
@@ -451,7 +486,9 @@ UnixMakefileGenerator::findLibraries()
                     if(!libdirs.contains(f))
                         libdirs.append(f);
                 } else if(opt.startsWith("-l")) {
-                    if (project->isActiveConfig("rvct_linker")) {
+                    if (!project->isEmpty("QMAKE_RVCT_LINKSTYLE")) {
+                        (*it) = opt.mid(2);
+                    } else if (project->isActiveConfig("rvct_linker") || project->isActiveConfig("armcc_linker")) {
                         (*it) = "lib" + opt.mid(2) + ".so";
                     } else {
                         stub = opt.mid(2);
@@ -491,26 +528,29 @@ UnixMakefileGenerator::findLibraries()
                 QStringList extens;
                 if(!extn.isNull())
                     extens << extn;
+                else if (!project->isEmpty("QMAKE_SYMBIAN_SHLIB"))
+                    // In Symbian you link to the stub .lib file, but run with the .dll file.
+                    extens << "lib";
                 else
                     extens << project->values("QMAKE_EXTENSION_SHLIB").first() << "a";
                 for(QStringList::Iterator extit = extens.begin(); extit != extens.end(); ++extit) {
                     if(dir.isNull()) {
-                        QString lib_stub;
                         for(QList<QMakeLocalFileName>::Iterator dep_it = libdirs.begin(); dep_it != libdirs.end(); ++dep_it) {
-                            if(exists((*dep_it).local() + Option::dir_sep + "lib" + stub +
-                                      "." + (*extit))) {
-                                lib_stub = stub;
+                            QString pathToLib = ((*dep_it).local() + Option::dir_sep
+                                    + project->values("QMAKE_PREFIX_SHLIB").first()
+                                    + stub + "." + (*extit));
+                            if(exists(pathToLib)) {
+                                if (!project->isEmpty("QMAKE_RVCT_LINKSTYLE"))
+                                    (*it) = pathToLib;
+                                else
+                                    (*it) = "-l" + stub;
+                                found = true;
                                 break;
                             }
                         }
-                        if(!lib_stub.isNull()) {
-                            (*it) = "-l" + lib_stub;
-                            found = true;
-                            break;
-                        }
                     } else {
-                        if(exists("lib" + stub + "." + (*extit))) {
-                            (*it) = "lib" + stub + "." + (*extit);
+                        if(exists(project->values("QMAKE_PREFIX_SHLIB").first() + stub + "." + (*extit))) {
+                            (*it) = project->values("QMAKE_PREFIX_SHLIB").first() + stub + "." + (*extit);
                             found = true;
                             break;
                         }
@@ -518,8 +558,8 @@ UnixMakefileGenerator::findLibraries()
                 }
                 if(!found && project->isActiveConfig("compile_libtool")) {
                     for(int dep_i = 0; dep_i < libdirs.size(); ++dep_i) {
-                        if(exists(libdirs[dep_i].local() + Option::dir_sep + "lib" + stub + Option::libtool_ext)) {
-                            (*it) = libdirs[dep_i].real() + Option::dir_sep + "lib" + stub + Option::libtool_ext;
+                        if(exists(libdirs[dep_i].local() + Option::dir_sep + project->values("QMAKE_PREFIX_SHLIB").first() + stub + Option::libtool_ext)) {
+                            (*it) = libdirs[dep_i].real() + Option::dir_sep + project->values("QMAKE_PREFIX_SHLIB").first() + stub + Option::libtool_ext;
                             found = true;
                             break;
                         }
@@ -544,7 +584,6 @@ UnixMakefileGenerator::processPrlFiles()
 {
     QList<QMakeLocalFileName> libdirs, frameworkdirs;
     frameworkdirs.append(QMakeLocalFileName("/System/Library/Frameworks"));
-    frameworkdirs.append(QMakeLocalFileName("/Library/Frameworks"));
     const QString lflags[] = { "QMAKE_LIBDIR_FLAGS", "QMAKE_FRAMEWORKPATH_FLAGS", "QMAKE_LFLAGS", "QMAKE_LIBS", QString() };
     for(int i = 0; !lflags[i].isNull(); i++) {
         QStringList &l = project->values(lflags[i]);
@@ -560,7 +599,7 @@ UnixMakefileGenerator::processPrlFiles()
                     for(int dep_i = 0; dep_i < libdirs.size(); ++dep_i) {
                         const QMakeLocalFileName &lfn = libdirs[dep_i];
                         if(!project->isActiveConfig("compile_libtool")) { //give them the .libs..
-                            QString la = lfn.local() + Option::dir_sep + "lib" + lib + Option::libtool_ext;
+                            QString la = lfn.local() + Option::dir_sep + project->values("QMAKE_PREFIX_SHLIB").first() + lib + Option::libtool_ext;
                             if(exists(la) && QFile::exists(lfn.local() + Option::dir_sep + ".libs")) {
                                 QString dot_libs = lfn.real() + Option::dir_sep + ".libs";
                                 l.append("-L" + dot_libs);
@@ -568,7 +607,7 @@ UnixMakefileGenerator::processPrlFiles()
                             }
                         }
 
-                        QString prl = lfn.local() + Option::dir_sep + "lib" + lib;
+                        QString prl = lfn.local() + Option::dir_sep + project->values("QMAKE_PREFIX_SHLIB").first() + lib;
                         if(!project->isEmpty("QMAKE_" + lib.toUpper() + "_SUFFIX"))
                             prl += project->first("QMAKE_" + lib.toUpper() + "_SUFFIX");
                         if(processPrlFile(prl)) {
@@ -616,31 +655,44 @@ UnixMakefileGenerator::processPrlFiles()
 
         //merge them into a logical order
         if(!project->isActiveConfig("no_smart_library_merge") && !project->isActiveConfig("no_lflags_merge")) {
-            QStringList lflags;
+            QHash<QString, QStringList> lflags;
             for(int lit = 0; lit < l.size(); ++lit) {
+                QString arch("default");
                 QString opt = l.at(lit).trimmed();
                 if(opt.startsWith("-")) {
+                    if (Option::target_mode == Option::TARG_MACX_MODE && opt.startsWith("-Xarch")) {
+                        if (opt.length() > 7) {
+                            arch = opt.mid(7);
+                            opt = l.at(++lit);
+                        }
+                    }
+
                     if(opt.startsWith("-L") ||
                        (Option::target_mode == Option::TARG_MACX_MODE && opt.startsWith("-F"))) {
-                        if(lit == 0 || l.lastIndexOf(opt, lit-1) == -1)
-                            lflags.append(opt);
-                    } else if(opt.startsWith("-l")) {
-                        if(lit == l.size()-1 || l.indexOf(opt, lit+1) == -1)
-                            lflags.append(opt);
+                        if(!lflags[arch].contains(opt))
+                            lflags[arch].append(opt);
+                    } else if(opt.startsWith("-l") || opt == "-pthread") {
+                        // Make sure we keep the dependency-order of libraries
+                        if (lflags[arch].contains(opt))
+                            lflags[arch].removeAll(opt);
+                        lflags[arch].append(opt);
                     } else if(Option::target_mode == Option::TARG_MACX_MODE && opt.startsWith("-framework")) {
                         if(opt.length() > 11)
                             opt = opt.mid(11);
-                        else
+                        else {
                             opt = l.at(++lit);
+                            if (Option::target_mode == Option::TARG_MACX_MODE && opt.startsWith("-Xarch"))
+                                opt = l.at(++lit); // The user has done the right thing and prefixed each part
+                        }
                         bool found = false;
-                        for(int x = lit+1; x < l.size(); ++x) {
-                            QString xf = l.at(x);
+                        for(int x = 0; x < lflags[arch].size(); ++x) {
+                            QString xf = lflags[arch].at(x);
                             if(xf.startsWith("-framework")) {
                                 QString framework;
                                 if(xf.length() > 11)
                                     framework = xf.mid(11);
                                 else
-                                    framework = l.at(++x);
+                                    framework = lflags[arch].at(++x);
                                 if(framework == opt) {
                                     found = true;
                                     break;
@@ -648,18 +700,30 @@ UnixMakefileGenerator::processPrlFiles()
                             }
                         }
                         if(!found) {
-                            lflags.append("-framework");
-                            lflags.append(opt);
+                            lflags[arch].append("-framework");
+                            lflags[arch].append(opt);
                         }
                     } else {
-                        lflags.append(opt);
+                        lflags[arch].append(opt);
                     }
                 } else if(!opt.isNull()) {
-                    if(lit == 0 || l.lastIndexOf(opt, lit-1) == -1)
-                        lflags.append(opt);
+                    if(!lflags[arch].contains(opt))
+                        lflags[arch].append(opt);
                 }
             }
-            l = lflags;
+
+            l =  lflags.take("default");
+
+            // Process architecture specific options (Xarch)
+            QHash<QString, QStringList>::const_iterator archIterator = lflags.constBegin();
+            while (archIterator != lflags.constEnd()) {
+                const QStringList archOptions = archIterator.value();
+                for (int i = 0; i < archOptions.size(); ++i) {
+                    l.append(QLatin1String("-Xarch_") + archIterator.key());
+                    l.append(archOptions.at(i));
+                }
+                ++archIterator;
+            }
         }
     }
 }
@@ -779,10 +843,8 @@ UnixMakefileGenerator::defaultInstall(const QString &t)
             uninst.append("-$(DEL_FILE) \"" + dst_targ + "\"");
         if(!links.isEmpty()) {
             for(int i = 0; i < links.size(); ++i) {
-                if(Option::target_mode == Option::TARG_WIN_MODE ||
-                   Option::target_mode == Option::TARG_MAC9_MODE) {
-                } else if(Option::target_mode == Option::TARG_UNIX_MODE ||
-                          Option::target_mode == Option::TARG_MACX_MODE) {
+                if(Option::target_mode == Option::TARG_UNIX_MODE ||
+                   Option::target_mode == Option::TARG_MACX_MODE) {
                     QString link = Option::fixPathToTargetOS(destdir + links[i], false);
                     int lslash = link.lastIndexOf(Option::dir_sep);
                     if(lslash != -1)
