@@ -1,40 +1,40 @@
 /****************************************************************************
 **
-** Copyright (C) 2009 Nokia Corporation and/or its subsidiary(-ies).
+** Copyright (C) 2011 Nokia Corporation and/or its subsidiary(-ies).
 ** All rights reserved.
 ** Contact: Nokia Corporation (qt-info@nokia.com)
 **
 ** This file is part of the QtCore module of the Qt Toolkit.
 **
 ** $QT_BEGIN_LICENSE:LGPL$
-** Commercial Usage
-** Licensees holding valid Qt Commercial licenses may use this file in
-** accordance with the Qt Commercial License Agreement provided with the
-** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and Nokia.
-**
 ** GNU Lesser General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU Lesser
-** General Public License version 2.1 as published by the Free Software
-** Foundation and appearing in the file LICENSE.LGPL included in the
-** packaging of this file.  Please review the following information to
-** ensure the GNU Lesser General Public License version 2.1 requirements
-** will be met: http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
+** This file may be used under the terms of the GNU Lesser General Public
+** License version 2.1 as published by the Free Software Foundation and
+** appearing in the file LICENSE.LGPL included in the packaging of this
+** file. Please review the following information to ensure the GNU Lesser
+** General Public License version 2.1 requirements will be met:
+** http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
 **
 ** In addition, as a special exception, Nokia gives you certain additional
-** rights.  These rights are described in the Nokia Qt LGPL Exception
+** rights. These rights are described in the Nokia Qt LGPL Exception
 ** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
 **
 ** GNU General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU
-** General Public License version 3.0 as published by the Free Software
-** Foundation and appearing in the file LICENSE.GPL included in the
-** packaging of this file.  Please review the following information to
-** ensure the GNU General Public License version 3.0 requirements will be
-** met: http://www.gnu.org/copyleft/gpl.html.
+** Alternatively, this file may be used under the terms of the GNU General
+** Public License version 3.0 as published by the Free Software Foundation
+** and appearing in the file LICENSE.GPL included in the packaging of this
+** file. Please review the following information to ensure the GNU General
+** Public License version 3.0 requirements will be met:
+** http://www.gnu.org/copyleft/gpl.html.
+**
+** Other Usage
+** Alternatively, this file may be used in accordance with the terms and
+** conditions contained in a signed written agreement between you and Nokia.
+**
+**
+**
+**
 **
-** If you have questions regarding the use of this file, please contact
-** Nokia at qt-info@nokia.com.
 ** $QT_END_LICENSE$
 **
 ****************************************************************************/
@@ -46,7 +46,11 @@
 #include "qlibraryinfo.h"
 #include "qscopedpointer.h"
 
-#ifdef QT_BUILD_QMAKE
+#if defined(QT_BUILD_QMAKE) || defined(QT_BOOTSTRAPPED)
+# define BOOTSTRAPPING
+#endif
+
+#ifdef BOOTSTRAPPING
 QT_BEGIN_NAMESPACE
 extern QString qmake_libraryInfoFile();
 QT_END_NAMESPACE
@@ -58,10 +62,16 @@ QT_END_NAMESPACE
 #  include "private/qcore_mac_p.h"
 #endif
 
+#ifdef QLIBRARYINFO_EPOCROOT
+# include "symbian/epocroot_p.h"
+#endif
+
 #include "qconfig.cpp"
 
 QT_BEGIN_NAMESPACE
 
+extern void qDumpCPUFeatures(); // in qsimd.cpp
+
 #ifndef QT_NO_SETTINGS
 
 struct QLibrarySettings
@@ -91,7 +101,7 @@ public:
 QLibrarySettings::QLibrarySettings()
     : settings(QLibraryInfoPrivate::findConfiguration())
 {
-#ifndef QT_BUILD_QMAKE
+#ifndef BOOTSTRAPPING
     qAddPostRoutine(QLibraryInfoPrivate::cleanup);
 #endif
 }
@@ -99,7 +109,7 @@ QLibrarySettings::QLibrarySettings()
 QSettings *QLibraryInfoPrivate::findConfiguration()
 {
     QString qtconfig = QLatin1String(":/qt/etc/qt.conf");
-#ifdef QT_BUILD_QMAKE
+#ifdef BOOTSTRAPPING
     if(!QFile::exists(qtconfig))
         qtconfig = qmake_libraryInfoFile();
 #else
@@ -262,6 +272,11 @@ QLibraryInfo::location(LibraryLocation loc)
             path = QT_CONFIGURE_PLUGINS_PATH;
             break;
 #endif
+#ifdef QT_CONFIGURE_IMPORTS_PATH
+        case ImportsPath:
+            path = QT_CONFIGURE_IMPORTS_PATH;
+            break;
+#endif
 #ifdef QT_CONFIGURE_DATA_PATH
         case DataPath:
             path = QT_CONFIGURE_DATA_PATH;
@@ -320,6 +335,10 @@ QLibraryInfo::location(LibraryLocation loc)
             key = QLatin1String("Plugins");
             defaultValue = QLatin1String("plugins");
             break;
+        case ImportsPath:
+            key = QLatin1String("Imports");
+            defaultValue = QLatin1String("imports");
+            break;
         case DataPath:
             key = QLatin1String("Data");
             break;
@@ -418,15 +437,24 @@ QLibraryInfo::location(LibraryLocation loc)
                             QString::fromLocal8Bit(qgetenv(ret.mid(rep + 2,
                                 reg_var.matchedLength() - 3).toLatin1().constData()).constData()));
             }
+
+#ifdef QLIBRARYINFO_EPOCROOT
+            // $${EPOCROOT} is a special case, resolve it similarly to qmake.
+            QRegExp epocrootMatcher(QLatin1String("\\$\\$\\{EPOCROOT\\}"));
+            if ((rep = epocrootMatcher.indexIn(ret)) != -1)
+                ret.replace(rep, epocrootMatcher.matchedLength(), qt_epocRoot());
+#endif
+
             config->endGroup();
         }
     }
 
     if (QDir::isRelativePath(ret)) {
+        QString baseDir;
         if (loc == PrefixPath) {
             // we make the prefix path absolute to the executable's directory
-#ifdef QT_BUILD_QMAKE
-            return QFileInfo(qmake_libraryInfoFile()).absolutePath();
+#ifdef BOOTSTRAPPING
+            baseDir = QFileInfo(qmake_libraryInfoFile()).absolutePath();
 #else
             if (QCoreApplication::instance()) {
 #ifdef Q_OS_MAC
@@ -435,19 +463,20 @@ QLibraryInfo::location(LibraryLocation loc)
                     QCFType<CFURLRef> urlRef = CFBundleCopyBundleURL(bundleRef);
                     if (urlRef) {
                         QCFString path = CFURLCopyFileSystemPath(urlRef, kCFURLPOSIXPathStyle);
-                        return QDir::cleanPath(path + QLatin1String("/Contents/") + ret);
+                        return QDir::cleanPath(QString(path) + QLatin1String("/Contents/") + ret);
                     }
                 }
 #endif
-                return QDir(QCoreApplication::applicationDirPath()).absoluteFilePath(ret);
+                baseDir = QCoreApplication::applicationDirPath();
             } else {
-                return QDir::current().absoluteFilePath(ret);
+                baseDir = QDir::currentPath();
             }
 #endif
         } else {
             // we make any other path absolute to the prefix directory
-            return QDir(location(PrefixPath)).absoluteFilePath(ret);
+            baseDir = location(PrefixPath);
         }
+        ret = QDir::cleanPath(baseDir + QLatin1Char('/') + ret);
     }
     return ret;
 }
@@ -463,9 +492,10 @@ QLibraryInfo::location(LibraryLocation loc)
     \value PrefixPath The default prefix for all paths.
     \value DocumentationPath The location for documentation upon install.
     \value HeadersPath The location for all headers.
-    \value LibrariesPath The location of installed librarires.
+    \value LibrariesPath The location of installed libraries.
     \value BinariesPath The location of installed Qt binaries (tools and applications).
     \value PluginsPath The location of installed Qt plugins.
+    \value ImportsPath The location of installed QML extensions to import.
     \value DataPath The location of general Qt data.
     \value TranslationsPath The location of translation information for Qt strings.
     \value SettingsPath The location for Qt settings.
@@ -490,10 +520,21 @@ extern "C" void qt_core_boilerplate();
 void qt_core_boilerplate()
 {
     printf("This is the QtCore library version " QT_VERSION_STR "\n"
-           "Copyright (C) 2009 Nokia Corporation and/or its subsidiary(-ies).\n"
+           "Copyright (C) 2011 Nokia Corporation and/or its subsidiary(-ies).\n"
            "Contact: Nokia Corporation (qt-info@nokia.com)\n"
            "\n"
            "Build key:           " QT_BUILD_KEY "\n"
+           "Compat build key:    "
+#ifdef QT_BUILD_KEY_COMPAT
+           "| " QT_BUILD_KEY_COMPAT " "
+#endif
+#ifdef QT_BUILD_KEY_COMPAT2
+           "| " QT_BUILD_KEY_COMPAT2 " "
+#endif
+#ifdef QT_BUILD_KEY_COMPAT3
+           "| " QT_BUILD_KEY_COMPAT3 " "
+#endif
+           "|\n"
            "Build date:          %s\n"
            "Installation prefix: %s\n"
            "Library path:        %s\n"
@@ -503,6 +544,8 @@ void qt_core_boilerplate()
            qt_configure_libraries_path_str + 12,
            qt_configure_headers_path_str + 12);
 
+    QT_PREPEND_NAMESPACE(qDumpCPUFeatures)();
+
 #ifdef QT_EVAL
     extern void qt_core_eval_init(uint);
     qt_core_eval_init(1);
