@@ -1,40 +1,40 @@
 /****************************************************************************
 **
-** Copyright (C) 2009 Nokia Corporation and/or its subsidiary(-ies).
+** Copyright (C) 2011 Nokia Corporation and/or its subsidiary(-ies).
 ** All rights reserved.
 ** Contact: Nokia Corporation (qt-info@nokia.com)
 **
 ** This file is part of the QtCore module of the Qt Toolkit.
 **
 ** $QT_BEGIN_LICENSE:LGPL$
-** Commercial Usage
-** Licensees holding valid Qt Commercial licenses may use this file in
-** accordance with the Qt Commercial License Agreement provided with the
-** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and Nokia.
-**
 ** GNU Lesser General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU Lesser
-** General Public License version 2.1 as published by the Free Software
-** Foundation and appearing in the file LICENSE.LGPL included in the
-** packaging of this file.  Please review the following information to
-** ensure the GNU Lesser General Public License version 2.1 requirements
-** will be met: http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
+** This file may be used under the terms of the GNU Lesser General Public
+** License version 2.1 as published by the Free Software Foundation and
+** appearing in the file LICENSE.LGPL included in the packaging of this
+** file. Please review the following information to ensure the GNU Lesser
+** General Public License version 2.1 requirements will be met:
+** http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
 **
 ** In addition, as a special exception, Nokia gives you certain additional
-** rights.  These rights are described in the Nokia Qt LGPL Exception
+** rights. These rights are described in the Nokia Qt LGPL Exception
 ** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
 **
 ** GNU General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU
-** General Public License version 3.0 as published by the Free Software
-** Foundation and appearing in the file LICENSE.GPL included in the
-** packaging of this file.  Please review the following information to
-** ensure the GNU General Public License version 3.0 requirements will be
-** met: http://www.gnu.org/copyleft/gpl.html.
+** Alternatively, this file may be used under the terms of the GNU General
+** Public License version 3.0 as published by the Free Software Foundation
+** and appearing in the file LICENSE.GPL included in the packaging of this
+** file. Please review the following information to ensure the GNU General
+** Public License version 3.0 requirements will be met:
+** http://www.gnu.org/copyleft/gpl.html.
+**
+** Other Usage
+** Alternatively, this file may be used in accordance with the terms and
+** conditions contained in a signed written agreement between you and Nokia.
+**
+**
+**
+**
 **
-** If you have questions regarding the use of this file, please contact
-** Nokia at qt-info@nokia.com.
 ** $QT_END_LICENSE$
 **
 ****************************************************************************/
@@ -45,6 +45,8 @@
 #include "qfilesystemwatcher.h"
 #include "qfilesystemwatcher_fsevents_p.h"
 
+#ifndef QT_NO_FILESYSTEMWATCHER
+
 #include <qdebug.h>
 #include <qfile.h>
 #include <qdatetime.h>
@@ -55,13 +57,15 @@
 #include <sys/types.h>
 #include <CoreFoundation/CFRunLoop.h>
 #include <CoreFoundation/CFUUID.h>
+#if !defined( QT_NO_CORESERVICES )
 #include <CoreServices/CoreServices.h>
+#endif
 #include <AvailabilityMacros.h>
 #include <private/qcore_mac_p.h>
 
 QT_BEGIN_NAMESPACE
 
-#if MAC_OS_X_VERSION_MAX_ALLOWED >= MAC_OS_X_VERSION_10_5
+#if MAC_OS_X_VERSION_MAX_ALLOWED >= MAC_OS_X_VERSION_10_5 && !defined( QT_NO_CORESERVICES )
 // Static operator overloading so for the sake of some convieniece.
 // They only live in this compilation unit to avoid polluting Qt in general.
 static bool operator==(const struct ::timespec &left, const struct ::timespec &right)
@@ -94,7 +98,7 @@ static void addPathToHash(PathHash &pathHash, const QString &key, const QFileInf
 {
     PathInfoList &list = pathHash[key];
     list.push_back(PathInfo(path,
-                            fileInfo.absoluteFilePath().normalized(QString::NormalizationForm_D).toUtf8()));
+                            fileInfo.canonicalFilePath().normalized(QString::NormalizationForm_D).toUtf8()));
     pathHash.insert(key, list);
 }
 
@@ -150,7 +154,7 @@ QFSEventsFileSystemWatcherEngine::QFSEventsFileSystemWatcherEngine()
 
 QFSEventsFileSystemWatcherEngine::~QFSEventsFileSystemWatcherEngine()
 {
-#if MAC_OS_X_VERSION_MAX_ALLOWED >= MAC_OS_X_VERSION_10_5
+#if MAC_OS_X_VERSION_MAX_ALLOWED >= MAC_OS_X_VERSION_10_5 && !defined( QT_NO_CORESERVICES )
     // I assume that at this point, QFileSystemWatcher has already called stop
     // on me, so I don't need to invalidate or stop my stream, simply
     // release it.
@@ -169,8 +173,9 @@ QStringList QFSEventsFileSystemWatcherEngine::addPaths(const QStringList &paths,
                                                        QStringList *files,
                                                        QStringList *directories)
 {
-#if MAC_OS_X_VERSION_MAX_ALLOWED >= MAC_OS_X_VERSION_10_5
+#if MAC_OS_X_VERSION_MAX_ALLOWED >= MAC_OS_X_VERSION_10_5 && !defined( QT_NO_CORESERVICES )
     stop();
+    wait();
     QMutexLocker locker(&mutex);
     QStringList failedToAdd;
     // if we have a running FSStreamEvent, we have to kill it, we'll re-add the stream soon.
@@ -206,7 +211,7 @@ QStringList QFSEventsFileSystemWatcherEngine::addPaths(const QStringList &paths,
             } else {
                 directories->append(path);
                 // Full file path for dirs.
-                QCFString cfpath(createFSStreamPath(fileInfo.absoluteFilePath()));
+                QCFString cfpath(createFSStreamPath(fileInfo.canonicalFilePath()));
                 addPathToHash(dirPathInfoHash, cfpath, fileInfo, path);
                 CFArrayAppendValue(tmpArray, cfpath);
             }
@@ -216,7 +221,7 @@ QStringList QFSEventsFileSystemWatcherEngine::addPaths(const QStringList &paths,
                 continue;
             } else {
                 // Just the absolute path (minus it's filename) for files.
-                QCFString cfpath(createFSStreamPath(fileInfo.absolutePath()));
+                QCFString cfpath(createFSStreamPath(fileInfo.canonicalPath()));
                 files->append(path);
                 addPathToHash(filePathInfoHash, cfpath, fileInfo, path);
                 CFArrayAppendValue(tmpArray, cfpath);
@@ -254,7 +259,7 @@ QStringList QFSEventsFileSystemWatcherEngine::addPaths(const QStringList &paths,
 
 void QFSEventsFileSystemWatcherEngine::warmUpFSEvents()
 {
-#if MAC_OS_X_VERSION_MAX_ALLOWED >= MAC_OS_X_VERSION_10_5
+#if MAC_OS_X_VERSION_MAX_ALLOWED >= MAC_OS_X_VERSION_10_5 && !defined( QT_NO_CORESERVICES )
     // This function assumes that the mutex has already been grabbed before calling it.
     // It exits with the mutex still locked (Q_ASSERT(mutex.isLocked()) ;-).
     start();
@@ -266,8 +271,9 @@ QStringList QFSEventsFileSystemWatcherEngine::removePaths(const QStringList &pat
                                                           QStringList *files,
                                                           QStringList *directories)
 {
-#if MAC_OS_X_VERSION_MAX_ALLOWED >= MAC_OS_X_VERSION_10_5
+#if MAC_OS_X_VERSION_MAX_ALLOWED >= MAC_OS_X_VERSION_10_5 && !defined( QT_NO_CORESERVICES )
     stop();
+    wait();
     QMutexLocker locker(&mutex);
     // short circuit for smarties that call remove before add and we have nothing.
     if (pathsToWatch == 0)
@@ -293,7 +299,7 @@ QStringList QFSEventsFileSystemWatcherEngine::removePaths(const QStringList &pat
         itemCount = CFArrayGetCount(tmpArray);
         const QString &path = paths.at(i);
         QFileInfo fi(path);
-        QCFString cfpath(createFSStreamPath(fi.absolutePath()));
+        QCFString cfpath(createFSStreamPath(fi.canonicalPath()));
 
         CFIndex index = CFArrayGetFirstIndexOfValue(tmpArray, CFRangeMake(0, itemCount), cfpath);
         if (index != -1) {
@@ -302,7 +308,7 @@ QStringList QFSEventsFileSystemWatcherEngine::removePaths(const QStringList &pat
             removePathFromHash(filePathInfoHash, cfpath, path);
         } else {
             // Could be a directory we are watching instead.
-            QCFString cfdirpath(createFSStreamPath(fi.absoluteFilePath()));
+            QCFString cfdirpath(createFSStreamPath(fi.canonicalFilePath()));
             index = CFArrayGetFirstIndexOfValue(tmpArray, CFRangeMake(0, itemCount), cfdirpath);
             if (index != -1) {
                 CFArrayRemoveValueAtIndex(tmpArray, index);
@@ -332,7 +338,7 @@ QStringList QFSEventsFileSystemWatcherEngine::removePaths(const QStringList &pat
 #endif
 }
 
-#if MAC_OS_X_VERSION_MAX_ALLOWED >= MAC_OS_X_VERSION_10_5
+#if MAC_OS_X_VERSION_MAX_ALLOWED >= MAC_OS_X_VERSION_10_5 && !defined( QT_NO_CORESERVICES )
 void QFSEventsFileSystemWatcherEngine::updateList(PathInfoList &list, bool directory, bool emitSignals)
 {
     PathInfoList::iterator End = list.end();
@@ -392,7 +398,7 @@ void QFSEventsFileSystemWatcherEngine::fseventsCallback(ConstFSEventStreamRef ,
                                                         const FSEventStreamEventFlags eventFlags[],
                                                         const FSEventStreamEventId [])
 {
-#if MAC_OS_X_VERSION_MAX_ALLOWED >= MAC_OS_X_VERSION_10_5
+#if MAC_OS_X_VERSION_MAX_ALLOWED >= MAC_OS_X_VERSION_10_5 && !defined( QT_NO_CORESERVICES )
     QFSEventsFileSystemWatcherEngine *watcher = static_cast<QFSEventsFileSystemWatcherEngine *>(clientCallBackInfo);
     QMutexLocker locker(&watcher->mutex);
     CFArrayRef paths = static_cast<CFArrayRef>(eventPaths);
@@ -427,7 +433,7 @@ void QFSEventsFileSystemWatcherEngine::fseventsCallback(ConstFSEventStreamRef ,
 
 void QFSEventsFileSystemWatcherEngine::stop()
 {
-#if MAC_OS_X_VERSION_MAX_ALLOWED >= MAC_OS_X_VERSION_10_5
+#if MAC_OS_X_VERSION_MAX_ALLOWED >= MAC_OS_X_VERSION_10_5 && !defined( QT_NO_CORESERVICES )
     QMutexLocker locker(&mutex);
     stopFSStream(fsStream);
     if (threadsRunLoop) {
@@ -439,13 +445,22 @@ void QFSEventsFileSystemWatcherEngine::stop()
 
 void QFSEventsFileSystemWatcherEngine::updateFiles()
 {
-#if MAC_OS_X_VERSION_MAX_ALLOWED >= MAC_OS_X_VERSION_10_5
+#if MAC_OS_X_VERSION_MAX_ALLOWED >= MAC_OS_X_VERSION_10_5 && !defined( QT_NO_CORESERVICES )
     QMutexLocker locker(&mutex);
     updateHash(filePathInfoHash);
     updateHash(dirPathInfoHash);
     if (filePathInfoHash.isEmpty() && dirPathInfoHash.isEmpty()) {
         // Everything disappeared before we got to start, don't bother.
-        stop();
+#if MAC_OS_X_VERSION_MAX_ALLOWED >= MAC_OS_X_VERSION_10_5 && !defined( QT_NO_CORESERVICES )
+        // Code duplicated from stop(), with the exception that we
+        // don't wait on waitForStop here. Doing this will lead to
+        // a deadlock since this function is called from the worker
+        // thread. (waitForStop.wakeAll() is only called from the
+        // end of run()).
+        stopFSStream(fsStream);
+        if (threadsRunLoop)
+            CFRunLoopStop(threadsRunLoop);
+#endif
         cleanupFSStream(fsStream);
     }
     waitCondition.wakeAll();
@@ -454,7 +469,7 @@ void QFSEventsFileSystemWatcherEngine::updateFiles()
 
 void QFSEventsFileSystemWatcherEngine::run()
 {
-#if MAC_OS_X_VERSION_MAX_ALLOWED >= MAC_OS_X_VERSION_10_5
+#if MAC_OS_X_VERSION_MAX_ALLOWED >= MAC_OS_X_VERSION_10_5 && !defined( QT_NO_CORESERVICES )
     threadsRunLoop = CFRunLoopGetCurrent();
     FSEventStreamScheduleWithRunLoop(fsStream, threadsRunLoop, kCFRunLoopDefaultMode);
     bool startedOK = FSEventStreamStart(fsStream);
@@ -476,3 +491,4 @@ void QFSEventsFileSystemWatcherEngine::run()
 }
 
 QT_END_NAMESPACE
+#endif //QT_NO_FILESYSTEMWATCHER
