@@ -1,40 +1,40 @@
 /****************************************************************************
 **
-** Copyright (C) 2010 Nokia Corporation and/or its subsidiary(-ies).
+** Copyright (C) 2011 Nokia Corporation and/or its subsidiary(-ies).
 ** All rights reserved.
 ** Contact: Nokia Corporation (qt-info@nokia.com)
 **
 ** This file is part of the QtCore module of the Qt Toolkit.
 **
 ** $QT_BEGIN_LICENSE:LGPL$
-** Commercial Usage
-** Licensees holding valid Qt Commercial licenses may use this file in
-** accordance with the Qt Commercial License Agreement provided with the
-** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and Nokia.
-**
 ** GNU Lesser General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU Lesser
-** General Public License version 2.1 as published by the Free Software
-** Foundation and appearing in the file LICENSE.LGPL included in the
-** packaging of this file.  Please review the following information to
-** ensure the GNU Lesser General Public License version 2.1 requirements
-** will be met: http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
+** This file may be used under the terms of the GNU Lesser General Public
+** License version 2.1 as published by the Free Software Foundation and
+** appearing in the file LICENSE.LGPL included in the packaging of this
+** file. Please review the following information to ensure the GNU Lesser
+** General Public License version 2.1 requirements will be met:
+** http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
 **
 ** In addition, as a special exception, Nokia gives you certain additional
-** rights.  These rights are described in the Nokia Qt LGPL Exception
+** rights. These rights are described in the Nokia Qt LGPL Exception
 ** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
 **
 ** GNU General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU
-** General Public License version 3.0 as published by the Free Software
-** Foundation and appearing in the file LICENSE.GPL included in the
-** packaging of this file.  Please review the following information to
-** ensure the GNU General Public License version 3.0 requirements will be
-** met: http://www.gnu.org/copyleft/gpl.html.
+** Alternatively, this file may be used under the terms of the GNU General
+** Public License version 3.0 as published by the Free Software Foundation
+** and appearing in the file LICENSE.GPL included in the packaging of this
+** file. Please review the following information to ensure the GNU General
+** Public License version 3.0 requirements will be met:
+** http://www.gnu.org/copyleft/gpl.html.
+**
+** Other Usage
+** Alternatively, this file may be used in accordance with the terms and
+** conditions contained in a signed written agreement between you and Nokia.
+**
+**
+**
+**
 **
-** If you have questions regarding the use of this file, please contact
-** Nokia at qt-info@nokia.com.
 ** $QT_END_LICENSE$
 **
 ****************************************************************************/
@@ -53,7 +53,6 @@
 #include "qabstracteventdispatcher_p.h"
 #include "qcoreapplication_p.h"
 #include <private/qthread_p.h>
-#include <private/qmutexpool_p.h>
 
 QT_BEGIN_NAMESPACE
 
@@ -78,13 +77,16 @@ extern uint qGlobalPostedEventsCount();
 #ifndef QT_NO_GESTURES
 #ifndef WM_GESTURE
 #  define WM_GESTURE 0x0119
+#endif
+#ifndef WM_GESTURENOTIFY
 #  define WM_GESTURENOTIFY 0x011A
 #endif
 #endif // QT_NO_GESTURES
 
 enum {
     WM_QT_SOCKETNOTIFIER = WM_USER,
-    WM_QT_SENDPOSTEDEVENTS = WM_USER + 1
+    WM_QT_SENDPOSTEDEVENTS = WM_USER + 1,
+    SendPostedEventsWindowsTimerId = ~1u
 };
 
 #if defined(Q_OS_WINCE)
@@ -319,19 +321,17 @@ static void resolveTimerAPI()
 {
     static bool triedResolve = false;
     if (!triedResolve) {
-#ifndef QT_NO_THREAD
-        QMutexLocker locker(QMutexPool::globalInstanceGet(&triedResolve));
-        if (triedResolve)
-            return;
-#endif
-        triedResolve = true;
-#if !defined(Q_OS_WINCE)
-        qtimeSetEvent = (ptimeSetEvent)QSystemLibrary::resolve(QLatin1String("winmm"), "timeSetEvent");
-        qtimeKillEvent = (ptimeKillEvent)QSystemLibrary::resolve(QLatin1String("winmm"), "timeKillEvent");
+#ifndef Q_OS_WINCE
+        QSystemLibrary library(QLatin1String("Mmtimer"));
 #else
-        qtimeSetEvent = (ptimeSetEvent)QSystemLibrary::resolve(QLatin1String("Mmtimer"), "timeSetEvent");
-        qtimeKillEvent = (ptimeKillEvent)QSystemLibrary::resolve(QLatin1String("Mmtimer"), "timeKillEvent");
+        QSystemLibrary library(QLatin1String("winmm"));
 #endif
+        if (library.load()) {
+            qtimeSetEvent = (ptimeSetEvent)library.resolve("timeSetEvent");
+            qtimeKillEvent = (ptimeKillEvent)library.resolve("timeKillEvent");
+        }
+
+        triedResolve = true;
     }
 }
 
@@ -353,10 +353,7 @@ public:
 
     // for controlling when to send posted events
     QAtomicInt serialNumber;
-    int lastSerialNumber;
-#ifndef Q_OS_WINCE
-    int lastMessageTime;
-#endif
+    int lastSerialNumber, sendPostedEventsWindowsTimerId;
     QAtomicInt wakeUps;
 
     // timers
@@ -381,11 +378,7 @@ public:
 
 QEventDispatcherWin32Private::QEventDispatcherWin32Private()
     : threadId(GetCurrentThreadId()), interrupt(false), internalHwnd(0), getMessageHook(0),
-      serialNumber(0), lastSerialNumber(0),
-#ifndef Q_OS_WINCE
-      lastMessageTime(0),
-#endif
-      wakeUps(0)
+      serialNumber(0), lastSerialNumber(0), sendPostedEventsWindowsTimerId(0), wakeUps(0)
 {
     resolveTimerAPI();
 }
@@ -492,20 +485,21 @@ LRESULT QT_WIN_CALLBACK qt_internal_proc(HWND hwnd, UINT message, WPARAM wp, LPA
             }
         }
         return 0;
-    } else if (message == WM_TIMER) {    
-        Q_ASSERT(d != 0);
-        d->sendTimerEvent(wp);
-        return 0;
-    } else if (message == WM_QT_SENDPOSTEDEVENTS) {
+    } else if (message == WM_QT_SENDPOSTEDEVENTS
+               // we also use a Windows timer to send posted events when the message queue is full
+               || (message == WM_TIMER
+                   && d->sendPostedEventsWindowsTimerId != 0
+                   && wp == (uint)d->sendPostedEventsWindowsTimerId)) {
         int localSerialNumber = d->serialNumber;
         if (localSerialNumber != d->lastSerialNumber) {
             d->lastSerialNumber = localSerialNumber;
-#ifndef Q_OS_WINCE
-            d->lastMessageTime = GetMessageTime();
-#endif
             QCoreApplicationPrivate::sendPostedEvents(0, 0, d->threadData);
         }
         return 0;
+    } else if (message == WM_TIMER) {
+        Q_ASSERT(d != 0);
+        d->sendTimerEvent(wp);
+        return 0;
     }
 
     return DefWindowProc(hwnd, message, wp, lp);
@@ -517,25 +511,33 @@ LRESULT QT_WIN_CALLBACK qt_GetMessageHook(int code, WPARAM wp, LPARAM lp)
         QEventDispatcherWin32 *q = qobject_cast<QEventDispatcherWin32 *>(QAbstractEventDispatcher::instance());
         Q_ASSERT(q != 0);
         if (q) {
+            MSG *msg = (MSG *) lp;
             QEventDispatcherWin32Private *d = q->d_func();
             int localSerialNumber = d->serialNumber;
-            MSG unused;
-            if ((HIWORD(GetQueueStatus(QS_INPUT | QS_RAWINPUT)) == 0
-                 && PeekMessage(&unused, 0, WM_TIMER, WM_TIMER, PM_NOREMOVE) == 0)
-#ifndef Q_OS_WINCE
-                || GetMessageTime() - d->lastMessageTime >= 10
-#endif
-                ) {
-                // no more input or timer events in the message queue or more than 10ms has elapsed since
-                // we send posted events, we can allow posted events to be sent now
+            if (HIWORD(GetQueueStatus(QS_TIMER | QS_INPUT | QS_RAWINPUT)) == 0) {
+                // no more input or timer events in the message queue, we can allow posted events to be sent normally now
+                if (d->sendPostedEventsWindowsTimerId != 0) {
+                    // stop the timer to send posted events, since we now allow the WM_QT_SENDPOSTEDEVENTS message
+                    KillTimer(d->internalHwnd, d->sendPostedEventsWindowsTimerId);
+                    d->sendPostedEventsWindowsTimerId = 0;
+                }
                 (void) d->wakeUps.fetchAndStoreRelease(0);
-                MSG *msg = (MSG *) lp;
                 if (localSerialNumber != d->lastSerialNumber
                     // if this message IS the one that triggers sendPostedEvents(), no need to post it again
                     && (msg->hwnd != d->internalHwnd
                         || msg->message != WM_QT_SENDPOSTEDEVENTS)) {
                     PostMessage(d->internalHwnd, WM_QT_SENDPOSTEDEVENTS, 0, 0);
                 }
+            } else if (d->sendPostedEventsWindowsTimerId == 0
+                       && localSerialNumber != d->lastSerialNumber) {
+                // start a special timer to continue delivering posted events while
+                // there are still input and timer messages in the message queue
+                d->sendPostedEventsWindowsTimerId = SetTimer(d->internalHwnd,
+                                                             SendPostedEventsWindowsTimerId,
+                                                             0, // we specify zero, but Windows uses USER_TIMER_MINIMUM
+                                                             NULL);
+                // we don't check the return value of SetTimer()... if creating the timer failed, there's little
+                // we can do. we just have to accept that posted events will be starved
             }
         }
     }
