@@ -1,40 +1,40 @@
 /****************************************************************************
 **
-** Copyright (C) 2009 Nokia Corporation and/or its subsidiary(-ies).
+** Copyright (C) 2011 Nokia Corporation and/or its subsidiary(-ies).
 ** All rights reserved.
 ** Contact: Nokia Corporation (qt-info@nokia.com)
 **
 ** This file is part of the QtCore module of the Qt Toolkit.
 **
 ** $QT_BEGIN_LICENSE:LGPL$
-** Commercial Usage
-** Licensees holding valid Qt Commercial licenses may use this file in
-** accordance with the Qt Commercial License Agreement provided with the
-** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and Nokia.
-**
 ** GNU Lesser General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU Lesser
-** General Public License version 2.1 as published by the Free Software
-** Foundation and appearing in the file LICENSE.LGPL included in the
-** packaging of this file.  Please review the following information to
-** ensure the GNU Lesser General Public License version 2.1 requirements
-** will be met: http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
+** This file may be used under the terms of the GNU Lesser General Public
+** License version 2.1 as published by the Free Software Foundation and
+** appearing in the file LICENSE.LGPL included in the packaging of this
+** file. Please review the following information to ensure the GNU Lesser
+** General Public License version 2.1 requirements will be met:
+** http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
 **
 ** In addition, as a special exception, Nokia gives you certain additional
-** rights.  These rights are described in the Nokia Qt LGPL Exception
+** rights. These rights are described in the Nokia Qt LGPL Exception
 ** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
 **
 ** GNU General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU
-** General Public License version 3.0 as published by the Free Software
-** Foundation and appearing in the file LICENSE.GPL included in the
-** packaging of this file.  Please review the following information to
-** ensure the GNU General Public License version 3.0 requirements will be
-** met: http://www.gnu.org/copyleft/gpl.html.
+** Alternatively, this file may be used under the terms of the GNU General
+** Public License version 3.0 as published by the Free Software Foundation
+** and appearing in the file LICENSE.GPL included in the packaging of this
+** file. Please review the following information to ensure the GNU General
+** Public License version 3.0 requirements will be met:
+** http://www.gnu.org/copyleft/gpl.html.
+**
+** Other Usage
+** Alternatively, this file may be used in accordance with the terms and
+** conditions contained in a signed written agreement between you and Nokia.
+**
+**
+**
+**
 **
-** If you have questions regarding the use of this file, please contact
-** Nokia at qt-info@nokia.com.
 ** $QT_END_LICENSE$
 **
 ****************************************************************************/
@@ -218,12 +218,20 @@ QObject *QMetaObject::newInstance(QGenericArgument val0,
 */
 int QMetaObject::static_metacall(Call cl, int idx, void **argv) const
 {
-    if (priv(d.data)->revision < 2)
-        return 0;
-    const QMetaObjectExtraData *extra = (const QMetaObjectExtraData*)(d.extradata);
-    if (!extra || !extra->static_metacall)
-        return 0;
-    return extra->static_metacall(cl, idx, argv);
+    const QMetaObjectExtraData *extra = reinterpret_cast<const QMetaObjectExtraData *>(d.extradata);
+    if (priv(d.data)->revision >= 6) {
+        if (!extra || !extra->static_metacall)
+            return 0;
+        extra->static_metacall(0, cl, idx, argv);
+        return -1;
+    } else if (priv(d.data)->revision >= 2) {
+        if (!extra || !extra->static_metacall)
+            return 0;
+        typedef int (*OldMetacall)(QMetaObject::Call, int, void **);
+        OldMetacall o = reinterpret_cast<OldMetacall>(extra->static_metacall);
+        return o(cl, idx, argv);
+    }
+    return 0;
 }
 
 /*!
@@ -266,7 +274,25 @@ QObject *QMetaObject::cast(QObject *obj) const
         const QMetaObject *m = obj->metaObject();
         do {
             if (m == this)
-                return const_cast<QObject*>(obj);
+                return obj;
+        } while ((m = m->d.superdata));
+    }
+    return 0;
+}
+
+/*!
+    \internal
+
+    Returns \a obj if object \a obj inherits from this
+    meta-object; otherwise returns 0.
+*/
+const QObject *QMetaObject::cast(const QObject *obj) const
+{
+    if (obj) {
+        const QMetaObject *m = obj->metaObject();
+        do {
+            if (m == this)
+                return obj;
         } while ((m = m->d.superdata));
     }
     return 0;
@@ -406,9 +432,10 @@ int QMetaObject::constructorCount() const
 }
 
 /*!
-    Returns the number of methods in this class, including the number of
-    properties provided by each base class. These include signals and slots
-    as well as normal member functions.
+    Returns the number of methods known to the meta-object system in this class,
+    including the number of properties provided by each base class. These
+    include signals and slots as well as member functions declared with the
+    Q_INVOKABLE macro.
 
     Use code like the following to obtain a QStringList containing the methods
     specific to a given class:
@@ -482,6 +509,45 @@ int QMetaObject::classInfoCount() const
     return n;
 }
 
+/** \internal
+* helper function for indexOf{Method,Slot,Signal}, returns the relative index of the method within
+* the baseObject
+* \a MethodType might be MethodSignal or MethodSlot, or 0 to match everything.
+* \a normalizeStringData set to true if we should do a second pass for old moc generated files normalizing all the symbols.
+*/
+template<int MethodType>
+static inline int indexOfMethodRelative(const QMetaObject **baseObject,
+                                        const char *method,
+                                        bool normalizeStringData)
+{
+    for (const QMetaObject *m = *baseObject; m; m = m->d.superdata) {
+        int i = (MethodType == MethodSignal && priv(m->d.data)->revision >= 4)
+                ? (priv(m->d.data)->signalCount - 1) : (priv(m->d.data)->methodCount - 1);
+        const int end = (MethodType == MethodSlot && priv(m->d.data)->revision >= 4)
+                        ? (priv(m->d.data)->signalCount) : 0;
+        if (!normalizeStringData) {
+            for (; i >= end; --i) {
+                const char *stringdata = m->d.stringdata + m->d.data[priv(m->d.data)->methodData + 5*i];
+                if (method[0] == stringdata[0] && strcmp(method + 1, stringdata + 1) == 0) {
+                    *baseObject = m;
+                    return i;
+                }
+            }
+        } else if (priv(m->d.data)->revision < 5) {
+            for (; i >= end; --i) {
+                const char *stringdata = (m->d.stringdata + m->d.data[priv(m->d.data)->methodData + 5 * i]);
+                const QByteArray normalizedSignature = QMetaObject::normalizedSignature(stringdata);
+                if (normalizedSignature == method) {
+                    *baseObject = m;
+                    return i;
+                }
+            }
+        }
+    }
+    return -1;
+}
+
+
 /*!
     \since 4.5
 
@@ -497,8 +563,8 @@ int QMetaObject::indexOfConstructor(const char *constructor) const
     if (priv(d.data)->revision < 2)
         return -1;
     for (int i = priv(d.data)->constructorCount-1; i >= 0; --i) {
-        if (strcmp(constructor, d.stringdata
-                   + d.data[priv(d.data)->constructorData + 5*i]) == 0) {
+        const char *data = d.stringdata + d.data[priv(d.data)->constructorData + 5*i];
+        if (data[0] == constructor[0] && strcmp(constructor + 1, data + 1) == 0) {
             return i;
         }
     }
@@ -515,17 +581,14 @@ int QMetaObject::indexOfConstructor(const char *constructor) const
 */
 int QMetaObject::indexOfMethod(const char *method) const
 {
-    int i = -1;
     const QMetaObject *m = this;
-    while (m && i < 0) {
-        for (i = priv(m->d.data)->methodCount-1; i >= 0; --i)
-            if (strcmp(method, m->d.stringdata
-                       + m->d.data[priv(m->d.data)->methodData + 5*i]) == 0) {
-                i += m->methodOffset();
-                break;
-            }
-        m = m->d.superdata;
+    int i = indexOfMethodRelative<0>(&m, method, false);
+    if (i < 0) {
+        m = this;
+        i = indexOfMethodRelative<0>(&m, method, true);
     }
+    if (i >= 0)
+        i += m->methodOffset();
     return i;
 }
 
@@ -543,7 +606,11 @@ int QMetaObject::indexOfMethod(const char *method) const
 int QMetaObject::indexOfSignal(const char *signal) const
 {
     const QMetaObject *m = this;
-    int i = QMetaObjectPrivate::indexOfSignalRelative(&m, signal);
+    int i = QMetaObjectPrivate::indexOfSignalRelative(&m, signal, false);
+    if (i < 0) {
+        m = this;
+        i = QMetaObjectPrivate::indexOfSignalRelative(&m, signal, true);
+    }
     if (i >= 0)
         i += m->methodOffset();
     return i;
@@ -554,21 +621,11 @@ int QMetaObject::indexOfSignal(const char *signal) const
 
     \a baseObject will be adjusted to the enclosing QMetaObject, or 0 if the signal is not found
 */
-int QMetaObjectPrivate::indexOfSignalRelative(const QMetaObject **baseObject, const char *signal)
+int QMetaObjectPrivate::indexOfSignalRelative(const QMetaObject **baseObject,
+                                              const char *signal,
+                                              bool normalizeStringData)
 {
-    int i = -1;
-    while (*baseObject) {
-        const QMetaObject *const m = *baseObject;
-        for (i = priv(m->d.data)->methodCount-1; i >= 0; --i)
-            if ((m->d.data[priv(m->d.data)->methodData + 5*i + 4] & MethodTypeMask) == MethodSignal
-                && strcmp(signal, m->d.stringdata
-                + m->d.data[priv(m->d.data)->methodData + 5*i]) == 0) {
-                break;
-            }
-        if (i >= 0)
-            break;
-        *baseObject = m->d.superdata;
-    }
+    int i = indexOfMethodRelative<MethodSignal>(baseObject, signal, normalizeStringData);
 #ifndef QT_NO_DEBUG
     const QMetaObject *m = *baseObject;
     if (i >= 0 && m && m->d.superdata) {
@@ -581,7 +638,6 @@ int QMetaObjectPrivate::indexOfSignalRelative(const QMetaObject **baseObject, co
     return i;
 }
 
-
 /*!
     Finds \a slot and returns its index; otherwise returns -1.
 
@@ -592,21 +648,23 @@ int QMetaObjectPrivate::indexOfSignalRelative(const QMetaObject **baseObject, co
 */
 int QMetaObject::indexOfSlot(const char *slot) const
 {
-    int i = -1;
     const QMetaObject *m = this;
-    while (m && i < 0) {
-        for (i = priv(m->d.data)->methodCount-1; i >= 0; --i)
-            if ((m->d.data[priv(m->d.data)->methodData + 5*i + 4] & MethodTypeMask) == MethodSlot
-                && strcmp(slot, m->d.stringdata
-                       + m->d.data[priv(m->d.data)->methodData + 5*i]) == 0) {
-                i += m->methodOffset();
-                break;
-            }
-        m = m->d.superdata;
-    }
+    int i = QMetaObjectPrivate::indexOfSlotRelative(&m, slot, false);
+    if (i < 0)
+        i = QMetaObjectPrivate::indexOfSlotRelative(&m, slot, true);
+    if (i >= 0)
+        i += m->methodOffset();
     return i;
 }
 
+// same as indexOfSignalRelative but for slots.
+int QMetaObjectPrivate::indexOfSlotRelative(const QMetaObject **m,
+                                    const char *slot,
+                                    bool normalizeStringData)
+{
+    return indexOfMethodRelative<MethodSlot>(m, slot, normalizeStringData);
+}
+
 static const QMetaObject *QMetaObject_findMetaObject(const QMetaObject *self, const char *name)
 {
     while (self) {
@@ -651,18 +709,19 @@ static const QMetaObject *QMetaObject_findMetaObject(const QMetaObject *self, co
 */
 int QMetaObject::indexOfEnumerator(const char *name) const
 {
-    int i = -1;
     const QMetaObject *m = this;
-    while (m && i < 0) {
-        for (i = priv(m->d.data)->enumeratorCount-1; i >= 0; --i)
-            if (strcmp(name, m->d.stringdata
-                       + m->d.data[priv(m->d.data)->enumeratorData + 4*i]) == 0) {
+    while (m) {
+        const QMetaObjectPrivate *d = priv(m->d.data);
+        for (int i = d->enumeratorCount - 1; i >= 0; --i) {
+            const char *prop = m->d.stringdata + m->d.data[d->enumeratorData + 4*i];
+            if (name[0] == prop[0] && strcmp(name + 1, prop + 1) == 0) {
                 i += m->enumeratorOffset();
-                break;
+                return i;
             }
+        }
         m = m->d.superdata;
     }
-    return i;
+    return -1;
 }
 
 /*!
@@ -673,26 +732,27 @@ int QMetaObject::indexOfEnumerator(const char *name) const
 */
 int QMetaObject::indexOfProperty(const char *name) const
 {
-    int i = -1;
     const QMetaObject *m = this;
-    while (m && i < 0) {
-        for (i = priv(m->d.data)->propertyCount-1; i >= 0; --i)
-            if (strcmp(name, m->d.stringdata
-                       + m->d.data[priv(m->d.data)->propertyData + 3*i]) == 0) {
+    while (m) {
+        const QMetaObjectPrivate *d = priv(m->d.data);
+        for (int i = d->propertyCount-1; i >= 0; --i) {
+            const char *prop = m->d.stringdata + m->d.data[d->propertyData + 3*i];
+            if (name[0] == prop[0] && strcmp(name + 1, prop + 1) == 0) {
                 i += m->propertyOffset();
-                break;
+                return i;
             }
+        }
         m = m->d.superdata;
     }
 
-    if (i == -1 && priv(this->d.data)->revision >= 3 && (priv(this->d.data)->flags & DynamicMetaObject)){
+    if (priv(this->d.data)->revision >= 3 && (priv(this->d.data)->flags & DynamicMetaObject)) {
         QAbstractDynamicMetaObject *me = 
             const_cast<QAbstractDynamicMetaObject *>(static_cast<const QAbstractDynamicMetaObject *>(this));
 
-        i = me->createProperty(name, 0);
+        return me->createProperty(name, 0);
     }
 
-    return i;
+    return -1;
 }
 
 /*!
@@ -943,7 +1003,7 @@ QByteArray QMetaObject::normalizedType(const char *type)
     if (!type || !*type)
         return result;
 
-    QVarLengthArray<char> stackbuf(int(strlen(type)) + 1);
+    QVarLengthArray<char> stackbuf(qstrlen(type) + 1);
     qRemoveWhitespace(type, stackbuf.data());
     int templdepth = 0;
     qNormalizeType(stackbuf.data(), templdepth, result);
@@ -977,8 +1037,11 @@ QByteArray QMetaObject::normalizedSignature(const char *method)
     int argdepth = 0;
     int templdepth = 0;
     while (*d) {
-        if (argdepth == 1)
+        if (argdepth == 1) {
             d = qNormalizeType(d, templdepth, result);
+            if (!*d) //most likely an invalid signature.
+                break;
+        }
         if (*d == '(')
             ++argdepth;
         if (*d == ')')
@@ -1106,8 +1169,11 @@ bool QMetaObject::invokeMethod(QObject *obj,
         idx = obj->metaObject()->indexOfMethod(norm.constData());
     }
 
-    if (idx < 0 || idx >= obj->metaObject()->methodCount())
+    if (idx < 0 || idx >= obj->metaObject()->methodCount()) {
+        qWarning("QMetaObject::invokeMethod: No such method %s::%s",
+                 obj->metaObject()->className(), sig.constData());
         return false;
+    }
     QMetaMethod method = obj->metaObject()->method(idx);
     return method.invoke(obj, type, ret,
                          val0, val1, val2, val3, val4, val5, val6, val7, val8, val9);
@@ -1180,6 +1246,10 @@ bool QMetaObject::invokeMethod(QObject *obj,
     tag(), and an access() specifier. You can use invoke() to invoke
     the method on an arbitrary QObject.
 
+    A method will only be registered with the meta-object system if it
+    is a slot, a signal, or declared with the Q_INVOKABLE macro.
+    Constructors can also be registered with Q_INVOKABLE.
+
     \sa QMetaObject, QMetaEnum, QMetaProperty, {Qt's Property System}
 */
 
@@ -1330,6 +1400,25 @@ int QMetaMethod::methodIndex() const
     return ((handle - priv(mobj->d.data)->methodData) / 5) + mobj->methodOffset();
 }
 
+/*!
+    \internal
+
+    Returns the method revision if one was
+    specified by Q_REVISION, otherwise returns 0.
+ */
+int QMetaMethod::revision() const
+{
+    if (!mobj)
+        return 0;
+    if ((QMetaMethod::Access)(mobj->d.data[handle + 4] & MethodRevisioned)) {
+        int offset = priv(mobj->d.data)->methodData
+                     + priv(mobj->d.data)->methodCount * 5
+                     + (handle - priv(mobj->d.data)->methodData) / 5;
+        return mobj->d.data[offset];
+    }
+    return 0;
+}
+
 /*!
     Returns the access specification of this method (private,
     protected, or public).
@@ -1419,6 +1508,11 @@ QMetaMethod::MethodType QMetaMethod::methodType() const
     If the "compute" slot does not take exactly one QString, one int
     and one double in the specified order, the call will fail.
 
+    \warning this method will not test the validity of the arguments: \a object
+    must be an instance of the class of the QMetaObject of which this QMetaMethod
+    has been constructed with.  The arguments must have the same type as the ones
+    expected by the method, else, the behaviour is undefined.
+
     \sa Q_ARG(), Q_RETURN_ARG(), qRegisterMetaType(), QMetaObject::invokeMethod()
 */
 bool QMetaMethod::invoke(QObject *object,
@@ -1438,6 +1532,8 @@ bool QMetaMethod::invoke(QObject *object,
     if (!object || !mobj)
         return false;
 
+    Q_ASSERT(mobj->cast(object));
+
     // check return type
     if (returnValue.data()) {
         const char *retType = typeName();
@@ -1513,6 +1609,12 @@ bool QMetaMethod::invoke(QObject *object,
                          : Qt::QueuedConnection;
     }
 
+#ifdef QT_NO_THREAD
+    if (connectionType == Qt::BlockingQueuedConnection) {
+        connectionType = Qt::DirectConnection;
+    }
+#endif
+
     // invoke!
     void *param[] = {
         returnValue.data(),
@@ -1528,10 +1630,20 @@ bool QMetaMethod::invoke(QObject *object,
         val9.data()
     };
     // recompute the methodIndex by reversing the arithmetic in QMetaObject::property()
-    int methodIndex = ((handle - priv(mobj->d.data)->methodData) / 5) + mobj->methodOffset();
+    int idx_relative = ((handle - priv(mobj->d.data)->methodData) / 5);
+    int idx_offset =  mobj->methodOffset();
+    QObjectPrivate::StaticMetaCallFunction callFunction =
+        (QMetaObjectPrivate::get(mobj)->revision >= 6 && mobj->d.extradata)
+        ? reinterpret_cast<const QMetaObjectExtraData *>(mobj->d.extradata)->static_metacall : 0;
+
     if (connectionType == Qt::DirectConnection) {
-        return QMetaObject::metacall(object, QMetaObject::InvokeMetaMethod, methodIndex, param) < 0;
-    } else {
+        if (callFunction) {
+            callFunction(object, QMetaObject::InvokeMetaMethod, idx_relative, param);
+            return true;
+        } else {
+            return QMetaObject::metacall(object, QMetaObject::InvokeMetaMethod, idx_relative + idx_offset, param) < 0;
+        }
+    } else if (connectionType == Qt::QueuedConnection) {
         if (returnValue.data()) {
             qWarning("QMetaMethod::invoke: Unable to invoke methods with return values in "
                      "queued connections");
@@ -1564,40 +1676,21 @@ bool QMetaMethod::invoke(QObject *object,
             }
         }
 
-        if (connectionType == Qt::QueuedConnection) {
-            QCoreApplication::postEvent(object, new QMetaCallEvent(methodIndex,
-                                                                   0,
-                                                                   -1,
-                                                                   nargs,
-                                                                   types,
-                                                                   args));
-        } else {
-            if (currentThread == objectThread) {
-                qWarning("QMetaMethod::invoke: Dead lock detected in "
-                         "BlockingQueuedConnection: Receiver is %s(%p)",
-                         mobj->className(), object);
-            }
-
-            // blocking queued connection
-#ifdef QT_NO_THREAD
-            QCoreApplication::postEvent(object, new QMetaCallEvent(methodIndex,
-                                                                   0,
-                                                                   -1,
-                                                                   nargs,
-                                                                   types,
-                                                                   args));
-#else
-            QSemaphore semaphore;
-            QCoreApplication::postEvent(object, new QMetaCallEvent(methodIndex,
-                                                                   0,
-                                                                   -1,
-                                                                   nargs,
-                                                                   types,
-                                                                   args,
-                                                                   &semaphore));
-            semaphore.acquire();
-#endif // QT_NO_THREAD
+        QCoreApplication::postEvent(object, new QMetaCallEvent(idx_offset, idx_relative, callFunction,
+                                                        0, -1, nargs, types, args));
+    } else { // blocking queued connection
+#ifndef QT_NO_THREAD
+        if (currentThread == objectThread) {
+            qWarning("QMetaMethod::invoke: Dead lock detected in "
+                        "BlockingQueuedConnection: Receiver is %s(%p)",
+                        mobj->className(), object);
         }
+
+        QSemaphore semaphore;
+        QCoreApplication::postEvent(object, new QMetaCallEvent(idx_offset, idx_relative, callFunction,
+                                                        0, -1, 0, 0, param, &semaphore));
+        semaphore.acquire();
+#endif // QT_NO_THREAD
     }
     return true;
 }
@@ -2343,6 +2436,35 @@ int QMetaProperty::notifySignalIndex() const
     }
 }
 
+/*!
+    \internal
+
+    Returns the property revision if one was
+    specified by REVISION, otherwise returns 0.
+ */
+int QMetaProperty::revision() const
+{
+    if (!mobj)
+        return 0;
+    int flags = mobj->d.data[handle + 2];
+    if (flags & Revisioned) {
+        int offset = priv(mobj->d.data)->propertyData +
+                     priv(mobj->d.data)->propertyCount * 3 + idx;
+        // Revision data is placed after NOTIFY data, if present.
+        // Iterate through properties to discover whether we have NOTIFY signals.
+        for (int i = 0; i < priv(mobj->d.data)->propertyCount; ++i) {
+            int handle = priv(mobj->d.data)->propertyData + 3*i;
+            if (mobj->d.data[handle + 2] & Notify) {
+                offset += priv(mobj->d.data)->propertyCount;
+                break;
+            }
+        }
+        return mobj->d.data[offset];
+    } else {
+        return 0;
+    }
+}
+
 /*!
     Returns true if this property is writable; otherwise returns
     false.
@@ -2648,6 +2770,7 @@ const char* QMetaClassInfo::value() const
  */
 int QMetaObjectPrivate::originalClone(const QMetaObject *mobj, int local_method_index)
 {
+    Q_ASSERT(local_method_index < get(mobj)->methodCount);
     int handle = get(mobj)->methodData + 5 * local_method_index;
     while (mobj->d.data[handle + 4] & MethodCloned) {
         Q_ASSERT(local_method_index > 0);
