@@ -1,40 +1,40 @@
 /****************************************************************************
 **
-** Copyright (C) 2009 Nokia Corporation and/or its subsidiary(-ies).
+** Copyright (C) 2011 Nokia Corporation and/or its subsidiary(-ies).
 ** All rights reserved.
 ** Contact: Nokia Corporation (qt-info@nokia.com)
 **
 ** This file is part of the QtCore module of the Qt Toolkit.
 **
 ** $QT_BEGIN_LICENSE:LGPL$
-** Commercial Usage
-** Licensees holding valid Qt Commercial licenses may use this file in
-** accordance with the Qt Commercial License Agreement provided with the
-** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and Nokia.
-**
 ** GNU Lesser General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU Lesser
-** General Public License version 2.1 as published by the Free Software
-** Foundation and appearing in the file LICENSE.LGPL included in the
-** packaging of this file.  Please review the following information to
-** ensure the GNU Lesser General Public License version 2.1 requirements
-** will be met: http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
+** This file may be used under the terms of the GNU Lesser General Public
+** License version 2.1 as published by the Free Software Foundation and
+** appearing in the file LICENSE.LGPL included in the packaging of this
+** file. Please review the following information to ensure the GNU Lesser
+** General Public License version 2.1 requirements will be met:
+** http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
 **
 ** In addition, as a special exception, Nokia gives you certain additional
-** rights.  These rights are described in the Nokia Qt LGPL Exception
+** rights. These rights are described in the Nokia Qt LGPL Exception
 ** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
 **
 ** GNU General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU
-** General Public License version 3.0 as published by the Free Software
-** Foundation and appearing in the file LICENSE.GPL included in the
-** packaging of this file.  Please review the following information to
-** ensure the GNU General Public License version 3.0 requirements will be
-** met: http://www.gnu.org/copyleft/gpl.html.
+** Alternatively, this file may be used under the terms of the GNU General
+** Public License version 3.0 as published by the Free Software Foundation
+** and appearing in the file LICENSE.GPL included in the packaging of this
+** file. Please review the following information to ensure the GNU General
+** Public License version 3.0 requirements will be met:
+** http://www.gnu.org/copyleft/gpl.html.
+**
+** Other Usage
+** Alternatively, this file may be used in accordance with the terms and
+** conditions contained in a signed written agreement between you and Nokia.
+**
+**
+**
+**
 **
-** If you have questions regarding the use of this file, please contact
-** Nokia at qt-info@nokia.com.
 ** $QT_END_LICENSE$
 **
 ****************************************************************************/
@@ -42,32 +42,31 @@
 // Don't define it while compiling this module, or USERS of Qt will
 // not be able to link.
 #ifdef QT_NO_CAST_FROM_ASCII
-#undef QT_NO_CAST_FROM_ASCII
+#  undef QT_NO_CAST_FROM_ASCII
 #endif
 #ifdef QT_NO_CAST_TO_ASCII
-#undef QT_NO_CAST_TO_ASCII
+#  undef QT_NO_CAST_TO_ASCII
 #endif
 #include "qchar.h"
+
 #include "qdatastream.h"
 #include "qtextcodec.h"
 
 #include "qunicodetables_p.h"
-
 #include "qunicodetables.cpp"
 
 QT_BEGIN_NAMESPACE
 
-#define LAST_UNICODE_CHAR 0x10ffff
-
 #ifndef QT_NO_CODEC_FOR_C_STRINGS
-#ifdef QT_NO_TEXTCODEC
-#define QT_NO_CODEC_FOR_C_STRINGS
-#endif
+#  ifdef QT_NO_TEXTCODEC
+#    define QT_NO_CODEC_FOR_C_STRINGS
+#  endif
 #endif
 
 #define FLAG(x) (1 << (x))
 
-/*! \class QLatin1Char
+/*!
+    \class QLatin1Char
     \brief The QLatin1Char class provides an 8-bit ASCII/Latin-1 character.
 
     \ingroup string-processing
@@ -381,8 +380,12 @@ QT_BEGIN_NAMESPACE
 
     \value Null A QChar with this value isNull().
     \value Nbsp Non-breaking space.
-    \value ReplacementCharacter
-    \value ObjectReplacementCharacter The character shown when a font has no glyph for a certain codepoint. The square character is normally used.
+    \value ReplacementCharacter The character shown when a font has no glyph
+           for a certain codepoint. A special question mark character is often
+           used. Codecs use this codepoint when input data cannot be
+           represented in Unicode.
+    \value ObjectReplacementCharacter Used to represent an object such as an
+           image when such objects cannot be presented.
     \value ByteOrderMark
     \value ByteOrderSwapped
     \value ParagraphSeparator
@@ -550,7 +553,7 @@ bool QChar::isSpace() const
 /*!
     Returns true if the character is a mark (Mark_* categories);
     otherwise returns false.
-    
+
     See QChar::Category for more information regarding marks.
 */
 bool QChar::isMark() const
@@ -647,45 +650,71 @@ bool QChar::isSymbol() const
 }
 
 /*!
-  \fn bool QChar::isHighSurrogate() const
+    \fn bool QChar::isHighSurrogate() const
 
-  Returns true if the QChar is the high part of a utf16 surrogate
-  (ie. if its code point is between 0xd800 and 0xdbff).
+    Returns true if the QChar is the high part of a utf16 surrogate
+    (ie. if its code point is between 0xd800 and 0xdbff, inclusive).
 */
 
 /*!
-  \fn bool QChar::isLowSurrogate() const
+    \fn bool QChar::isLowSurrogate() const
 
-  Returns true if the QChar is the low part of a utf16 surrogate
-  (ie. if its code point is between 0xdc00 and 0xdfff).
+    Returns true if the QChar is the low part of a utf16 surrogate
+    (ie. if its code point is between 0xdc00 and 0xdfff, inclusive).
 */
 
 /*!
-  \fn static uint QChar::surrogateToUcs4(ushort high, ushort low)
+    \fn static bool QChar::isHighSurrogate(uint ucs4)
+    \since 4.7
 
-  Converts a UTF16 surrogate pair with the given \a high and \a low values
-  to its UCS-4 code point.
+    Returns true if the UCS-4-encoded character specified by \a ucs4
+    is the high part of a utf16 surrogate
+    (ie. if its code point is between 0xd800 and 0xdbff, inclusive).
 */
 
 /*!
-  \fn static uint QChar::surrogateToUcs4(QChar high, QChar low)
+    \fn static bool QChar::isLowSurrogate(uint ucs4)
+    \since 4.7
 
-  Converts a utf16 surrogate pair (\a high, \a low) to its ucs4 code
-  point.
+    Returns true if the UCS-4-encoded character specified by \a ucs4
+    is the low part of a utf16 surrogate
+    (ie. if its code point is between 0xdc00 and 0xdfff, inclusive).
 */
 
 /*!
-  \fn static ushort QChar::highSurrogate(uint ucs4)
+    \fn static bool QChar::requiresSurrogates(uint ucs4)
+    \since 4.7
 
-  Returns the high surrogate value of a ucs4 code point.
-  The returned result is undefined if \a ucs4 is smaller than 0x10000.
+    Returns true if the UCS-4-encoded character specified by \a ucs4
+    can be split into the high and low parts of a utf16 surrogate
+    (ie. if its code point is greater than or equals to 0x10000).
 */
 
 /*!
-  \fn static ushort QChar::lowSurrogate(uint ucs4)
+    \fn static uint QChar::surrogateToUcs4(ushort high, ushort low)
 
-  Returns the low surrogate value of a ucs4 code point.
-  The returned result is undefined if \a ucs4 is smaller than 0x10000.
+    Converts a UTF16 surrogate pair with the given \a high and \a low values
+    to its UCS-4 code point.
+*/
+
+/*!
+    \fn static uint QChar::surrogateToUcs4(QChar high, QChar low)
+
+    Converts a utf16 surrogate pair (\a high, \a low) to its ucs4 code point.
+*/
+
+/*!
+    \fn static ushort QChar::highSurrogate(uint ucs4)
+
+    Returns the high surrogate value of a ucs4 code point.
+    The returned result is undefined if \a ucs4 is smaller than 0x10000.
+*/
+
+/*!
+    \fn static ushort QChar::lowSurrogate(uint ucs4)
+
+    Returns the low surrogate value of a ucs4 code point.
+    The returned result is undefined if \a ucs4 is smaller than 0x10000.
 */
 
 /*!
@@ -714,7 +743,7 @@ int QChar::digitValue(ushort ucs2)
 */
 int QChar::digitValue(uint ucs4)
 {
-    if (ucs4 > LAST_UNICODE_CHAR)
+    if (ucs4 > UNICODE_LAST_CODEPOINT)
         return 0;
     return qGetProp(ucs4)->digitValue;
 }
@@ -727,22 +756,22 @@ QChar::Category QChar::category() const
     return (QChar::Category) qGetProp(ucs)->category;
 }
 
-/*! 
+/*!
     \overload
     \since 4.3
     Returns the category of the UCS-4-encoded character specified by \a ucs4.
- */
+*/
 QChar::Category QChar::category(uint ucs4)
 {
-    if (ucs4 > LAST_UNICODE_CHAR)
+    if (ucs4 > UNICODE_LAST_CODEPOINT)
         return QChar::NoCategory;
     return (QChar::Category) qGetProp(ucs4)->category;
 }
 
-/*! 
+/*!
     \overload
     Returns the category of the UCS-2-encoded character specified by \a ucs2.
- */
+*/
 QChar::Category QChar::category(ushort ucs2)
 {
     return (QChar::Category) qGetProp(ucs2)->category;
@@ -757,21 +786,21 @@ QChar::Direction QChar::direction() const
     return (QChar::Direction) qGetProp(ucs)->direction;
 }
 
-/*! 
-\overload
-Returns the direction of the UCS-4-encoded character specified by \a ucs4.
- */
+/*!
+    \overload
+    Returns the direction of the UCS-4-encoded character specified by \a ucs4.
+*/
 QChar::Direction QChar::direction(uint ucs4)
 {
-    if (ucs4 > LAST_UNICODE_CHAR)
+    if (ucs4 > UNICODE_LAST_CODEPOINT)
         return QChar::DirL;
     return (QChar::Direction) qGetProp(ucs4)->direction;
 }
 
-/*! 
-\overload
-Returns the direction of the UCS-2-encoded character specified by \a ucs2.
- */
+/*!
+    \overload
+    Returns the direction of the UCS-2-encoded character specified by \a ucs2.
+*/
 QChar::Direction QChar::direction(ushort ucs2)
 {
     return (QChar::Direction) qGetProp(ucs2)->direction;
@@ -786,25 +815,25 @@ QChar::Joining QChar::joining() const
     return (QChar::Joining) qGetProp(ucs)->joining;
 }
 
-/*! 
-\overload
-Returns information about the joining properties of the UCS-4-encoded
-character specified by \a ucs4 (needed for certain languages such as
-Arabic).
- */
+/*!
+    \overload
+    Returns information about the joining properties of the UCS-4-encoded
+    character specified by \a ucs4 (needed for certain languages such as
+    Arabic).
+*/
 QChar::Joining QChar::joining(uint ucs4)
 {
-    if (ucs4 > LAST_UNICODE_CHAR)
+    if (ucs4 > UNICODE_LAST_CODEPOINT)
         return QChar::OtherJoining;
     return (QChar::Joining) qGetProp(ucs4)->joining;
 }
 
-/*! 
-\overload
-Returns information about the joining properties of the UCS-2-encoded
-character specified by \a ucs2 (needed for certain languages such as
-Arabic).
- */
+/*!
+    \overload
+    Returns information about the joining properties of the UCS-2-encoded
+    character specified by \a ucs2 (needed for certain languages such as
+    Arabic).
+*/
 QChar::Joining QChar::joining(ushort ucs2)
 {
     return (QChar::Joining) qGetProp(ucs2)->joining;
@@ -863,26 +892,27 @@ QChar QChar::mirroredChar() const
     return ucs + qGetProp(ucs)->mirrorDiff;
 }
 
-/*! \overload
-Returns the mirrored character if the UCS-4-encoded character specified
-by \a ucs4 is a mirrored character; otherwise returns the character itself.
+/*!
+    \overload
+    Returns the mirrored character if the UCS-4-encoded character specified
+    by \a ucs4 is a mirrored character; otherwise returns the character itself.
 
-\sa hasMirrored()
- */
+    \sa hasMirrored()
+*/
 uint QChar::mirroredChar(uint ucs4)
 {
-    if (ucs4 > LAST_UNICODE_CHAR)
+    if (ucs4 > UNICODE_LAST_CODEPOINT)
         return ucs4;
     return ucs4 + qGetProp(ucs4)->mirrorDiff;
 }
 
-/*! 
-\overload
-Returns the mirrored character if the UCS-2-encoded character specified
-by \a ucs2 is a mirrored character; otherwise returns the character itself.
+/*!
+    \overload
+    Returns the mirrored character if the UCS-2-encoded character specified
+    by \a ucs2 is a mirrored character; otherwise returns the character itself.
 
-\sa hasMirrored()
- */
+    \sa hasMirrored()
+*/
 ushort QChar::mirroredChar(ushort ucs2)
 {
     return ucs2 + qGetProp(ucs2)->mirrorDiff;
@@ -906,7 +936,7 @@ static const unsigned short * QT_FASTCALL decompositionHelper
     (uint ucs4, int *length, int *tag, unsigned short *buffer)
 {
     *length = 0;
-    if (ucs4 > LAST_UNICODE_CHAR)
+    if (ucs4 > UNICODE_LAST_CODEPOINT)
         return 0;
     if (ucs4 >= Hangul_SBase && ucs4 < Hangul_SBase + Hangul_SCount) {
         int SIndex = ucs4 - Hangul_SBase;
@@ -936,11 +966,11 @@ QString QChar::decomposition() const
     return decomposition(ucs);
 }
 
-/*! 
-\overload
-Decomposes the UCS-4-encoded character specified by \a ucs4 into its
-constituent parts. Returns an empty string if no decomposition exists.
- */
+/*!
+    \overload
+    Decomposes the UCS-4-encoded character specified by \a ucs4 into its
+    constituent parts. Returns an empty string if no decomposition exists.
+*/
 QString QChar::decomposition(uint ucs4)
 {
     unsigned short buffer[3];
@@ -959,14 +989,14 @@ QChar::Decomposition QChar::decompositionTag() const
     return decompositionTag(ucs);
 }
 
-/*! 
-\overload
-Returns the tag defining the composition of the UCS-4-encoded character
-specified by \a ucs4. Returns QChar::Single if no decomposition exists.
- */
+/*!
+    \overload
+    Returns the tag defining the composition of the UCS-4-encoded character
+    specified by \a ucs4. Returns QChar::Single if no decomposition exists.
+*/
 QChar::Decomposition QChar::decompositionTag(uint ucs4)
 {
-    if (ucs4 > LAST_UNICODE_CHAR)
+    if (ucs4 > UNICODE_LAST_CODEPOINT)
         return QChar::NoDecomposition;
     const unsigned short index = GET_DECOMPOSITION_INDEX(ucs4);
     if (index == 0xffff)
@@ -987,27 +1017,28 @@ unsigned char QChar::combiningClass() const
     return (unsigned char) qGetProp(ucs)->combiningClass;
 }
 
-/*! \overload
-Returns the combining class for the UCS-4-encoded character specified by
-\a ucs4, as defined in the Unicode standard.
- */
+/*!
+    \overload
+    Returns the combining class for the UCS-4-encoded character specified by
+    \a ucs4, as defined in the Unicode standard.
+*/
 unsigned char QChar::combiningClass(uint ucs4)
 {
-    if (ucs4 > LAST_UNICODE_CHAR)
+    if (ucs4 > UNICODE_LAST_CODEPOINT)
         return 0;
     return (unsigned char) qGetProp(ucs4)->combiningClass;
 }
 
-/*! \overload
-Returns the combining class for the UCS-2-encoded character specified by
-\a ucs2, as defined in the Unicode standard.
- */
+/*!
+    \overload
+    Returns the combining class for the UCS-2-encoded character specified by
+    \a ucs2, as defined in the Unicode standard.
+*/
 unsigned char QChar::combiningClass(ushort ucs2)
 {
     return (unsigned char) qGetProp(ucs2)->combiningClass;
 }
 
-
 /*!
     Returns the Unicode version that introduced this character.
 */
@@ -1016,26 +1047,37 @@ QChar::UnicodeVersion QChar::unicodeVersion() const
     return (QChar::UnicodeVersion) qGetProp(ucs)->unicodeVersion;
 }
 
-/*! \overload
-Returns the Unicode version that introduced the character specified in
-its UCS-4-encoded form as \a ucs4.
- */
+/*!
+    \overload
+    Returns the Unicode version that introduced the character specified in
+    its UCS-4-encoded form as \a ucs4.
+*/
 QChar::UnicodeVersion QChar::unicodeVersion(uint ucs4)
 {
-    if (ucs4 > LAST_UNICODE_CHAR)
+    if (ucs4 > UNICODE_LAST_CODEPOINT)
         return QChar::Unicode_Unassigned;
     return (QChar::UnicodeVersion) qGetProp(ucs4)->unicodeVersion;
 }
 
-/*! \overload
-Returns the Unicode version that introduced the character specified in
-its UCS-2-encoded form as \a ucs2.
- */
+/*!
+    \overload
+    Returns the Unicode version that introduced the character specified in
+    its UCS-2-encoded form as \a ucs2.
+*/
 QChar::UnicodeVersion QChar::unicodeVersion(ushort ucs2)
 {
     return (QChar::UnicodeVersion) qGetProp(ucs2)->unicodeVersion;
 }
 
+/*!
+    \since 4.8
+
+    Returns the most recent supported Unicode version.
+*/
+QChar::UnicodeVersion QChar::currentUnicodeVersion()
+{
+    return UNICODE_DATA_VERSION;
+}
 
 /*!
     Returns the lowercase equivalent if the character is uppercase or titlecase;
@@ -1049,14 +1091,15 @@ QChar QChar::toLower() const
     return ucs;
 }
 
-/*! \overload
-Returns the lowercase equivalent of the UCS-4-encoded character specified
-by \a ucs4 if the character is uppercase or titlecase; otherwise returns
-the character itself.
- */
+/*!
+    \overload
+    Returns the lowercase equivalent of the UCS-4-encoded character specified
+    by \a ucs4 if the character is uppercase or titlecase; otherwise returns
+    the character itself.
+*/
 uint QChar::toLower(uint ucs4)
 {
-    if (ucs4 > LAST_UNICODE_CHAR)
+    if (ucs4 > UNICODE_LAST_CODEPOINT)
         return ucs4;
     const QUnicodeTables::Properties *p = qGetProp(ucs4);
     if (!p->lowerCaseSpecial)
@@ -1064,11 +1107,12 @@ uint QChar::toLower(uint ucs4)
     return ucs4;
 }
 
-/*! \overload
-Returns the lowercase equivalent of the UCS-2-encoded character specified
-by \a ucs2 if the character is uppercase or titlecase; otherwise returns
-the character itself.
- */
+/*!
+    \overload
+    Returns the lowercase equivalent of the UCS-2-encoded character specified
+    by \a ucs2 if the character is uppercase or titlecase; otherwise returns
+    the character itself.
+*/
 ushort QChar::toLower(ushort ucs2)
 {
     const QUnicodeTables::Properties *p = qGetProp(ucs2);
@@ -1089,14 +1133,15 @@ QChar QChar::toUpper() const
     return ucs;
 }
 
-/*! \overload
-Returns the uppercase equivalent of the UCS-4-encoded character specified
-by \a ucs4 if the character is lowercase or titlecase; otherwise returns
-the character itself.
- */
+/*!
+    \overload
+    Returns the uppercase equivalent of the UCS-4-encoded character specified
+    by \a ucs4 if the character is lowercase or titlecase; otherwise returns
+    the character itself.
+*/
 uint QChar::toUpper(uint ucs4)
 {
-    if (ucs4 > LAST_UNICODE_CHAR)
+    if (ucs4 > UNICODE_LAST_CODEPOINT)
         return ucs4;
     const QUnicodeTables::Properties *p = qGetProp(ucs4);
     if (!p->upperCaseSpecial)
@@ -1104,11 +1149,12 @@ uint QChar::toUpper(uint ucs4)
     return ucs4;
 }
 
-/*! \overload
-Returns the uppercase equivalent of the UCS-2-encoded character specified
-by \a ucs2 if the character is lowercase or titlecase; otherwise returns
-the character itself.
- */
+/*!
+    \overload
+    Returns the uppercase equivalent of the UCS-2-encoded character specified
+    by \a ucs2 if the character is lowercase or titlecase; otherwise returns
+    the character itself.
+*/
 ushort QChar::toUpper(ushort ucs2)
 {
     const QUnicodeTables::Properties *p = qGetProp(ucs2);
@@ -1137,7 +1183,7 @@ QChar QChar::toTitleCase() const
 */
 uint QChar::toTitleCase(uint ucs4)
 {
-    if (ucs4 > LAST_UNICODE_CHAR)
+    if (ucs4 > UNICODE_LAST_CODEPOINT)
         return ucs4;
     const QUnicodeTables::Properties *p = qGetProp(ucs4);
     if (!p->titleCaseSpecial)
@@ -1198,7 +1244,7 @@ QChar QChar::toCaseFolded() const
 */
 uint QChar::toCaseFolded(uint ucs4)
 {
-    if (ucs4 > LAST_UNICODE_CHAR)
+    if (ucs4 > UNICODE_LAST_CODEPOINT)
         return ucs4;
     return ucs4 + qGetProp(ucs4)->caseFoldDiff;
 }
@@ -1292,28 +1338,25 @@ QChar QChar::fromAscii(char c)
 
 #ifndef QT_NO_DATASTREAM
 /*!
-  \relates QChar
+    \relates QChar
 
-  Writes the char \a chr to the stream \a out.
-
-  \sa {Format of the QDataStream operators}
- */
+    Writes the char \a chr to the stream \a out.
 
+    \sa {Serializing Qt Data Types}
+*/
 QDataStream &operator<<(QDataStream &out, const QChar &chr)
 {
     out << quint16(chr.unicode());
     return out;
 }
 
-
 /*!
-  \relates QChar
+    \relates QChar
 
-  Reads a char from the stream \a in into char \a chr.
-
-  \sa {Format of the QDataStream operators}
- */
+    Reads a char from the stream \a in into char \a chr.
 
+    \sa {Serializing Qt Data Types}
+*/
 QDataStream &operator>>(QDataStream &in, QChar &chr)
 {
     quint16 u;
@@ -1438,7 +1481,8 @@ static void decomposeHelper(QString *str, bool canonical, QChar::UnicodeVersion
                 ucs4 = QChar::surrogateToUcs4(high, ucs4);
             }
         }
-        if (QChar::unicodeVersion(ucs4) > version)
+        QChar::UnicodeVersion v = QChar::unicodeVersion(ucs4);
+        if (v == QChar::Unicode_Unassigned || v > version)
             continue;
         int length;
         int tag;
@@ -1446,15 +1490,25 @@ static void decomposeHelper(QString *str, bool canonical, QChar::UnicodeVersion
         if (!d || (canonical && tag != QChar::Canonical))
             continue;
 
-        s.replace(uc - utf16, ucs4 > 0x10000 ? 2 : 1, (const QChar *)d, length);
-        // since the insert invalidates the pointers and we do decomposition recursive
         int pos = uc - utf16;
+        s.replace(pos, QChar::requiresSurrogates(ucs4) ? 2 : 1, reinterpret_cast<const QChar *>(d), length);
+        // since the insert invalidates the pointers and we do decomposition recursive
         utf16 = reinterpret_cast<unsigned short *>(s.data());
         uc = utf16 + pos + length;
     }
 }
 
 
+struct UCS2Pair {
+    ushort u1;
+    ushort u2;
+};
+
+inline bool operator<(ushort u1, const UCS2Pair &ligature)
+{ return u1 < ligature.u1; }
+inline bool operator<(const UCS2Pair &ligature, ushort u1)
+{ return ligature.u1 < u1; }
+
 static ushort ligatureHelper(ushort u1, ushort u2)
 {
     // hangul L-V pair
@@ -1477,16 +1531,18 @@ static ushort ligatureHelper(ushort u1, ushort u2)
     if (index == 0xffff)
         return 0;
     const unsigned short *ligatures = uc_ligature_map+index;
-    ushort length = *ligatures;
-    ++ligatures;
-    // ### use bsearch
-    for (uint i = 0; i < length; ++i)
-        if (ligatures[2*i] == u1)
-            return ligatures[2*i+1];
+    ushort length = *ligatures++;
+    {
+        const UCS2Pair *data = reinterpret_cast<const UCS2Pair *>(ligatures);
+        const UCS2Pair *r = qBinaryFind(data, data + length, u1);
+        if (r != data + length)
+            return r->u2;
+    }
+
     return 0;
 }
 
-static void composeHelper(QString *str, int from)
+static void composeHelper(QString *str, QChar::UnicodeVersion version, int from)
 {
     QString &s = *str;
 
@@ -1506,7 +1562,14 @@ static void composeHelper(QString *str, int from)
                 ++pos;
             }
         }
-        int combining = QChar::combiningClass(uc);
+        const QUnicodeTables::Properties *p = qGetProp(uc);
+        if (p->unicodeVersion == QChar::Unicode_Unassigned || p->unicodeVersion > version) {
+            starter = -1; // to prevent starter == pos - 1
+            lastCombining = 0;
+            ++pos;
+            continue;
+        }
+        int combining = p->combiningClass;
         if (starter == pos - 1 || combining > lastCombining) {
             // allowed to form ligature with S
             QChar ligature = ligatureHelper(s.at(starter).unicode(), uc);
@@ -1533,46 +1596,52 @@ static void canonicalOrderHelper(QString *str, QChar::UnicodeVersion version, in
         int p2 = pos+1;
         uint u1 = s.at(pos).unicode();
         if (QChar(u1).isHighSurrogate()) {
-            ushort low = s.at(pos+1).unicode();
+            ushort low = s.at(p2).unicode();
             if (QChar(low).isLowSurrogate()) {
-                p2++;
                 u1 = QChar::surrogateToUcs4(u1, low);
                 if (p2 >= l)
                     break;
+                ++p2;
             }
         }
         uint u2 = s.at(p2).unicode();
-        if (QChar(u2).isHighSurrogate() && p2 < l-1) {
+        if (QChar(u2).isHighSurrogate() && p2 < l) {
             ushort low = s.at(p2+1).unicode();
             if (QChar(low).isLowSurrogate()) {
-                p2++;
                 u2 = QChar::surrogateToUcs4(u2, low);
+                ++p2;
             }
         }
 
-        int c2 = QChar::combiningClass(u2);
-        if (QChar::unicodeVersion(u2) > version)
-            c2 = 0;
-
+        ushort c2 = 0;
+        {
+            const QUnicodeTables::Properties *p = qGetProp(u2);
+            if (p->unicodeVersion != QChar::Unicode_Unassigned && p->unicodeVersion <= version)
+                c2 = p->combiningClass;
+        }
         if (c2 == 0) {
             pos = p2+1;
             continue;
         }
-        int c1 = QChar::combiningClass(u1);
-        if (QChar::unicodeVersion(u1) > version)
-            c1 = 0;
+
+        ushort c1 = 0;
+        {
+            const QUnicodeTables::Properties *p = qGetProp(u1);
+            if (p->unicodeVersion != QChar::Unicode_Unassigned && p->unicodeVersion <= version)
+                c1 = p->combiningClass;
+        }
 
         if (c1 > c2) {
             QChar *uc = s.data();
             int p = pos;
             // exchange characters
-            if (u2 < 0x10000) {
+            if (!QChar::requiresSurrogates(u2)) {
                 uc[p++] = u2;
             } else {
                 uc[p++] = QChar::highSurrogate(u2);
                 uc[p++] = QChar::lowSurrogate(u2);
             }
-            if (u1 < 0x10000) {
+            if (!QChar::requiresSurrogates(u1)) {
                 uc[p++] = u1;
             } else {
                 uc[p++] = QChar::highSurrogate(u1);
@@ -1584,29 +1653,10 @@ static void canonicalOrderHelper(QString *str, QChar::UnicodeVersion version, in
                 --pos;
         } else {
             ++pos;
-            if (u1 > 0x10000)
+            if (QChar::requiresSurrogates(u1))
                 ++pos;
         }
     }
 }
 
-int QT_FASTCALL QUnicodeTables::script(unsigned int uc)
-{
-    if (uc > 0xffff)
-        return Common;
-    int script = uc_scripts[uc >> 7];
-    if (script < ScriptSentinel)
-        return script;
-    script = (((script - ScriptSentinel) * UnicodeBlockSize) + UnicodeBlockCount);
-    script = uc_scripts[script + (uc & 0x7f)];
-    return script;
-}
-
-
-Q_CORE_EXPORT QUnicodeTables::LineBreakClass QT_FASTCALL QUnicodeTables::lineBreakClass(uint ucs4)
-{
-    return (QUnicodeTables::LineBreakClass) qGetProp(ucs4)->line_break_class;
-}
-
-
 QT_END_NAMESPACE
