@@ -1,47 +1,51 @@
 /****************************************************************************
 **
-** Copyright (C) 2009 Nokia Corporation and/or its subsidiary(-ies).
+** Copyright (C) 2011 Nokia Corporation and/or its subsidiary(-ies).
 ** All rights reserved.
 ** Contact: Nokia Corporation (qt-info@nokia.com)
 **
 ** This file is part of the QtDBus module of the Qt Toolkit.
 **
 ** $QT_BEGIN_LICENSE:LGPL$
-** Commercial Usage
-** Licensees holding valid Qt Commercial licenses may use this file in
-** accordance with the Qt Commercial License Agreement provided with the
-** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and Nokia.
-**
 ** GNU Lesser General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU Lesser
-** General Public License version 2.1 as published by the Free Software
-** Foundation and appearing in the file LICENSE.LGPL included in the
-** packaging of this file.  Please review the following information to
-** ensure the GNU Lesser General Public License version 2.1 requirements
-** will be met: http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
+** This file may be used under the terms of the GNU Lesser General Public
+** License version 2.1 as published by the Free Software Foundation and
+** appearing in the file LICENSE.LGPL included in the packaging of this
+** file. Please review the following information to ensure the GNU Lesser
+** General Public License version 2.1 requirements will be met:
+** http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
 **
 ** In addition, as a special exception, Nokia gives you certain additional
-** rights.  These rights are described in the Nokia Qt LGPL Exception
+** rights. These rights are described in the Nokia Qt LGPL Exception
 ** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
 **
 ** GNU General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU
-** General Public License version 3.0 as published by the Free Software
-** Foundation and appearing in the file LICENSE.GPL included in the
-** packaging of this file.  Please review the following information to
-** ensure the GNU General Public License version 3.0 requirements will be
-** met: http://www.gnu.org/copyleft/gpl.html.
+** Alternatively, this file may be used under the terms of the GNU General
+** Public License version 3.0 as published by the Free Software Foundation
+** and appearing in the file LICENSE.GPL included in the packaging of this
+** file. Please review the following information to ensure the GNU General
+** Public License version 3.0 requirements will be met:
+** http://www.gnu.org/copyleft/gpl.html.
+**
+** Other Usage
+** Alternatively, this file may be used in accordance with the terms and
+** conditions contained in a signed written agreement between you and Nokia.
+**
+**
+**
+**
 **
-** If you have questions regarding the use of this file, please contact
-** Nokia at qt-info@nokia.com.
 ** $QT_END_LICENSE$
 **
 ****************************************************************************/
 
 #include "qdbusargument_p.h"
+#include "qdbusconnection.h"
+#include "qdbusmetatype_p.h"
 #include "qdbusutil_p.h"
 
+#ifndef QT_NO_DBUS
+
 QT_BEGIN_NAMESPACE
 
 static void qIterAppend(DBusMessageIter *it, QByteArray *ba, int type, const void *arg)
@@ -135,6 +139,16 @@ inline void QDBusMarshaller::append(const QDBusSignature &arg)
     qIterAppend(&iterator, ba, DBUS_TYPE_SIGNATURE, &cdata);
 }
 
+inline void QDBusMarshaller::append(const QDBusUnixFileDescriptor &arg)
+{
+    int fd = arg.fileDescriptor();
+    if (!ba && fd == -1) {
+        error(QLatin1String("Invalid file descriptor passed in arguments"));
+    } else {
+        qIterAppend(&iterator, ba, DBUS_TYPE_UNIX_FD, &fd);
+    }
+}
+
 inline void QDBusMarshaller::append(const QByteArray &arg)
 {
     if (ba) {
@@ -167,7 +181,7 @@ inline bool QDBusMarshaller::append(const QDBusVariant &arg)
 
     QByteArray tmpSignature;
     const char *signature = 0;
-    if (int(id) == qMetaTypeId<QDBusArgument>()) {
+    if (int(id) == QDBusMetaTypeId::argument) {
         // take the signature from the QDBusArgument object we're marshalling
         tmpSignature =
             qvariant_cast<QDBusArgument>(value).currentSignature().toLatin1();
@@ -185,7 +199,7 @@ inline bool QDBusMarshaller::append(const QDBusVariant &arg)
         return false;
     }
 
-    QDBusMarshaller sub;
+    QDBusMarshaller sub(capabilities);
     open(sub, DBUS_TYPE_VARIANT, signature);
     bool isOk = sub.appendVariantInternal(value);
     // don't call sub.close(): it auto-closes
@@ -200,7 +214,7 @@ inline void QDBusMarshaller::append(const QStringList &arg)
         return;
     }
 
-    QDBusMarshaller sub;
+    QDBusMarshaller sub(capabilities);
     open(sub, DBUS_TYPE_ARRAY, DBUS_TYPE_STRING_AS_STRING);
     QStringList::ConstIterator it = arg.constBegin();
     QStringList::ConstIterator end = arg.constEnd();
@@ -240,7 +254,7 @@ inline QDBusMarshaller *QDBusMarshaller::beginMap(int kid, int vid)
               .arg(QLatin1String(QVariant::typeToName(QVariant::Type(kid)))));
         return this;
     }
-    if (ksignature[1] != 0 || !q_dbus_type_is_basic(*ksignature)) {
+    if (ksignature[1] != 0 || !QDBusUtil::isValidBasicType(*ksignature)) {
         qWarning("QDBusMarshaller: type '%s' (%d) cannot be used as the key type in a D-BUS map.",
                  QVariant::typeToName( QVariant::Type(kid) ), kid);
         error(QString::fromLatin1("Type %1 passed in arguments cannot be used as a key in a map")
@@ -277,6 +291,7 @@ void QDBusMarshaller::open(QDBusMarshaller &sub, int code, const char *signature
     sub.parent = this;
     sub.ba = ba;
     sub.ok = true;
+    sub.capabilities = capabilities;
 
     if (ba)
         switch (code) {
@@ -300,7 +315,7 @@ void QDBusMarshaller::open(QDBusMarshaller &sub, int code, const char *signature
 
 QDBusMarshaller *QDBusMarshaller::beginCommon(int code, const char *signature)
 {
-    QDBusMarshaller *d = new QDBusMarshaller;
+    QDBusMarshaller *d = new QDBusMarshaller(capabilities);
     open(*d, code, signature);
     return d;
 }
@@ -353,13 +368,13 @@ bool QDBusMarshaller::appendVariantInternal(const QVariant &arg)
     }
 
     // intercept QDBusArgument parameters here
-    if (id == qMetaTypeId<QDBusArgument>()) {
+    if (id == QDBusMetaTypeId::argument) {
         QDBusArgument dbusargument = qvariant_cast<QDBusArgument>(arg);
         QDBusArgumentPrivate *d = QDBusArgumentPrivate::d(dbusargument);
         if (!d->message)
             return false;       // can't append this one...
 
-        QDBusDemarshaller demarshaller;
+        QDBusDemarshaller demarshaller(capabilities);
         demarshaller.message = q_dbus_message_ref(d->message);
 
         if (d->direction == Demarshalling) {
@@ -387,7 +402,6 @@ bool QDBusMarshaller::appendVariantInternal(const QVariant &arg)
     switch (*signature) {
 #ifdef __OPTIMIZE__
     case DBUS_TYPE_BYTE:
-    case DBUS_TYPE_BOOLEAN:
     case DBUS_TYPE_INT16:
     case DBUS_TYPE_UINT16:
     case DBUS_TYPE_INT32:
@@ -397,6 +411,9 @@ bool QDBusMarshaller::appendVariantInternal(const QVariant &arg)
     case DBUS_TYPE_DOUBLE:
         qIterAppend(&iterator, ba, *signature, arg.constData());
         return true;
+    case DBUS_TYPE_BOOLEAN:
+        append( arg.toBool() );
+        return true;
 #else
     case DBUS_TYPE_BYTE:
         append( qvariant_cast<uchar>(arg) );
@@ -468,6 +485,13 @@ bool QDBusMarshaller::appendVariantInternal(const QVariant &arg)
         qFatal("QDBusMarshaller::appendVariantInternal got a DICT_ENTRY!");
         return false;
 
+    case DBUS_TYPE_UNIX_FD:
+        if (capabilities & QDBusConnection::UnixFileDescriptorPassing || ba) {
+            append(qvariant_cast<QDBusUnixFileDescriptor>(arg));
+            return true;
+        }
+        // fall through
+
     default:
         qWarning("QDBusMarshaller::appendVariantInternal: Found unknown D-BUS type '%s'",
                  signature);
@@ -487,7 +511,7 @@ bool QDBusMarshaller::appendRegisteredType(const QVariant &arg)
 bool QDBusMarshaller::appendCrossMarshalling(QDBusDemarshaller *demarshaller)
 {
     int code = q_dbus_message_iter_get_arg_type(&demarshaller->iterator);
-    if (q_dbus_type_is_basic(code)) {
+    if (QDBusUtil::isValidBasicType(code)) {
         // easy: just append
         // do exactly like the D-BUS docs suggest
         // (see apidocs for q_dbus_message_iter_get_basic)
@@ -501,8 +525,8 @@ bool QDBusMarshaller::appendCrossMarshalling(QDBusDemarshaller *demarshaller)
 
     if (code == DBUS_TYPE_ARRAY) {
         int element = q_dbus_message_iter_get_element_type(&demarshaller->iterator);
-        if (q_dbus_type_is_fixed(element)) {
-            // another optimisation: fixed size arrays
+        if (QDBusUtil::isValidFixedType(element) && element != DBUS_TYPE_UNIX_FD) {
+            // another optimization: fixed size arrays
             // code is exactly like QDBusDemarshaller::toByteArray
             DBusMessageIter sub;
             q_dbus_message_iter_recurse(&demarshaller->iterator, &sub);
@@ -511,7 +535,7 @@ bool QDBusMarshaller::appendCrossMarshalling(QDBusDemarshaller *demarshaller)
             void* data;
             q_dbus_message_iter_get_fixed_array(&sub,&data,&len);
 
-            char signature[2] = { element, 0 };
+            char signature[2] = { char(element), 0 };
             q_dbus_message_iter_open_container(&iterator, DBUS_TYPE_ARRAY, signature, &sub);
             q_dbus_message_iter_append_fixed_array(&sub, element, &data, len);
             q_dbus_message_iter_close_container(&iterator, &sub);
@@ -523,7 +547,7 @@ bool QDBusMarshaller::appendCrossMarshalling(QDBusDemarshaller *demarshaller)
     // We have to recurse
     QDBusDemarshaller *drecursed = demarshaller->beginCommon();
 
-    QDBusMarshaller mrecursed;  // create on the stack makes it autoclose
+    QDBusMarshaller mrecursed(capabilities);  // create on the stack makes it autoclose
     QByteArray subSignature;
     const char *sig = 0;
     if (code == DBUS_TYPE_VARIANT || code == DBUS_TYPE_ARRAY) {
@@ -545,3 +569,5 @@ bool QDBusMarshaller::appendCrossMarshalling(QDBusDemarshaller *demarshaller)
 }
 
 QT_END_NAMESPACE
+
+#endif // QT_NO_DBUS
