@@ -1,40 +1,40 @@
 /****************************************************************************
 **
-** Copyright (C) 2009 Nokia Corporation and/or its subsidiary(-ies).
+** Copyright (C) 2011 Nokia Corporation and/or its subsidiary(-ies).
 ** All rights reserved.
 ** Contact: Nokia Corporation (qt-info@nokia.com)
 **
 ** This file is part of the QtDBus module of the Qt Toolkit.
 **
 ** $QT_BEGIN_LICENSE:LGPL$
-** Commercial Usage
-** Licensees holding valid Qt Commercial licenses may use this file in
-** accordance with the Qt Commercial License Agreement provided with the
-** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and Nokia.
-**
 ** GNU Lesser General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU Lesser
-** General Public License version 2.1 as published by the Free Software
-** Foundation and appearing in the file LICENSE.LGPL included in the
-** packaging of this file.  Please review the following information to
-** ensure the GNU Lesser General Public License version 2.1 requirements
-** will be met: http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
+** This file may be used under the terms of the GNU Lesser General Public
+** License version 2.1 as published by the Free Software Foundation and
+** appearing in the file LICENSE.LGPL included in the packaging of this
+** file. Please review the following information to ensure the GNU Lesser
+** General Public License version 2.1 requirements will be met:
+** http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
 **
 ** In addition, as a special exception, Nokia gives you certain additional
-** rights.  These rights are described in the Nokia Qt LGPL Exception
+** rights. These rights are described in the Nokia Qt LGPL Exception
 ** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
 **
 ** GNU General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU
-** General Public License version 3.0 as published by the Free Software
-** Foundation and appearing in the file LICENSE.GPL included in the
-** packaging of this file.  Please review the following information to
-** ensure the GNU General Public License version 3.0 requirements will be
-** met: http://www.gnu.org/copyleft/gpl.html.
+** Alternatively, this file may be used under the terms of the GNU General
+** Public License version 3.0 as published by the Free Software Foundation
+** and appearing in the file LICENSE.GPL included in the packaging of this
+** file. Please review the following information to ensure the GNU General
+** Public License version 3.0 requirements will be met:
+** http://www.gnu.org/copyleft/gpl.html.
+**
+** Other Usage
+** Alternatively, this file may be used in accordance with the terms and
+** conditions contained in a signed written agreement between you and Nokia.
+**
+**
+**
+**
 **
-** If you have questions regarding the use of this file, please contact
-** Nokia at qt-info@nokia.com.
 ** $QT_END_LICENSE$
 **
 ****************************************************************************/
@@ -44,7 +44,7 @@
 #include <qdebug.h>
 #include <qstringlist.h>
 
-#include <qdbus_symbols_p.h>
+#include "qdbus_symbols_p.h"
 
 #include "qdbusargument_p.h"
 #include "qdbuserror.h"
@@ -53,6 +53,8 @@
 #include "qdbusconnection_p.h"
 #include "qdbusutil_p.h"
 
+#ifndef QT_NO_DBUS
+
 QT_BEGIN_NAMESPACE
 
 static inline const char *data(const QByteArray &arr)
@@ -63,7 +65,7 @@ static inline const char *data(const QByteArray &arr)
 QDBusMessagePrivate::QDBusMessagePrivate()
     : msg(0), reply(0), type(DBUS_MESSAGE_TYPE_INVALID),
       timeout(-1), localReply(0), ref(1), delayedReply(false), localMessage(false),
-      parametersValidated(false)
+      parametersValidated(false), autoStartService(true)
 {
 }
 
@@ -93,14 +95,15 @@ QString QDBusMessage::errorMessage() const
 
 /*!
     \internal
-    Constructs a DBusMessage object from this object. The returned value must be de-referenced
-    with q_dbus_message_unref.
+    Constructs a DBusMessage object from \a message. The returned value must be de-referenced
+    with q_dbus_message_unref. The \a capabilities flags indicates which capabilities to use.
 
     The \a error object is set to indicate the error if anything went wrong with the
     marshalling. Usually, this error message will be placed in the reply, as if the call failed.
     The \a error pointer must not be null.
 */
-DBusMessage *QDBusMessagePrivate::toDBusMessage(const QDBusMessage &message, QDBusError *error)
+DBusMessage *QDBusMessagePrivate::toDBusMessage(const QDBusMessage &message, QDBusConnection::ConnectionCapabilities capabilities,
+                                                QDBusError *error)
 {
     if (!qdbus_loadLibDBus()) {
         *error = QDBusError(QDBusError::Failed, QLatin1String("Could not open lidbus-1 library"));
@@ -129,6 +132,7 @@ DBusMessage *QDBusMessagePrivate::toDBusMessage(const QDBusMessage &message, QDB
 
         msg = q_dbus_message_new_method_call(data(d_ptr->service.toUtf8()), d_ptr->path.toUtf8(),
                                              data(d_ptr->interface.toUtf8()), d_ptr->name.toUtf8());
+        q_dbus_message_set_auto_start( msg, d_ptr->autoStartService );
         break;
     case DBUS_MESSAGE_TYPE_METHOD_RETURN:
         msg = q_dbus_message_new(DBUS_MESSAGE_TYPE_METHOD_RETURN);
@@ -174,7 +178,7 @@ DBusMessage *QDBusMessagePrivate::toDBusMessage(const QDBusMessage &message, QDB
     // we can record this fact
     d_ptr->parametersValidated = true;
 
-    QDBusMarshaller marshaller;
+    QDBusMarshaller marshaller(capabilities);
     QVariantList::ConstIterator it =  d_ptr->arguments.constBegin();
     QVariantList::ConstIterator cend = d_ptr->arguments.constEnd();
     q_dbus_message_iter_init_append(msg, &marshaller.iterator);
@@ -219,7 +223,7 @@ DBUS_DISABLE_CHECKS
     \internal
     Constructs a QDBusMessage by parsing the given DBusMessage object.
 */
-QDBusMessage QDBusMessagePrivate::fromDBusMessage(DBusMessage *dmsg)
+QDBusMessage QDBusMessagePrivate::fromDBusMessage(DBusMessage *dmsg, QDBusConnection::ConnectionCapabilities capabilities)
 {
     QDBusMessage message;
     if (!dmsg)
@@ -235,7 +239,7 @@ QDBusMessage QDBusMessagePrivate::fromDBusMessage(DBusMessage *dmsg)
     message.d_ptr->signature = QString::fromUtf8(q_dbus_message_get_signature(dmsg));
     message.d_ptr->msg = q_dbus_message_ref(dmsg);
 
-    QDBusDemarshaller demarshaller;
+    QDBusDemarshaller demarshaller(capabilities);
     demarshaller.message = q_dbus_message_ref(dmsg);
     if (q_dbus_message_iter_init(demarshaller.message, &demarshaller.iterator))
         while (!demarshaller.atEnd())
@@ -269,7 +273,7 @@ QDBusMessage QDBusMessagePrivate::makeLocal(const QDBusConnectionPrivate &conn,
             // we must marshall and demarshall again so as to create QDBusArgument
             // entries for the complex types
             QDBusError error;
-            DBusMessage *message = toDBusMessage(asSent, &error);
+            DBusMessage *message = toDBusMessage(asSent, conn.capabilities, &error);
             if (!message) {
                 // failed to marshall, so it's a call error
                 return QDBusMessage::createError(error);
@@ -277,7 +281,7 @@ QDBusMessage QDBusMessagePrivate::makeLocal(const QDBusConnectionPrivate &conn,
 
             q_dbus_message_set_sender(message, conn.baseService.toUtf8());
 
-            QDBusMessage retval = fromDBusMessage(message);
+            QDBusMessage retval = fromDBusMessage(message, conn.capabilities);
             retval.d_ptr->localMessage = true;
             q_dbus_message_unref(message);
             if (retval.d_ptr->service.isEmpty())
@@ -289,7 +293,7 @@ QDBusMessage QDBusMessagePrivate::makeLocal(const QDBusConnectionPrivate &conn,
     }
 
     // no complex types seen
-    // optimise by using the variant list itself
+    // optimize by using the variant list itself
     QDBusMessage retval;
     QDBusMessagePrivate *d = retval.d_ptr;
     d->arguments = asSent.d_ptr->arguments;
@@ -449,7 +453,7 @@ QDBusMessage QDBusMessage::createReply(const QVariantList &arguments) const
         d_ptr->localReply = new QDBusMessage(reply); // keep an internal copy
     }
 
-    // the reply must have a msg or be a local-loop optimisation
+    // the reply must have a msg or be a local-loop optimization
     Q_ASSERT(reply.d_ptr->reply || reply.d_ptr->localMessage);
     return reply;
 }
@@ -468,7 +472,7 @@ QDBusMessage QDBusMessage::createErrorReply(const QString name, const QString &m
         d_ptr->localReply = new QDBusMessage(reply); // keep an internal copy
     }
 
-    // the reply must have a msg or be a local-loop optimisation
+    // the reply must have a msg or be a local-loop optimization
     Q_ASSERT(reply.d_ptr->reply || reply.d_ptr->localMessage);
     return reply;
 }
@@ -643,6 +647,45 @@ bool QDBusMessage::isDelayedReply() const
     return d_ptr->delayedReply;
 }
 
+/*!
+    Sets the auto start flag to \a enable. This flag only makes sense
+    for method call messages, where it tells the D-Bus server to
+    either auto start the service responsible for the service name, or
+    not to auto start it.
+
+    By default this flag is true, i.e. a service is autostarted.
+    This means:
+
+    When the service that this method call is sent to is already
+    running, the method call is sent to it. If the service is not
+    running yet, the D-Bus daemon is requested to autostart the
+    service that is assigned to this service name. This is
+    handled by .service files that are placed in a directory known
+    to the D-Bus server. These files then each contain a service
+    name and the path to a program that should be executed when
+    this service name is requested.
+
+    \since 4.7
+*/
+void QDBusMessage::setAutoStartService(bool enable)
+{
+    d_ptr->autoStartService = enable;
+}
+
+/*!
+    Returns the auto start flag, as set by setAutoStartService(). By default, this
+    flag is true, which means QtDBus will auto start a service, if it is
+    not running already.
+
+    \sa setAutoStartService()
+
+    \since 4.7
+*/
+bool QDBusMessage::autoStartService() const
+{
+    return d_ptr->autoStartService;
+}
+
 /*!
     Sets the arguments that are going to be sent over D-Bus to \a arguments. Those
     will be the arguments to a method call or the parameters in the signal.
@@ -759,3 +802,4 @@ QDebug operator<<(QDebug dbg, const QDBusMessage &msg)
 
 QT_END_NAMESPACE
 
+#endif // QT_NO_DBUS
