@@ -1,51 +1,54 @@
 /****************************************************************************
 **
-** Copyright (C) 2009 Nokia Corporation and/or its subsidiary(-ies).
+** Copyright (C) 2011 Nokia Corporation and/or its subsidiary(-ies).
 ** All rights reserved.
 ** Contact: Nokia Corporation (qt-info@nokia.com)
 **
 ** This file is part of the QtDBus module of the Qt Toolkit.
 **
 ** $QT_BEGIN_LICENSE:LGPL$
-** Commercial Usage
-** Licensees holding valid Qt Commercial licenses may use this file in
-** accordance with the Qt Commercial License Agreement provided with the
-** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and Nokia.
-**
 ** GNU Lesser General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU Lesser
-** General Public License version 2.1 as published by the Free Software
-** Foundation and appearing in the file LICENSE.LGPL included in the
-** packaging of this file.  Please review the following information to
-** ensure the GNU Lesser General Public License version 2.1 requirements
-** will be met: http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
+** This file may be used under the terms of the GNU Lesser General Public
+** License version 2.1 as published by the Free Software Foundation and
+** appearing in the file LICENSE.LGPL included in the packaging of this
+** file. Please review the following information to ensure the GNU Lesser
+** General Public License version 2.1 requirements will be met:
+** http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
 **
 ** In addition, as a special exception, Nokia gives you certain additional
-** rights.  These rights are described in the Nokia Qt LGPL Exception
+** rights. These rights are described in the Nokia Qt LGPL Exception
 ** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
 **
 ** GNU General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU
-** General Public License version 3.0 as published by the Free Software
-** Foundation and appearing in the file LICENSE.GPL included in the
-** packaging of this file.  Please review the following information to
-** ensure the GNU General Public License version 3.0 requirements will be
-** met: http://www.gnu.org/copyleft/gpl.html.
+** Alternatively, this file may be used under the terms of the GNU General
+** Public License version 3.0 as published by the Free Software Foundation
+** and appearing in the file LICENSE.GPL included in the packaging of this
+** file. Please review the following information to ensure the GNU General
+** Public License version 3.0 requirements will be met:
+** http://www.gnu.org/copyleft/gpl.html.
+**
+** Other Usage
+** Alternatively, this file may be used in accordance with the terms and
+** conditions contained in a signed written agreement between you and Nokia.
+**
+**
+**
+**
 **
-** If you have questions regarding the use of this file, please contact
-** Nokia at qt-info@nokia.com.
 ** $QT_END_LICENSE$
 **
 ****************************************************************************/
 
 #include "qdbusutil_p.h"
 
-#include <qdbus_symbols_p.h>
+#include "qdbus_symbols_p.h"
 
 #include <QtCore/qstringlist.h>
 
 #include "qdbusargument.h"
+#include "qdbusunixfiledescriptor.h"
+
+#ifndef QT_NO_DBUS
 
 QT_BEGIN_NAMESPACE
 
@@ -128,6 +131,10 @@ static bool variantToString(const QVariant &arg, QString &out)
     } else if (argType == qMetaTypeId<QDBusSignature>()) {
         out += QLatin1String("[Signature: ") + qvariant_cast<QDBusSignature>(arg).signature();
         out += QLatin1Char(']');
+    } else if (argType == qMetaTypeId<QDBusUnixFileDescriptor>()) {
+        out += QLatin1String("[Unix FD: ");
+        out += QLatin1String(qvariant_cast<QDBusUnixFileDescriptor>(arg).isValid() ? "valid" : "not valid");
+        out += QLatin1Char(']');
     } else if (argType == qMetaTypeId<QDBusVariant>()) {
         const QVariant v = qvariant_cast<QDBusVariant>(arg).variant();
         out += QLatin1String("[Variant");
@@ -231,6 +238,68 @@ bool argToString(const QDBusArgument &busArg, QString &out)
     return true;
 }
 
+//------- D-Bus Types --------
+static const char oneLetterTypes[] = "vsogybnqiuxtdh";
+static const char basicTypes[] =      "sogybnqiuxtdh";
+static const char fixedTypes[] =         "ybnqiuxtdh";
+
+static bool isBasicType(int c)
+{
+    return c != DBUS_TYPE_INVALID && strchr(basicTypes, c) != NULL;
+}
+
+static bool isFixedType(int c)
+{
+    return c != DBUS_TYPE_INVALID && strchr(fixedTypes, c) != NULL;
+}
+
+// Returns a pointer to one-past-end of this type if it's valid;
+// returns NULL if it isn't valid.
+static const char *validateSingleType(const char *signature)
+{
+    register char c = *signature;
+    if (c == DBUS_TYPE_INVALID)
+        return false;
+
+    // is it one of the one-letter types?
+    if (strchr(oneLetterTypes, c) != NULL)
+        return signature + 1;
+
+    // is it an array?
+    if (c == DBUS_TYPE_ARRAY) {
+        // then it's valid if the next type is valid
+        // or if it's a dict-entry
+        c = *++signature;
+        if (c == DBUS_DICT_ENTRY_BEGIN_CHAR) {
+            // beginning of a dictionary entry
+            // a dictionary entry has a key which is of basic types
+            // and a free value
+            c = *++signature;
+            if (!isBasicType(c))
+                return 0;
+            signature = validateSingleType(signature + 1);
+            return signature && *signature == DBUS_DICT_ENTRY_END_CHAR ? signature + 1 : 0;
+        }
+
+        return validateSingleType(signature);
+    }
+
+    if (c == DBUS_STRUCT_BEGIN_CHAR) {
+        // beginning of a struct
+        ++signature;
+        while (true) {
+            signature = validateSingleType(signature);
+            if (!signature)
+                return 0;
+            if (*signature == DBUS_STRUCT_END_CHAR)
+                return signature + 1;
+        }
+    }
+
+    // invalid/unknown type
+    return 0;
+}
+
 /*!
     \namespace QDBusUtil
     \inmodule QtDBus
@@ -307,7 +376,7 @@ namespace QDBusUtil
         Returns true if \a connName is a valid unique connection name.
 
         Unique connection names start with a colon (":") and are followed by a list of dot-separated
-        components composed of ASCII letters, digits, the hypen or the underscore ("_") character.
+        components composed of ASCII letters, digits, the hyphen or the underscore ("_") character.
     */
     bool isValidUniqueConnectionName(const QString &connName)
     {
@@ -439,6 +508,25 @@ namespace QDBusUtil
         return true;
     }
 
+    /*!
+        \fn bool QDBusUtil::isValidBasicType(int type)
+        Returns true if \a c is a valid, basic D-Bus type.
+     */
+    bool isValidBasicType(int c)
+    {
+        return isBasicType(c);
+    }
+
+    /*!
+        \fn bool QDBusUtil::isValidFixedType(int type)
+        Returns true if \a c is a valid, fixed D-Bus type.
+     */
+    bool isValidFixedType(int c)
+    {
+        return isFixedType(c);
+    }
+
+
     /*!
         \fn bool QDBusUtil::isValidSignature(const QString &signature)
         Returns true if \a signature is a valid D-Bus type signature for one or more types.
@@ -449,7 +537,15 @@ namespace QDBusUtil
     */
     bool isValidSignature(const QString &signature)
     {
-        return q_dbus_signature_validate(signature.toUtf8(), 0);
+        QByteArray ba = signature.toLatin1();
+        const char *data = ba.constData();
+        while (true) {
+            data = validateSingleType(data);
+            if (!data)
+                return false;
+            if (*data == '\0')
+                return true;
+        }
     }
 
     /*!
@@ -460,9 +556,13 @@ namespace QDBusUtil
     */
     bool isValidSingleSignature(const QString &signature)
     {
-        return q_dbus_signature_validate_single(signature.toUtf8(), 0);
+        QByteArray ba = signature.toLatin1();
+        const char *data = validateSingleType(ba.constData());
+        return data && *data == '\0';
     }
 
 } // namespace QDBusUtil
 
 QT_END_NAMESPACE
+
+#endif // QT_NO_DBUS
