@@ -1,40 +1,40 @@
 /****************************************************************************
 **
-** Copyright (C) 2010 Nokia Corporation and/or its subsidiary(-ies).
+** Copyright (C) 2011 Nokia Corporation and/or its subsidiary(-ies).
 ** All rights reserved.
 ** Contact: Nokia Corporation (qt-info@nokia.com)
 **
 ** This file is part of the QtDeclarative module of the Qt Toolkit.
 **
 ** $QT_BEGIN_LICENSE:LGPL$
-** Commercial Usage
-** Licensees holding valid Qt Commercial licenses may use this file in
-** accordance with the Qt Commercial License Agreement provided with the
-** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and Nokia.
-**
 ** GNU Lesser General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU Lesser
-** General Public License version 2.1 as published by the Free Software
-** Foundation and appearing in the file LICENSE.LGPL included in the
-** packaging of this file.  Please review the following information to
-** ensure the GNU Lesser General Public License version 2.1 requirements
-** will be met: http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
+** This file may be used under the terms of the GNU Lesser General Public
+** License version 2.1 as published by the Free Software Foundation and
+** appearing in the file LICENSE.LGPL included in the packaging of this
+** file. Please review the following information to ensure the GNU Lesser
+** General Public License version 2.1 requirements will be met:
+** http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
 **
 ** In addition, as a special exception, Nokia gives you certain additional
-** rights.  These rights are described in the Nokia Qt LGPL Exception
+** rights. These rights are described in the Nokia Qt LGPL Exception
 ** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
 **
 ** GNU General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU
-** General Public License version 3.0 as published by the Free Software
-** Foundation and appearing in the file LICENSE.GPL included in the
-** packaging of this file.  Please review the following information to
-** ensure the GNU General Public License version 3.0 requirements will be
-** met: http://www.gnu.org/copyleft/gpl.html.
+** Alternatively, this file may be used under the terms of the GNU General
+** Public License version 3.0 as published by the Free Software Foundation
+** and appearing in the file LICENSE.GPL included in the packaging of this
+** file. Please review the following information to ensure the GNU General
+** Public License version 3.0 requirements will be met:
+** http://www.gnu.org/copyleft/gpl.html.
+**
+** Other Usage
+** Alternatively, this file may be used in accordance with the terms and
+** conditions contained in a signed written agreement between you and Nokia.
+**
+**
+**
+**
 **
-** If you have questions regarding the use of this file, please contact
-** Nokia at qt-info@nokia.com.
 ** $QT_END_LICENSE$
 **
 ****************************************************************************/
@@ -46,11 +46,16 @@
 #include <QtCore/qfileinfo.h>
 #include <QtCore/qpluginloader.h>
 #include <QtCore/qlibraryinfo.h>
+#include <QtCore/qalgorithms.h>
 #include <QtDeclarative/qdeclarativeextensioninterface.h>
 #include <private/qdeclarativeglobal_p.h>
 #include <private/qdeclarativetypenamecache_p.h>
 #include <private/qdeclarativeengine_p.h>
 
+#ifdef Q_OS_SYMBIAN
+#include "private/qcore_symbian_p.h"
+#endif
+
 QT_BEGIN_NAMESPACE
 
 DEFINE_BOOL_CONFIG_OPTION(qmlImportTrace, QML_IMPORT_TRACE)
@@ -75,16 +80,16 @@ public:
     QList<QDeclarativeDirComponents> qmlDirComponents;
 
 
-    bool find_helper(int i, const QByteArray& type, int *vmajor, int *vminor,
+    bool find_helper(QDeclarativeTypeLoader *typeLoader, int i, const QByteArray& type, int *vmajor, int *vminor,
                                  QDeclarativeType** type_return, QUrl* url_return,
                                  QUrl *base = 0, bool *typeRecursionDetected = 0);
-    bool find(const QByteArray& type, int *vmajor, int *vminor, QDeclarativeType** type_return,
+    bool find(QDeclarativeTypeLoader *typeLoader, const QByteArray& type, int *vmajor, int *vminor, QDeclarativeType** type_return,
               QUrl* url_return, QUrl *base = 0, QString *errorString = 0);
 };
 
 class QDeclarativeImportsPrivate {
 public:
-    QDeclarativeImportsPrivate();
+    QDeclarativeImportsPrivate(QDeclarativeTypeLoader *loader);
     ~QDeclarativeImportsPrivate();
 
     bool importExtension(const QString &absoluteFilePath, const QString &uri, 
@@ -107,6 +112,7 @@ public:
     QSet<QString> qmlDirFilesForWhichPluginsHaveBeenLoaded;
     QDeclarativeImportedNamespace unqualifiedset;
     QHash<QString,QDeclarativeImportedNamespace* > set;
+    QDeclarativeTypeLoader *typeLoader;
 };
 
 /*!
@@ -130,9 +136,12 @@ QDeclarativeImports::operator =(const QDeclarativeImports &copy)
     return *this;
 }
 
-QDeclarativeImports::QDeclarativeImports() 
-: d(new QDeclarativeImportsPrivate)
-{
+QDeclarativeImports::QDeclarativeImports()
+    : d(new QDeclarativeImportsPrivate(0)){
+}
+
+QDeclarativeImports::QDeclarativeImports(QDeclarativeTypeLoader *typeLoader)
+    : d(new QDeclarativeImportsPrivate(typeLoader)){
 }
 
 QDeclarativeImports::~QDeclarativeImports()
@@ -264,10 +273,11 @@ bool QDeclarativeImports::resolveType(QDeclarativeImportedNamespace* ns, const Q
                                       QDeclarativeType** type_return, QUrl* url_return, 
                                       int *vmaj, int *vmin) const
 {
-    return ns->find(type,vmaj,vmin,type_return,url_return);
+    Q_ASSERT(d->typeLoader);
+    return ns->find(d->typeLoader,type,vmaj,vmin,type_return,url_return);
 }
 
-bool QDeclarativeImportedNamespace::find_helper(int i, const QByteArray& type, int *vmajor, int *vminor,
+bool QDeclarativeImportedNamespace::find_helper(QDeclarativeTypeLoader *typeLoader, int i, const QByteArray& type, int *vmajor, int *vminor,
                                  QDeclarativeType** type_return, QUrl* url_return,
                                  QUrl *base, bool *typeRecursionDetected)
 {
@@ -287,7 +297,6 @@ bool QDeclarativeImportedNamespace::find_helper(int i, const QByteArray& type, i
         return true;
     }
 
-    QUrl url = QUrl(urls.at(i) + QLatin1Char('/') + QString::fromUtf8(type) + QLatin1String(".qml"));
     QDeclarativeDirComponents qmldircomponents = qmlDirComponents.at(i);
 
     bool typeWasDeclaredInQmldir = false;
@@ -299,6 +308,7 @@ bool QDeclarativeImportedNamespace::find_helper(int i, const QByteArray& type, i
 
                 // importing version -1 means import ALL versions
                 if ((vmaj == -1) || (c.majorVersion < vmaj || (c.majorVersion == vmaj && vmin >= c.minorVersion))) {
+                    QUrl url = QUrl(urls.at(i) + QLatin1Char('/') + QString::fromUtf8(type) + QLatin1String(".qml"));
                     QUrl candidate = url.resolved(QUrl(c.fileName));
                     if (c.internal && base) {
                         if (base->resolved(QUrl(c.fileName)) != candidate)
@@ -319,8 +329,9 @@ bool QDeclarativeImportedNamespace::find_helper(int i, const QByteArray& type, i
 
     if (!typeWasDeclaredInQmldir  && !isLibrary.at(i)) {
         // XXX search non-files too! (eg. zip files, see QT-524)
-        QFileInfo f(QDeclarativeEnginePrivate::urlToLocalFileOrQrc(url));
-        if (f.exists()) {
+        QUrl url = QUrl(urls.at(i) + QLatin1Char('/') + QString::fromUtf8(type) + QLatin1String(".qml"));
+        QString file = QDeclarativeEnginePrivate::urlToLocalFileOrQrc(url);
+        if (!typeLoader->absoluteFilePath(file).isEmpty()) {
             if (base && *base == url) { // no recursion
                 if (typeRecursionDetected)
                     *typeRecursionDetected = true;
@@ -334,9 +345,8 @@ bool QDeclarativeImportedNamespace::find_helper(int i, const QByteArray& type, i
     return false;
 }
 
-QDeclarativeImportsPrivate::QDeclarativeImportsPrivate() 
-: ref(1)
-{
+QDeclarativeImportsPrivate::QDeclarativeImportsPrivate(QDeclarativeTypeLoader *loader)
+    : ref(1), typeLoader(loader){
 }
 
 QDeclarativeImportsPrivate::~QDeclarativeImportsPrivate()
@@ -349,32 +359,31 @@ bool QDeclarativeImportsPrivate::importExtension(const QString &absoluteFilePath
                                                  QDeclarativeImportDatabase *database, 
                                                  QDeclarativeDirComponents* components, QString *errorString) 
 {
-    QFile file(absoluteFilePath);
-    QString filecontent;
-    if (file.open(QFile::ReadOnly)) {
-        filecontent = QString::fromUtf8(file.readAll());
-        if (qmlImportTrace())
-            qDebug().nospace() << "QDeclarativeImports(" << qPrintable(base.toString()) << "::importExtension: "
-                               << "loaded " << absoluteFilePath;
-    } else {
-        if (errorString)
-            *errorString = QDeclarativeImportDatabase::tr("module \"%1\" definition \"%2\" not readable").arg(uri).arg(absoluteFilePath);
+    Q_ASSERT(typeLoader);
+    const QDeclarativeDirParser *qmldirParser = typeLoader->qmlDirParser(absoluteFilePath);
+    if (qmldirParser->hasError()) {
+        if (errorString) {
+            const QList<QDeclarativeError> qmldirErrors = qmldirParser->errors(uri);
+            for (int i = 0; i < qmldirErrors.size(); ++i)
+                *errorString += qmldirErrors.at(i).description();
+        }
         return false;
     }
-    QDir dir = QFileInfo(file).dir();
-
-    QDeclarativeDirParser qmldirParser;
-    qmldirParser.setSource(filecontent);
-    qmldirParser.parse();
 
     if (! qmlDirFilesForWhichPluginsHaveBeenLoaded.contains(absoluteFilePath)) {
         qmlDirFilesForWhichPluginsHaveBeenLoaded.insert(absoluteFilePath);
 
-
-        foreach (const QDeclarativeDirParser::Plugin &plugin, qmldirParser.plugins()) {
+        QDir dir = QFileInfo(absoluteFilePath).dir();
+        foreach (const QDeclarativeDirParser::Plugin &plugin, qmldirParser->plugins()) {
 
             QString resolvedFilePath = database->resolvePlugin(dir, plugin.path, plugin.name);
-
+#if defined(QT_LIBINFIX) && defined(Q_OS_SYMBIAN)
+            if (resolvedFilePath.isEmpty()) {
+                // In case of libinfixed build, attempt to load libinfixed version, too.
+                QString infixedPluginName = plugin.name + QLatin1String(QT_LIBINFIX);
+                resolvedFilePath = database->resolvePlugin(dir, plugin.path, infixedPluginName);
+            }
+#endif
             if (!resolvedFilePath.isEmpty()) {
                 if (!database->importPlugin(resolvedFilePath, uri, errorString)) {
                     if (errorString)
@@ -390,7 +399,7 @@ bool QDeclarativeImportsPrivate::importExtension(const QString &absoluteFilePath
     }
 
     if (components)
-        *components = qmldirParser.components();
+        *components = qmldirParser->components();
 
     return true;
 }
@@ -411,8 +420,18 @@ QString QDeclarativeImportsPrivate::resolvedUri(const QString &dir_arg, QDeclara
             break;
         }
     }
+
+    stableRelativePath.replace(QLatin1Char('\\'), QLatin1Char('/'));
+
+    // remove optional versioning in dot notation from uri
+    int lastSlash = stableRelativePath.lastIndexOf(QLatin1Char('/'));
+    if (lastSlash >= 0) {
+        int versionDot = stableRelativePath.indexOf(QLatin1Char('.'), lastSlash);
+        if (versionDot >= 0)
+            stableRelativePath = stableRelativePath.left(versionDot);
+    }
+
     stableRelativePath.replace(QLatin1Char('/'), QLatin1Char('.'));
-    stableRelativePath.replace(QLatin1Char('\\'), QLatin1Char('.'));
     return stableRelativePath;
 }
 
@@ -421,6 +440,7 @@ bool QDeclarativeImportsPrivate::add(const QDeclarativeDirComponents &qmldircomp
                                      QDeclarativeScriptParser::Import::Type importType, 
                                      QDeclarativeImportDatabase *database, QString *errorString)
 {
+    Q_ASSERT(typeLoader);
     QDeclarativeDirComponents qmldircomponents = qmldircomponentsnetwork;
     QString uri = uri_arg;
     QDeclarativeImportedNamespace *s;
@@ -433,41 +453,84 @@ bool QDeclarativeImportsPrivate::add(const QDeclarativeDirComponents &qmldircomp
     }
 
     QString url = uri;
+    bool versionFound = false;
     if (importType == QDeclarativeScriptParser::Import::Library) {
         url.replace(QLatin1Char('.'), QLatin1Char('/'));
         bool found = false;
         QString dir;
+        QString qmldir;
 
+        // step 1: search for extension with fully encoded version number
+        if (vmaj >= 0 && vmin >= 0) {
+            foreach (const QString &p, database->fileImportPath) {
+                dir = p+QLatin1Char('/')+url;
+                qmldir = dir+QString(QLatin1String(".%1.%2")).arg(vmaj).arg(vmin)+QLatin1String("/qmldir");
 
-        foreach (const QString &p, database->fileImportPath) {
-            dir = p+QLatin1Char('/')+url;
+                QString absoluteFilePath = typeLoader->absoluteFilePath(qmldir);
+                if (!absoluteFilePath.isEmpty()) {
+                    found = true;
 
-            QFileInfo fi(dir+QLatin1String("/qmldir"));
-            const QString absoluteFilePath = fi.absoluteFilePath();
+                    QString absolutePath = absoluteFilePath.left(absoluteFilePath.lastIndexOf(QLatin1Char('/')));
+                    url = QUrl::fromLocalFile(absolutePath).toString();
+                    uri = resolvedUri(dir, database);
+                    if (!importExtension(absoluteFilePath, uri, database, &qmldircomponents, errorString))
+                        return false;
+                    break;
+                }
+            }
+        }
+        // step 2: search for extension with encoded version major
+        if (vmaj >= 0 && vmin >= 0) {
+            foreach (const QString &p, database->fileImportPath) {
+                dir = p+QLatin1Char('/')+url;
+                qmldir = dir+QString(QLatin1String(".%1")).arg(vmaj)+QLatin1String("/qmldir");
 
-            if (fi.isFile()) {
-                found = true;
+                QString absoluteFilePath = typeLoader->absoluteFilePath(qmldir);
+                if (!absoluteFilePath.isEmpty()) {
+                    found = true;
 
-                url = QUrl::fromLocalFile(fi.absolutePath()).toString();
-                uri = resolvedUri(dir, database);
-                if (!importExtension(absoluteFilePath, uri, database, &qmldircomponents, errorString))
-                    return false;
-                break;
+                    QString absolutePath = absoluteFilePath.left(absoluteFilePath.lastIndexOf(QLatin1Char('/')));
+                    url = QUrl::fromLocalFile(absolutePath).toString();
+                    uri = resolvedUri(dir, database);
+                    if (!importExtension(absoluteFilePath, uri, database, &qmldircomponents, errorString))
+                        return false;
+                    break;
+                }
+            }
+        }
+        if (!found) {
+            // step 3: search for extension without version number
+
+            foreach (const QString &p, database->fileImportPath) {
+                dir = p+QLatin1Char('/')+url;
+                qmldir = dir+QLatin1String("/qmldir");
+
+                QString absoluteFilePath = typeLoader->absoluteFilePath(qmldir);
+                if (!absoluteFilePath.isEmpty()) {
+                    found = true;
+
+                    QString absolutePath = absoluteFilePath.left(absoluteFilePath.lastIndexOf(QLatin1Char('/')));
+                    url = QUrl::fromLocalFile(absolutePath).toString();
+                    uri = resolvedUri(dir, database);
+                    if (!importExtension(absoluteFilePath, uri, database, &qmldircomponents, errorString))
+                        return false;
+                    break;
+                }
             }
         }
 
-        if (!found) {
-            found = QDeclarativeMetaType::isModule(uri.toUtf8(), vmaj, vmin);
-            if (!found) {
-                if (errorString) {
-                    bool anyversion = QDeclarativeMetaType::isModule(uri.toUtf8(), -1, -1);
-                    if (anyversion)
-                        *errorString = QDeclarativeImportDatabase::tr("module \"%1\" version %2.%3 is not installed").arg(uri_arg).arg(vmaj).arg(vmin);
-                    else
-                        *errorString = QDeclarativeImportDatabase::tr("module \"%1\" is not installed").arg(uri_arg);
-                }
-                return false;
+        if (QDeclarativeMetaType::isModule(uri.toUtf8(), vmaj, vmin))
+            versionFound = true;
+
+        if (!versionFound && qmldircomponents.isEmpty()) {
+            if (errorString) {
+                bool anyversion = QDeclarativeMetaType::isModule(uri.toUtf8(), -1, -1);
+                if (anyversion)
+                    *errorString = QDeclarativeImportDatabase::tr("module \"%1\" version %2.%3 is not installed").arg(uri_arg).arg(vmaj).arg(vmin);
+                else
+                    *errorString = QDeclarativeImportDatabase::tr("module \"%1\" is not installed").arg(uri_arg);
             }
+            return false;
         }
     } else {
 
@@ -485,7 +548,7 @@ bool QDeclarativeImportsPrivate::add(const QDeclarativeDirComponents &qmldircomp
                 uri = resolvedUri(QDeclarativeEnginePrivate::urlToLocalFileOrQrc(base.resolved(QUrl(uri))), database);
                 if (uri.endsWith(QLatin1Char('/')))
                     uri.chop(1);
-                if (QFile::exists(localFileOrQrc)) {
+                if (!typeLoader->absoluteFilePath(localFileOrQrc).isEmpty()) {
                     if (!importExtension(localFileOrQrc,uri,database,&qmldircomponents,errorString))
                         return false;
                 }
@@ -512,7 +575,7 @@ bool QDeclarativeImportsPrivate::add(const QDeclarativeDirComponents &qmldircomp
             url.chop(1);
     }
 
-    if (vmaj > -1 && vmin > -1 && !qmldircomponents.isEmpty()) {
+    if (!versionFound && vmaj > -1 && vmin > -1 && !qmldircomponents.isEmpty()) {
         QList<QDeclarativeDirParser::Component>::ConstIterator it = qmldircomponents.begin();
         int lowest_maj = INT_MAX;
         int lowest_min = INT_MAX;
@@ -548,6 +611,7 @@ bool QDeclarativeImportsPrivate::add(const QDeclarativeDirComponents &qmldircomp
 bool QDeclarativeImportsPrivate::find(const QByteArray& type, int *vmajor, int *vminor, QDeclarativeType** type_return,
                                       QUrl* url_return, QString *errorString)
 {
+    Q_ASSERT(typeLoader);
     QDeclarativeImportedNamespace *s = 0;
     int slash = type.indexOf('/');
     if (slash >= 0) {
@@ -569,7 +633,7 @@ bool QDeclarativeImportsPrivate::find(const QByteArray& type, int *vmajor, int *
     }
     QByteArray unqualifiedtype = slash < 0 ? type : type.mid(slash+1); // common-case opt (QString::mid works fine, but slower)
     if (s) {
-        if (s->find(unqualifiedtype,vmajor,vminor,type_return,url_return, &base, errorString))
+        if (s->find(typeLoader, unqualifiedtype,vmajor,vminor,type_return,url_return, &base, errorString))
             return true;
         if (s->urls.count() == 1 && !s->isLibrary[0] && url_return && s != &unqualifiedset) {
             // qualified, and only 1 url
@@ -586,16 +650,16 @@ QDeclarativeImportedNamespace *QDeclarativeImportsPrivate::findNamespace(const Q
     return set.value(type);
 }
 
-bool QDeclarativeImportedNamespace::find(const QByteArray& type, int *vmajor, int *vminor, QDeclarativeType** type_return,
+bool QDeclarativeImportedNamespace::find(QDeclarativeTypeLoader *typeLoader, const QByteArray& type, int *vmajor, int *vminor, QDeclarativeType** type_return,
           QUrl* url_return, QUrl *base, QString *errorString)
 {
     bool typeRecursionDetected = false;
     for (int i=0; i<urls.count(); ++i) {
-        if (find_helper(i, type, vmajor, vminor, type_return, url_return, base, &typeRecursionDetected)) {
+        if (find_helper(typeLoader, i, type, vmajor, vminor, type_return, url_return, base, &typeRecursionDetected)) {
             if (qmlCheckTypes()) {
                 // check for type clashes
                 for (int j = i+1; j<urls.count(); ++j) {
-                    if (find_helper(j, type, vmajor, vminor, 0, 0, base)) {
+                    if (find_helper(typeLoader, j, type, vmajor, vminor, 0, 0, base)) {
                         if (errorString) {
                             QString u1 = urls.at(i);
                             QString u2 = urls.at(j);
@@ -646,6 +710,7 @@ bool QDeclarativeImportedNamespace::find(const QByteArray& type, int *vmajor, in
 /*!
 \class QDeclarativeImportDatabase
 \brief The QDeclarativeImportDatabase class manages the QML imports for a QDeclarativeEngine.
+\internal
 */
 QDeclarativeImportDatabase::QDeclarativeImportDatabase(QDeclarativeEngine *e)
 : engine(e)
@@ -654,7 +719,42 @@ QDeclarativeImportDatabase::QDeclarativeImportDatabase(QDeclarativeEngine *e)
 
     // Search order is applicationDirPath(), $QML_IMPORT_PATH, QLibraryInfo::ImportsPath
 
-    addImportPath(QLibraryInfo::location(QLibraryInfo::ImportsPath));
+#ifndef QT_NO_SETTINGS
+    QString installImportsPath =  QLibraryInfo::location(QLibraryInfo::ImportsPath);
+
+#if defined(Q_OS_SYMBIAN)
+    // Append imports path for all available drives in Symbian
+    if (installImportsPath.at(1) != QChar(QLatin1Char(':'))) {
+        QString tempPath = installImportsPath;
+        if (tempPath.at(tempPath.length() - 1) != QDir::separator()) {
+            tempPath += QDir::separator();
+        }
+        RFs& fs = qt_s60GetRFs();
+        TPtrC tempPathPtr(reinterpret_cast<const TText*> (tempPath.constData()));
+        // Symbian searches should start from Y:. Fix start drive otherwise TFindFile starts from the session drive
+        _LIT(KStartDir, "Y:");
+        TFileName dirPath(KStartDir);
+        dirPath.Append(tempPathPtr);
+        TFindFile finder(fs);
+        TInt err = finder.FindByDir(tempPathPtr, dirPath);
+        while (err == KErrNone) {
+            QString foundDir(reinterpret_cast<const QChar *>(finder.File().Ptr()),
+                             finder.File().Length());
+            foundDir = QDir(foundDir).canonicalPath();
+            addImportPath(foundDir);
+            err = finder.Find();
+        }
+        // TFindFile found the directories in the order we want, but addImportPath reverses it.
+        // Reverse the order again to get it right.
+        QAlgorithmsPrivate::qReverse(fileImportPath.begin(), fileImportPath.end());
+    } else {
+        addImportPath(installImportsPath);
+    }
+#else
+    addImportPath(installImportsPath);
+#endif
+
+#endif // QT_NO_SETTINGS
 
     // env import paths
     QByteArray envImportPath = qgetenv("QML_IMPORT_PATH");
@@ -836,23 +936,33 @@ QString QDeclarativeImportDatabase::resolvePlugin(const QDir &qmldirPath, const
 #endif
 }
 
+/*!
+    \internal
+*/
 QStringList QDeclarativeImportDatabase::pluginPathList() const
 {
     return filePluginPath;
 }
 
+/*!
+    \internal
+*/
 void QDeclarativeImportDatabase::setPluginPathList(const QStringList &paths)
 {
     filePluginPath = paths;
 }
 
+/*!
+    \internal
+*/
 void QDeclarativeImportDatabase::addPluginPath(const QString& path)
 {
     if (qmlImportTrace())
         qDebug().nospace() << "QDeclarativeImportDatabase::addPluginPath: " << path;
 
     QUrl url = QUrl(path);
-    if (url.isRelative() || url.scheme() == QLatin1String("file")) {
+    if (url.isRelative() || url.scheme() == QLatin1String("file")
+            || (url.scheme().length() == 1 && QFile::exists(path)) ) {  // windows path
         QDir dir = QDir(path);
         filePluginPath.prepend(dir.canonicalPath());
     } else {
@@ -860,6 +970,9 @@ void QDeclarativeImportDatabase::addPluginPath(const QString& path)
     }
 }
 
+/*!
+    \internal
+*/
 void QDeclarativeImportDatabase::addImportPath(const QString& path)
 {
     if (qmlImportTrace())
@@ -871,7 +984,8 @@ void QDeclarativeImportDatabase::addImportPath(const QString& path)
     QUrl url = QUrl(path);
     QString cPath;
 
-    if (url.isRelative() || url.scheme() == QLatin1String("file")) {
+    if (url.isRelative() || url.scheme() == QLatin1String("file")
+            || (url.scheme().length() == 1 && QFile::exists(path)) ) {  // windows path
         QDir dir = QDir(path);
         cPath = dir.canonicalPath();
     } else {
@@ -884,22 +998,31 @@ void QDeclarativeImportDatabase::addImportPath(const QString& path)
         fileImportPath.prepend(cPath);
 }
 
+/*!
+    \internal
+*/
 QStringList QDeclarativeImportDatabase::importPathList() const
 {
     return fileImportPath;
 }
 
+/*!
+    \internal
+*/
 void QDeclarativeImportDatabase::setImportPathList(const QStringList &paths)
 {
     fileImportPath = paths;
 }
 
-
+/*!
+    \internal
+*/
 bool QDeclarativeImportDatabase::importPlugin(const QString &filePath, const QString &uri, QString *errorString)
 {
     if (qmlImportTrace())
         qDebug().nospace() << "QDeclarativeImportDatabase::importPlugin: " << uri << " from " << filePath;
 
+#ifndef QT_NO_LIBRARY
     QFileInfo fileInfo(filePath);
     const QString absoluteFilePath = fileInfo.absoluteFilePath();
 
@@ -913,6 +1036,11 @@ bool QDeclarativeImportDatabase::importPlugin(const QString &filePath, const QSt
     }
 
     if (!engineInitialized || !typesRegistered) {
+        if (!QDeclarative_isFileCaseCorrect(absoluteFilePath)) {
+            if (errorString) 
+                *errorString = tr("File name case mismatch for \"%1\"").arg(absoluteFilePath);
+            return false;
+        }
         QPluginLoader loader(absoluteFilePath);
 
         if (!loader.load()) {
@@ -946,6 +1074,9 @@ bool QDeclarativeImportDatabase::importPlugin(const QString &filePath, const QSt
     }
 
     return true;
+#else
+    return false;
+#endif
 }
 
 
