@@ -1,40 +1,40 @@
 /****************************************************************************
 **
-** Copyright (C) 2010 Nokia Corporation and/or its subsidiary(-ies).
+** Copyright (C) 2011 Nokia Corporation and/or its subsidiary(-ies).
 ** All rights reserved.
 ** Contact: Nokia Corporation (qt-info@nokia.com)
 **
 ** This file is part of the QtDeclarative module of the Qt Toolkit.
 **
 ** $QT_BEGIN_LICENSE:LGPL$
-** Commercial Usage
-** Licensees holding valid Qt Commercial licenses may use this file in
-** accordance with the Qt Commercial License Agreement provided with the
-** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and Nokia.
-**
 ** GNU Lesser General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU Lesser
-** General Public License version 2.1 as published by the Free Software
-** Foundation and appearing in the file LICENSE.LGPL included in the
-** packaging of this file.  Please review the following information to
-** ensure the GNU Lesser General Public License version 2.1 requirements
-** will be met: http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
+** This file may be used under the terms of the GNU Lesser General Public
+** License version 2.1 as published by the Free Software Foundation and
+** appearing in the file LICENSE.LGPL included in the packaging of this
+** file. Please review the following information to ensure the GNU Lesser
+** General Public License version 2.1 requirements will be met:
+** http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
 **
 ** In addition, as a special exception, Nokia gives you certain additional
-** rights.  These rights are described in the Nokia Qt LGPL Exception
+** rights. These rights are described in the Nokia Qt LGPL Exception
 ** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
 **
 ** GNU General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU
-** General Public License version 3.0 as published by the Free Software
-** Foundation and appearing in the file LICENSE.GPL included in the
-** packaging of this file.  Please review the following information to
-** ensure the GNU General Public License version 3.0 requirements will be
-** met: http://www.gnu.org/copyleft/gpl.html.
+** Alternatively, this file may be used under the terms of the GNU General
+** Public License version 3.0 as published by the Free Software Foundation
+** and appearing in the file LICENSE.GPL included in the packaging of this
+** file. Please review the following information to ensure the GNU General
+** Public License version 3.0 requirements will be met:
+** http://www.gnu.org/copyleft/gpl.html.
+**
+** Other Usage
+** Alternatively, this file may be used in accordance with the terms and
+** conditions contained in a signed written agreement between you and Nokia.
+**
+**
+**
+**
 **
-** If you have questions regarding the use of this file, please contact
-** Nokia at qt-info@nokia.com.
 ** $QT_END_LICENSE$
 **
 ****************************************************************************/
@@ -42,13 +42,12 @@
 #include "private/qdeclarativevaluetype_p.h"
 
 #include "private/qdeclarativemetatype_p.h"
+#include "private/qfont_p.h"
 
 #include <QtCore/qdebug.h>
 
 QT_BEGIN_NAMESPACE
 
-Q_GUI_EXPORT int qt_defaultDpi();
-
 template<typename T>
 int qmlRegisterValueTypeEnums(const char *uri, int versionMajor, int versionMinor, const char *qmlName)
 {
@@ -71,6 +70,7 @@ int qmlRegisterValueTypeEnums(const char *uri, int versionMajor, int versionMino
 
         0, 0,
 
+        0,
         0
     };
 
@@ -109,36 +109,54 @@ void QDeclarativeValueTypeFactory::registerValueTypes()
 
 QDeclarativeValueType *QDeclarativeValueTypeFactory::valueType(int t)
 {
+    QDeclarativeValueType *rv = 0;
+
     switch (t) {
     case QVariant::Point:
-        return new QDeclarativePointValueType;
+        rv = new QDeclarativePointValueType;
+        break;
     case QVariant::PointF:
-        return new QDeclarativePointFValueType;
+        rv = new QDeclarativePointFValueType;
+        break;
     case QVariant::Size:
-        return new QDeclarativeSizeValueType;
+        rv = new QDeclarativeSizeValueType;
+        break;
     case QVariant::SizeF:
-        return new QDeclarativeSizeFValueType;
+        rv = new QDeclarativeSizeFValueType;
+        break;
     case QVariant::Rect:
-        return new QDeclarativeRectValueType;
+        rv = new QDeclarativeRectValueType;
+        break;
     case QVariant::RectF:
-        return new QDeclarativeRectFValueType;
+        rv = new QDeclarativeRectFValueType;
+        break;
     case QVariant::Vector2D:
-        return new QDeclarativeVector2DValueType;
+        rv = new QDeclarativeVector2DValueType;
+        break;
     case QVariant::Vector3D:
-        return new QDeclarativeVector3DValueType;
+        rv = new QDeclarativeVector3DValueType;
+        break;
     case QVariant::Vector4D:
-        return new QDeclarativeVector4DValueType;
+        rv = new QDeclarativeVector4DValueType;
+        break;
     case QVariant::Quaternion:
-        return new QDeclarativeQuaternionValueType;
+        rv = new QDeclarativeQuaternionValueType;
+        break;
     case QVariant::Matrix4x4:
-        return new QDeclarativeMatrix4x4ValueType;
+        rv = new QDeclarativeMatrix4x4ValueType;
+        break;
     case QVariant::EasingCurve:
-        return new QDeclarativeEasingValueType;
+        rv = new QDeclarativeEasingValueType;
+        break;
     case QVariant::Font:
-        return new QDeclarativeFontValueType;
+        rv = new QDeclarativeFontValueType;
+        break;
     default:
-        return 0;
+        break;
     }
+
+    Q_ASSERT(!rv || rv->metaObject()->propertyCount() < 32);
+    return rv;
 }
 
 QDeclarativeValueType::QDeclarativeValueType(QObject *parent)
