@@ -1,40 +1,40 @@
 /****************************************************************************
 **
-** Copyright (C) 2010 Nokia Corporation and/or its subsidiary(-ies).
+** Copyright (C) 2011 Nokia Corporation and/or its subsidiary(-ies).
 ** All rights reserved.
 ** Contact: Nokia Corporation (qt-info@nokia.com)
 **
 ** This file is part of the QtDeclarative module of the Qt Toolkit.
 **
 ** $QT_BEGIN_LICENSE:LGPL$
-** Commercial Usage
-** Licensees holding valid Qt Commercial licenses may use this file in
-** accordance with the Qt Commercial License Agreement provided with the
-** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and Nokia.
-**
 ** GNU Lesser General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU Lesser
-** General Public License version 2.1 as published by the Free Software
-** Foundation and appearing in the file LICENSE.LGPL included in the
-** packaging of this file.  Please review the following information to
-** ensure the GNU Lesser General Public License version 2.1 requirements
-** will be met: http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
+** This file may be used under the terms of the GNU Lesser General Public
+** License version 2.1 as published by the Free Software Foundation and
+** appearing in the file LICENSE.LGPL included in the packaging of this
+** file. Please review the following information to ensure the GNU Lesser
+** General Public License version 2.1 requirements will be met:
+** http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
 **
 ** In addition, as a special exception, Nokia gives you certain additional
-** rights.  These rights are described in the Nokia Qt LGPL Exception
+** rights. These rights are described in the Nokia Qt LGPL Exception
 ** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
 **
 ** GNU General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU
-** General Public License version 3.0 as published by the Free Software
-** Foundation and appearing in the file LICENSE.GPL included in the
-** packaging of this file.  Please review the following information to
-** ensure the GNU General Public License version 3.0 requirements will be
-** met: http://www.gnu.org/copyleft/gpl.html.
+** Alternatively, this file may be used under the terms of the GNU General
+** Public License version 3.0 as published by the Free Software Foundation
+** and appearing in the file LICENSE.GPL included in the packaging of this
+** file. Please review the following information to ensure the GNU General
+** Public License version 3.0 requirements will be met:
+** http://www.gnu.org/copyleft/gpl.html.
+**
+** Other Usage
+** Alternatively, this file may be used in accordance with the terms and
+** conditions contained in a signed written agreement between you and Nokia.
+**
+**
+**
+**
 **
-** If you have questions regarding the use of this file, please contact
-** Nokia at qt-info@nokia.com.
 ** $QT_END_LICENSE$
 **
 ****************************************************************************/
@@ -83,6 +83,13 @@ public:
     QDeclarativeNullableValue<QDeclarativeScriptString> scaleString;
     QDeclarativeNullableValue<QDeclarativeScriptString> rotationString;
 
+    QDeclarativeNullableValue<qreal> x;
+    QDeclarativeNullableValue<qreal> y;
+    QDeclarativeNullableValue<qreal> width;
+    QDeclarativeNullableValue<qreal> height;
+    QDeclarativeNullableValue<qreal> scale;
+    QDeclarativeNullableValue<qreal> rotation;
+
     void doChange(QDeclarativeItem *targetParent, QDeclarativeItem *stackBefore = 0);
 };
 
@@ -116,7 +123,7 @@ void QDeclarativeParentChangePrivate::doChange(QDeclarativeItem *targetParent, Q
             }
 
             if (scale != 0)
-                rotation = atan2(transform.m12()/scale, transform.m11()/scale) * 180/M_PI;
+                rotation = atan2(transform.m12()/scale, transform.m11()/scale) * 180/qreal(M_PI);
             else {
                 qmlInfo(q) << QDeclarativeParentChange::tr("Unable to preserve appearance under scale of 0");
                 ok = false;
@@ -213,10 +220,21 @@ QDeclarativeScriptString QDeclarativeParentChange::x() const
     return d->xString.value;
 }
 
+void tryReal(QDeclarativeNullableValue<qreal> &value, const QString &string)
+{
+    bool ok = false;
+    qreal realValue = string.toFloat(&ok);
+    if (ok)
+        value = realValue;
+    else
+        value.invalidate();
+}
+
 void QDeclarativeParentChange::setX(QDeclarativeScriptString x)
 {
     Q_D(QDeclarativeParentChange);
     d->xString = x;
+    tryReal(d->x, x.script());
 }
 
 bool QDeclarativeParentChange::xIsSet() const
@@ -235,6 +253,7 @@ void QDeclarativeParentChange::setY(QDeclarativeScriptString y)
 {
     Q_D(QDeclarativeParentChange);
     d->yString = y;
+    tryReal(d->y, y.script());
 }
 
 bool QDeclarativeParentChange::yIsSet() const
@@ -253,6 +272,7 @@ void QDeclarativeParentChange::setWidth(QDeclarativeScriptString width)
 {
     Q_D(QDeclarativeParentChange);
     d->widthString = width;
+    tryReal(d->width, width.script());
 }
 
 bool QDeclarativeParentChange::widthIsSet() const
@@ -271,6 +291,7 @@ void QDeclarativeParentChange::setHeight(QDeclarativeScriptString height)
 {
     Q_D(QDeclarativeParentChange);
     d->heightString = height;
+    tryReal(d->height, height.script());
 }
 
 bool QDeclarativeParentChange::heightIsSet() const
@@ -289,6 +310,7 @@ void QDeclarativeParentChange::setScale(QDeclarativeScriptString scale)
 {
     Q_D(QDeclarativeParentChange);
     d->scaleString = scale;
+    tryReal(d->scale, scale.script());
 }
 
 bool QDeclarativeParentChange::scaleIsSet() const
@@ -307,6 +329,7 @@ void QDeclarativeParentChange::setRotation(QDeclarativeScriptString rotation)
 {
     Q_D(QDeclarativeParentChange);
     d->rotationString = rotation;
+    tryReal(d->rotation, rotation.script());
 }
 
 bool QDeclarativeParentChange::rotationIsSet() const
@@ -367,16 +390,15 @@ QDeclarativeStateOperation::ActionList QDeclarativeParentChange::actions()
     a.event = this;
     actions << a;
 
+    QDeclarativeContext *ctxt = qmlContext(this);
+
     if (d->xString.isValid()) {
-        bool ok = false;
-        QString script = d->xString.value.script();
-        qreal x = script.toFloat(&ok);
-        if (ok) {
-            QDeclarativeAction xa(d->target, QLatin1String("x"), x);
+        if (d->x.isValid()) {
+            QDeclarativeAction xa(d->target, QLatin1String("x"), ctxt, d->x.value);
             actions << xa;
         } else {
-            QDeclarativeBinding *newBinding = new QDeclarativeBinding(script, d->target, qmlContext(this));
-            newBinding->setTarget(QDeclarativeProperty(d->target, QLatin1String("x")));
+            QDeclarativeBinding *newBinding = new QDeclarativeBinding(d->xString.value.script(), d->target, ctxt);
+            newBinding->setTarget(QDeclarativeProperty(d->target, QLatin1String("x"), ctxt));
             QDeclarativeAction xa;
             xa.property = newBinding->property();
             xa.toBinding = newBinding;
@@ -387,15 +409,12 @@ QDeclarativeStateOperation::ActionList QDeclarativeParentChange::actions()
     }
 
     if (d->yString.isValid()) {
-        bool ok = false;
-        QString script = d->yString.value.script();
-        qreal y = script.toFloat(&ok);
-        if (ok) {
-            QDeclarativeAction ya(d->target, QLatin1String("y"), y);
+        if (d->y.isValid()) {
+            QDeclarativeAction ya(d->target, QLatin1String("y"), ctxt, d->y.value);
             actions << ya;
         } else {
-            QDeclarativeBinding *newBinding = new QDeclarativeBinding(script, d->target, qmlContext(this));
-            newBinding->setTarget(QDeclarativeProperty(d->target, QLatin1String("y")));
+            QDeclarativeBinding *newBinding = new QDeclarativeBinding(d->yString.value.script(), d->target, ctxt);
+            newBinding->setTarget(QDeclarativeProperty(d->target, QLatin1String("y"), ctxt));
             QDeclarativeAction ya;
             ya.property = newBinding->property();
             ya.toBinding = newBinding;
@@ -406,15 +425,12 @@ QDeclarativeStateOperation::ActionList QDeclarativeParentChange::actions()
     }
 
     if (d->scaleString.isValid()) {
-        bool ok = false;
-        QString script = d->scaleString.value.script();
-        qreal scale = script.toFloat(&ok);
-        if (ok) {
-            QDeclarativeAction sa(d->target, QLatin1String("scale"), scale);
+        if (d->scale.isValid()) {
+            QDeclarativeAction sa(d->target, QLatin1String("scale"), ctxt, d->scale.value);
             actions << sa;
         } else {
-            QDeclarativeBinding *newBinding = new QDeclarativeBinding(script, d->target, qmlContext(this));
-            newBinding->setTarget(QDeclarativeProperty(d->target, QLatin1String("scale")));
+            QDeclarativeBinding *newBinding = new QDeclarativeBinding(d->scaleString.value.script(), d->target, ctxt);
+            newBinding->setTarget(QDeclarativeProperty(d->target, QLatin1String("scale"), ctxt));
             QDeclarativeAction sa;
             sa.property = newBinding->property();
             sa.toBinding = newBinding;
@@ -425,15 +441,12 @@ QDeclarativeStateOperation::ActionList QDeclarativeParentChange::actions()
     }
 
     if (d->rotationString.isValid()) {
-        bool ok = false;
-        QString script = d->rotationString.value.script();
-        qreal rotation = script.toFloat(&ok);
-        if (ok) {
-            QDeclarativeAction ra(d->target, QLatin1String("rotation"), rotation);
+        if (d->rotation.isValid()) {
+            QDeclarativeAction ra(d->target, QLatin1String("rotation"), ctxt, d->rotation.value);
             actions << ra;
         } else {
-            QDeclarativeBinding *newBinding = new QDeclarativeBinding(script, d->target, qmlContext(this));
-            newBinding->setTarget(QDeclarativeProperty(d->target, QLatin1String("rotation")));
+            QDeclarativeBinding *newBinding = new QDeclarativeBinding(d->rotationString.value.script(), d->target, ctxt);
+            newBinding->setTarget(QDeclarativeProperty(d->target, QLatin1String("rotation"), ctxt));
             QDeclarativeAction ra;
             ra.property = newBinding->property();
             ra.toBinding = newBinding;
@@ -444,15 +457,12 @@ QDeclarativeStateOperation::ActionList QDeclarativeParentChange::actions()
     }
 
     if (d->widthString.isValid()) {
-        bool ok = false;
-        QString script = d->widthString.value.script();
-        qreal width = script.toFloat(&ok);
-        if (ok) {
-            QDeclarativeAction wa(d->target, QLatin1String("width"), width);
+        if (d->width.isValid()) {
+            QDeclarativeAction wa(d->target, QLatin1String("width"), ctxt, d->width.value);
             actions << wa;
         } else {
-            QDeclarativeBinding *newBinding = new QDeclarativeBinding(script, d->target, qmlContext(this));
-            newBinding->setTarget(QDeclarativeProperty(d->target, QLatin1String("width")));
+            QDeclarativeBinding *newBinding = new QDeclarativeBinding(d->widthString.value.script(), d->target, ctxt);
+            newBinding->setTarget(QDeclarativeProperty(d->target, QLatin1String("width"), ctxt));
             QDeclarativeAction wa;
             wa.property = newBinding->property();
             wa.toBinding = newBinding;
@@ -463,15 +473,12 @@ QDeclarativeStateOperation::ActionList QDeclarativeParentChange::actions()
     }
 
     if (d->heightString.isValid()) {
-        bool ok = false;
-        QString script = d->heightString.value.script();
-        qreal height = script.toFloat(&ok);
-        if (ok) {
-            QDeclarativeAction ha(d->target, QLatin1String("height"), height);
+        if (d->height.isValid()) {
+            QDeclarativeAction ha(d->target, QLatin1String("height"), ctxt, d->height.value);
             actions << ha;
         } else {
-            QDeclarativeBinding *newBinding = new QDeclarativeBinding(script, d->target, qmlContext(this));
-            newBinding->setTarget(QDeclarativeProperty(d->target, QLatin1String("height")));
+            QDeclarativeBinding *newBinding = new QDeclarativeBinding(d->heightString.value.script(), d->target, ctxt);
+            newBinding->setTarget(QDeclarativeProperty(d->target, QLatin1String("height"), ctxt));
             QDeclarativeAction ha;
             ha.property = newBinding->property();
             ha.toBinding = newBinding;
@@ -599,25 +606,7 @@ public:
     ScriptAction to specify the point in the transition at which
     the StateChangeScript should to be run.
 
-    \qml
-    State {
-        name "state1"
-        StateChangeScript {
-            name: "myScript"
-            script: doStateStuff();
-        }
-        ...
-    }
-    ...
-    Transition {
-        to: "state1"
-        SequentialAnimation {
-            NumberAnimation { ... }
-            ScriptAction { scriptName: "myScript" }
-            NumberAnimation { ... }
-        }
-    }
-    \endqml
+    \snippet snippets/declarative/states/statechangescript.qml state and transition
 
     \sa ScriptAction
 */
@@ -648,11 +637,11 @@ void QDeclarativeStateChangeScript::setScript(const QDeclarativeScriptString &s)
 }
 
 /*!
-    \qmlproperty script StateChangeScript::script
+    \qmlproperty string StateChangeScript::name
     This property holds the name of the script. This name can be used by a
     ScriptAction to target a specific script.
 
-    \sa ScriptAction::stateChangeScriptName
+    \sa ScriptAction::scriptName
 */
 QString QDeclarativeStateChangeScript::name() const
 {
@@ -1075,32 +1064,34 @@ QDeclarativeAnchorChanges::ActionList QDeclarativeAnchorChanges::actions()
     d->vCenterProp = QDeclarativeProperty(d->target, QLatin1String("anchors.verticalCenter"));
     d->baselineProp = QDeclarativeProperty(d->target, QLatin1String("anchors.baseline"));
 
+    QDeclarativeContext *ctxt = qmlContext(this);
+
     if (d->anchorSet->d_func()->usedAnchors & QDeclarativeAnchors::LeftAnchor) {
-        d->leftBinding = new QDeclarativeBinding(d->anchorSet->d_func()->leftScript.script(), d->target, qmlContext(this));
+        d->leftBinding = new QDeclarativeBinding(d->anchorSet->d_func()->leftScript.script(), d->target, ctxt);
         d->leftBinding->setTarget(d->leftProp);
     }
     if (d->anchorSet->d_func()->usedAnchors & QDeclarativeAnchors::RightAnchor) {
-        d->rightBinding = new QDeclarativeBinding(d->anchorSet->d_func()->rightScript.script(), d->target, qmlContext(this));
+        d->rightBinding = new QDeclarativeBinding(d->anchorSet->d_func()->rightScript.script(), d->target, ctxt);
         d->rightBinding->setTarget(d->rightProp);
     }
     if (d->anchorSet->d_func()->usedAnchors & QDeclarativeAnchors::HCenterAnchor) {
-        d->hCenterBinding = new QDeclarativeBinding(d->anchorSet->d_func()->hCenterScript.script(), d->target, qmlContext(this));
+        d->hCenterBinding = new QDeclarativeBinding(d->anchorSet->d_func()->hCenterScript.script(), d->target, ctxt);
         d->hCenterBinding->setTarget(d->hCenterProp);
     }
     if (d->anchorSet->d_func()->usedAnchors & QDeclarativeAnchors::TopAnchor) {
-        d->topBinding = new QDeclarativeBinding(d->anchorSet->d_func()->topScript.script(), d->target, qmlContext(this));
+        d->topBinding = new QDeclarativeBinding(d->anchorSet->d_func()->topScript.script(), d->target, ctxt);
         d->topBinding->setTarget(d->topProp);
     }
     if (d->anchorSet->d_func()->usedAnchors & QDeclarativeAnchors::BottomAnchor) {
-        d->bottomBinding = new QDeclarativeBinding(d->anchorSet->d_func()->bottomScript.script(), d->target, qmlContext(this));
+        d->bottomBinding = new QDeclarativeBinding(d->anchorSet->d_func()->bottomScript.script(), d->target, ctxt);
         d->bottomBinding->setTarget(d->bottomProp);
     }
     if (d->anchorSet->d_func()->usedAnchors & QDeclarativeAnchors::VCenterAnchor) {
-        d->vCenterBinding = new QDeclarativeBinding(d->anchorSet->d_func()->vCenterScript.script(), d->target, qmlContext(this));
+        d->vCenterBinding = new QDeclarativeBinding(d->anchorSet->d_func()->vCenterScript.script(), d->target, ctxt);
         d->vCenterBinding->setTarget(d->vCenterProp);
     }
     if (d->anchorSet->d_func()->usedAnchors & QDeclarativeAnchors::BaselineAnchor) {
-        d->baselineBinding = new QDeclarativeBinding(d->anchorSet->d_func()->baselineScript.script(), d->target, qmlContext(this));
+        d->baselineBinding = new QDeclarativeBinding(d->anchorSet->d_func()->baselineScript.script(), d->target, ctxt);
         d->baselineBinding->setTarget(d->baselineProp);
     }
 
@@ -1380,24 +1371,25 @@ QList<QDeclarativeAction> QDeclarativeAnchorChanges::additionalActions()
     bool vChange = combined & QDeclarativeAnchors::Vertical_Mask;
 
     if (d->target) {
+        QDeclarativeContext *ctxt = qmlContext(this);
         QDeclarativeAction a;
         if (hChange && d->fromX != d->toX) {
-            a.property = QDeclarativeProperty(d->target, QLatin1String("x"));
+            a.property = QDeclarativeProperty(d->target, QLatin1String("x"), ctxt);
             a.toValue = d->toX;
             extra << a;
         }
         if (vChange && d->fromY != d->toY) {
-            a.property = QDeclarativeProperty(d->target, QLatin1String("y"));
+            a.property = QDeclarativeProperty(d->target, QLatin1String("y"), ctxt);
             a.toValue = d->toY;
             extra << a;
         }
         if (hChange && d->fromWidth != d->toWidth) {
-            a.property = QDeclarativeProperty(d->target, QLatin1String("width"));
+            a.property = QDeclarativeProperty(d->target, QLatin1String("width"), ctxt);
             a.toValue = d->toWidth;
             extra << a;
         }
         if (vChange && d->fromHeight != d->toHeight) {
-            a.property = QDeclarativeProperty(d->target, QLatin1String("height"));
+            a.property = QDeclarativeProperty(d->target, QLatin1String("height"), ctxt);
             a.toValue = d->toHeight;
             extra << a;
         }
