@@ -1,40 +1,40 @@
 /****************************************************************************
 **
-** Copyright (C) 2010 Nokia Corporation and/or its subsidiary(-ies).
+** Copyright (C) 2011 Nokia Corporation and/or its subsidiary(-ies).
 ** All rights reserved.
 ** Contact: Nokia Corporation (qt-info@nokia.com)
 **
 ** This file is part of the QtDeclarative module of the Qt Toolkit.
 **
 ** $QT_BEGIN_LICENSE:LGPL$
-** Commercial Usage
-** Licensees holding valid Qt Commercial licenses may use this file in
-** accordance with the Qt Commercial License Agreement provided with the
-** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and Nokia.
-**
 ** GNU Lesser General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU Lesser
-** General Public License version 2.1 as published by the Free Software
-** Foundation and appearing in the file LICENSE.LGPL included in the
-** packaging of this file.  Please review the following information to
-** ensure the GNU Lesser General Public License version 2.1 requirements
-** will be met: http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
+** This file may be used under the terms of the GNU Lesser General Public
+** License version 2.1 as published by the Free Software Foundation and
+** appearing in the file LICENSE.LGPL included in the packaging of this
+** file. Please review the following information to ensure the GNU Lesser
+** General Public License version 2.1 requirements will be met:
+** http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
 **
 ** In addition, as a special exception, Nokia gives you certain additional
-** rights.  These rights are described in the Nokia Qt LGPL Exception
+** rights. These rights are described in the Nokia Qt LGPL Exception
 ** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
 **
 ** GNU General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU
-** General Public License version 3.0 as published by the Free Software
-** Foundation and appearing in the file LICENSE.GPL included in the
-** packaging of this file.  Please review the following information to
-** ensure the GNU General Public License version 3.0 requirements will be
-** met: http://www.gnu.org/copyleft/gpl.html.
+** Alternatively, this file may be used under the terms of the GNU General
+** Public License version 3.0 as published by the Free Software Foundation
+** and appearing in the file LICENSE.GPL included in the packaging of this
+** file. Please review the following information to ensure the GNU General
+** Public License version 3.0 requirements will be met:
+** http://www.gnu.org/copyleft/gpl.html.
+**
+** Other Usage
+** Alternatively, this file may be used in accordance with the terms and
+** conditions contained in a signed written agreement between you and Nokia.
+**
+**
+**
+**
 **
-** If you have questions regarding the use of this file, please contact
-** Nokia at qt-info@nokia.com.
 ** $QT_END_LICENSE$
 **
 ****************************************************************************/
@@ -46,11 +46,9 @@
 
 #include <QDebug>
 #include <QStringList>
-#include <QQueue>
+#include <QMap>
 #include <QApplication>
 #include <QThread>
-#include <QMutex>
-#include <QWaitCondition>
 #include <QXmlQuery>
 #include <QXmlResultItems>
 #include <QXmlNodeModelIndex>
@@ -58,6 +56,7 @@
 #include <QNetworkRequest>
 #include <QNetworkReply>
 #include <QTimer>
+#include <QMutex>
 
 #include <private/qobject_p.h>
 
@@ -90,10 +89,15 @@ typedef QPair<int, int> QDeclarativeXmlListRange;
     \qml
     XmlListModel {
         id: xmlModel
-        ...
-        XmlRole { name: "title"; query: "title/string()" }
+        // ...
+        XmlRole {
+            name: "title"
+            query: "title/string()"
+        }
     }
+    \endqml
 
+    \qml
     ListView {
         model: xmlModel
         delegate: Text { text: title }
@@ -140,139 +144,229 @@ struct XmlQueryJob
     QList<void*> roleQueryErrorId; // the ptr to send back if there is an error
     QStringList keyRoleQueries;
     QStringList keyRoleResultsCache;
+    QString prefix;
 };
 
-class QDeclarativeXmlQuery : public QThread
+
+class QDeclarativeXmlQueryEngine;
+class QDeclarativeXmlQueryThreadObject : public QObject
 {
     Q_OBJECT
 public:
-    QDeclarativeXmlQuery(QObject *parent=0)
-        : QThread(parent), m_quit(false), m_abortQueryId(-1), m_queryIds(XMLLISTMODEL_CLEAR_ID + 1) {
-        qRegisterMetaType<QDeclarativeXmlQueryResult>("QDeclarativeXmlQueryResult");
-        m_currentJob.queryId = -1;
-    }
+    QDeclarativeXmlQueryThreadObject(QDeclarativeXmlQueryEngine *);
 
-    ~QDeclarativeXmlQuery() {
-        m_mutex.lock();
-        m_quit = true;
-        m_condition.wakeOne();
-        m_mutex.unlock();
+    void processJobs();
+    virtual bool event(QEvent *e);
 
-        wait();
-    }
+private:
+    QDeclarativeXmlQueryEngine *m_queryEngine;
+};
 
-    void abort(int id) {
-        QMutexLocker locker(&m_mutex);
-        QQueue<XmlQueryJob>::iterator it;
-        for (it = m_jobs.begin(); it != m_jobs.end(); ++it) {
-            if ((*it).queryId == id) {
-                m_jobs.erase(it);
-                return;
-            }
-        }
-        m_abortQueryId = id;
-    }
 
-    int doQuery(QString query, QString namespaces, QByteArray data, QList<QDeclarativeXmlListModelRole *> *roleObjects, QStringList keyRoleResultsCache) {
-        QMutexLocker locker(&m_mutex);
+class QDeclarativeXmlQueryEngine : public QThread
+{
+    Q_OBJECT
+public:
+    QDeclarativeXmlQueryEngine(QDeclarativeEngine *eng);
+    ~QDeclarativeXmlQueryEngine();
 
-        XmlQueryJob job;
-        job.queryId = m_queryIds;
-        job.data = data;
-        job.query = QLatin1String("doc($src)") + query;
-        job.namespaces = namespaces;
-        job.keyRoleResultsCache = keyRoleResultsCache;
+    int doQuery(QString query, QString namespaces, QByteArray data, QList<QDeclarativeXmlListModelRole *>* roleObjects, QStringList keyRoleResultsCache);
+    void abort(int id);
 
-        for (int i=0; i<roleObjects->count(); i++) {
-            if (!roleObjects->at(i)->isValid()) {
-                job.roleQueries << QString();
-                continue;
-            }
-            job.roleQueries << roleObjects->at(i)->query();
-            job.roleQueryErrorId << static_cast<void*>(roleObjects->at(i));
-            if (roleObjects->at(i)->isKey())
-                job.keyRoleQueries << job.roleQueries.last();
-        }
-        m_jobs.enqueue(job);
-        m_queryIds++;
+    void processJobs();
 
-        if (!isRunning())
-            start(QThread::IdlePriority);
-        else
-            m_condition.wakeOne();
-        return job.queryId;
-    }
+    static QDeclarativeXmlQueryEngine *instance(QDeclarativeEngine *engine);
 
-Q_SIGNALS:
+signals:
     void queryCompleted(const QDeclarativeXmlQueryResult &);
     void error(void*, const QString&);
 
 protected:
-    void run() {
-        m_mutex.lock();
-
-        while (!m_quit) {
-            if (!m_jobs.isEmpty())
-                m_currentJob = m_jobs.dequeue();
-            m_mutex.unlock();
-
-            QDeclarativeXmlQueryResult r;
-            if (m_currentJob.queryId != -1) {
-                doQueryJob();
-                doSubQueryJob();
-                r.queryId = m_currentJob.queryId;
-                r.size = m_size;
-                r.data = m_modelData;
-                r.inserted = m_insertedItemRanges;
-                r.removed = m_removedItemRanges;
-                r.keyRoleResultsCache = m_currentJob.keyRoleResultsCache;
-            }
-
-            m_mutex.lock();
-            if (m_currentJob.queryId != -1 && m_abortQueryId != m_currentJob.queryId)
-                emit queryCompleted(r);
-            if (m_jobs.isEmpty() && !m_quit)
-                m_condition.wait(&m_mutex);
-            m_currentJob.queryId = -1;
-            m_abortQueryId = -1;
-        }
-
-        m_mutex.unlock();
-    }
+    void run();
 
 private:
-    void doQueryJob();
-    void doSubQueryJob();
-    void getValuesOfKeyRoles(QStringList *values, QXmlQuery *query) const;
+    void processQuery(XmlQueryJob *job);
+    void doQueryJob(XmlQueryJob *job, QDeclarativeXmlQueryResult *currentResult);
+    void doSubQueryJob(XmlQueryJob *job, QDeclarativeXmlQueryResult *currentResult);
+    void getValuesOfKeyRoles(const XmlQueryJob& currentJob, QStringList *values, QXmlQuery *query) const;
     void addIndexToRangeList(QList<QDeclarativeXmlListRange> *ranges, int index) const;
 
-private:
     QMutex m_mutex;
-    QWaitCondition m_condition;
-    QQueue<XmlQueryJob> m_jobs;
-    XmlQueryJob m_currentJob;
-    bool m_quit;
-    int m_abortQueryId;
-    QString m_prefix;
-    int m_size;
-    int m_queryIds;
-    QList<QList<QVariant> > m_modelData;
-    QList<QDeclarativeXmlListRange> m_insertedItemRanges;
-    QList<QDeclarativeXmlListRange> m_removedItemRanges;
+    QDeclarativeXmlQueryThreadObject *m_threadObject;
+    QList<XmlQueryJob> m_jobs;
+    QSet<int> m_cancelledJobs;
+    QAtomicInt m_queryIds;
+
+    QDeclarativeEngine *m_engine;
+    QObject *m_eventLoopQuitHack;
+
+    static QHash<QDeclarativeEngine *,QDeclarativeXmlQueryEngine*> queryEngines;
+    static QMutex queryEnginesMutex;
 };
+QHash<QDeclarativeEngine *,QDeclarativeXmlQueryEngine*> QDeclarativeXmlQueryEngine::queryEngines;
+QMutex QDeclarativeXmlQueryEngine::queryEnginesMutex;
 
-Q_GLOBAL_STATIC(QDeclarativeXmlQuery, globalXmlQuery)
 
-void QDeclarativeXmlQuery::doQueryJob()
+QDeclarativeXmlQueryThreadObject::QDeclarativeXmlQueryThreadObject(QDeclarativeXmlQueryEngine *e)
+    : m_queryEngine(e)
 {
-    Q_ASSERT(m_currentJob.queryId != -1);
+}
+
+void QDeclarativeXmlQueryThreadObject::processJobs()
+{
+    QCoreApplication::postEvent(this, new QEvent(QEvent::User));
+}
+
+bool QDeclarativeXmlQueryThreadObject::event(QEvent *e)
+{
+    if (e->type() == QEvent::User) {
+        m_queryEngine->processJobs();
+        return true;
+    } else {
+        return QObject::event(e);
+    }
+}
+
+
+
+QDeclarativeXmlQueryEngine::QDeclarativeXmlQueryEngine(QDeclarativeEngine *eng)
+: QThread(eng), m_threadObject(0), m_queryIds(XMLLISTMODEL_CLEAR_ID + 1), m_engine(eng), m_eventLoopQuitHack(0)
+{
+    qRegisterMetaType<QDeclarativeXmlQueryResult>("QDeclarativeXmlQueryResult");
+
+    m_eventLoopQuitHack = new QObject;
+    m_eventLoopQuitHack->moveToThread(this);
+    connect(m_eventLoopQuitHack, SIGNAL(destroyed(QObject*)), SLOT(quit()), Qt::DirectConnection);
+    start(QThread::IdlePriority);
+}
+
+QDeclarativeXmlQueryEngine::~QDeclarativeXmlQueryEngine()
+{
+    queryEnginesMutex.lock();
+    queryEngines.remove(m_engine);
+    queryEnginesMutex.unlock();
+
+    m_eventLoopQuitHack->deleteLater();
+    wait();
+}
+
+int QDeclarativeXmlQueryEngine::doQuery(QString query, QString namespaces, QByteArray data, QList<QDeclarativeXmlListModelRole *>* roleObjects, QStringList keyRoleResultsCache) {
+    {
+        QMutexLocker m1(&m_mutex);
+        m_queryIds.ref();
+        if (m_queryIds <= 0)
+            m_queryIds = 1;
+    }
+
+    XmlQueryJob job;
+    job.queryId = m_queryIds;
+    job.data = data;
+    job.query = QLatin1String("doc($src)") + query;
+    job.namespaces = namespaces;
+    job.keyRoleResultsCache = keyRoleResultsCache;
+
+    for (int i=0; i<roleObjects->count(); i++) {
+        if (!roleObjects->at(i)->isValid()) {
+            job.roleQueries << QString();
+            continue;
+        }
+        job.roleQueries << roleObjects->at(i)->query();
+        job.roleQueryErrorId << static_cast<void*>(roleObjects->at(i));
+        if (roleObjects->at(i)->isKey())
+            job.keyRoleQueries << job.roleQueries.last();
+    }
+
+    {
+        QMutexLocker ml(&m_mutex);
+        m_jobs.append(job);
+        if (m_threadObject)
+            m_threadObject->processJobs();
+    }
+
+    return job.queryId;
+}
+
+void QDeclarativeXmlQueryEngine::abort(int id)
+{
+    QMutexLocker ml(&m_mutex);
+    if (id != -1)
+        m_cancelledJobs.insert(id);
+}
+
+void QDeclarativeXmlQueryEngine::run()
+{
+    m_mutex.lock();
+    m_threadObject = new QDeclarativeXmlQueryThreadObject(this);
+    m_mutex.unlock();
+
+    processJobs();
+    exec();
+
+    delete m_threadObject;
+    m_threadObject = 0;
+}
+
+void QDeclarativeXmlQueryEngine::processJobs()
+{
+    QMutexLocker locker(&m_mutex);
+
+    while (true) {
+        if (m_jobs.isEmpty())
+            return;
+
+        XmlQueryJob currentJob = m_jobs.takeLast();
+        while (m_cancelledJobs.remove(currentJob.queryId)) {
+            if (m_jobs.isEmpty())
+              return;
+            currentJob = m_jobs.takeLast();
+        }
+        
+        locker.unlock();
+        processQuery(&currentJob);
+        locker.relock();
+    }
+}
+
+QDeclarativeXmlQueryEngine *QDeclarativeXmlQueryEngine::instance(QDeclarativeEngine *engine)
+{
+    queryEnginesMutex.lock();
+    QDeclarativeXmlQueryEngine *queryEng = queryEngines.value(engine);
+    if (!queryEng) {
+        queryEng = new QDeclarativeXmlQueryEngine(engine);
+        queryEngines.insert(engine, queryEng);
+    }
+    queryEnginesMutex.unlock();
+
+    return queryEng;
+}
+
+void QDeclarativeXmlQueryEngine::processQuery(XmlQueryJob *job)
+{
+    QDeclarativeXmlQueryResult result;
+    result.queryId = job->queryId;
+    doQueryJob(job, &result);
+    doSubQueryJob(job, &result);
+
+    {
+        QMutexLocker ml(&m_mutex);
+        if (m_cancelledJobs.contains(job->queryId)) {
+            m_cancelledJobs.remove(job->queryId);
+        } else {
+            emit queryCompleted(result);
+        }
+    }
+}
+
+void QDeclarativeXmlQueryEngine::doQueryJob(XmlQueryJob *currentJob, QDeclarativeXmlQueryResult *currentResult)
+{
+    Q_ASSERT(currentJob->queryId != -1);
 
     QString r;
     QXmlQuery query;
-    QBuffer buffer(&m_currentJob.data);
+    QBuffer buffer(&currentJob->data);
     buffer.open(QIODevice::ReadOnly);
     query.bindVariable(QLatin1String("src"), &buffer);
-    query.setQuery(m_currentJob.namespaces + m_currentJob.query);
+    query.setQuery(currentJob->namespaces + currentJob->query);
     query.evaluateTo(&r);
 
     //always need a single root element
@@ -280,9 +374,9 @@ void QDeclarativeXmlQuery::doQueryJob()
     QBuffer b(&xml);
     b.open(QIODevice::ReadOnly);
 
-    QString namespaces = QLatin1String("declare namespace dummy=\"http://qtsotware.com/dummy\";\n") + m_currentJob.namespaces;
+    QString namespaces = QLatin1String("declare namespace dummy=\"http://qtsotware.com/dummy\";\n") + currentJob->namespaces;
     QString prefix = QLatin1String("doc($inputDocument)/dummy:items") +
-                     m_currentJob.query.mid(m_currentJob.query.lastIndexOf(QLatin1Char('/')));
+                     currentJob->query.mid(currentJob->query.lastIndexOf(QLatin1Char('/')));
 
     //figure out how many items we are dealing with
     int count = -1;
@@ -297,23 +391,19 @@ void QDeclarativeXmlQuery::doQueryJob()
             count = item.toAtomicValue().toInt();
     }
 
-    m_currentJob.data = xml;
-    m_prefix = namespaces + prefix + QLatin1Char('/');
-    m_size = 0;
-    if (count > 0)
-        m_size = count;
+    currentJob->data = xml;
+    currentJob->prefix = namespaces + prefix + QLatin1Char('/');
+    currentResult->size = (count > 0 ? count : 0);
 }
 
-void QDeclarativeXmlQuery::getValuesOfKeyRoles(QStringList *values, QXmlQuery *query) const
+void QDeclarativeXmlQueryEngine::getValuesOfKeyRoles(const XmlQueryJob& currentJob, QStringList *values, QXmlQuery *query) const
 {
-    Q_ASSERT(m_currentJob.queryId != -1);
-
-    const QStringList &keysQueries = m_currentJob.keyRoleQueries;
+    const QStringList &keysQueries = currentJob.keyRoleQueries;
     QString keysQuery;
     if (keysQueries.count() == 1)
-        keysQuery = m_prefix + keysQueries[0];
+        keysQuery = currentJob.prefix + keysQueries[0];
     else if (keysQueries.count() > 1)
-        keysQuery = m_prefix + QLatin1String("concat(") + keysQueries.join(QLatin1String(",")) + QLatin1String(")");
+        keysQuery = currentJob.prefix + QLatin1String("concat(") + keysQueries.join(QLatin1String(",")) + QLatin1String(")");
 
     if (!keysQuery.isEmpty()) {
         query->setQuery(keysQuery);
@@ -327,7 +417,7 @@ void QDeclarativeXmlQuery::getValuesOfKeyRoles(QStringList *values, QXmlQuery *q
     }
 }
 
-void QDeclarativeXmlQuery::addIndexToRangeList(QList<QDeclarativeXmlListRange> *ranges, int index) const {
+void QDeclarativeXmlQueryEngine::addIndexToRangeList(QList<QDeclarativeXmlListRange> *ranges, int index) const {
     if (ranges->isEmpty())
         ranges->append(qMakePair(index, 1));
     else if (ranges->last().first + ranges->last().second == index)
@@ -336,53 +426,50 @@ void QDeclarativeXmlQuery::addIndexToRangeList(QList<QDeclarativeXmlListRange> *
         ranges->append(qMakePair(index, 1));
 }
 
-void QDeclarativeXmlQuery::doSubQueryJob()
+void QDeclarativeXmlQueryEngine::doSubQueryJob(XmlQueryJob *currentJob, QDeclarativeXmlQueryResult *currentResult)
 {
-    Q_ASSERT(m_currentJob.queryId != -1);
-    m_modelData.clear();
+    Q_ASSERT(currentJob->queryId != -1);
 
-    QBuffer b(&m_currentJob.data);
+    QBuffer b(&currentJob->data);
     b.open(QIODevice::ReadOnly);
 
     QXmlQuery subquery;
     subquery.bindVariable(QLatin1String("inputDocument"), &b);
 
     QStringList keyRoleResults;
-    getValuesOfKeyRoles(&keyRoleResults, &subquery);
+    getValuesOfKeyRoles(*currentJob, &keyRoleResults, &subquery);
 
     // See if any values of key roles have been inserted or removed.
 
-    m_insertedItemRanges.clear();
-    m_removedItemRanges.clear();
-    if (m_currentJob.keyRoleResultsCache.isEmpty()) {
-        m_insertedItemRanges << qMakePair(0, m_size);
+    if (currentJob->keyRoleResultsCache.isEmpty()) {
+        currentResult->inserted << qMakePair(0, currentResult->size);
     } else {
-        if (keyRoleResults != m_currentJob.keyRoleResultsCache) {
+        if (keyRoleResults != currentJob->keyRoleResultsCache) {
             QStringList temp;
-            for (int i=0; i<m_currentJob.keyRoleResultsCache.count(); i++) {
-                if (!keyRoleResults.contains(m_currentJob.keyRoleResultsCache[i]))
-                    addIndexToRangeList(&m_removedItemRanges, i);
+            for (int i=0; i<currentJob->keyRoleResultsCache.count(); i++) {
+                if (!keyRoleResults.contains(currentJob->keyRoleResultsCache[i]))
+                    addIndexToRangeList(&currentResult->removed, i);
                 else 
-                    temp << m_currentJob.keyRoleResultsCache[i];
+                    temp << currentJob->keyRoleResultsCache[i];
             }
 
             for (int i=0; i<keyRoleResults.count(); i++) {
                 if (temp.count() == i || keyRoleResults[i] != temp[i]) {
                     temp.insert(i, keyRoleResults[i]);
-                    addIndexToRangeList(&m_insertedItemRanges, i);
+                    addIndexToRangeList(&currentResult->inserted, i);
                 }
             }
         }
     }
-    m_currentJob.keyRoleResultsCache = keyRoleResults;
+    currentResult->keyRoleResultsCache = keyRoleResults;
 
     // Get the new values for each role.
     //### we might be able to condense even further (query for everything in one go)
-    const QStringList &queries = m_currentJob.roleQueries;
+    const QStringList &queries = currentJob->roleQueries;
     for (int i = 0; i < queries.size(); ++i) {
         QList<QVariant> resultList;
         if (!queries[i].isEmpty()) {
-            subquery.setQuery(m_prefix + QLatin1String("(let $v := ") + queries[i] + QLatin1String(" return if ($v) then ") + queries[i] + QLatin1String(" else \"\")"));
+            subquery.setQuery(currentJob->prefix + QLatin1String("(let $v := string(") + queries[i] + QLatin1String(") return if ($v) then ") + queries[i] + QLatin1String(" else \"\")"));
             if (subquery.isValid()) {
                 QXmlResultItems resultItems;
                 subquery.evaluateTo(&resultItems);
@@ -392,13 +479,13 @@ void QDeclarativeXmlQuery::doSubQueryJob()
                     item = resultItems.next();
                 }
             } else {
-                emit error(m_currentJob.roleQueryErrorId.at(i), queries[i]);
+                emit error(currentJob->roleQueryErrorId.at(i), queries[i]);
             }
         }
         //### should warn here if things have gone wrong.
-        while (resultList.count() < m_size)
+        while (resultList.count() < currentResult->size)
             resultList << QVariant();
-        m_modelData << resultList;
+        currentResult->data << resultList;
         b.seek(0);
     }
 
@@ -443,13 +530,22 @@ public:
 
     void notifyQueryStarted(bool remoteSource) {
         Q_Q(QDeclarativeXmlListModel);
-        progress = remoteSource ? 0.0 : 1.0;
+        progress = remoteSource ? qreal(0.0) : qreal(1.0);
         status = QDeclarativeXmlListModel::Loading;
         errorString.clear();
         emit q->progressChanged(progress);
         emit q->statusChanged(status);
     }
 
+    void deleteReply() {
+        Q_Q(QDeclarativeXmlListModel);
+        if (reply) {
+            QObject::disconnect(reply, 0, q, 0);
+            reply->deleteLater();
+            reply = 0;
+        }
+    }
+
     bool isComponentComplete;
     QUrl src;
     QString xml;
@@ -459,6 +555,7 @@ public:
     QList<int> roles;
     QStringList roleNames;
     int highestRole;
+
     QNetworkReply *reply;
     QDeclarativeXmlListModel::Status status;
     QString errorString;
@@ -466,6 +563,7 @@ public:
     int queryId;
     QStringList keyRoleResultsCache;
     QList<QDeclarativeXmlListModelRole *> roleObjects;
+
     static void append_role(QDeclarativeListProperty<QDeclarativeXmlListModelRole> *list, QDeclarativeXmlListModelRole *role);
     static void clear_role(QDeclarativeListProperty<QDeclarativeXmlListModelRole> *list);
     QList<QList<QVariant> > data;
@@ -476,7 +574,7 @@ public:
 void QDeclarativeXmlListModelPrivate::append_role(QDeclarativeListProperty<QDeclarativeXmlListModelRole> *list, QDeclarativeXmlListModelRole *role)
 {
     QDeclarativeXmlListModel *_this = qobject_cast<QDeclarativeXmlListModel *>(list->object);
-    if (_this) {
+    if (_this && role) {
         int i = _this->d_func()->roleObjects.count();
         _this->d_func()->roleObjects.append(role);
         if (_this->d_func()->roleNames.contains(role->name())) {
@@ -503,9 +601,9 @@ void QDeclarativeXmlListModelPrivate::clear_role(QDeclarativeListProperty<QDecla
     \qmlclass XmlListModel QDeclarativeXmlListModel
     \ingroup qml-working-with-data
   \since 4.7
-    \brief The XmlListModel element is used to specify a model using XPath expressions.
+    \brief The XmlListModel element is used to specify a read-only model using XPath expressions.
 
-    XmlListModel is used to create a model from XML data. It can be used as a data source
+    XmlListModel is used to create a read-only model from XML data. It can be used as a data source
     for view elements (such as ListView, PathView, GridView) and other elements that interact with model
     data (such as \l Repeater).
 
@@ -599,10 +697,6 @@ void QDeclarativeXmlListModelPrivate::clear_role(QDeclarativeListProperty<QDecla
 QDeclarativeXmlListModel::QDeclarativeXmlListModel(QObject *parent)
     : QListModelInterface(*(new QDeclarativeXmlListModelPrivate), parent)
 {
-    connect(globalXmlQuery(), SIGNAL(queryCompleted(QDeclarativeXmlQueryResult)),
-            this, SLOT(queryCompleted(QDeclarativeXmlQueryResult)));
-    connect(globalXmlQuery(), SIGNAL(error(void*,QString)),
-            this, SLOT(queryError(void*,QString)));
 }
 
 QDeclarativeXmlListModel::~QDeclarativeXmlListModel()
@@ -792,9 +886,9 @@ void QDeclarativeXmlListModel::setNamespaceDeclarations(const QString &declarati
 
     This will access the \c title value for the first item in the model:
 
-    \qml
-        var title = model.get(0).title;
-    \endqml
+    \js
+    var title = model.get(0).title;
+    \endjs
 */
 QScriptValue QDeclarativeXmlListModel::get(int index) const
 {
@@ -806,7 +900,7 @@ QScriptValue QDeclarativeXmlListModel::get(int index) const
 
     QScriptValue sv = sengine->newObject();
     for (int i=0; i<d->roleObjects.count(); i++) 
-        sv.setProperty(d->roleObjects[i]->name(), qScriptValueFromValue(sengine, d->data.value(i).value(index)));
+        sv.setProperty(d->roleObjects[i]->name(), sengine->toScriptValue(d->data.value(i).value(index)));
     return sv;    
 }
 
@@ -868,6 +962,12 @@ void QDeclarativeXmlListModel::classBegin()
 {
     Q_D(QDeclarativeXmlListModel);
     d->isComponentComplete = false;
+
+    QDeclarativeXmlQueryEngine *queryEngine = QDeclarativeXmlQueryEngine::instance(qmlEngine(this));
+    connect(queryEngine, SIGNAL(queryCompleted(QDeclarativeXmlQueryResult)),
+            SLOT(queryCompleted(QDeclarativeXmlQueryResult)));
+    connect(queryEngine, SIGNAL(error(void*,QString)),
+            SLOT(queryError(void*,QString)));
 }
 
 void QDeclarativeXmlListModel::componentComplete()
@@ -897,7 +997,7 @@ void QDeclarativeXmlListModel::reload()
     if (!d->isComponentComplete)
         return;
 
-    globalXmlQuery()->abort(d->queryId);
+    QDeclarativeXmlQueryEngine::instance(qmlEngine(this))->abort(d->queryId);
     d->queryId = -1;
 
     if (d->size < 0)
@@ -905,15 +1005,11 @@ void QDeclarativeXmlListModel::reload()
 
     if (d->reply) {
         d->reply->abort();
-        if (d->reply) {
-            // abort will generally have already done this (and more)
-            d->reply->deleteLater();
-            d->reply = 0;
-        }
+        d->deleteReply();
     }
 
     if (!d->xml.isEmpty()) {
-        d->queryId = globalXmlQuery()->doQuery(d->query, d->namespaces, d->xml.toUtf8(), &d->roleObjects, d->keyRoleResultsCache);
+        d->queryId = QDeclarativeXmlQueryEngine::instance(qmlEngine(this))->doQuery(d->query, d->namespaces, d->xml.toUtf8(), &d->roleObjects, d->keyRoleResultsCache);
         d->notifyQueryStarted(false);
 
     } else if (d->src.isEmpty()) {
@@ -924,6 +1020,7 @@ void QDeclarativeXmlListModel::reload()
     } else {
         d->notifyQueryStarted(true);
         QNetworkRequest req(d->src);
+        req.setRawHeader("Accept", "application/xml,*/*");
         d->reply = qmlContext(this)->engine()->networkAccessManager()->get(req);
         QObject::connect(d->reply, SIGNAL(finished()), this, SLOT(requestFinished()));
         QObject::connect(d->reply, SIGNAL(downloadProgress(qint64,qint64)),
@@ -942,8 +1039,7 @@ void QDeclarativeXmlListModel::requestFinished()
         QVariant redirect = d->reply->attribute(QNetworkRequest::RedirectionTargetAttribute);
         if (redirect.isValid()) {
             QUrl url = d->reply->url().resolved(redirect.toUrl());
-            d->reply->deleteLater();
-            d->reply = 0;
+            d->deleteReply();
             setSource(url);
             return;
         }
@@ -952,9 +1048,7 @@ void QDeclarativeXmlListModel::requestFinished()
 
     if (d->reply->error() != QNetworkReply::NoError) {
         d->errorString = d->reply->errorString();
-        disconnect(d->reply, 0, this, 0);
-        d->reply->deleteLater();
-        d->reply = 0;
+        d->deleteReply();
 
         int count = this->count();
         d->data.clear();
@@ -973,11 +1067,9 @@ void QDeclarativeXmlListModel::requestFinished()
             d->queryId = XMLLISTMODEL_CLEAR_ID;
             QTimer::singleShot(0, this, SLOT(dataCleared()));
         } else {
-            d->queryId = globalXmlQuery()->doQuery(d->query, d->namespaces, data, &d->roleObjects, d->keyRoleResultsCache);
+            d->queryId = QDeclarativeXmlQueryEngine::instance(qmlEngine(this))->doQuery(d->query, d->namespaces, data, &d->roleObjects, d->keyRoleResultsCache);
         }
-        disconnect(d->reply, 0, this, 0);
-        d->reply->deleteLater();
-        d->reply = 0;
+        d->deleteReply();
 
         d->progress = 1.0;
         emit progressChanged(d->progress);
