@@ -1,40 +1,40 @@
 /****************************************************************************
 **
-** Copyright (C) 2009 Nokia Corporation and/or its subsidiary(-ies).
+** Copyright (C) 2011 Nokia Corporation and/or its subsidiary(-ies).
 ** All rights reserved.
 ** Contact: Nokia Corporation (qt-info@nokia.com)
 **
 ** This file is part of the QtGui module of the Qt Toolkit.
 **
 ** $QT_BEGIN_LICENSE:LGPL$
-** Commercial Usage
-** Licensees holding valid Qt Commercial licenses may use this file in
-** accordance with the Qt Commercial License Agreement provided with the
-** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and Nokia.
-**
 ** GNU Lesser General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU Lesser
-** General Public License version 2.1 as published by the Free Software
-** Foundation and appearing in the file LICENSE.LGPL included in the
-** packaging of this file.  Please review the following information to
-** ensure the GNU Lesser General Public License version 2.1 requirements
-** will be met: http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
+** This file may be used under the terms of the GNU Lesser General Public
+** License version 2.1 as published by the Free Software Foundation and
+** appearing in the file LICENSE.LGPL included in the packaging of this
+** file. Please review the following information to ensure the GNU Lesser
+** General Public License version 2.1 requirements will be met:
+** http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
 **
 ** In addition, as a special exception, Nokia gives you certain additional
-** rights.  These rights are described in the Nokia Qt LGPL Exception
+** rights. These rights are described in the Nokia Qt LGPL Exception
 ** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
 **
 ** GNU General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU
-** General Public License version 3.0 as published by the Free Software
-** Foundation and appearing in the file LICENSE.GPL included in the
-** packaging of this file.  Please review the following information to
-** ensure the GNU General Public License version 3.0 requirements will be
-** met: http://www.gnu.org/copyleft/gpl.html.
+** Alternatively, this file may be used under the terms of the GNU General
+** Public License version 3.0 as published by the Free Software Foundation
+** and appearing in the file LICENSE.GPL included in the packaging of this
+** file. Please review the following information to ensure the GNU General
+** Public License version 3.0 requirements will be met:
+** http://www.gnu.org/copyleft/gpl.html.
+**
+** Other Usage
+** Alternatively, this file may be used in accordance with the terms and
+** conditions contained in a signed written agreement between you and Nokia.
+**
+**
+**
+**
 **
-** If you have questions regarding the use of this file, please contact
-** Nokia at qt-info@nokia.com.
 ** $QT_END_LICENSE$
 **
 ****************************************************************************/
@@ -95,7 +95,7 @@ class QInputDialogSpinBox : public QSpinBox
 public:
     QInputDialogSpinBox(QWidget *parent)
         : QSpinBox(parent) {
-        connect(lineEdit(), SIGNAL(textChanged(const QString&)), this, SLOT(notifyTextChanged()));
+        connect(lineEdit(), SIGNAL(textChanged(QString)), this, SLOT(notifyTextChanged()));
         connect(this, SIGNAL(editingFinished()), this, SLOT(notifyTextChanged()));
     }
 
@@ -130,7 +130,7 @@ class QInputDialogDoubleSpinBox : public QDoubleSpinBox
 public:
     QInputDialogDoubleSpinBox(QWidget *parent = 0)
         : QDoubleSpinBox(parent) {
-        connect(lineEdit(), SIGNAL(textChanged(const QString&)), this, SLOT(notifyTextChanged()));
+        connect(lineEdit(), SIGNAL(textChanged(QString)), this, SLOT(notifyTextChanged()));
         connect(this, SIGNAL(editingFinished()), this, SLOT(notifyTextChanged()));
     }
 
@@ -231,7 +231,12 @@ void QInputDialogPrivate::ensureLayout()
     QObject::connect(buttonBox, SIGNAL(rejected()), q, SLOT(reject()));
 
     mainLayout = new QVBoxLayout(q);
+    //we want to let the input dialog grow to available size on Symbian.
+#ifndef Q_OS_SYMBIAN
     mainLayout->setSizeConstraint(QLayout::SetMinAndMaxSize);
+#else
+    label->setSizePolicy(QSizePolicy::Expanding, QSizePolicy::Expanding);
+#endif
     mainLayout->addWidget(label);
     mainLayout->addWidget(inputWidget);
     mainLayout->addWidget(buttonBox);
@@ -244,9 +249,12 @@ void QInputDialogPrivate::ensureLineEdit()
     Q_Q(QInputDialog);
     if (!lineEdit) {
         lineEdit = new QLineEdit(q);
+#ifndef QT_NO_IM
+        qt_widget_private(lineEdit)->inheritsInputMethodHints = 1;
+#endif
         lineEdit->hide();
-        QObject::connect(lineEdit, SIGNAL(textChanged(const QString&)),
-                         q, SLOT(_q_textChanged(const QString&)));
+        QObject::connect(lineEdit, SIGNAL(textChanged(QString)),
+                         q, SLOT(_q_textChanged(QString)));
     }
 }
 
@@ -255,11 +263,14 @@ void QInputDialogPrivate::ensureComboBox()
     Q_Q(QInputDialog);
     if (!comboBox) {
         comboBox = new QComboBox(q);
+#ifndef QT_NO_IM
+        qt_widget_private(comboBox)->inheritsInputMethodHints = 1;
+#endif
         comboBox->hide();
-        QObject::connect(comboBox, SIGNAL(editTextChanged(const QString&)),
-                         q, SLOT(_q_textChanged(const QString&)));
-        QObject::connect(comboBox, SIGNAL(currentIndexChanged(const QString&)),
-                         q, SLOT(_q_textChanged(const QString&)));
+        QObject::connect(comboBox, SIGNAL(editTextChanged(QString)),
+                         q, SLOT(_q_textChanged(QString)));
+        QObject::connect(comboBox, SIGNAL(currentIndexChanged(QString)),
+                         q, SLOT(_q_textChanged(QString)));
     }
 }
 
@@ -276,8 +287,8 @@ void QInputDialogPrivate::ensureListView()
         listView->setModel(comboBox->model());
         listView->setCurrentIndex(QModelIndex()); // ###
         QObject::connect(listView->selectionModel(),
-                         SIGNAL(currentRowChanged(const QModelIndex&, const QModelIndex&)),
-                         q, SLOT(_q_currentRowChanged(const QModelIndex&, const QModelIndex&)));
+                         SIGNAL(currentRowChanged(QModelIndex,QModelIndex)),
+                         q, SLOT(_q_currentRowChanged(QModelIndex,QModelIndex)));
     }
 }
 
@@ -552,6 +563,9 @@ void QInputDialog::setLabelText(const QString &text)
     } else {
         d->label->setText(text);
     }
+#ifdef Q_OS_SYMBIAN
+    d->label->setWordWrap(true);
+#endif
 }
 
 QString QInputDialog::labelText() const
@@ -1122,6 +1136,8 @@ void QInputDialog::done(int result)
     be entered).
     \a text is the default text which is placed in the line edit.
     \a mode is the echo mode the line edit will use.
+    \a inputMethodHints is the input method hints that will be used in the
+    edit widget if an input method is active.
 
     If \a ok is nonnull \e *\a ok will be set to true if the user pressed
     \gui OK and to false if the user pressed \gui Cancel. The dialog's parent
@@ -1144,13 +1160,14 @@ void QInputDialog::done(int result)
 
 QString QInputDialog::getText(QWidget *parent, const QString &title, const QString &label,
                               QLineEdit::EchoMode mode, const QString &text, bool *ok,
-                              Qt::WindowFlags flags)
+                              Qt::WindowFlags flags, Qt::InputMethodHints inputMethodHints)
 {
     QInputDialog dialog(parent, flags);
     dialog.setWindowTitle(title);
     dialog.setLabelText(label);
     dialog.setTextValue(text);
     dialog.setTextEchoMode(mode);
+    dialog.setInputMethodHints(inputMethodHints);
 
     int ret = dialog.exec();
     if (ok)
@@ -1162,6 +1179,17 @@ QString QInputDialog::getText(QWidget *parent, const QString &title, const QStri
     }
 }
 
+/*!
+    \internal
+*/
+// ### Qt 5: Use only the version above.
+QString QInputDialog::getText(QWidget *parent, const QString &title, const QString &label,
+                              QLineEdit::EchoMode mode, const QString &text, bool *ok,
+                              Qt::WindowFlags flags)
+{
+    return getText(parent, title, label, mode, text, ok, flags, Qt::ImhNone);
+}
+
 /*!
     \since 4.5
 
@@ -1272,6 +1300,8 @@ double QInputDialog::getDouble(QWidget *parent, const QString &title, const QStr
     be entered).
     \a items is the string list which is inserted into the combobox.
     \a current is the number  of the item which should be the current item.
+    \a inputMethodHints is the input method hints that will be used if the
+    combobox is editable and an input method is active.
 
     If \a editable is true the user can enter their own text; otherwise the
     user may only select one of the existing items.
@@ -1296,7 +1326,7 @@ double QInputDialog::getDouble(QWidget *parent, const QString &title, const QStr
 
 QString QInputDialog::getItem(QWidget *parent, const QString &title, const QString &label,
                               const QStringList &items, int current, bool editable, bool *ok,
-                              Qt::WindowFlags flags)
+                              Qt::WindowFlags flags, Qt::InputMethodHints inputMethodHints)
 {
     QString text(items.value(current));
 
@@ -1306,6 +1336,7 @@ QString QInputDialog::getItem(QWidget *parent, const QString &title, const QStri
     dialog.setComboBoxItems(items);
     dialog.setTextValue(text);
     dialog.setComboBoxEditable(editable);
+    dialog.setInputMethodHints(inputMethodHints);
 
     int ret = dialog.exec();
     if (ok)
@@ -1317,6 +1348,17 @@ QString QInputDialog::getItem(QWidget *parent, const QString &title, const QStri
     }
 }
 
+/*!
+    \internal
+*/
+// ### Qt 5: Use only the version above.
+QString QInputDialog::getItem(QWidget *parent, const QString &title, const QString &label,
+                              const QStringList &items, int current, bool editable, bool *ok,
+                              Qt::WindowFlags flags)
+{
+    return getItem(parent, title, label, items, current, editable, ok, flags, Qt::ImhNone);
+}
+
 /*!
     \obsolete
 
