@@ -1,40 +1,40 @@
 /****************************************************************************
 **
-** Copyright (C) 2009 Nokia Corporation and/or its subsidiary(-ies).
+** Copyright (C) 2011 Nokia Corporation and/or its subsidiary(-ies).
 ** All rights reserved.
 ** Contact: Nokia Corporation (qt-info@nokia.com)
 **
 ** This file is part of the QtGui module of the Qt Toolkit.
 **
 ** $QT_BEGIN_LICENSE:LGPL$
-** Commercial Usage
-** Licensees holding valid Qt Commercial licenses may use this file in
-** accordance with the Qt Commercial License Agreement provided with the
-** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and Nokia.
-**
 ** GNU Lesser General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU Lesser
-** General Public License version 2.1 as published by the Free Software
-** Foundation and appearing in the file LICENSE.LGPL included in the
-** packaging of this file.  Please review the following information to
-** ensure the GNU Lesser General Public License version 2.1 requirements
-** will be met: http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
+** This file may be used under the terms of the GNU Lesser General Public
+** License version 2.1 as published by the Free Software Foundation and
+** appearing in the file LICENSE.LGPL included in the packaging of this
+** file. Please review the following information to ensure the GNU Lesser
+** General Public License version 2.1 requirements will be met:
+** http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
 **
 ** In addition, as a special exception, Nokia gives you certain additional
-** rights.  These rights are described in the Nokia Qt LGPL Exception
+** rights. These rights are described in the Nokia Qt LGPL Exception
 ** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
 **
 ** GNU General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU
-** General Public License version 3.0 as published by the Free Software
-** Foundation and appearing in the file LICENSE.GPL included in the
-** packaging of this file.  Please review the following information to
-** ensure the GNU General Public License version 3.0 requirements will be
-** met: http://www.gnu.org/copyleft/gpl.html.
+** Alternatively, this file may be used under the terms of the GNU General
+** Public License version 3.0 as published by the Free Software Foundation
+** and appearing in the file LICENSE.GPL included in the packaging of this
+** file. Please review the following information to ensure the GNU General
+** Public License version 3.0 requirements will be met:
+** http://www.gnu.org/copyleft/gpl.html.
+**
+** Other Usage
+** Alternatively, this file may be used in accordance with the terms and
+** conditions contained in a signed written agreement between you and Nokia.
+**
+**
+**
+**
 **
-** If you have questions regarding the use of this file, please contact
-** Nokia at qt-info@nokia.com.
 ** $QT_END_LICENSE$
 **
 ****************************************************************************/
@@ -58,12 +58,17 @@
 #include <QtGui/qtextdocument.h>
 #include <QtGui/qapplication.h>
 #include <QtGui/qtextedit.h>
+#include <QtGui/qtextbrowser.h>
 #include <QtGui/qmenu.h>
 #include "qdialog_p.h"
 #include <QtGui/qfont.h>
 #include <QtGui/qfontmetrics.h>
 #include <QtGui/qclipboard.h>
 
+#ifndef QT_NO_STYLE_S60
+#include <qs60style.h>
+#endif
+
 #ifdef Q_WS_WINCE
 extern bool qt_wince_is_mobile();    //defined in qguifunctions_wince.cpp
 extern bool qt_wince_is_smartphone();//defined in qguifunctions_wince.cpp
@@ -91,8 +96,8 @@ public:
         {
 #ifndef QT_NO_CONTEXTMENU
             QMenu *menu = createStandardContextMenu();
-            menu->exec(e->globalPos());
-            delete menu;
+            menu->setAttribute(Qt::WA_DeleteOnClose);
+            menu->popup(e->globalPos());
 #else
             Q_UNUSED(e);
 #endif
@@ -117,14 +122,44 @@ public:
     }
     void setText(const QString &text) { textEdit->setPlainText(text); }
     QString text() const { return textEdit->toPlainText(); }
-    QString label(DetailButtonLabel label)
-        { return label == ShowLabel ? QMessageBox::tr("Show Details...")
-                                    : QMessageBox::tr("Hide Details..."); }
 private:
     TextEdit *textEdit;
 };
 #endif // QT_NO_TEXTEDIT
 
+class DetailButton : public QPushButton
+{
+public:
+    DetailButton(QWidget *parent) : QPushButton(label(ShowLabel), parent)
+    {
+        setSizePolicy(QSizePolicy::Fixed, QSizePolicy::Fixed);
+    }
+
+    QString label(DetailButtonLabel label) const
+    { return label == ShowLabel ? QMessageBox::tr("Show Details...") : QMessageBox::tr("Hide Details..."); }
+
+    void setLabel(DetailButtonLabel lbl)
+    { setText(label(lbl)); }
+
+    QSize sizeHint() const
+    {
+        ensurePolished();
+        QStyleOptionButton opt;
+        initStyleOption(&opt);
+        const QFontMetrics fm = fontMetrics();
+        opt.text = label(ShowLabel);
+        QSize sz = fm.size(Qt::TextShowMnemonic, opt.text);
+        QSize ret = style()->sizeFromContents(QStyle::CT_PushButton, &opt, sz, this).
+                      expandedTo(QApplication::globalStrut());
+        opt.text = label(HideLabel);
+        sz = fm.size(Qt::TextShowMnemonic, opt.text);
+        ret = ret.expandedTo(style()->sizeFromContents(QStyle::CT_PushButton, &opt, sz, this).
+                      expandedTo(QApplication::globalStrut()));
+        return ret;
+    }
+};
+
+
 class QMessageBoxPrivate : public QDialogPrivate
 {
     Q_DECLARE_PUBLIC(QMessageBox)
@@ -180,7 +215,7 @@ public:
     QAbstractButton *escapeButton;
     QPushButton *defaultButton;
     QAbstractButton *clickedButton;
-    QPushButton *detailsButton;
+    DetailButton *detailsButton;
 #ifndef QT_NO_TEXTEDIT
     QMessageBoxDetailsText *detailsText;
 #endif
@@ -188,6 +223,9 @@ public:
     bool autoAddOkButton;
     QAbstractButton *detectedEscapeButton;
     QLabel *informativeLabel;
+#if defined(Q_OS_SYMBIAN) || defined(Q_WS_MAEMO_5)
+    QTextBrowser *textBrowser;
+#endif
     QPointer<QObject> receiverToDisconnectOnClose;
     QByteArray memberToDisconnectOnClose;
     QByteArray signalToDisconnectOnClose;
@@ -221,8 +259,15 @@ void QMessageBoxPrivate::init(const QString &title, const QString &text)
 
     QGridLayout *grid = new QGridLayout;
 #ifndef Q_WS_MAC
-    grid->addWidget(iconLabel, 0, 0, 2, 1, Qt::AlignTop);
-    grid->addWidget(label, 0, 1, 1, 1);
+#ifdef Q_WS_S60
+    const int preferredIconColumn = (QApplication::layoutDirection() == Qt::LeftToRight) ? 1 : 0;
+    const int preferredTextColumn = (QApplication::layoutDirection() == Qt::LeftToRight) ? 0 : 1;
+#else
+    const int preferredIconColumn = 0;
+    const int preferredTextColumn = 1;
+#endif
+    grid->addWidget(iconLabel, 0, preferredIconColumn, 2, 1, Qt::AlignTop);
+    grid->addWidget(label, 0, preferredTextColumn, 1, 1);
     // -- leave space for information label --
     grid->addWidget(buttonBox, 2, 0, 1, 2);
 #else
@@ -310,6 +355,7 @@ void QMessageBoxPrivate::updateSize()
             }
             width = hardLimit;
         }
+    }
 #ifdef Q_WS_S60
         // in S60 portait messageBoxes should always occupy maximum width
         if (QApplication::desktop()->size().height() > QApplication::desktop()->size().width()){
@@ -319,7 +365,6 @@ void QMessageBoxPrivate::updateSize()
             width = qMin(QApplication::desktop()->size().height(), hardLimit);
         }
 #endif
-    }
 
     if (informativeLabel) {
         label->setSizePolicy(QSizePolicy::Ignored, QSizePolicy::Ignored);
@@ -349,6 +394,16 @@ void QMessageBoxPrivate::updateSize()
     int height = (layout->hasHeightForWidth())
                      ? layout->totalHeightForWidth(width)
                      : layout->totalMinimumSize().height();
+
+#ifndef QT_NO_STYLE_S60
+        QS60Style *s60Style = 0;
+        s60Style = qobject_cast<QS60Style *>(QApplication::style());
+
+        //use custom pixel metric to deduce the minimum height of the messagebox
+        if (s60Style)
+            height = qMax(height, s60Style->pixelMetric((QStyle::PixelMetric)PM_MessageBoxHeight));
+#endif
+
     q->setFixedSize(width, height);
     QCoreApplication::removePostedEvents(q, QEvent::LayoutRequest);
 }
@@ -364,7 +419,7 @@ void QMessageBoxPrivate::updateSize()
 void QMessageBoxPrivate::hideSpecial()
 {
     Q_Q(QMessageBox);
-    QList<QPushButton*> list = qFindChildren<QPushButton*>(q);
+    QList<QPushButton*> list = q->findChildren<QPushButton*>();
         for (int i=0; i<list.size(); ++i) {
             QPushButton *pb = list.at(i);
             QString text = pb->text();
@@ -417,7 +472,7 @@ void QMessageBoxPrivate::_q_buttonClicked(QAbstractButton *button)
     Q_Q(QMessageBox);
 #ifndef QT_NO_TEXTEDIT
     if (detailsButton && detailsText && button == detailsButton) {
-        detailsButton->setText(detailsText->isHidden() ? detailsText->label(HideLabel) : detailsText->label(ShowLabel));
+        detailsButton->setLabel(detailsText->isHidden() ? HideLabel : ShowLabel);
         detailsText->setHidden(!detailsText->isHidden());
         updateSize();
     } else
@@ -493,8 +548,8 @@ void QMessageBoxPrivate::_q_buttonClicked(QAbstractButton *button)
     \snippet doc/src/snippets/code/src_gui_dialogs_qmessagebox.cpp 6
 
     This is the approach recommended in the
-    \l{http://developer.apple.com/documentation/UserExperience/Conceptual/AppleHIGuidelines/XHIGWindows/chapter_18_section_7.html}
-    {Mac OS X Guidlines}. Similar guidlines apply for the other
+    \l{http://developer.apple.com/library/mac/documentation/UserExperience/Conceptual/AppleHIGuidelines/Windows/Windows.html#//apple_ref/doc/uid/20000961-BABCAJID}
+    {Mac OS X Guidelines}. Similar guidelines apply for the other
     platforms, but note the different ways the
     \l{QMessageBox::informativeText} {informative text} is handled for
     different platforms.
@@ -728,10 +783,8 @@ QMessageBox::QMessageBox(QWidget *parent)
     added at any time using addButton(). The \a parent and \a f
     arguments are passed to the QDialog constructor.
 
-    If \a parent is 0, the message box is an \l{Qt::ApplicationModal}
-    {application modal} dialog box. If \a parent is a widget, the
-    message box is \l{Qt::WindowModal} {window modal} relative to \a
-    parent.
+    The message box is an \l{Qt::ApplicationModal} {application modal}
+    dialog box.
 
     On Mac OS X, if \a parent is not 0 and you want your message box
     to appear as a Qt::Sheet of that parent, set the message box's
@@ -1218,7 +1271,7 @@ bool QMessageBox::event(QEvent *e)
               (e->type() == QEvent::OkRequest)
               ? QApplication::translate("QMessageBox", "OK")
               : QApplication::translate("QMessageBox", "Help");
-          QList<QPushButton*> list = qFindChildren<QPushButton*>(this);
+          QList<QPushButton*> list = findChildren<QPushButton*>();
           for (int i=0; i<list.size(); ++i) {
               QPushButton *pb = list.at(i);
               if (pb->text() == bName) {
@@ -1465,7 +1518,7 @@ static QMessageBox::StandardButton showNewMessageBox(QWidget *parent,
                                                             int(defaultButton), 0);
 
     QMessageBox msgBox(icon, title, text, QMessageBox::NoButton, parent);
-    QDialogButtonBox *buttonBox = qFindChild<QDialogButtonBox*>(&msgBox);
+    QDialogButtonBox *buttonBox = msgBox.findChild<QDialogButtonBox*>();
     Q_ASSERT(buttonBox != 0);
 
     uint mask = QMessageBox::FirstButton;
@@ -1490,10 +1543,12 @@ static QMessageBox::StandardButton showNewMessageBox(QWidget *parent,
 /*!
     \since 4.2
 
-    Opens an information message box with the specified \a title and
-    \a text. The standard \a buttons are added to the message box. \a
-    defaultButton specifies the button used when \key Enter is
-    pressed. \a defaultButton must refer to a button that was given in \a buttons.
+    Opens an information message box with the given \a title and
+    \a text in front of the specified \a parent widget.
+
+    The standard \a buttons are added to the message box.
+    \a defaultButton specifies the button used when \key Enter is pressed.
+    \a defaultButton must refer to a button that was given in \a buttons.
     If \a defaultButton is QMessageBox::NoButton, QMessageBox
     chooses a suitable default automatically.
 
@@ -1501,10 +1556,12 @@ static QMessageBox::StandardButton showNewMessageBox(QWidget *parent,
     \key Esc was pressed instead, the \l{Default and Escape Keys}
     {escape button} is returned.
 
-    If \a parent is 0, the message box is an \l{Qt::ApplicationModal}
-    {application modal} dialog box. If \a parent is a widget, the
-    message box is \l{Qt::WindowModal} {window modal} relative to \a
-    parent.
+    The message box is an \l{Qt::ApplicationModal}{application modal}
+    dialog box.
+
+    \warning Do not delete \a parent during the execution of the dialog.
+             If you want to do this, you should create the dialog
+             yourself using one of the QMessageBox constructors.
 
     \sa question(), warning(), critical()
 */
@@ -1520,8 +1577,10 @@ QMessageBox::StandardButton QMessageBox::information(QWidget *parent, const QStr
 /*!
     \since 4.2
 
-    Opens a question message box with the specified \a title and \a
-    text. The standard \a buttons are added to the message box. \a
+    Opens a question message box with the given \a title and \a
+    text in front of the specified \a parent widget.
+
+    The standard \a buttons are added to the message box. \a
     defaultButton specifies the button used when \key Enter is
     pressed. \a defaultButton must refer to a button that was given in \a buttons.
     If \a defaultButton is QMessageBox::NoButton, QMessageBox
@@ -1531,10 +1590,12 @@ QMessageBox::StandardButton QMessageBox::information(QWidget *parent, const QStr
     \key Esc was pressed instead, the \l{Default and Escape Keys}
     {escape button} is returned.
 
-    If \a parent is 0, the message box is an \l{Qt::ApplicationModal}
-    {application modal} dialog box. If \a parent is a widget, the
-    message box is \l{Qt::WindowModal} {window modal} relative to \a
-    parent.
+    The message box is an \l{Qt::ApplicationModal} {application modal}
+    dialog box.
+
+    \warning Do not delete \a parent during the execution of the dialog.
+             If you want to do this, you should create the dialog
+             yourself using one of the QMessageBox constructors.
 
     \sa information(), warning(), critical()
 */
@@ -1548,8 +1609,10 @@ QMessageBox::StandardButton QMessageBox::question(QWidget *parent, const QString
 /*!
     \since 4.2
 
-    Opens a warning message box with the specified \a title and \a
-    text. The standard \a buttons are added to the message box. \a
+    Opens a warning message box with the given \a title and \a
+    text in front of the specified \a parent widget.
+
+    The standard \a buttons are added to the message box. \a
     defaultButton specifies the button used when \key Enter is
     pressed. \a defaultButton must refer to a button that was given in \a buttons.
     If \a defaultButton is QMessageBox::NoButton, QMessageBox
@@ -1559,10 +1622,12 @@ QMessageBox::StandardButton QMessageBox::question(QWidget *parent, const QString
     \key Esc was pressed instead, the \l{Default and Escape Keys}
     {escape button} is returned.
 
-    If \a parent is 0, the message box is an \l{Qt::ApplicationModal}
-    {application modal} dialog box. If \a parent is a widget, the
-    message box is \l{Qt::WindowModal} {window modal} relative to \a
-    parent.
+    The message box is an \l{Qt::ApplicationModal} {application modal}
+    dialog box.
+
+    \warning Do not delete \a parent during the execution of the dialog.
+             If you want to do this, you should create the dialog
+             yourself using one of the QMessageBox constructors.
 
     \sa question(), information(), critical()
 */
@@ -1576,8 +1641,10 @@ QMessageBox::StandardButton QMessageBox::warning(QWidget *parent, const QString
 /*!
     \since 4.2
 
-    Opens a critical message box with the specified \a title and \a
-    text. The standard \a buttons are added to the message box. \a
+    Opens a critical message box with the given \a title and \a
+    text in front of the specified \a parent widget.
+
+    The standard \a buttons are added to the message box. \a
     defaultButton specifies the button used when \key Enter is
     pressed. \a defaultButton must refer to a button that was given in \a buttons.
     If \a defaultButton is QMessageBox::NoButton, QMessageBox
@@ -1587,14 +1654,12 @@ QMessageBox::StandardButton QMessageBox::warning(QWidget *parent, const QString
     \key Esc was pressed instead, the \l{Default and Escape Keys}
     {escape button} is returned.
 
-    If \a parent is 0, the message box is an \l{Qt::ApplicationModal}
-    {application modal} dialog box. If \a parent is a widget, the
-    message box is \l{Qt::WindowModal} {window modal} relative to \a
-    parent.
+    The message box is an \l{Qt::ApplicationModal} {application modal}
+    dialog box.
 
-  \warning Do not delete \a parent during the execution of the dialog.
-           If you want to do this, you should create the dialog
-           yourself using one of the QMessageBox constructors.
+    \warning Do not delete \a parent during the execution of the dialog.
+             If you want to do this, you should create the dialog
+             yourself using one of the QMessageBox constructors.
 
     \sa question(), warning(), information()
 */
@@ -1622,7 +1687,7 @@ QMessageBox::StandardButton QMessageBox::critical(QWidget *parent, const QString
 
     The about box has a single button labelled "OK". On Mac OS X, the
     about box is popped up as a modeless window; on other platforms,
-    it is currently a window modal.
+    it is currently application modal.
 
     \sa QWidget::windowIcon(), QApplication::activeWindow()
 */
@@ -1675,7 +1740,7 @@ void QMessageBox::about(QWidget *parent, const QString &title, const QString &te
     QApplication provides this functionality as a slot.
 
     On Mac OS X, the about box is popped up as a modeless window; on
-    other platforms, it is currently window modal.
+    other platforms, it is currently application modal.
 
     \sa QApplication::aboutQt()
 */
@@ -1723,7 +1788,7 @@ void QMessageBox::aboutQt(QWidget *parent, const QString &title)
         "to comply with the terms of the GNU GPL version 3.0.</p>"
         "<p>Please see <a href=\"http://qt.nokia.com/products/licensing\">qt.nokia.com/products/licensing</a> "
         "for an overview of Qt licensing.</p>"
-        "<p>Copyright (C) 2009 Nokia Corporation and/or its subsidiary(-ies).</p>"
+        "<p>Copyright (C) 2011 Nokia Corporation and/or its subsidiary(-ies).</p>"
         "<p>Qt is a Nokia product. See <a href=\"http://qt.nokia.com/\">qt.nokia.com</a> "
         "for more information.</p>"
         );
@@ -1887,7 +1952,7 @@ void QMessageBoxPrivate::retranslateStrings()
 {
 #ifndef QT_NO_TEXTEDIT
     if (detailsButton)
-        detailsButton->setText(detailsText->isHidden() ? detailsText->label(HideLabel) : detailsText->label(ShowLabel));
+        detailsButton->setLabel(detailsText->isHidden() ? ShowLabel : HideLabel);
 #endif
 }
 
@@ -1935,10 +2000,8 @@ void QMessageBoxPrivate::retranslateStrings()
 
     \snippet doc/src/snippets/dialogs/dialogs.cpp 2
 
-    If \a parent is 0, the message box is an \l{Qt::ApplicationModal}
-    {application modal} dialog box. If \a parent is a widget, the
-    message box is \l{Qt::WindowModal} {window modal} relative to \a
-    parent.
+    The message box is an \l{Qt::ApplicationModal} {application modal}
+    dialog box.
 
     The \a parent and \a f arguments are passed to
     the QDialog constructor.
@@ -1987,10 +2050,8 @@ QMessageBox::QMessageBox(const QString &title, const QString &text, Icon icon,
     Returns the identity (QMessageBox::Ok, or QMessageBox::No, etc.)
     of the button that was clicked.
 
-    If \a parent is 0, the message box is an \l{Qt::ApplicationModal}
-    {application modal} dialog box. If \a parent is a widget, the
-    message box is \l{Qt::WindowModal} {window modal} relative to \a
-    parent.
+    The message box is an \l{Qt::ApplicationModal} {application modal}
+    dialog box.
 
   \warning Do not delete \a parent during the execution of the dialog.
            If you want to do this, you should create the dialog
@@ -2025,10 +2086,8 @@ int QMessageBox::information(QWidget *parent, const QString &title, const QStrin
     supply 0, 1 or 2 to make pressing \key Esc equivalent to clicking
     the relevant button.
 
-    If \a parent is 0, the message box is an \l{Qt::ApplicationModal}
-    {application modal} dialog box. If \a parent is a widget, the
-    message box is \l{Qt::WindowModal} {window modal} relative to \a
-    parent.
+    The message box is an \l{Qt::ApplicationModal} {application modal}
+    dialog box.
 
   \warning Do not delete \a parent during the execution of the dialog.
            If you want to do this, you should create the dialog
@@ -2077,10 +2136,8 @@ int QMessageBox::information(QWidget *parent, const QString &title, const QStrin
     Returns the identity (QMessageBox::Yes, or QMessageBox::No, etc.)
     of the button that was clicked.
 
-    If \a parent is 0, the message box is an \l{Qt::ApplicationModal}
-    {application modal} dialog box. If \a parent is a widget, the
-    message box is \l{Qt::WindowModal} {window modal} relative to \a
-    parent.
+    The message box is an \l{Qt::ApplicationModal} {application modal}
+    dialog box.
 
   \warning Do not delete \a parent during the execution of the dialog.
            If you want to do this, you should create the dialog
@@ -2115,10 +2172,8 @@ int QMessageBox::question(QWidget *parent, const QString &title, const QString&
     supply 0, 1 or 2 to make pressing Escape equivalent to clicking
     the relevant button.
 
-    If \a parent is 0, the message box is an \l{Qt::ApplicationModal}
-    {application modal} dialog box. If \a parent is a widget, the
-    message box is \l{Qt::WindowModal} {window modal} relative to \a
-    parent.
+    The message box is an \l{Qt::ApplicationModal} {application modal}
+    dialog box.
 
   \warning Do not delete \a parent during the execution of the dialog.
            If you want to do this, you should create the dialog
@@ -2167,10 +2222,8 @@ int QMessageBox::question(QWidget *parent, const QString &title, const QString&
     Returns the identity (QMessageBox::Ok or QMessageBox::No or ...)
     of the button that was clicked.
 
-    If \a parent is 0, the message box is an \l{Qt::ApplicationModal}
-    {application modal} dialog box. If \a parent is a widget, the
-    message box is \l{Qt::WindowModal} {window modal} relative to \a
-    parent.
+    The message box is an \l{Qt::ApplicationModal} {application modal}
+    dialog box.
 
   \warning Do not delete \a parent during the execution of the dialog.
            If you want to do this, you should create the dialog
@@ -2205,10 +2258,8 @@ int QMessageBox::warning(QWidget *parent, const QString &title, const QString& t
     supply 0, 1, or 2 to make pressing Escape equivalent to clicking
     the relevant button.
 
-    If \a parent is 0, the message box is an \l{Qt::ApplicationModal}
-    {application modal} dialog box. If \a parent is a widget, the
-    message box is \l{Qt::WindowModal} {window modal} relative to \a
-    parent.
+    The message box is an \l{Qt::ApplicationModal} {application modal}
+    dialog box.
 
   \warning Do not delete \a parent during the execution of the dialog.
            If you want to do this, you should create the dialog
@@ -2256,10 +2307,8 @@ int QMessageBox::warning(QWidget *parent, const QString &title, const QString& t
     Returns the identity (QMessageBox::Ok, or QMessageBox::No, etc.)
     of the button that was clicked.
 
-    If \a parent is 0, the message box is an \l{Qt::ApplicationModal}
-    {application modal} dialog box. If \a parent is a widget, the
-    message box is \l{Qt::WindowModal} {window modal} relative to \a
-    parent.
+    The message box is an \l{Qt::ApplicationModal} {application modal}
+    dialog box.
 
   \warning Do not delete \a parent during the execution of the dialog.
            If you want to do this, you should create the dialog
@@ -2295,10 +2344,8 @@ int QMessageBox::critical(QWidget *parent, const QString &title, const QString&
     supply 0, 1, or 2 to make pressing Escape equivalent to clicking
     the relevant button.
 
-    If \a parent is 0, the message box is an \l{Qt::ApplicationModal}
-    {application modal} dialog box. If \a parent is a widget, the
-    message box is \l{Qt::WindowModal} {window modal} relative to \a
-    parent.
+    The message box is an \l{Qt::ApplicationModal} {application modal}
+    dialog box.
 
   \warning Do not delete \a parent during the execution of the dialog.
            If you want to do this, you should create the dialog
@@ -2395,11 +2442,8 @@ void QMessageBox::setDetailedText(const QString &text)
             grid->addWidget(d->detailsText, grid->rowCount(), 0, 1, grid->columnCount());
         d->detailsText->hide();
     }
-    if (!d->detailsButton) {
-        d->detailsButton = new QPushButton(d->detailsText->label(ShowLabel), this);
-        QPushButton hideDetails(d->detailsText->label(HideLabel));
-        d->detailsButton->setFixedSize(d->detailsButton->sizeHint().expandedTo(hideDetails.sizeHint()));
-    }
+    if (!d->detailsButton)
+        d->detailsButton = new DetailButton(this);
     d->detailsText->setText(text);
 }
 #endif // QT_NO_TEXTEDIT
@@ -2442,7 +2486,7 @@ void QMessageBox::setInformativeText(const QString &text)
     }
 
     if (!d->informativeLabel) {
-        QLabel *label = new QLabel;
+        QLabel *label = new QLabel(this);
         label->setObjectName(QLatin1String("qt_msgbox_informativelabel"));
         label->setTextInteractionFlags(Qt::TextInteractionFlags(style()->styleHint(QStyle::SH_MessageBox_TextInteractionFlags, 0, this)));
         label->setAlignment(Qt::AlignTop | Qt::AlignLeft);
@@ -2459,10 +2503,29 @@ void QMessageBox::setInformativeText(const QString &text)
 #endif
         label->setWordWrap(true);
         QGridLayout *grid = static_cast<QGridLayout *>(layout());
+#if defined(Q_OS_SYMBIAN) || defined(Q_WS_MAEMO_5)
+        label->hide();
+        QTextBrowser *textBrowser = new QTextBrowser(this);
+        textBrowser->setOpenExternalLinks(true);
+#if defined(Q_OS_SYMBIAN)
+        const int preferredTextColumn = (QApplication::layoutDirection() == Qt::LeftToRight) ? 0 : 1;
+#else
+        const int preferredTextColumn = 1;
+#endif
+        grid->addWidget(textBrowser, 1, preferredTextColumn, 1, 1);
+        d->textBrowser = textBrowser;
+#else
         grid->addWidget(label, 1, 1, 1, 1);
+#endif
         d->informativeLabel = label;
     }
     d->informativeLabel->setText(text);
+
+#if defined(Q_OS_SYMBIAN) || defined(Q_WS_MAEMO_5)
+    //We need to put the informative label inside textBrowser to enable scrolling of long texts.
+    d->textBrowser->setText(d->informativeLabel->text());
+#endif
+
     d->updateSize();
 }
 
