@@ -1,40 +1,40 @@
 /****************************************************************************
 **
-** Copyright (C) 2009 Nokia Corporation and/or its subsidiary(-ies).
+** Copyright (C) 2011 Nokia Corporation and/or its subsidiary(-ies).
 ** All rights reserved.
 ** Contact: Nokia Corporation (qt-info@nokia.com)
 **
 ** This file is part of the QtGui module of the Qt Toolkit.
 **
 ** $QT_BEGIN_LICENSE:LGPL$
-** Commercial Usage
-** Licensees holding valid Qt Commercial licenses may use this file in
-** accordance with the Qt Commercial License Agreement provided with the
-** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and Nokia.
-**
 ** GNU Lesser General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU Lesser
-** General Public License version 2.1 as published by the Free Software
-** Foundation and appearing in the file LICENSE.LGPL included in the
-** packaging of this file.  Please review the following information to
-** ensure the GNU Lesser General Public License version 2.1 requirements
-** will be met: http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
+** This file may be used under the terms of the GNU Lesser General Public
+** License version 2.1 as published by the Free Software Foundation and
+** appearing in the file LICENSE.LGPL included in the packaging of this
+** file. Please review the following information to ensure the GNU Lesser
+** General Public License version 2.1 requirements will be met:
+** http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
 **
 ** In addition, as a special exception, Nokia gives you certain additional
-** rights.  These rights are described in the Nokia Qt LGPL Exception
+** rights. These rights are described in the Nokia Qt LGPL Exception
 ** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
 **
 ** GNU General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU
-** General Public License version 3.0 as published by the Free Software
-** Foundation and appearing in the file LICENSE.GPL included in the
-** packaging of this file.  Please review the following information to
-** ensure the GNU General Public License version 3.0 requirements will be
-** met: http://www.gnu.org/copyleft/gpl.html.
+** Alternatively, this file may be used under the terms of the GNU General
+** Public License version 3.0 as published by the Free Software Foundation
+** and appearing in the file LICENSE.GPL included in the packaging of this
+** file. Please review the following information to ensure the GNU General
+** Public License version 3.0 requirements will be met:
+** http://www.gnu.org/copyleft/gpl.html.
+**
+** Other Usage
+** Alternatively, this file may be used in accordance with the terms and
+** conditions contained in a signed written agreement between you and Nokia.
+**
+**
+**
+**
 **
-** If you have questions regarding the use of this file, please contact
-** Nokia at qt-info@nokia.com.
 ** $QT_END_LICENSE$
 **
 ****************************************************************************/
@@ -48,6 +48,7 @@
 #include "qgraphicslayoutitem.h"
 #include "qgraphicslayoutitem_p.h"
 #include "qwidget.h"
+#include "qgraphicswidget.h"
 
 #include <QtDebug>
 
@@ -127,28 +128,103 @@ QGraphicsLayoutItemPrivate::~QGraphicsLayoutItemPrivate()
 void QGraphicsLayoutItemPrivate::init()
 {
     sizeHintCacheDirty = true;
-    sizePolicy = QSizePolicy(QSizePolicy::Preferred, QSizePolicy::Preferred);
+    sizeHintWithConstraintCacheDirty = true;
 }
 
 /*!
     \internal
+
+    effectiveSizeHint has a quirky behavior, one of the quirkinesses is when the hfw function is
+    combined with user-specified min/max sizes. The input to hfw function (e.g width) must be within
+    the min/max width constraint, and the output must be within the min/max height. This sets up a
+    loose dependency between minimum width and maximum height (or minimum height, depending on the
+    type of hfw function). Note that its only the concrete subclass that implements that hfw
+    function that knows if this dependency means that the height will increase or decrease when the
+    width is increased.
+
+    The application should try to ensure that the user-defined sizes are within the range so that
+    they don't conflict with the hfw function.
+
+    Suppose, for instance that the hfw function is:
+
+        height = 2000/width
+
+    and the item has these user-defined sizes:
+
+        min  ( 5,  5)
+        pref(100, 10)
+        max (500,100)
+
+    what is the return value if one calls item->effectiveSizeHint(Qt::MinimumSize, QSizeF(10, -1)); ?
+    The sizeHint() function would return QSizeF(10, 200), but it would be bounded down to 100 due
+    to the max value, so it would return (10, 100). This is not what the item expects, since it
+    really wants that its hfw is respected. If this is a label with wrapped text, this would most
+    likely lead to that some text is clipped. This is certainly not what the app developer wants.
+    Now, it would be better if the user changed those constraints to match the hfw function:
+
+        min ( 20,  5)
+        pref(100, 10)
+        max (500,100)
+
+    here, it says that the width cannot be smaller than 20. This is because if it becomes smaller
+    than 20 the result of the hfw function would violate the max height (100).
+
+    However, there is a similar problem if the width passed to the hfw function reaches *max* width:
+
+    the sizeHint() function would now return QSizeF(500, 4), but 4 is smaller than the minimum
+    height (5), so effectiveSizeHint() would return (500, 5), which would leave too much space.
+    In this case, setting the max width to 400 fixes the problem:
+
+        min ( 20,  5)
+        pref(100, 10)
+        max (400,100)
+
+
+    The implementor of a hfw widget must be aware of this when sizeHint() is reimplemented, so that
+    the default min and max sizes works sensible. (unfortunately the implementor does not have the
+    control over user-set values).
+
 */
 QSizeF *QGraphicsLayoutItemPrivate::effectiveSizeHints(const QSizeF &constraint) const
 {
     Q_Q(const QGraphicsLayoutItem);
-    if (!sizeHintCacheDirty && cachedConstraint == constraint)
-        return cachedSizeHints;
+    QSizeF *sizeHintCache;
+    const bool hasConstraint = constraint.width() >= 0 || constraint.height() >= 0;
+    QSizeF adjustedConstraint = constraint;
+    if (hasConstraint) {
+        if (!sizeHintWithConstraintCacheDirty && constraint == cachedConstraint)
+            return cachedSizeHintsWithConstraints;
 
-    for (int i = 0; i < Qt::NSizeHints; ++i) {
-        cachedSizeHints[i] = constraint;
-        if (userSizeHints)
-            combineSize(cachedSizeHints[i], userSizeHints[i]);
+        const QSizeF *hintsWithoutConstraint = effectiveSizeHints(QSizeF(-1,-1));
+
+        if (adjustedConstraint.width() >= 0)
+            adjustedConstraint.setWidth( qBound( hintsWithoutConstraint[Qt::MinimumSize].width(),
+                                                 adjustedConstraint.width(),
+                                                 hintsWithoutConstraint[Qt::MaximumSize].width()));
+        if (adjustedConstraint.height() >= 0)
+            adjustedConstraint.setHeight( qBound( hintsWithoutConstraint[Qt::MinimumSize].height(),
+                                                  adjustedConstraint.height(),
+                                                  hintsWithoutConstraint[Qt::MaximumSize].height()));
+
+        if (!sizeHintWithConstraintCacheDirty && adjustedConstraint == cachedConstraint)
+            return cachedSizeHintsWithConstraints;
+        sizeHintCache = cachedSizeHintsWithConstraints;
+    } else {
+        if (!sizeHintCacheDirty)
+            return cachedSizeHints;
+        sizeHintCache = cachedSizeHints;
     }
 
-    QSizeF &minS = cachedSizeHints[Qt::MinimumSize];
-    QSizeF &prefS = cachedSizeHints[Qt::PreferredSize];
-    QSizeF &maxS = cachedSizeHints[Qt::MaximumSize];
-    QSizeF &descentS = cachedSizeHints[Qt::MinimumDescent];
+    for (int i = 0; i < Qt::NSizeHints; ++i) {
+        sizeHintCache[i] = adjustedConstraint;
+        if (userSizeHints)
+            combineSize(sizeHintCache[i], userSizeHints[i]);
+    }
+
+    QSizeF &minS = sizeHintCache[Qt::MinimumSize];
+    QSizeF &prefS = sizeHintCache[Qt::PreferredSize];
+    QSizeF &maxS = sizeHintCache[Qt::MaximumSize];
+    QSizeF &descentS = sizeHintCache[Qt::MinimumDescent];
 
     normalizeHints(minS.rwidth(), prefS.rwidth(), maxS.rwidth(), descentS.rwidth());
     normalizeHints(minS.rheight(), prefS.rheight(), maxS.rheight(), descentS.rheight());
@@ -174,9 +250,13 @@ QSizeF *QGraphicsLayoutItemPrivate::effectiveSizeHints(const QSizeF &constraint)
     // Not supported yet
     // COMBINE_SIZE(descentS, q->sizeHint(Qt::MinimumDescent, constraint));
 
-    cachedConstraint = constraint;
-    sizeHintCacheDirty = false;
-    return cachedSizeHints;
+    if (hasConstraint) {
+        cachedConstraint = adjustedConstraint;
+        sizeHintWithConstraintCacheDirty = false;
+    } else {
+        sizeHintCacheDirty = false;
+    }
+    return sizeHintCache;
 }
 
 
@@ -231,7 +311,7 @@ void QGraphicsLayoutItemPrivate::setSize(Qt::SizeHint which, const QSizeF &size)
     if (userSizeHints) {
         if (size == userSizeHints[which])
             return;
-    } else if (!size.isValid()) {
+    } else if (size.width() < 0 && size.height() < 0) {
         return;
     }
 
@@ -259,6 +339,47 @@ void QGraphicsLayoutItemPrivate::setSizeComponent(
     q->updateGeometry();
 }
 
+
+bool QGraphicsLayoutItemPrivate::hasHeightForWidth() const
+{
+    Q_Q(const QGraphicsLayoutItem);
+    if (isLayout) {
+        const QGraphicsLayout *l = static_cast<const QGraphicsLayout *>(q);
+        for (int i = l->count() - 1; i >= 0; --i) {
+            if (QGraphicsLayoutItemPrivate::get(l->itemAt(i))->hasHeightForWidth())
+                return true;
+        }
+    } else if (QGraphicsItem *item = q->graphicsItem()) {
+        if (item->isWidget()) {
+            QGraphicsWidget *w = static_cast<QGraphicsWidget *>(item);
+            if (w->layout()) {
+                return QGraphicsLayoutItemPrivate::get(w->layout())->hasHeightForWidth();
+            }
+        }
+    }
+    return q->sizePolicy().hasHeightForWidth();
+}
+
+bool QGraphicsLayoutItemPrivate::hasWidthForHeight() const
+{
+    Q_Q(const QGraphicsLayoutItem);
+    if (isLayout) {
+        const QGraphicsLayout *l = static_cast<const QGraphicsLayout *>(q);
+        for (int i = l->count() - 1; i >= 0; --i) {
+            if (QGraphicsLayoutItemPrivate::get(l->itemAt(i))->hasWidthForHeight())
+                return true;
+        }
+    } else if (QGraphicsItem *item = q->graphicsItem()) {
+        if (item->isWidget()) {
+            QGraphicsWidget *w = static_cast<QGraphicsWidget *>(item);
+            if (w->layout()) {
+                return QGraphicsLayoutItemPrivate::get(w->layout())->hasWidthForHeight();
+            }
+        }
+    }
+    return q->sizePolicy().hasWidthForHeight();
+}
+
 /*!
     \class QGraphicsLayoutItem
     \brief The QGraphicsLayoutItem class can be inherited to allow your custom
@@ -322,7 +443,7 @@ void QGraphicsLayoutItemPrivate::setSizeComponent(
     layout, or false otherwise.
 
     Qt uses QGraphicsLayoutItem to provide layout functionality in the
-    \l{The Graphics View Framework}, but in the future its use may spread
+    \l{Graphics View Framework}, but in the future its use may spread
     throughout Qt itself.
 
     \sa QGraphicsWidget, QGraphicsLayout, QGraphicsLinearLayout,
@@ -339,6 +460,7 @@ QGraphicsLayoutItem::QGraphicsLayoutItem(QGraphicsLayoutItem *parent, bool isLay
 {
     Q_D(QGraphicsLayoutItem);
     d->init();
+    d->sizePolicy = QSizePolicy(QSizePolicy::Preferred, QSizePolicy::Preferred);
     d->q_ptr = this;
 }
 
@@ -349,6 +471,7 @@ QGraphicsLayoutItem::QGraphicsLayoutItem(QGraphicsLayoutItemPrivate &dd)
     : d_ptr(&dd)
 {
     Q_D(QGraphicsLayoutItem);
+    d->init();
     d->q_ptr = this;
 }
 
@@ -648,7 +771,7 @@ void QGraphicsLayoutItem::setMaximumHeight(qreal height)
     is equivalent to the item's position in parent coordinates).
 
     You must reimplement this function in a subclass of QGraphicsLayoutItem to
-    receive geometry updates. The layout will call this function when it does a 
+    receive geometry updates. The layout will call this function when it does a
     rearrangement.
 
     If \a rect is outside of the bounds of minimumSize and maximumSize, it
@@ -768,6 +891,7 @@ void QGraphicsLayoutItem::updateGeometry()
 {
     Q_D(QGraphicsLayoutItem);
     d->sizeHintCacheDirty = true;
+    d->sizeHintWithConstraintCacheDirty = true;
 }
 
 /*!
