@@ -1,40 +1,40 @@
 /****************************************************************************
 **
-** Copyright (C) 2009 Nokia Corporation and/or its subsidiary(-ies).
+** Copyright (C) 2011 Nokia Corporation and/or its subsidiary(-ies).
 ** All rights reserved.
 ** Contact: Nokia Corporation (qt-info@nokia.com)
 **
 ** This file is part of the QtGui module of the Qt Toolkit.
 **
 ** $QT_BEGIN_LICENSE:LGPL$
-** Commercial Usage
-** Licensees holding valid Qt Commercial licenses may use this file in
-** accordance with the Qt Commercial License Agreement provided with the
-** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and Nokia.
-**
 ** GNU Lesser General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU Lesser
-** General Public License version 2.1 as published by the Free Software
-** Foundation and appearing in the file LICENSE.LGPL included in the
-** packaging of this file.  Please review the following information to
-** ensure the GNU Lesser General Public License version 2.1 requirements
-** will be met: http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
+** This file may be used under the terms of the GNU Lesser General Public
+** License version 2.1 as published by the Free Software Foundation and
+** appearing in the file LICENSE.LGPL included in the packaging of this
+** file. Please review the following information to ensure the GNU Lesser
+** General Public License version 2.1 requirements will be met:
+** http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
 **
 ** In addition, as a special exception, Nokia gives you certain additional
-** rights.  These rights are described in the Nokia Qt LGPL Exception
+** rights. These rights are described in the Nokia Qt LGPL Exception
 ** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
 **
 ** GNU General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU
-** General Public License version 3.0 as published by the Free Software
-** Foundation and appearing in the file LICENSE.GPL included in the
-** packaging of this file.  Please review the following information to
-** ensure the GNU General Public License version 3.0 requirements will be
-** met: http://www.gnu.org/copyleft/gpl.html.
+** Alternatively, this file may be used under the terms of the GNU General
+** Public License version 3.0 as published by the Free Software Foundation
+** and appearing in the file LICENSE.GPL included in the packaging of this
+** file. Please review the following information to ensure the GNU General
+** Public License version 3.0 requirements will be met:
+** http://www.gnu.org/copyleft/gpl.html.
+**
+** Other Usage
+** Alternatively, this file may be used in accordance with the terms and
+** conditions contained in a signed written agreement between you and Nokia.
+**
+**
+**
+**
 **
-** If you have questions regarding the use of this file, please contact
-** Nokia at qt-info@nokia.com.
 ** $QT_END_LICENSE$
 **
 ****************************************************************************/
@@ -44,7 +44,9 @@
 #ifndef QT_NO_GRAPHICSVIEW
 
 #include <QtCore/qdebug.h>
+#include <QtCore/qnumeric.h>
 #include "qgraphicswidget_p.h"
+#include "qgraphicslayoutitem_p.h"
 #include "qgraphicslayout.h"
 #include "qgraphicsscene_p.h"
 #include <QtGui/qapplication.h>
@@ -70,14 +72,18 @@ void QGraphicsWidgetPrivate::init(QGraphicsItem *parentItem, Qt::WindowFlags wFl
     adjustWindowFlags(&wFlags);
     windowFlags = wFlags;
 
-    q->setParentItem(parentItem);
+    if (parentItem)
+        setParentItemHelper(parentItem, 0, 0);
+
     q->setSizePolicy(QSizePolicy(QSizePolicy::Preferred, QSizePolicy::Preferred, QSizePolicy::DefaultType));
     q->setGraphicsItem(q);
 
     resolveLayoutDirection();
     q->unsetWindowFrameMargins();
-    q->setFlag(QGraphicsItem::ItemUsesExtendedStyleOption);
-    q->setFlag(QGraphicsItem::ItemSendsGeometryChanges);
+    flags |= QGraphicsItem::ItemUsesExtendedStyleOption;
+    flags |= QGraphicsItem::ItemSendsGeometryChanges;
+    if (windowFlags & Qt::Window)
+        flags |= QGraphicsItem::ItemIsPanel;
 }
 
 qreal QGraphicsWidgetPrivate::titleBarHeight(const QStyleOptionTitleBar &options) const
@@ -227,6 +233,18 @@ void QGraphicsWidgetPrivate::resolveLayoutDirection()
     }
 }
 
+/* private slot */
+void QGraphicsWidgetPrivate::_q_relayout()
+{
+    --refCountInvokeRelayout;
+    if (refCountInvokeRelayout == 0) {
+        Q_Q(QGraphicsWidget);
+        bool wasResized = q->testAttribute(Qt::WA_Resized);
+        q->resize(q->size()); // this will restrict the size
+        q->setAttribute(Qt::WA_Resized, wasResized);
+    }
+}
+
 QPalette QGraphicsWidgetPrivate::naturalWidgetPalette() const
 {
     Q_Q(const QGraphicsWidget);
@@ -249,7 +267,10 @@ void QGraphicsWidgetPrivate::setFont_helper(const QFont &font)
 
 void QGraphicsWidgetPrivate::resolveFont(uint inheritedMask)
 {
+    Q_Q(QGraphicsWidget);
     inheritedFontResolveMask = inheritedMask;
+    if (QGraphicsWidget *p = q->parentWidget())
+        inheritedFontResolveMask |= p->d_func()->inheritedFontResolveMask;
     QFont naturalFont = naturalWidgetFont();
     QFont resolvedFont = font.resolve(naturalFont);
     updateFont(resolvedFont);
@@ -407,8 +428,7 @@ static qreal minimumHeightForWidth(qreal width, qreal minh, qreal maxh,
                                    bool heightForWidth = true)
 {
     qreal minimumHeightForWidth = -1;
-    const QSizePolicy sp = widget->layout() ? widget->layout()->sizePolicy() : widget->sizePolicy();
-    const bool hasHFW = sp.hasHeightForWidth();
+    const bool hasHFW = QGraphicsLayoutItemPrivate::get(widget)->hasHeightForWidth();
     if (hasHFW == heightForWidth) {
         minimumHeightForWidth = hasHFW
                                 ? widget->effectiveSizeHint(Qt::MinimumSize, QSizeF(width, -1)).height()
@@ -462,8 +482,8 @@ static QSizeF closestAcceptableSize(const QSizeF &proposed,
             minw = maxw;
             minh = maxh;
         }
-        middlew = minw + (maxw - minw)/2.0;
-        middleh = minh + (maxh - minh)/2.0;
+        middlew = minw + (maxw - minw)/qreal(2.0);
+        middleh = minh + (maxh - minh)/qreal(2.0);
 
         min_hfw = minimumHeightForWidth(middlew, minh, maxh, widget);
 
@@ -497,21 +517,31 @@ static void _q_boundGeometryToSizeConstraints(const QRectF &startGeometry,
     qreal width = qBound(min.width(), proposedRect.width(), max.width());
     qreal height = qBound(min.height(), proposedRect.height(), max.height());
 
-    QSizePolicy sp = widget->sizePolicy();
-    if (const QGraphicsLayout *l = widget->layout()) {
-        sp = l->sizePolicy();
-    }
-    const bool hasHFW = sp.hasHeightForWidth(); // || sp.hasWidthForHeight();
+    const bool hasHFW = QGraphicsLayoutItemPrivate::get(widget)->hasHeightForWidth();
+    const bool hasWFH = QGraphicsLayoutItemPrivate::get(widget)->hasWidthForHeight();
 
-    const bool widthChanged = proposedRect.width() < widget->size().width();
-    const bool heightChanged = proposedRect.height() < widget->size().height();
+    const bool widthChanged = proposedRect.width() != widget->size().width();
+    const bool heightChanged = proposedRect.height() != widget->size().height();
 
-    if (hasHFW) {
+    if (hasHFW || hasWFH) {
         if (widthChanged || heightChanged) {
-            const qreal minh = min.height();
-            const qreal maxh = max.height();
-            const qreal proposedHFW = minimumHeightForWidth(width, minh, maxh, widget);
-            if (proposedHFW > proposedRect.height()) {
+            qreal minExtent;
+            qreal maxExtent;
+            qreal constraint;
+            qreal proposed;
+            if (hasHFW) {
+                minExtent = min.height();
+                maxExtent = max.height();
+                constraint = width;
+                proposed = proposedRect.height();
+            } else {
+                // width for height
+                minExtent = min.width();
+                maxExtent = max.width();
+                constraint = height;
+                proposed = proposedRect.width();
+            }
+            if (minimumHeightForWidth(constraint, minExtent, maxExtent, widget, hasHFW) > proposed) {
                 QSizeF effectiveSize = closestAcceptableSize(QSizeF(width, height), widget);
                 width = effectiveSize.width();
                 height = effectiveSize.height();
@@ -743,7 +773,7 @@ bool QGraphicsWidgetPrivate::hasDecoration() const
 /**
  * is called after a reparent has taken place to fix up the focus chain(s)
  */
-void QGraphicsWidgetPrivate::fixFocusChainBeforeReparenting(QGraphicsWidget *newParent, QGraphicsScene *newScene)
+void QGraphicsWidgetPrivate::fixFocusChainBeforeReparenting(QGraphicsWidget *newParent, QGraphicsScene *oldScene, QGraphicsScene *newScene)
 {
     Q_Q(QGraphicsWidget);
 
@@ -756,7 +786,7 @@ void QGraphicsWidgetPrivate::fixFocusChainBeforeReparenting(QGraphicsWidget *new
 
     QGraphicsWidget *firstOld = 0;
     bool wasPreviousNew = true;
-    
+
     while (w != q) {
         bool isCurrentNew = q->isAncestorOf(w);
         if (isCurrentNew) {
@@ -789,9 +819,9 @@ void QGraphicsWidgetPrivate::fixFocusChainBeforeReparenting(QGraphicsWidget *new
     // update tabFocusFirst for oldScene if the item is going to be removed from oldScene
     if (newParent)
         newScene = newParent->scene();
-    QGraphicsScene *oldScene = q->scene();
+
     if (oldScene && newScene != oldScene)
-        oldScene->d_func()->tabFocusFirst = firstOld;
+        oldScene->d_func()->tabFocusFirst = (firstOld && firstOld->scene() == oldScene) ? firstOld : 0;
 
     QGraphicsItem *topLevelItem = newParent ? newParent->topLevelItem() : 0;
     QGraphicsWidget *topLevel = 0;
@@ -825,6 +855,66 @@ void QGraphicsWidgetPrivate::setLayout_helper(QGraphicsLayout *l)
     }
 }
 
+qreal QGraphicsWidgetPrivate::width() const
+{
+    Q_Q(const QGraphicsWidget);
+    return q->geometry().width();
+}
+
+void QGraphicsWidgetPrivate::setWidth(qreal w)
+{
+    if (qIsNaN(w))
+        return;
+    Q_Q(QGraphicsWidget);
+    if (q->geometry().width() == w)
+        return;
+
+    q->setGeometry(QRectF(q->x(), q->y(), w, height()));
+}
+
+void QGraphicsWidgetPrivate::resetWidth()
+{
+    Q_Q(QGraphicsWidget);
+    q->setGeometry(QRectF(q->x(), q->y(), 0, height()));
+}
+
+qreal QGraphicsWidgetPrivate::height() const
+{
+    Q_Q(const QGraphicsWidget);
+    return q->geometry().height();
+}
+
+void QGraphicsWidgetPrivate::setHeight(qreal h)
+{
+    if (qIsNaN(h))
+        return;
+    Q_Q(QGraphicsWidget);
+    if (q->geometry().height() == h)
+        return;
+
+    q->setGeometry(QRectF(q->x(), q->y(), width(), h));
+}
+
+void QGraphicsWidgetPrivate::resetHeight()
+{
+    Q_Q(QGraphicsWidget);
+    q->setGeometry(QRectF(q->x(), q->y(), width(), 0));
+}
+
+void QGraphicsWidgetPrivate::setGeometryFromSetPos()
+{
+    if (inSetGeometry)
+        return;
+    Q_Q(QGraphicsWidget);
+    inSetPos = 1;
+    // Ensure setGeometry is called (avoid recursion when setPos is
+    // called from within setGeometry).
+    q->setGeometry(QRectF(pos, q->size()));
+    inSetPos = 0 ;
+}
+
 QT_END_NAMESPACE
 
+#include "moc_qgraphicswidget.cpp"
+
 #endif //QT_NO_GRAPHICSVIEW
