@@ -1,40 +1,40 @@
 /****************************************************************************
 **
-** Copyright (C) 2009 Nokia Corporation and/or its subsidiary(-ies).
+** Copyright (C) 2011 Nokia Corporation and/or its subsidiary(-ies).
 ** All rights reserved.
 ** Contact: Nokia Corporation (qt-info@nokia.com)
 **
 ** This file is part of the QtGui module of the Qt Toolkit.
 **
 ** $QT_BEGIN_LICENSE:LGPL$
-** Commercial Usage
-** Licensees holding valid Qt Commercial licenses may use this file in
-** accordance with the Qt Commercial License Agreement provided with the
-** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and Nokia.
-**
 ** GNU Lesser General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU Lesser
-** General Public License version 2.1 as published by the Free Software
-** Foundation and appearing in the file LICENSE.LGPL included in the
-** packaging of this file.  Please review the following information to
-** ensure the GNU Lesser General Public License version 2.1 requirements
-** will be met: http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
+** This file may be used under the terms of the GNU Lesser General Public
+** License version 2.1 as published by the Free Software Foundation and
+** appearing in the file LICENSE.LGPL included in the packaging of this
+** file. Please review the following information to ensure the GNU Lesser
+** General Public License version 2.1 requirements will be met:
+** http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
 **
 ** In addition, as a special exception, Nokia gives you certain additional
-** rights.  These rights are described in the Nokia Qt LGPL Exception
+** rights. These rights are described in the Nokia Qt LGPL Exception
 ** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
 **
 ** GNU General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU
-** General Public License version 3.0 as published by the Free Software
-** Foundation and appearing in the file LICENSE.GPL included in the
-** packaging of this file.  Please review the following information to
-** ensure the GNU General Public License version 3.0 requirements will be
-** met: http://www.gnu.org/copyleft/gpl.html.
+** Alternatively, this file may be used under the terms of the GNU General
+** Public License version 3.0 as published by the Free Software Foundation
+** and appearing in the file LICENSE.GPL included in the packaging of this
+** file. Please review the following information to ensure the GNU General
+** Public License version 3.0 requirements will be met:
+** http://www.gnu.org/copyleft/gpl.html.
+**
+** Other Usage
+** Alternatively, this file may be used in accordance with the terms and
+** conditions contained in a signed written agreement between you and Nokia.
+**
+**
+**
+**
 **
-** If you have questions regarding the use of this file, please contact
-** Nokia at qt-info@nokia.com.
 ** $QT_END_LICENSE$
 **
 ****************************************************************************/
@@ -47,7 +47,7 @@
 #include <qlayout.h>
 #include <qstyle.h>
 #include <qstyleoption.h>
-#include <qdatetime.h>
+#include <qelapsedtimer.h>
 #include <qpointer.h>
 #include <qdebug.h>
 #include <qx11info_x11.h>
@@ -415,8 +415,9 @@ static Bool functor(Display *display, XEvent *event, XPointer arg)
 	status = XGetWindowProperty(display, data->id, ATOM(WM_STATE), 0, 2, False, ATOM(WM_STATE),
 				    &ret, &format, &nitems, &after, &retval );
 	if (status == Success && ret == ATOM(WM_STATE) && format == 32 && nitems > 0) {
-            long *state = (long *)retval;
-	    if (state[0] == WithdrawnState) {
+            long state = *(long *)retval;
+            XFree(retval);
+            if (state == WithdrawnState) {
                 data->clearedWmState = true;
 		return true;
             }
@@ -512,7 +513,7 @@ QX11EmbedWidget::~QX11EmbedWidget()
                  << "from container with winId" << d->container;
 #endif
         XUnmapWindow(x11Info().display(), internalWinId());
-        XReparentWindow(x11Info().display(), internalWinId(), x11Info().appRootWindow(), 0, 0);
+        XReparentWindow(x11Info().display(), internalWinId(), x11Info().appRootWindow(x11Info().screen()), 0, 0);
     }
 
 #ifdef QX11EMBED_DEBUG
@@ -791,13 +792,13 @@ bool QX11EmbedWidget::x11Event(XEvent *event)
         qDebug() << "QX11EmbedWidget::x11Event: client"
                  << (void *)this << "with winId" << winId()
                  << "received a ReparentNotify to"
-                 << ((event->xreparent.parent == x11Info().appRootWindow())
+                 << ((event->xreparent.parent == x11Info().appRootWindow(x11Info().screen()))
                      ? QString::fromLatin1("root") : QString::number(event->xreparent.parent));
 #endif
         // If the container shuts down, we will be reparented to the
         // root window. We must also consider the case that we may be
         // reparented from one container to another.
-        if (event->xreparent.parent == x11Info().appRootWindow()) {
+        if (event->xreparent.parent == x11Info().appRootWindow(x11Info().screen())) {
             if (((QHackWidget *)this)->topData()->embedded) {
                 d->container = 0;
                 emit containerClosed();
@@ -833,6 +834,8 @@ bool QX11EmbedWidget::x11Event(XEvent *event)
                         XUnmapWindow(x11Info().display(), internalWinId());
                     }
                 }
+                if (prop_return)
+                    XFree(prop_return);
             }
         }
 
@@ -1070,6 +1073,7 @@ QX11EmbedContainer::QX11EmbedContainer(QWidget *parent)
     d->focusProxy = new QWidget(this);
     d->focusProxy->setAttribute(Qt::WA_NativeWindow);
     d->focusProxy->setAttribute(Qt::WA_DontCreateNativeAncestors);
+    d->focusProxy->createWinId();
     d->focusProxy->setGeometry(-1, -1, 1, 1);
 
     // We need events from the window (activation status) and
@@ -1114,7 +1118,7 @@ QX11EmbedContainer::~QX11EmbedContainer()
     Q_D(QX11EmbedContainer);
     if (d->client) {
 	XUnmapWindow(x11Info().display(), d->client);
-	XReparentWindow(x11Info().display(), d->client, x11Info().appRootWindow(), 0, 0);
+	XReparentWindow(x11Info().display(), d->client, x11Info().appRootWindow(x11Info().screen()), 0, 0);
     }
 
     if (d->xgrab)
@@ -1135,7 +1139,7 @@ void QX11EmbedContainer::paintEvent(QPaintEvent *)
 
 /*! \internal
 
-    Returns wether or not the windows' embedded flag is set.
+    Returns whether or not the windows' embedded flag is set.
 */
 bool QX11EmbedContainerPrivate::isEmbedded() const
 {
@@ -1231,7 +1235,7 @@ void QX11EmbedContainer::embedClient(WId id)
       For safety, we will not wait more than 500 ms, so that we can
       preemptively workaround buggy window managers.
     */
-    QTime t;
+    QElapsedTimer t;
     t.start();
 
     functorData data;
@@ -1378,7 +1382,7 @@ bool QX11EmbedContainer::eventFilter(QObject *o, QEvent *event)
 	    // Wait until the messages have been processed. Then ask
 	    // the window manager to delete the window.
 	    XUnmapWindow(x11Info().display(), d->client);
-	    XReparentWindow(x11Info().display(), d->client, x11Info().appRootWindow(), 0, 0);
+	    XReparentWindow(x11Info().display(), d->client, x11Info().appRootWindow(x11Info().screen()), 0, 0);
 	    XSync(x11Info().display(), false);
 
 	    XEvent ev;
@@ -1626,7 +1630,7 @@ void QX11EmbedContainerPrivate::rejectClient(WId window)
     Q_Q(QX11EmbedContainer);
     q->setEnabled(false);
     XRemoveFromSaveSet(q->x11Info().display(), client);
-    XReparentWindow(q->x11Info().display(), window, q->x11Info().appRootWindow(), 0, 0);
+    XReparentWindow(q->x11Info().display(), window, q->x11Info().appRootWindow(q->x11Info().screen()), 0, 0);
 }
 
 /*! \internal
