@@ -1,40 +1,40 @@
 /****************************************************************************
 **
-** Copyright (C) 2010 Nokia Corporation and/or its subsidiary(-ies).
+** Copyright (C) 2011 Nokia Corporation and/or its subsidiary(-ies).
 ** All rights reserved.
 ** Contact: Nokia Corporation (qt-info@nokia.com)
 **
 ** This file is part of the QtGui module of the Qt Toolkit.
 **
 ** $QT_BEGIN_LICENSE:LGPL$
-** Commercial Usage
-** Licensees holding valid Qt Commercial licenses may use this file in
-** accordance with the Qt Commercial License Agreement provided with the
-** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and Nokia.
-**
 ** GNU Lesser General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU Lesser
-** General Public License version 2.1 as published by the Free Software
-** Foundation and appearing in the file LICENSE.LGPL included in the
-** packaging of this file.  Please review the following information to
-** ensure the GNU Lesser General Public License version 2.1 requirements
-** will be met: http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
+** This file may be used under the terms of the GNU Lesser General Public
+** License version 2.1 as published by the Free Software Foundation and
+** appearing in the file LICENSE.LGPL included in the packaging of this
+** file. Please review the following information to ensure the GNU Lesser
+** General Public License version 2.1 requirements will be met:
+** http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
 **
 ** In addition, as a special exception, Nokia gives you certain additional
-** rights.  These rights are described in the Nokia Qt LGPL Exception
+** rights. These rights are described in the Nokia Qt LGPL Exception
 ** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
 **
 ** GNU General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU
-** General Public License version 3.0 as published by the Free Software
-** Foundation and appearing in the file LICENSE.GPL included in the
-** packaging of this file.  Please review the following information to
-** ensure the GNU General Public License version 3.0 requirements will be
-** met: http://www.gnu.org/copyleft/gpl.html.
+** Alternatively, this file may be used under the terms of the GNU General
+** Public License version 3.0 as published by the Free Software Foundation
+** and appearing in the file LICENSE.GPL included in the packaging of this
+** file. Please review the following information to ensure the GNU General
+** Public License version 3.0 requirements will be met:
+** http://www.gnu.org/copyleft/gpl.html.
+**
+** Other Usage
+** Alternatively, this file may be used in accordance with the terms and
+** conditions contained in a signed written agreement between you and Nokia.
+**
+**
+**
+**
 **
-** If you have questions regarding the use of this file, please contact
-** Nokia at qt-info@nokia.com.
 ** $QT_END_LICENSE$
 **
 ****************************************************************************/
@@ -98,6 +98,21 @@ static inline void qt_flush(QWidget *widget, const QRegion &region, QWindowSurfa
         QWidgetBackingStore::showYellowThing(widget, region, flushUpdate * 10, false);
 #endif
 
+    //The performance hit by doing this should be negligible. However, be aware that
+    //using this FPS when you have > 1 windowsurface can give you inaccurate FPS
+    static bool fpsDebug = qgetenv("QT_DEBUG_FPS").toInt();
+    if (fpsDebug) {
+        static QTime time = QTime::currentTime();
+        static int frames = 0;
+
+        frames++;
+
+        if(time.elapsed() > 5000) {
+            double fps = double(frames * 1000) /time.restart();
+            fprintf(stderr,"FPS: %.1f\n",fps);
+            frames = 0;
+        }
+    }
     if (widget != tlw)
         windowSurface->flush(widget, region, tlwOffset + widget->mapTo(tlw, QPoint()));
     else
@@ -271,7 +286,11 @@ bool QWidgetBackingStore::bltRect(const QRect &rect, int dx, int dy, QWidget *wi
 void QWidgetBackingStore::releaseBuffer()
 {
     if (windowSurface)
+#if defined(Q_WS_QPA)
+        windowSurface->resize(QSize());
+#else
         windowSurface->setGeometry(QRect());
+#endif
 #ifdef Q_BACKINGSTORE_SUBSURFACES
     for (int i = 0; i < subSurfaces.size(); ++i)
         subSurfaces.at(i)->setGeometry(QRect());
@@ -401,7 +420,11 @@ QRegion QWidgetBackingStore::dirtyRegion(QWidget *widget) const
 {
     const bool widgetDirty = widget && widget != tlw;
     const QRect tlwRect(topLevelRect());
+#if defined(Q_WS_QPA)
+    const QRect surfaceGeometry(tlwRect.topLeft(), windowSurface->size());
+#else
     const QRect surfaceGeometry(windowSurface->geometry());
+#endif
     if (fullUpdatePending || (surfaceGeometry != tlwRect && surfaceGeometry.size() != tlwRect.size())) {
         if (widgetDirty) {
             const QRect dirtyTlwRect = QRect(QPoint(), tlwRect.size());
@@ -446,13 +469,17 @@ QRegion QWidgetBackingStore::dirtyRegion(QWidget *widget) const
 
 /*!
     Returns the static content inside the \a parent if non-zero; otherwise the static content
-    for the entire backing store is returned. The content will be clipped to \a withingClipRect
+    for the entire backing store is returned. The content will be clipped to \a withinClipRect
     if non-empty.
 */
 QRegion QWidgetBackingStore::staticContents(QWidget *parent, const QRect &withinClipRect) const
 {
     if (!parent && tlw->testAttribute(Qt::WA_StaticContents)) {
+#if defined(Q_WS_QPA)
+        const QSize surfaceGeometry(windowSurface->size());
+#else
         const QRect surfaceGeometry(windowSurface->geometry());
+#endif
         QRect surfaceRect(0, 0, surfaceGeometry.width(), surfaceGeometry.height());
         if (!withinClipRect.isEmpty())
             surfaceRect &= withinClipRect;
@@ -561,7 +588,7 @@ void QWidgetBackingStore::markDirty(const QRegion &rgn, QWidget *widget, bool up
         return;
     }
 
-    if (!windowSurface->hasPartialUpdateSupport()) {
+    if (!windowSurface->hasFeature(QWindowSurface::PartialUpdates)) {
         fullUpdatePending = true;
         sendUpdateRequest(tlw, updateImmediately);
         return;
@@ -656,7 +683,7 @@ void QWidgetBackingStore::markDirty(const QRect &rect, QWidget *widget, bool upd
         return;
     }
 
-    if (!windowSurface->hasPartialUpdateSupport()) {
+    if (!windowSurface->hasFeature(QWindowSurface::PartialUpdates)) {
         fullUpdatePending = true;
         sendUpdateRequest(tlw, updateImmediately);
         return;
@@ -720,9 +747,8 @@ void QWidgetBackingStore::markDirtyOnScreen(const QRegion &region, QWidget *widg
     }
 
     // Alien widgets.
-    if (!widget->internalWinId()) {
-        QWidget *nativeParent = widget->nativeParentWidget();
-        // Alien widgets with the top-level as the native parent (common case).
+    if (!widget->internalWinId() && !widget->isWindow()) {
+        QWidget *nativeParent = widget->nativeParentWidget();        // Alien widgets with the top-level as the native parent (common case).
         if (nativeParent == tlw) {
             if (!widget->testAttribute(Qt::WA_WState_InPaintEvent))
                 dirtyOnScreen += region.translated(topLevelOffset);
@@ -1117,6 +1143,11 @@ void QWidgetBackingStore::sync(QWidget *exposedWidget, const QRegion &exposedReg
         return;
     }
 
+    // If there's no preserved contents support we always need
+    // to do a full repaint before flushing
+    if (!windowSurface->hasFeature(QWindowSurface::PreservedContents))
+        fullUpdatePending = true;
+
     // Nothing to repaint.
     if (!isDirty()) {
         qt_flush(exposedWidget, exposedRegion, windowSurface, tlw, tlwOffset);
@@ -1152,12 +1183,16 @@ void QWidgetBackingStore::sync()
         return;
     }
 
-    const bool inTopLevelResize = tlwExtra->inTopLevelResize;
     const bool updatesDisabled = !tlw->updatesEnabled();
-    const QRect tlwRect(topLevelRect());
-    const QRect surfaceGeometry(windowSurface->geometry());
     bool repaintAllWidgets = false;
 
+    const bool inTopLevelResize = tlwExtra->inTopLevelResize;
+    const QRect tlwRect(topLevelRect());
+#ifdef  Q_WS_QPA
+    const QRect surfaceGeometry(tlwRect.topLeft(), windowSurface->size());
+#else
+    const QRect surfaceGeometry(windowSurface->geometry());
+#endif
     if ((fullUpdatePending || inTopLevelResize || surfaceGeometry.size() != tlwRect.size()) && !updatesDisabled) {
         if (hasStaticContents()) {
             // Repaint existing dirty area and newly visible area.
@@ -1177,8 +1212,13 @@ void QWidgetBackingStore::sync()
         }
     }
 
+#ifdef Q_WS_QPA
+    if (inTopLevelResize || surfaceGeometry.size() != tlwRect.size())
+        windowSurface->resize(tlwRect.size());
+#else
     if (inTopLevelResize || surfaceGeometry != tlwRect)
         windowSurface->setGeometry(tlwRect);
+#endif
 
     if (updatesDisabled)
         return;
@@ -1585,7 +1625,11 @@ void QWidgetPrivate::repaint_sys(const QRegion &rgn)
         extra->staticContentsSize = data.crect.size();
     }
 
+#ifdef Q_WS_QPA //Don't even call q->p
+    QPaintEngine *engine = 0;
+#else
     QPaintEngine *engine = q->paintEngine();
+#endif
     // QGLWidget does not support partial updates if:
     // 1) The context is double buffered
     // 2) The context is single buffered and auto-fill background is enabled.
