@@ -1,40 +1,40 @@
 /****************************************************************************
 **
-** Copyright (C) 2009 Nokia Corporation and/or its subsidiary(-ies).
+** Copyright (C) 2011 Nokia Corporation and/or its subsidiary(-ies).
 ** All rights reserved.
 ** Contact: Nokia Corporation (qt-info@nokia.com)
 **
 ** This file is part of the QtGui module of the Qt Toolkit.
 **
 ** $QT_BEGIN_LICENSE:LGPL$
-** Commercial Usage
-** Licensees holding valid Qt Commercial licenses may use this file in
-** accordance with the Qt Commercial License Agreement provided with the
-** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and Nokia.
-**
 ** GNU Lesser General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU Lesser
-** General Public License version 2.1 as published by the Free Software
-** Foundation and appearing in the file LICENSE.LGPL included in the
-** packaging of this file.  Please review the following information to
-** ensure the GNU Lesser General Public License version 2.1 requirements
-** will be met: http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
+** This file may be used under the terms of the GNU Lesser General Public
+** License version 2.1 as published by the Free Software Foundation and
+** appearing in the file LICENSE.LGPL included in the packaging of this
+** file. Please review the following information to ensure the GNU Lesser
+** General Public License version 2.1 requirements will be met:
+** http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
 **
 ** In addition, as a special exception, Nokia gives you certain additional
-** rights.  These rights are described in the Nokia Qt LGPL Exception
+** rights. These rights are described in the Nokia Qt LGPL Exception
 ** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
 **
 ** GNU General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU
-** General Public License version 3.0 as published by the Free Software
-** Foundation and appearing in the file LICENSE.GPL included in the
-** packaging of this file.  Please review the following information to
-** ensure the GNU General Public License version 3.0 requirements will be
-** met: http://www.gnu.org/copyleft/gpl.html.
+** Alternatively, this file may be used under the terms of the GNU General
+** Public License version 3.0 as published by the Free Software Foundation
+** and appearing in the file LICENSE.GPL included in the packaging of this
+** file. Please review the following information to ensure the GNU General
+** Public License version 3.0 requirements will be met:
+** http://www.gnu.org/copyleft/gpl.html.
+**
+** Other Usage
+** Alternatively, this file may be used in accordance with the terms and
+** conditions contained in a signed written agreement between you and Nokia.
+**
+**
+**
+**
 **
-** If you have questions regarding the use of this file, please contact
-** Nokia at qt-info@nokia.com.
 ** $QT_END_LICENSE$
 **
 ****************************************************************************/
@@ -80,7 +80,7 @@ static void qt_insertWindowSurface(int winId, QWSWindowSurface *surface)
 
 inline bool isWidgetOpaque(const QWidget *w)
 {
-    return w->d_func()->isOpaque;
+    return w->d_func()->isOpaque && !w->testAttribute(Qt::WA_TranslucentBackground);
 }
 
 static inline QScreen *getScreen(const QWidget *w)
@@ -806,6 +806,10 @@ QWSMemorySurface::QWSMemorySurface(QWidget *w)
 
 QWSMemorySurface::~QWSMemorySurface()
 {
+#ifndef QT_NO_QWS_MULTIPROCESS
+    if (memlock != QWSDisplay::Data::getClientLock())
+        delete memlock;
+#endif
 }
 
 
@@ -852,9 +856,9 @@ void QWSMemorySurface::setLock(int lockId)
 {
     if (memlock && memlock->id() == lockId)
         return;
-    delete memlock;
+    if (memlock != QWSDisplay::Data::getClientLock())
+        delete memlock;
     memlock = (lockId == -1 ? 0 : new QWSLock(lockId));
-    return;
 }
 #endif // QT_NO_QWS_MULTIPROCESS
 
@@ -873,6 +877,25 @@ bool QWSMemorySurface::isValid() const
     return true;
 }
 
+// ### copied from qwindowsurface_raster.cpp -- should be cross-platform
+void QWSMemorySurface::beginPaint(const QRegion &rgn)
+{
+    if (!isWidgetOpaque(window())) {
+        QPainter p(&img);
+        p.setCompositionMode(QPainter::CompositionMode_Source);
+        const QVector<QRect> rects = rgn.rects();
+        const QColor blank = Qt::transparent;
+        for (QVector<QRect>::const_iterator it = rects.begin(); it != rects.end(); ++it) {
+            QRect r = *it;
+#ifdef Q_BACKINGSTORE_SUBSURFACES
+            r.translate(painterOffset());
+#endif
+            p.fillRect(r, blank);
+        }
+    }
+    QWSWindowSurface::beginPaint(rgn);
+}
+
 // from qwindowsurface.cpp
 extern void qt_scrollRectInImage(QImage &img, const QRect &rect, const QPoint &offset);
 
@@ -927,37 +950,39 @@ void QWSLocalMemSurface::setGeometry(const QRect &rect)
     }
 
     uchar *deleteLater = 0;
-    // In case of a Hide event we need to delete the memory after sending the
-    // event to the server in order to let the server animate the event.
-    if (size.isEmpty()) {
-        deleteLater = mem;
-        mem = 0;
-    }
 
     if (img.size() != size) {
-        delete[] mem;
         if (size.isEmpty()) {
+            if (memsize) {
+                // In case of a Hide event we need to delete the memory after sending the
+                // event to the server in order to let the server animate the event.
+                deleteLater = mem;
+                memsize = 0;
+            }
             mem = 0;
             img = QImage();
         } else {
             const QImage::Format format = preferredImageFormat(win);
             const int bpl = nextMulOf4(bytesPerPixel(format) * size.width());
-            const int memsize = bpl * size.height();
-            mem = new uchar[memsize];
+            const int imagesize = bpl * size.height();
+            if (memsize < imagesize) {
+                delete[] mem;
+                memsize = imagesize;
+                mem = new uchar[memsize];
+            }
             img = QImage(mem, size.width(), size.height(), bpl, format);
             setImageMetrics(img, win);
         }
     }
 
     QWSWindowSurface::setGeometry(rect);
+
     delete[] deleteLater;
 }
 
 QByteArray QWSLocalMemSurface::permanentState() const
 {
-    QByteArray array;
-    array.resize(sizeof(uchar*) + 3 * sizeof(int) +
-                 sizeof(SurfaceFlags));
+    QByteArray array(sizeof(uchar*) + 3 * sizeof(int) + sizeof(SurfaceFlags), Qt::Uninitialized);
 
     char *ptr = array.data();
 
@@ -978,6 +1003,11 @@ QByteArray QWSLocalMemSurface::permanentState() const
 
 void QWSLocalMemSurface::setPermanentState(const QByteArray &data)
 {
+    if (memsize) {
+        delete[] mem;
+        memsize = 0;
+    }
+
     int width;
     int height;
     QImage::Format format;
@@ -1004,6 +1034,10 @@ void QWSLocalMemSurface::setPermanentState(const QByteArray &data)
 
 void QWSLocalMemSurface::releaseSurface()
 {
+    if (memsize) {
+        delete[] mem;
+        memsize = 0;
+    }
     mem = 0;
     img = QImage();
 }
@@ -1031,10 +1065,12 @@ bool QWSSharedMemSurface::setMemory(int memId)
         return true;
 
     mem.detach();
-    if (!mem.attach(memId)) {
+
+    if (memId != -1 && !mem.attach(memId)) {
+#ifndef QT_NO_DEBUG
         perror("QWSSharedMemSurface: attaching to shared memory");
-        qCritical("QWSSharedMemSurface: Error attaching to"
-                  " shared memory 0x%x", memId);
+        qCritical("QWSSharedMemSurface: Error attaching to shared memory 0x%x", memId);
+#endif
         return false;
     }
 
@@ -1045,17 +1081,15 @@ bool QWSSharedMemSurface::setMemory(int memId)
 void QWSSharedMemSurface::setDirectRegion(const QRegion &r, int id)
 {
     QWSMemorySurface::setDirectRegion(r, id);
-    if(mem.address())
+    if (mem.address())
         *(uint *)mem.address() = id;
 }
 
 const QRegion QWSSharedMemSurface::directRegion() const
 {
-    QWSSharedMemory *cmem = const_cast<QWSSharedMemory *>(&mem);
-    if (cmem->address() && ((int*)cmem->address())[0] == directRegionId())
+    if (mem.address() && *(uint *)mem.address() == uint(directRegionId()))
         return QWSMemorySurface::directRegion();
-    else
-        return QRegion();
+    return QRegion();
 }
 #endif
 
@@ -1098,8 +1132,6 @@ void QWSSharedMemSurface::setGeometry(const QRect &rect)
             mem.detach();
             img = QImage();
         } else {
-            mem.detach();
-
             QWidget *win = window();
             const QImage::Format format = preferredImageFormat(win);
             const int bpl = nextMulOf4(bytesPerPixel(format) * size.width());
@@ -1108,9 +1140,12 @@ void QWSSharedMemSurface::setGeometry(const QRect &rect)
 #else
             const int imagesize = bpl * size.height();
 #endif
-            if (!mem.create(imagesize)) {
-                perror("QWSSharedMemSurface::setGeometry allocating shared memory");
-                qFatal("Error creating shared memory of size %d", imagesize);
+            if (mem.size() < imagesize) {
+                mem.detach();
+                if (!mem.create(imagesize)) {
+                    perror("QWSSharedMemSurface::setGeometry allocating shared memory");
+                    qFatal("Error creating shared memory of size %d", imagesize);
+                }
             }
 #ifdef QT_QWS_CLIENTBLIT
             *((uint *)mem.address()) = 0;
@@ -1128,8 +1163,7 @@ void QWSSharedMemSurface::setGeometry(const QRect &rect)
 
 QByteArray QWSSharedMemSurface::permanentState() const
 {
-    QByteArray array;
-    array.resize(6 * sizeof(int));
+    QByteArray array(6 * sizeof(int), Qt::Uninitialized);
 
     int *ptr = reinterpret_cast<int*>(array.data());
 
@@ -1203,8 +1237,8 @@ bool QWSOnScreenSurface::isValid() const
 
 QByteArray QWSOnScreenSurface::permanentState() const
 {
-    QByteArray array;
-    array.resize(sizeof(int));
+    QByteArray array(sizeof(int), Qt::Uninitialized);
+
     int *ptr = reinterpret_cast<int*>(array.data());
     ptr[0] = QApplication::desktop()->screenNumber(window());
     return array;
@@ -1244,8 +1278,7 @@ QWSYellowSurface::~QWSYellowSurface()
 
 QByteArray QWSYellowSurface::permanentState() const
 {
-    QByteArray array;
-    array.resize(2 * sizeof(int));
+    QByteArray array(2 * sizeof(int), Qt::Uninitialized);
 
     int *ptr = reinterpret_cast<int*>(array.data());
     ptr[0] = surfaceSize.width();
