@@ -1,40 +1,40 @@
 /****************************************************************************
 **
-** Copyright (C) 2010 Nokia Corporation and/or its subsidiary(-ies).
+** Copyright (C) 2011 Nokia Corporation and/or its subsidiary(-ies).
 ** All rights reserved.
 ** Contact: Nokia Corporation (qt-info@nokia.com)
 **
 ** This file is part of the Symbian application wrapper of the Qt Toolkit.
 **
 ** $QT_BEGIN_LICENSE:LGPL$
-** Commercial Usage
-** Licensees holding valid Qt Commercial licenses may use this file in
-** accordance with the Qt Commercial License Agreement provided with the
-** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and Nokia.
-**
 ** GNU Lesser General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU Lesser
-** General Public License version 2.1 as published by the Free Software
-** Foundation and appearing in the file LICENSE.LGPL included in the
-** packaging of this file.  Please review the following information to
-** ensure the GNU Lesser General Public License version 2.1 requirements
-** will be met: http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
+** This file may be used under the terms of the GNU Lesser General Public
+** License version 2.1 as published by the Free Software Foundation and
+** appearing in the file LICENSE.LGPL included in the packaging of this
+** file. Please review the following information to ensure the GNU Lesser
+** General Public License version 2.1 requirements will be met:
+** http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
 **
 ** In addition, as a special exception, Nokia gives you certain additional
-** rights.  These rights are described in the Nokia Qt LGPL Exception
+** rights. These rights are described in the Nokia Qt LGPL Exception
 ** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
 **
 ** GNU General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU
-** General Public License version 3.0 as published by the Free Software
-** Foundation and appearing in the file LICENSE.GPL included in the
-** packaging of this file.  Please review the following information to
-** ensure the GNU General Public License version 3.0 requirements will be
-** met: http://www.gnu.org/copyleft/gpl.html.
+** Alternatively, this file may be used under the terms of the GNU General
+** Public License version 3.0 as published by the Free Software Foundation
+** and appearing in the file LICENSE.GPL included in the packaging of this
+** file. Please review the following information to ensure the GNU General
+** Public License version 3.0 requirements will be met:
+** http://www.gnu.org/copyleft/gpl.html.
+**
+** Other Usage
+** Alternatively, this file may be used in accordance with the terms and
+** conditions contained in a signed written agreement between you and Nokia.
+**
+**
+**
+**
 **
-** If you have questions regarding the use of this file, please contact
-** Nokia at qt-info@nokia.com.
 ** $QT_END_LICENSE$
 **
 ****************************************************************************/
@@ -49,17 +49,8 @@
 #include <avkon.rsg>
 #endif
 #include <barsread.h>
+#include <coeutils.h>
 #include <qconfig.h>
-#ifdef Q_WS_S60
-#  if defined(QT_LIBINFIX_UNQUOTED)
-// Two level macro needed for proper expansion of libinfix
-#    define QT_S60MAIN_RSG_2(x) <s60main##x##.rsg>
-#    define QT_S60MAIN_RSG(x) QT_S60MAIN_RSG_2(x)
-#    include QT_S60MAIN_RSG(QT_LIBINFIX_UNQUOTED)
-#  else
-#    include <s60main.rsg>
-#  endif
-#endif
 
 #include "qs60mainappui.h"
 #include <QtGui/qapplication.h>
@@ -68,12 +59,28 @@
 #include <private/qmenu_p.h>
 #include <private/qt_s60_p.h>
 #include <qdebug.h>
+#include "qs60keycapture_p.h"
 
 //Animated wallpapers in Qt applications are not supported.
 const TInt KAknDisableAnimationBackground = 0x02000000;
+const TInt KAknSingleClickCompatible = 0x01000000;
 
 QT_BEGIN_NAMESPACE
 
+static QS60KeyCapture *qt_S60KeyCapture = 0;
+
+static void installS60KeyCapture(CCoeEnv *env)
+{
+    if (QApplication::testAttribute(Qt::AA_CaptureMultimediaKeys))
+        qt_S60KeyCapture = new QS60KeyCapture(env);
+}
+
+static void removeS60KeyCapture()
+{
+    delete qt_S60KeyCapture;
+    qt_S60KeyCapture = 0;
+}
+
 /*!
   \class QS60MainAppUi
   \since 4.6
@@ -125,12 +132,17 @@ void QS60MainAppUi::ConstructL()
 #ifdef Q_WS_S60
     flags |= CAknAppUi::EAknEnableSkin;
     // After 5th Edition S60, native side supports animated wallpapers.
-	// However, there is no support for that feature on Qt side, so indicate to
-	// native UI framework that this application will not support background animations.
-    if (QSysInfo::s60Version() > QSysInfo::SV_S60_5_0)
-        flags |= KAknDisableAnimationBackground;
+    // However, there is no support for that feature on Qt side, so indicate to
+    // native UI framework that this application will not support background animations.
+
+    // Also, add support for single touch for post 5th edition platforms.
+    // This has only impact when launching native dialogs/menus from inside QApplication.
+    if (QSysInfo::s60Version() > QSysInfo::SV_S60_5_0) {
+        flags |= (KAknDisableAnimationBackground | KAknSingleClickCompatible);
+    }
 #endif
     BaseConstructL(flags);
+    installS60KeyCapture(iCoeEnv);
 }
 
 /*!
@@ -146,6 +158,7 @@ QS60MainAppUi::QS60MainAppUi()
  */
 QS60MainAppUi::~QS60MainAppUi()
 {
+    removeS60KeyCapture();
 }
 
 /*!
@@ -244,7 +257,7 @@ void QS60MainAppUi::DynInitMenuBarL(TInt /* resourceId */, CEikMenuBar * /* menu
 void QS60MainAppUi::DynInitMenuPaneL(TInt resourceId, CEikMenuPane *menuPane)
 {
 #ifdef Q_WS_S60
-    if (resourceId == R_QT_WRAPPERAPP_MENU) {
+    if (resourceId == R_AVKON_MENUPANE_EMPTY) {
         if (menuPane->NumberOfItemsInPane() <= 1)
             QT_TRYCATCH_LEAVING(qt_symbian_show_toplevel(menuPane));
 
@@ -274,7 +287,17 @@ void QS60MainAppUi::RestoreMenuL(CCoeControl *menuWindow, TInt resourceId, TMenu
             DynInitMenuPaneL(resourceId, (CEikMenuPane*)menuWindow);
         else
             DynInitMenuBarL(resourceId, (CEikMenuBar*)menuWindow);
-    } else
+    } else if(resourceId == R_AVKON_MENUPANE_EMPTY) {
+        CEikMenuBarTitle *title = new(ELeave) CEikMenuBarTitle;
+        CleanupStack::PushL(title);
+
+        title->iData.iMenuPaneResourceId = R_AVKON_MENUPANE_EMPTY;
+        title->iTitleFlags = 0;
+
+        S60->menuBar()->TitleArray()->AddTitleL(title);
+        CleanupStack::Pop( title );
+    }
+    else
 #endif
     {
         QS60MainAppUiBase::RestoreMenuL(menuWindow, resourceId, menuType);
@@ -310,7 +333,21 @@ TRect QS60MainAppUi::ApplicationRect() const
 */
 void QS60MainAppUi::HandleScreenDeviceChangedL()
 {
+    // This function triggers AppUi relayout which also generates
+    // HandleStatusPaneSizeChange(). We don't want to handle
+    // status pane resizes at this point because it causes
+    // Qt window resize and thus EGL surface resize in the middle of
+    // incomplete layout process causing unnecessary overhead.
+    // To prevent status pane resize handling while layout is still
+    // in progress, we guard relayouting with handleStatusPaneResizeNotifications
+    // flag. QSymbianControl checks this flag before doing Qt window
+    // resize due to status pane change.
+    // Eventually when layout is ready, Symbian framework calls
+    // HandleResourceChangeL(KEikDynamicLayoutVariantSwitch) which triggers
+    // resize to Qt window and to its EGL surface.
+    S60->handleStatusPaneResizeNotifications = false;
     QS60MainAppUiBase::HandleScreenDeviceChangedL();
+    S60->handleStatusPaneResizeNotifications = true;
 }
 
 /*!
@@ -401,6 +438,16 @@ void QS60MainAppUi::HandleForegroundEventL(TBool aForeground)
     QS60MainAppUiBase::HandleForegroundEventL(aForeground);
 }
 
+/*!
+  \internal
+*/
+TBool QS60MainAppUi::ProcessCommandParametersL(TApaCommand /*aCommand*/, TFileName &/*aDocumentName*/, const TDesC8 &/*aTail*/)
+{
+    // bypass CEikAppUi::ProcessCommandParametersL(..) which modifies aDocumentName, preventing apparc document opening from working.
+    // The return value is effectively unused in Qt apps (see QS60MainDocument::OpenFileL)
+    return EFalse;
+}
+
 #ifndef Q_WS_S60
 
 void QS60StubAknAppUi::HandleViewDeactivation(const TVwsViewId &, const TVwsViewId &) {}
