@@ -1,46 +1,45 @@
 /****************************************************************************
 **
-** Copyright (C) 2010 Nokia Corporation and/or its subsidiary(-ies).
+** Copyright (C) 2011 Nokia Corporation and/or its subsidiary(-ies).
 ** All rights reserved.
 ** Contact: Nokia Corporation (qt-info@nokia.com)
 **
 ** This file is part of the QtGui module of the Qt Toolkit.
 **
 ** $QT_BEGIN_LICENSE:LGPL$
-** Commercial Usage
-** Licensees holding valid Qt Commercial licenses may use this file in
-** accordance with the Qt Commercial License Agreement provided with the
-** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and Nokia.
-**
 ** GNU Lesser General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU Lesser
-** General Public License version 2.1 as published by the Free Software
-** Foundation and appearing in the file LICENSE.LGPL included in the
-** packaging of this file.  Please review the following information to
-** ensure the GNU Lesser General Public License version 2.1 requirements
-** will be met: http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
+** This file may be used under the terms of the GNU Lesser General Public
+** License version 2.1 as published by the Free Software Foundation and
+** appearing in the file LICENSE.LGPL included in the packaging of this
+** file. Please review the following information to ensure the GNU Lesser
+** General Public License version 2.1 requirements will be met:
+** http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
 **
 ** In addition, as a special exception, Nokia gives you certain additional
-** rights.  These rights are described in the Nokia Qt LGPL Exception
+** rights. These rights are described in the Nokia Qt LGPL Exception
 ** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
 **
 ** GNU General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU
-** General Public License version 3.0 as published by the Free Software
-** Foundation and appearing in the file LICENSE.GPL included in the
-** packaging of this file.  Please review the following information to
-** ensure the GNU General Public License version 3.0 requirements will be
-** met: http://www.gnu.org/copyleft/gpl.html.
+** Alternatively, this file may be used under the terms of the GNU General
+** Public License version 3.0 as published by the Free Software Foundation
+** and appearing in the file LICENSE.GPL included in the packaging of this
+** file. Please review the following information to ensure the GNU General
+** Public License version 3.0 requirements will be met:
+** http://www.gnu.org/copyleft/gpl.html.
+**
+** Other Usage
+** Alternatively, this file may be used in accordance with the terms and
+** conditions contained in a signed written agreement between you and Nokia.
+**
+**
+**
+**
 **
-** If you have questions regarding the use of this file, please contact
-** Nokia at qt-info@nokia.com.
 ** $QT_END_LICENSE$
 **
 ****************************************************************************/
 
 #include "qfontengine_p.h"
-#include <private/qunicodetables_p.h>
 #include <qwsdisplay_qws.h>
 #include <qvarlengtharray.h>
 #include <private/qpainter_p.h>
@@ -51,15 +50,17 @@
 
 #include <qdebug.h>
 
-
 #ifndef QT_NO_QWS_QPF
 
+#include "qplatformdefs.h"
 #include "qfile.h"
-#include "qdir.h"
 
-#define QT_USE_MMAP
 #include <stdlib.h>
 
+#if !defined(Q_OS_INTEGRITY)
+#define QT_USE_MMAP
+#endif
+
 #ifdef QT_USE_MMAP
 // for mmap
 #include <unistd.h>
@@ -69,33 +70,25 @@
 #include <fcntl.h>
 #include <errno.h>
 
-#  if defined(QT_LINUXBASE) && !defined(MAP_FILE)
-     // LSB 3.2 does not define MAP_FILE
-#    define MAP_FILE 0
-#  endif
-
+#ifndef MAP_FILE
+#  define MAP_FILE 0
+#endif
+#ifndef MAP_FAILED
+#  define MAP_FAILED (void *)-1
 #endif
 
-#endif // QT_NO_QWS_QPF
+#endif // QT_USE_MMAP
 
 QT_BEGIN_NAMESPACE
 
-#ifndef QT_NO_QWS_QPF
-QT_BEGIN_INCLUDE_NAMESPACE
-#include "qplatformdefs.h"
-QT_END_INCLUDE_NAMESPACE
-
 static inline unsigned int getChar(const QChar *str, int &i, const int len)
 {
-    unsigned int uc = str[i].unicode();
-    if (uc >= 0xd800 && uc < 0xdc00 && i < len-1) {
-        uint low = str[i+1].unicode();
-       if (low >= 0xdc00 && low < 0xe000) {
-            uc = (uc - 0xd800)*0x400 + (low - 0xdc00) + 0x10000;
-            ++i;
-        }
+    uint ucs4 = str[i].unicode();
+    if (str[i].isHighSurrogate() && i < len-1 && str[i+1].isLowSurrogate()) {
+        ++i;
+        ucs4 = QChar::surrogateToUcs4(ucs4, str[i].unicode());
     }
-    return uc;
+    return ucs4;
 }
 
 #define FM_SMOOTH 1
@@ -161,17 +154,10 @@ public:
     QPFGlyphTree* more;
     QPFGlyph* glyph;
 public:
-#ifdef QT_USE_MMAP
     QPFGlyphTree(uchar*& data)
     {
         read(data);
     }
-#else
-    QPFGlyphTree(QIODevice& f)
-    {
-        read(f);
-    }
-#endif
 
     ~QPFGlyphTree()
     {
@@ -237,7 +223,6 @@ private:
     {
     }
 
-#ifdef QT_USE_MMAP
     void read(uchar*& data)
     {
         // All node data first
@@ -247,19 +232,7 @@ private:
         // Then all video data
         readData(data);
     }
-#else
-    void read(QIODevice& f)
-    {
-        // All node data first
-        readNode(f);
-        // Then all non-video data
-        readMetrics(f);
-        // Then all video data
-        readData(f);
-    }
-#endif
 
-#ifdef QT_USE_MMAP
     void readNode(uchar*& data)
     {
         uchar rw = *data++;
@@ -285,35 +258,7 @@ private:
         if ( more )
             more->readNode(data);
     }
-#else
-    void readNode(QIODevice& f)
-    {
-        uchar rw = f.getch();
-        uchar cl = f.getch();
-        min = (rw << 8) | cl;
-        rw = f.getch();
-        cl = f.getch();
-        max = (rw << 8) | cl;
-        int flags = f.getch();
-        if ( flags & 1 )
-            less = new QPFGlyphTree;
-        else
-            less = 0;
-        if ( flags & 2 )
-            more = new QPFGlyphTree;
-        else
-            more = 0;
-        int n = max-min+1;
-        glyph = new QPFGlyph[n];
 
-        if ( less )
-            less->readNode(f);
-        if ( more )
-            more->readNode(f);
-    }
-#endif
-
-#ifdef QT_USE_MMAP
     void readMetrics(uchar*& data)
     {
         int n = max-min+1;
@@ -326,22 +271,7 @@ private:
         if ( more )
             more->readMetrics(data);
     }
-#else
-    void readMetrics(QIODevice& f)
-    {
-        int n = max-min+1;
-        for (int i=0; i<n; i++) {
-            glyph[i].metrics = new QPFGlyphMetrics;
-            f.readBlock((char*)glyph[i].metrics, sizeof(QPFGlyphMetrics));
-        }
-        if ( less )
-            less->readMetrics(f);
-        if ( more )
-            more->readMetrics(f);
-    }
-#endif
 
-#ifdef QT_USE_MMAP
     void readData(uchar*& data)
     {
         int n = max-min+1;
@@ -356,24 +286,6 @@ private:
         if ( more )
             more->readData(data);
     }
-#else
-    void readData(QIODevice& f)
-    {
-        int n = max-min+1;
-        for (int i=0; i<n; i++) {
-            QSize s( glyph[i].metrics->width, glyph[i].metrics->height );
-            //############### s = qt_screen->mapToDevice( s );
-            uint datasize = glyph[i].metrics->linestep * s.height();
-            glyph[i].data = new uchar[datasize]; // ### deleted?
-            f.readBlock((char*)glyph[i].data, datasize);
-        }
-        if ( less )
-            less->readData(f);
-        if ( more )
-            more->readData(f);
-    }
-#endif
-
 };
 
 class QFontEngineQPF1Data
@@ -381,45 +293,58 @@ class QFontEngineQPF1Data
 public:
     QPFFontMetrics fm;
     QPFGlyphTree *tree;
-    void *mmapStart;
+    uchar *mmapStart;
     size_t mmapLength;
+#ifdef QT_USE_MMAP
+    bool used_mmap;
+#endif
 };
 
-
 QFontEngineQPF1::QFontEngineQPF1(const QFontDef&, const QString &fn)
 {
     cache_cost = 1;
 
-    int f = QT_OPEN( QFile::encodeName(fn), O_RDONLY, 0);
-    Q_ASSERT(f>=0);
+    int fd = QT_OPEN(QFile::encodeName(fn).constData(), O_RDONLY, 0);
+    if (fd == -1)
+        qFatal("Failed to open '%s'", QFile::encodeName(fn).constData());
+
     QT_STATBUF st;
-    if ( QT_FSTAT( f, &st ) )
-        qFatal("Failed to stat %s",QFile::encodeName(fn).data());
-    uchar* data = (uchar*)mmap( 0, // any address
-                                st.st_size, // whole file
-                                PROT_READ, // read-only memory
-#if !defined(Q_OS_SOLARIS) && !defined(Q_OS_QNX4) && !defined(Q_OS_INTEGRITY) && !defined(Q_OS_VXWORKS)
-                                MAP_FILE | MAP_PRIVATE, // swap-backed map from file
-#else
-                                MAP_PRIVATE,
-#endif
-                                f, 0 ); // from offset 0 of f
-#if defined(Q_OS_QNX4) && !defined(MAP_FAILED)
-#define MAP_FAILED ((void *)-1)
-#endif
-    if ( !data || data == (uchar*)MAP_FAILED )
-        qFatal("Failed to mmap %s",QFile::encodeName(fn).data());
-    QT_CLOSE(f);
+    if (QT_FSTAT(fd, &st) != 0)
+        qFatal("Failed to stat '%s'", QFile::encodeName(fn).constData());
 
     d = new QFontEngineQPF1Data;
-    d->mmapStart = data;
+    d->mmapStart = 0;
     d->mmapLength = st.st_size;
-    memcpy(reinterpret_cast<char*>(&d->fm),data,sizeof(d->fm));
 
-    data += sizeof(d->fm);
-    d->tree = new QPFGlyphTree(data);
-    glyphFormat = (d->fm.flags & FM_SMOOTH) ? QFontEngineGlyphCache::Raster_A8
-                  : QFontEngineGlyphCache::Raster_Mono;
+#ifdef QT_USE_MMAP
+    d->used_mmap = true;
+    d->mmapStart = (uchar *)::mmap(0, st.st_size,             // any address, whole file
+                                   PROT_READ,                 // read-only memory
+                                   MAP_FILE | MAP_PRIVATE,    // swap-backed map from file
+                                   fd, 0);                   // from offset 0 of fd
+    if (d->mmapStart == (uchar *)MAP_FAILED)
+        d->mmapStart = 0;
+#endif
+
+    if (!d->mmapStart) {
+#ifdef QT_USE_MMAP
+        d->used_mmap = false;
+#endif
+        d->mmapStart = new uchar[d->mmapLength];
+        if (QT_READ(fd, d->mmapStart, d->mmapLength) != d->mmapLength)
+            qFatal("Failed to read '%s'", QFile::encodeName(fn).constData());
+    }
+    QT_CLOSE(fd);
+
+    if (d->mmapStart) {
+        uchar* data = d->mmapStart;
+
+        memcpy(reinterpret_cast<char*>(&d->fm), data, sizeof(d->fm));
+        data += sizeof(d->fm);
+
+        d->tree = new QPFGlyphTree(data);
+        glyphFormat = (d->fm.flags & FM_SMOOTH) ? QFontEngineGlyphCache::Raster_A8
+                                                : QFontEngineGlyphCache::Raster_Mono;
 #if 0
     qDebug() << "font file" << fn
              << "ascent" << d->fm.ascent << "descent" << d->fm.descent
@@ -431,12 +356,19 @@ QFontEngineQPF1::QFontEngineQPF1(const QFontDef&, const QString &fn)
              << "underlinepos" << d->fm.underlinepos
              << "underlinewidth" << d->fm.underlinewidth;
 #endif
+    }
 }
 
 QFontEngineQPF1::~QFontEngineQPF1()
 {
-    if (d->mmapStart)
-        munmap(d->mmapStart, d->mmapLength);
+    if (d->mmapStart) {
+#if defined(QT_USE_MMAP)
+        if (d->used_mmap)
+            ::munmap(d->mmapStart, d->mmapLength);
+        else
+#endif
+            delete [] d->mmapStart;
+    }
     delete d->tree;
     delete d;
 }
