@@ -1,40 +1,40 @@
 /****************************************************************************
 **
-** Copyright (C) 2009 Nokia Corporation and/or its subsidiary(-ies).
+** Copyright (C) 2011 Nokia Corporation and/or its subsidiary(-ies).
 ** All rights reserved.
 ** Contact: Nokia Corporation (qt-info@nokia.com)
 **
 ** This file is part of the QtGui module of the Qt Toolkit.
 **
 ** $QT_BEGIN_LICENSE:LGPL$
-** Commercial Usage
-** Licensees holding valid Qt Commercial licenses may use this file in
-** accordance with the Qt Commercial License Agreement provided with the
-** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and Nokia.
-**
 ** GNU Lesser General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU Lesser
-** General Public License version 2.1 as published by the Free Software
-** Foundation and appearing in the file LICENSE.LGPL included in the
-** packaging of this file.  Please review the following information to
-** ensure the GNU Lesser General Public License version 2.1 requirements
-** will be met: http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
+** This file may be used under the terms of the GNU Lesser General Public
+** License version 2.1 as published by the Free Software Foundation and
+** appearing in the file LICENSE.LGPL included in the packaging of this
+** file. Please review the following information to ensure the GNU Lesser
+** General Public License version 2.1 requirements will be met:
+** http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
 **
 ** In addition, as a special exception, Nokia gives you certain additional
-** rights.  These rights are described in the Nokia Qt LGPL Exception
+** rights. These rights are described in the Nokia Qt LGPL Exception
 ** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
 **
 ** GNU General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU
-** General Public License version 3.0 as published by the Free Software
-** Foundation and appearing in the file LICENSE.GPL included in the
-** packaging of this file.  Please review the following information to
-** ensure the GNU General Public License version 3.0 requirements will be
-** met: http://www.gnu.org/copyleft/gpl.html.
+** Alternatively, this file may be used under the terms of the GNU General
+** Public License version 3.0 as published by the Free Software Foundation
+** and appearing in the file LICENSE.GPL included in the packaging of this
+** file. Please review the following information to ensure the GNU General
+** Public License version 3.0 requirements will be met:
+** http://www.gnu.org/copyleft/gpl.html.
+**
+** Other Usage
+** Alternatively, this file may be used in accordance with the terms and
+** conditions contained in a signed written agreement between you and Nokia.
+**
+**
+**
+**
 **
-** If you have questions regarding the use of this file, please contact
-** Nokia at qt-info@nokia.com.
 ** $QT_END_LICENSE$
 **
 ****************************************************************************/
@@ -616,6 +616,7 @@ void QTextFramePrivate::remove_me()
 
 /*!
     Returns an iterator pointing to the first document element inside the frame.
+    Please see the document \l{STL-style-Iterators} for more information.
 
     \sa end()
 */
@@ -628,8 +629,8 @@ QTextFrame::iterator QTextFrame::begin() const
 }
 
 /*!
-    Returns an iterator pointing to the last document element inside the frame.
-
+    Returns an iterator pointing to the position past the last document element inside the frame.
+    Please see the document \l{STL-Style Iterators} for more information.    
     \sa begin()
 */
 QTextFrame::iterator QTextFrame::end() const
@@ -1139,6 +1140,53 @@ int QTextBlock::charFormatIndex() const
     return p->blockCharFormatIndex(n);
 }
 
+/*!
+  \since 4.7
+
+  Returns the resolved text direction.
+
+  If the block has no explicit direction set, it will resolve the
+  direction from the blocks content. Returns either Qt::LeftToRight
+  or Qt::RightToLeft.
+
+  \sa QTextFormat::layoutDirection(), QString::isRightToLeft(), Qt::LayoutDirection
+*/
+Qt::LayoutDirection QTextBlock::textDirection() const
+{
+    Qt::LayoutDirection dir = blockFormat().layoutDirection();
+    if (dir != Qt::LayoutDirectionAuto)
+        return dir;
+
+    dir = p->defaultTextOption.textDirection();
+    if (dir != Qt::LayoutDirectionAuto)
+        return dir;
+
+    const QString buffer = p->buffer();
+
+    const int pos = position();
+    QTextDocumentPrivate::FragmentIterator it = p->find(pos);
+    QTextDocumentPrivate::FragmentIterator end = p->find(pos + length() - 1); // -1 to omit the block separator char
+    for (; it != end; ++it) {
+        const QTextFragmentData * const frag = it.value();
+        const QChar *p = buffer.constData() + frag->stringPosition;
+        const QChar * const end = p + frag->size_array[0];
+        while (p < end) {
+            switch(QChar::direction(p->unicode()))
+            {
+            case QChar::DirL:
+                return Qt::LeftToRight;
+            case QChar::DirR:
+            case QChar::DirAL:
+                return Qt::RightToLeft;
+            default:
+                break;
+            }
+            ++p;
+        }
+    }
+    return Qt::LeftToRight;
+}
+
 /*!
     Returns the block's contents as plain text.
 
@@ -1440,7 +1488,7 @@ QTextBlock::iterator QTextBlock::end() const
 */
 QTextBlock QTextBlock::next() const
 {
-    if (!p)
+    if (!isValid() || !p->blockMap().isValid(n))
         return QTextBlock();
 
     return QTextBlock(p, p->blockMap().next(n));
@@ -1607,6 +1655,37 @@ QTextBlock::iterator &QTextBlock::iterator::operator--()
     than the \a other text fragment; otherwise returns false.
 */
 
+/*!
+    Returns the glyphs of this text fragment. The positions of the glyphs are
+    relative to the position of the QTextBlock's layout.
+
+    \sa QGlyphRun, QTextBlock::layout(), QTextLayout::position(), QPainter::drawGlyphRun()
+*/
+#if !defined(QT_NO_RAWFONT)
+QList<QGlyphRun> QTextFragment::glyphRuns() const
+{
+    if (!p || !n)
+        return QList<QGlyphRun>();
+
+    int pos = position();
+    int len = length();
+    if (len == 0)
+        return QList<QGlyphRun>();
+
+    int blockNode = p->blockMap().findNode(pos);
+
+    const QTextBlockData *blockData = p->blockMap().fragment(blockNode);
+    QTextLayout *layout = blockData->layout;
+
+    QList<QGlyphRun> ret;
+    for (int i=0; i<layout->lineCount(); ++i) {
+        QTextLine textLine = layout->lineAt(i);
+        ret += textLine.glyphs(pos, len);
+    }
+
+    return ret;
+}
+#endif // QT_NO_RAWFONT
 
 /*!
     Returns the position of this text fragment in the document.
