@@ -1,40 +1,40 @@
 /****************************************************************************
 **
-** Copyright (C) 2009 Nokia Corporation and/or its subsidiary(-ies).
+** Copyright (C) 2011 Nokia Corporation and/or its subsidiary(-ies).
 ** All rights reserved.
 ** Contact: Nokia Corporation (qt-info@nokia.com)
 **
 ** This file is part of the QtGui module of the Qt Toolkit.
 **
 ** $QT_BEGIN_LICENSE:LGPL$
-** Commercial Usage
-** Licensees holding valid Qt Commercial licenses may use this file in
-** accordance with the Qt Commercial License Agreement provided with the
-** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and Nokia.
-**
 ** GNU Lesser General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU Lesser
-** General Public License version 2.1 as published by the Free Software
-** Foundation and appearing in the file LICENSE.LGPL included in the
-** packaging of this file.  Please review the following information to
-** ensure the GNU Lesser General Public License version 2.1 requirements
-** will be met: http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
+** This file may be used under the terms of the GNU Lesser General Public
+** License version 2.1 as published by the Free Software Foundation and
+** appearing in the file LICENSE.LGPL included in the packaging of this
+** file. Please review the following information to ensure the GNU Lesser
+** General Public License version 2.1 requirements will be met:
+** http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
 **
 ** In addition, as a special exception, Nokia gives you certain additional
-** rights.  These rights are described in the Nokia Qt LGPL Exception
+** rights. These rights are described in the Nokia Qt LGPL Exception
 ** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
 **
 ** GNU General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU
-** General Public License version 3.0 as published by the Free Software
-** Foundation and appearing in the file LICENSE.GPL included in the
-** packaging of this file.  Please review the following information to
-** ensure the GNU General Public License version 3.0 requirements will be
-** met: http://www.gnu.org/copyleft/gpl.html.
+** Alternatively, this file may be used under the terms of the GNU General
+** Public License version 3.0 as published by the Free Software Foundation
+** and appearing in the file LICENSE.GPL included in the packaging of this
+** file. Please review the following information to ensure the GNU General
+** Public License version 3.0 requirements will be met:
+** http://www.gnu.org/copyleft/gpl.html.
+**
+** Other Usage
+** Alternatively, this file may be used in accordance with the terms and
+** conditions contained in a signed written agreement between you and Nokia.
+**
+**
+**
+**
 **
-** If you have questions regarding the use of this file, please contact
-** Nokia at qt-info@nokia.com.
 ** $QT_END_LICENSE$
 **
 ****************************************************************************/
@@ -124,6 +124,7 @@ public:
         manifestWriter.writeNamespace(manifestNS, QString::fromLatin1("manifest"));
         manifestWriter.writeStartDocument();
         manifestWriter.writeStartElement(manifestNS, QString::fromLatin1("manifest"));
+        manifestWriter.writeAttribute(manifestNS, QString::fromLatin1("version"), QString::fromLatin1("1.2"));
         addFile(QString::fromLatin1("/"), QString::fromLatin1("application/vnd.oasis.opendocument.text"));
         addFile(QString::fromLatin1("content.xml"), QString::fromLatin1("text/xml"));
     }
@@ -447,18 +448,19 @@ void QTextOdfWriter::writeBlockFormat(QXmlStreamWriter &writer, QTextBlockFormat
     writer.writeStartElement(styleNS, QString::fromLatin1("paragraph-properties"));
 
     if (format.hasProperty(QTextFormat::BlockAlignment)) {
+        const Qt::Alignment alignment = format.alignment() & Qt::AlignHorizontal_Mask;
         QString value;
-        if (format.alignment() == Qt::AlignLeading)
+        if (alignment == Qt::AlignLeading)
             value = QString::fromLatin1("start");
-        else if (format.alignment() == Qt::AlignTrailing)
+        else if (alignment == Qt::AlignTrailing)
             value = QString::fromLatin1("end");
-        else if (format.alignment() == (Qt::AlignLeft | Qt::AlignAbsolute))
+        else if (alignment == (Qt::AlignLeft | Qt::AlignAbsolute))
             value = QString::fromLatin1("left");
-        else if (format.alignment() == (Qt::AlignRight | Qt::AlignAbsolute))
+        else if (alignment == (Qt::AlignRight | Qt::AlignAbsolute))
             value = QString::fromLatin1("right");
-        else if (format.alignment() == Qt::AlignHCenter)
+        else if (alignment == Qt::AlignHCenter)
             value = QString::fromLatin1("center");
-        else if (format.alignment() == Qt::AlignJustify)
+        else if (alignment == Qt::AlignJustify)
             value = QString::fromLatin1("justify");
         else
             qWarning() << "QTextOdfWriter: unsupported paragraph alignment; " << format.alignment();
@@ -483,15 +485,19 @@ void QTextOdfWriter::writeBlockFormat(QXmlStreamWriter &writer, QTextBlockFormat
         if (format.pageBreakPolicy() & QTextFormat::PageBreak_AlwaysAfter)
             writer.writeAttribute(foNS, QString::fromLatin1("break-after"), QString::fromLatin1("page"));
     }
+    if (format.hasProperty(QTextFormat::BackgroundBrush)) {
+        QBrush brush = format.background();
+        writer.writeAttribute(foNS, QString::fromLatin1("background-color"), brush.color().name());
+    }
     if (format.hasProperty(QTextFormat::BlockNonBreakableLines))
         writer.writeAttribute(foNS, QString::fromLatin1("keep-together"),
                 format.nonBreakableLines() ? QString::fromLatin1("true") : QString::fromLatin1("false"));
     if (format.hasProperty(QTextFormat::TabPositions)) {
         QList<QTextOption::Tab> tabs = format.tabPositions();
-        writer.writeStartElement(styleNS, QString::fromLatin1("style-tab-stops"));
+        writer.writeStartElement(styleNS, QString::fromLatin1("tab-stops"));
         QList<QTextOption::Tab>::Iterator iterator = tabs.begin();
         while(iterator != tabs.end()) {
-            writer.writeEmptyElement(styleNS, QString::fromLatin1("style-tab-stop"));
+            writer.writeEmptyElement(styleNS, QString::fromLatin1("tab-stop"));
             writer.writeAttribute(styleNS, QString::fromLatin1("position"), pixelToPoint(iterator->position) );
             QString type;
             switch(iterator->type) {
@@ -506,7 +512,7 @@ void QTextOdfWriter::writeBlockFormat(QXmlStreamWriter &writer, QTextBlockFormat
             ++iterator;
         }
 
-        writer.writeEndElement(); // style-tab-stops
+        writer.writeEndElement(); // tab-stops
     }
 
     writer.writeEndElement(); // paragraph-properties
@@ -551,8 +557,8 @@ void QTextOdfWriter::writeCharacterFormat(QXmlStreamWriter &writer, QTextCharFor
     }
     if (format.hasProperty(QTextFormat::FontLetterSpacing))
         writer.writeAttribute(foNS, QString::fromLatin1("letter-spacing"), pixelToPoint(format.fontLetterSpacing()));
-    if (format.hasProperty(QTextFormat::FontWordSpacing))
-        writer.writeAttribute(foNS, QString::fromLatin1("word-spacing"), pixelToPoint(format.fontWordSpacing()));
+    if (format.hasProperty(QTextFormat::FontWordSpacing) && format.fontWordSpacing() != 0)
+            writer.writeAttribute(foNS, QString::fromLatin1("word-spacing"), pixelToPoint(format.fontWordSpacing()));
     if (format.hasProperty(QTextFormat::FontUnderline))
         writer.writeAttribute(styleNS, QString::fromLatin1("text-underline-type"),
                 format.fontUnderline() ? QString::fromLatin1("single") : QString::fromLatin1("none"));
@@ -585,6 +591,7 @@ void QTextOdfWriter::writeCharacterFormat(QXmlStreamWriter &writer, QTextCharFor
         QString value;
         switch (format.verticalAlignment()) {
         case QTextCharFormat::AlignMiddle:
+        case QTextCharFormat::AlignBaseline:
         case QTextCharFormat::AlignNormal: value = QString::fromLatin1("0%"); break;
         case QTextCharFormat::AlignSuperScript: value = QString::fromLatin1("super"); break;
         case QTextCharFormat::AlignSubScript: value = QString::fromLatin1("sub"); break;
@@ -609,9 +616,12 @@ void QTextOdfWriter::writeCharacterFormat(QXmlStreamWriter &writer, QTextCharFor
     }
     if (format.hasProperty(QTextFormat::ForegroundBrush)) {
         QBrush brush = format.foreground();
-        // TODO
         writer.writeAttribute(foNS, QString::fromLatin1("color"), brush.color().name());
     }
+    if (format.hasProperty(QTextFormat::BackgroundBrush)) {
+        QBrush brush = format.background();
+        writer.writeAttribute(foNS, QString::fromLatin1("background-color"), brush.color().name());
+    }
 
     writer.writeEndElement(); // style
 }
@@ -628,7 +638,15 @@ void QTextOdfWriter::writeListFormat(QXmlStreamWriter &writer, QTextListFormat f
             || style == QTextListFormat::ListUpperRoman) {
         writer.writeStartElement(textNS, QString::fromLatin1("list-level-style-number"));
         writer.writeAttribute(styleNS, QString::fromLatin1("num-format"), bulletChar(style));
-        writer.writeAttribute(styleNS, QString::fromLatin1("num-suffix"), QString::fromLatin1("."));
+
+        if (format.hasProperty(QTextFormat::ListNumberSuffix))
+            writer.writeAttribute(styleNS, QString::fromLatin1("num-suffix"), format.numberSuffix());
+        else
+            writer.writeAttribute(styleNS, QString::fromLatin1("num-suffix"), QString::fromLatin1("."));
+
+        if (format.hasProperty(QTextFormat::ListNumberPrefix))
+            writer.writeAttribute(styleNS, QString::fromLatin1("num-prefix"), format.numberPrefix());
+
     } else {
         writer.writeStartElement(textNS, QString::fromLatin1("list-level-style-bullet"));
         writer.writeAttribute(textNS, QString::fromLatin1("bullet-char"), bulletChar(style));
@@ -690,11 +708,11 @@ void QTextOdfWriter::writeTableCellFormat(QXmlStreamWriter &writer, QTextTableCe
         if (padding > 0)
             writer.writeAttribute(foNS, QString::fromLatin1("padding-top"), pixelToPoint(padding));
         if (format.bottomPadding() > 0)
-            writer.writeAttribute(foNS, QString::fromLatin1("padding-top"), pixelToPoint(format.bottomPadding()));
+            writer.writeAttribute(foNS, QString::fromLatin1("padding-bottom"), pixelToPoint(format.bottomPadding()));
         if (format.leftPadding() > 0)
-            writer.writeAttribute(foNS, QString::fromLatin1("padding-top"), pixelToPoint(format.leftPadding()));
+            writer.writeAttribute(foNS, QString::fromLatin1("padding-left"), pixelToPoint(format.leftPadding()));
         if (format.rightPadding() > 0)
-            writer.writeAttribute(foNS, QString::fromLatin1("padding-top"), pixelToPoint(format.rightPadding()));
+            writer.writeAttribute(foNS, QString::fromLatin1("padding-right"), pixelToPoint(format.rightPadding()));
     }
 
     if (format.hasProperty(QTextFormat::TextVerticalAlignment)) {
@@ -770,6 +788,7 @@ bool QTextOdfWriter::writeAll()
     writer.writeNamespace(svgNS, QString::fromLatin1("svg"));
     writer.writeStartDocument();
     writer.writeStartElement(officeNS, QString::fromLatin1("document-content"));
+    writer.writeAttribute(officeNS, QString::fromLatin1("version"), QString::fromLatin1("1.2"));
 
     // add fragments. (for character formats)
     QTextDocumentPrivate::FragmentIterator fragIt = m_document->docHandle()->begin();
