@@ -1,40 +1,40 @@
 /****************************************************************************
 **
-** Copyright (C) 2009 Nokia Corporation and/or its subsidiary(-ies).
+** Copyright (C) 2011 Nokia Corporation and/or its subsidiary(-ies).
 ** All rights reserved.
 ** Contact: Nokia Corporation (qt-info@nokia.com)
 **
 ** This file is part of the QtGui module of the Qt Toolkit.
 **
 ** $QT_BEGIN_LICENSE:LGPL$
-** Commercial Usage
-** Licensees holding valid Qt Commercial licenses may use this file in
-** accordance with the Qt Commercial License Agreement provided with the
-** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and Nokia.
-**
 ** GNU Lesser General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU Lesser
-** General Public License version 2.1 as published by the Free Software
-** Foundation and appearing in the file LICENSE.LGPL included in the
-** packaging of this file.  Please review the following information to
-** ensure the GNU Lesser General Public License version 2.1 requirements
-** will be met: http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
+** This file may be used under the terms of the GNU Lesser General Public
+** License version 2.1 as published by the Free Software Foundation and
+** appearing in the file LICENSE.LGPL included in the packaging of this
+** file. Please review the following information to ensure the GNU Lesser
+** General Public License version 2.1 requirements will be met:
+** http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
 **
 ** In addition, as a special exception, Nokia gives you certain additional
-** rights.  These rights are described in the Nokia Qt LGPL Exception
+** rights. These rights are described in the Nokia Qt LGPL Exception
 ** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
 **
 ** GNU General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU
-** General Public License version 3.0 as published by the Free Software
-** Foundation and appearing in the file LICENSE.GPL included in the
-** packaging of this file.  Please review the following information to
-** ensure the GNU General Public License version 3.0 requirements will be
-** met: http://www.gnu.org/copyleft/gpl.html.
+** Alternatively, this file may be used under the terms of the GNU General
+** Public License version 3.0 as published by the Free Software Foundation
+** and appearing in the file LICENSE.GPL included in the packaging of this
+** file. Please review the following information to ensure the GNU General
+** Public License version 3.0 requirements will be met:
+** http://www.gnu.org/copyleft/gpl.html.
+**
+** Other Usage
+** Alternatively, this file may be used in accordance with the terms and
+** conditions contained in a signed written agreement between you and Nokia.
+**
+**
+**
+**
 **
-** If you have questions regarding the use of this file, please contact
-** Nokia at qt-info@nokia.com.
 ** $QT_END_LICENSE$
 **
 ****************************************************************************/
@@ -46,6 +46,7 @@
 #include <QtGui/qdialog.h>
 #include <QtGui/qapplication.h>
 #include <QtGui/private/qwidget_p.h>
+#include <QtGui/qaction.h>
 
 #include "qdialogbuttonbox.h"
 
@@ -93,6 +94,7 @@ QT_BEGIN_NAMESPACE
     specifying their role.
 
     \snippet examples/dialogs/extension/finddialog.cpp 1
+    \snippet examples/dialogs/extension/finddialog.cpp 6
 
     Alternatively, QDialogButtonBox provides several standard buttons (e.g. OK, Cancel, Save)
     that you can use. They exist as flags so you can OR them together in the constructor.
@@ -102,7 +104,7 @@ QT_BEGIN_NAMESPACE
     You can mix and match normal buttons and standard buttons.
 
     Currently the buttons are laid out in the following way if the button box is horizontal:
-    \table 100%
+    \table
     \row \o \inlineimage buttonbox-gnomelayout-horizontal.png GnomeLayout Horizontal
          \o Button box laid out in horizontal GnomeLayout
     \row \o \inlineimage buttonbox-kdelayout-horizontal.png KdeLayout Horizontal
@@ -115,25 +117,23 @@ QT_BEGIN_NAMESPACE
 
     The buttons are laid out the following way if the button box is vertical:
 
-    \table 100%
+    \table
+    \row \o GnomeLayout
+         \o KdeLayout
+         \o MacLayout
+         \o WinLayout
     \row \o \inlineimage buttonbox-gnomelayout-vertical.png GnomeLayout Vertical
-         \o Button box laid out in vertical GnomeLayout
-    \row \o \inlineimage buttonbox-kdelayout-vertical.png KdeLayout Vertical
-         \o Button box laid out in vertical KdeLayout
-    \row \o \inlineimage buttonbox-maclayout-vertical.png MacLayout Vertical
-         \o Button box laid out in vertical MacLayout
-    \row \o \inlineimage buttonbox-winlayout-vertical.png WinLayout Vertical
-         \o Button box laid out in vertical WinLayout
+         \o \inlineimage buttonbox-kdelayout-vertical.png KdeLayout Vertical
+         \o \inlineimage buttonbox-maclayout-vertical.png MacLayout Vertical
+         \o \inlineimage buttonbox-winlayout-vertical.png WinLayout Vertical
     \endtable
 
     Additionally, button boxes that contain only buttons with ActionRole or
-    HelpRole can be considered modeless and have an alternate look on the mac:
+    HelpRole can be considered modeless and have an alternate look on Mac OS X:
 
-    \table 100%
-    \row \o \inlineimage buttonbox-mac-modeless-horizontal.png Screenshot of modeless horizontal MacLayout
-         \o modeless horizontal MacLayout
-    \row \o \inlineimage buttonbox-mac-modeless-vertical.png Screenshot of modeless vertical MacLayout
-         \o modeless vertical MacLayout
+    \table
+    \row \o modeless horizontal MacLayout
+         \o \inlineimage buttonbox-mac-modeless-horizontal.png Screenshot of modeless horizontal MacLayout
     \endtable
 
     When a button is clicked in the button box, the clicked() signal is emitted
@@ -259,6 +259,32 @@ static const int layouts[2][5][14] =
     }
 };
 
+#if defined(QT_SOFTKEYS_ENABLED) && !defined(QT_NO_ACTION)
+class QDialogButtonEnabledProxy : public QObject
+{
+public:
+    QDialogButtonEnabledProxy(QObject *parent, QWidget *src, QAction *trg) : QObject(parent), source(src), target(trg)
+    {
+        source->installEventFilter(this);
+        target->setEnabled(source->isEnabled());
+    }
+    ~QDialogButtonEnabledProxy()
+    {
+        source->removeEventFilter(this);
+    }
+    bool eventFilter(QObject *object, QEvent *event)
+    {
+        if (object == source && event->type() == QEvent::EnabledChange) {
+            target->setEnabled(source->isEnabled());
+        }
+        return false;
+    };
+private:
+    QWidget *source;
+    QAction *target;
+};
+#endif
+
 class QDialogButtonBoxPrivate : public QWidgetPrivate
 {
     Q_DECLARE_PUBLIC(QDialogButtonBox)
@@ -290,7 +316,7 @@ public:
     void addButtonsToLayout(const QList<QAbstractButton *> &buttonList, bool reverse);
     void retranslateStrings();
     const char *standardButtonText(QDialogButtonBox::StandardButton sbutton) const;
-#ifdef QT_SOFTKEYS_ENABLED
+#if defined(QT_SOFTKEYS_ENABLED) && !defined(QT_NO_ACTION)
     QAction *createSoftKey(QAbstractButton *button, QDialogButtonBox::ButtonRole role);
 #endif
 };
@@ -537,6 +563,14 @@ QPushButton *QDialogButtonBoxPrivate::createButton(QDialogButtonBox::StandardBut
     } else {
         qWarning("QDialogButtonBox::createButton: Invalid ButtonRole, button not added");
     }
+
+#ifdef Q_WS_MAC
+    // Since mnemonics is off by default on Mac, we add a Cmd-D
+    // shortcut here to e.g. make the "Don't Save" button work nativly:
+    if (sbutton == QDialogButtonBox::Discard)
+        button->setShortcut(QKeySequence(QLatin1String("Ctrl+D")));
+#endif
+
     return button;
 }
 
@@ -547,14 +581,16 @@ void QDialogButtonBoxPrivate::addButton(QAbstractButton *button, QDialogButtonBo
     QObject::connect(button, SIGNAL(clicked()), q, SLOT(_q_handleButtonClicked()));
     QObject::connect(button, SIGNAL(destroyed()), q, SLOT(_q_handleButtonDestroyed()));
     buttonLists[role].append(button);
-#ifdef QT_SOFTKEYS_ENABLED
-    softKeyActions.insert(button, createSoftKey(button, role));
+#if defined(QT_SOFTKEYS_ENABLED) && !defined(QT_NO_ACTION)
+    QAction *action = createSoftKey(button, role);
+    softKeyActions.insert(button, action);
+    new QDialogButtonEnabledProxy(action, button, action);
 #endif
     if (doLayout)
         layoutButtons();
 }
 
-#ifdef QT_SOFTKEYS_ENABLED
+#if defined(QT_SOFTKEYS_ENABLED) && !defined(QT_NO_ACTION)
 QAction* QDialogButtonBoxPrivate::createSoftKey(QAbstractButton *button, QDialogButtonBox::ButtonRole role)
 {
     Q_Q(QDialogButtonBox);
@@ -692,7 +728,7 @@ void QDialogButtonBoxPrivate::retranslateStrings()
         if (buttonText) {
             QPushButton *button = it.key();
             button->setText(QDialogButtonBox::tr(buttonText));
-#ifdef QT_SOFTKEYS_ENABLED
+#if defined(QT_SOFTKEYS_ENABLED) && !defined(QT_NO_ACTION)
             QAction *action = softKeyActions.value(button, 0);
             if (action)
                 action->setText(button->text());
@@ -971,7 +1007,7 @@ void QDialogButtonBox::removeButton(QAbstractButton *button)
             }
         }
     }
-#ifdef QT_SOFTKEYS_ENABLED
+#if defined(QT_SOFTKEYS_ENABLED) && !defined(QT_NO_ACTION)
     QAction *action = d->softKeyActions.value(button, 0);
     if (action) {
         d->softKeyActions.remove(button);
@@ -989,6 +1025,8 @@ void QDialogButtonBox::removeButton(QAbstractButton *button)
     If the button has already been added, it is removed and added again with the
     new role.
 
+    \note The button box takes ownership of the button.
+
     \sa removeButton(), clear()
 */
 void QDialogButtonBox::addButton(QAbstractButton *button, ButtonRole role)
@@ -1200,7 +1238,7 @@ bool QDialogButtonBox::event(QEvent *event)
                 break;
         }
 
-        foreach (QPushButton *pb, qFindChildren<QPushButton *>(dialog ? dialog : this)) {
+        foreach (QPushButton *pb, (dialog ? dialog : this)->findChildren<QPushButton *>()) {
             if (pb->isDefault() && pb != firstAcceptButton) {
                 hasDefault = true;
                 break;
@@ -1215,7 +1253,7 @@ bool QDialogButtonBox::event(QEvent *event)
     }else if (event->type() == QEvent::LanguageChange) {
         d->retranslateStrings();
     }
-#ifdef QT_SOFTKEYS_ENABLED
+#if defined(QT_SOFTKEYS_ENABLED) && !defined(QT_NO_ACTION)
     else if (event->type() == QEvent::ParentChange) {
         QWidget *dialog = 0;
         QWidget *p = this;
