@@ -1,40 +1,40 @@
 /****************************************************************************
 **
-** Copyright (C) 2009 Nokia Corporation and/or its subsidiary(-ies).
+** Copyright (C) 2011 Nokia Corporation and/or its subsidiary(-ies).
 ** All rights reserved.
 ** Contact: Nokia Corporation (qt-info@nokia.com)
 **
 ** This file is part of the QtGui module of the Qt Toolkit.
 **
 ** $QT_BEGIN_LICENSE:LGPL$
-** Commercial Usage
-** Licensees holding valid Qt Commercial licenses may use this file in
-** accordance with the Qt Commercial License Agreement provided with the
-** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and Nokia.
-**
 ** GNU Lesser General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU Lesser
-** General Public License version 2.1 as published by the Free Software
-** Foundation and appearing in the file LICENSE.LGPL included in the
-** packaging of this file.  Please review the following information to
-** ensure the GNU Lesser General Public License version 2.1 requirements
-** will be met: http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
+** This file may be used under the terms of the GNU Lesser General Public
+** License version 2.1 as published by the Free Software Foundation and
+** appearing in the file LICENSE.LGPL included in the packaging of this
+** file. Please review the following information to ensure the GNU Lesser
+** General Public License version 2.1 requirements will be met:
+** http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
 **
 ** In addition, as a special exception, Nokia gives you certain additional
-** rights.  These rights are described in the Nokia Qt LGPL Exception
+** rights. These rights are described in the Nokia Qt LGPL Exception
 ** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
 **
 ** GNU General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU
-** General Public License version 3.0 as published by the Free Software
-** Foundation and appearing in the file LICENSE.GPL included in the
-** packaging of this file.  Please review the following information to
-** ensure the GNU General Public License version 3.0 requirements will be
-** met: http://www.gnu.org/copyleft/gpl.html.
+** Alternatively, this file may be used under the terms of the GNU General
+** Public License version 3.0 as published by the Free Software Foundation
+** and appearing in the file LICENSE.GPL included in the packaging of this
+** file. Please review the following information to ensure the GNU General
+** Public License version 3.0 requirements will be met:
+** http://www.gnu.org/copyleft/gpl.html.
+**
+** Other Usage
+** Alternatively, this file may be used in accordance with the terms and
+** conditions contained in a signed written agreement between you and Nokia.
+**
+**
+**
+**
 **
-** If you have questions regarding the use of this file, please contact
-** Nokia at qt-info@nokia.com.
 ** $QT_END_LICENSE$
 **
 ****************************************************************************/
@@ -59,13 +59,29 @@
 
 QT_BEGIN_NAMESPACE
 
+#ifdef QT_GUI_PASSWORD_ECHO_DELAY
+static const int qt_passwordEchoDelay = QT_GUI_PASSWORD_ECHO_DELAY;
+#endif
+
+/*!
+    \macro QT_GUI_PASSWORD_ECHO_DELAY
+
+    \internal
+
+    Defines the amount of time in milliseconds the last entered character
+    should be displayed unmasked in the Password echo mode.
+
+    If not defined in qplatformdefs.h there will be no delay in masking
+    password characters.
+*/
+
 /*!
     \internal
 
     Updates the display text based of the current edit text
     If the text has changed will emit displayTextChanged()
 */
-void QLineControl::updateDisplayText()
+void QLineControl::updateDisplayText(bool forceUpdate)
 {
     QString orig = m_textLayout.text();
     QString str;
@@ -74,9 +90,25 @@ void QLineControl::updateDisplayText()
     else
         str = m_text;
 
-    if (m_echoMode == QLineEdit::Password || (m_echoMode == QLineEdit::PasswordEchoOnEdit
-                && !m_passwordEchoEditing))
+    if (m_echoMode == QLineEdit::Password) {
         str.fill(m_passwordCharacter);
+#ifdef QT_GUI_PASSWORD_ECHO_DELAY
+        if (m_passwordEchoTimer != 0 && m_cursor > 0 && m_cursor <= m_text.length()) {
+            int cursor = m_cursor - 1;
+            QChar uc = m_text.at(cursor);
+            str[cursor] = uc;
+            if (cursor > 0 && uc.unicode() >= 0xdc00 && uc.unicode() < 0xe000) {
+                // second half of a surrogate, check if we have the first half as well,
+                // if yes restore both at once
+                uc = m_text.at(cursor - 1);
+                if (uc.unicode() >= 0xd800 && uc.unicode() < 0xdc00)
+                    str[cursor - 1] = uc;
+            }
+        }
+#endif
+    } else if (m_echoMode == QLineEdit::PasswordEchoOnEdit && !m_passwordEchoEditing) {
+        str.fill(m_passwordCharacter);
+    }
 
     // replace certain non-printable characters with spaces (to avoid
     // drawing boxes when using fonts that don't have glyphs for such
@@ -102,7 +134,7 @@ void QLineControl::updateDisplayText()
     m_textLayout.endLayout();
     m_ascent = qRound(l.ascent());
 
-    if (str != orig)
+    if (str != orig || forceUpdate)
         emit displayTextChanged(str);
 }
 
@@ -112,7 +144,7 @@ void QLineControl::updateDisplayText()
 
     Copies the currently selected text into the clipboard using the given
     \a mode.
-  
+
     \note If the echo mode is set to a mode other than Normal then copy
     will not work.  This is to prevent using copy as a method of bypassing
     password features of the line control.
@@ -136,9 +168,14 @@ void QLineControl::copy(QClipboard::Mode mode) const
 
     \sa insert()
 */
-void QLineControl::paste()
+void QLineControl::paste(QClipboard::Mode clipboardMode)
 {
-    insert(QApplication::clipboard()->text(QClipboard::Clipboard));
+    QString clip = QApplication::clipboard()->text(clipboardMode);
+    if (!clip.isEmpty() || hasSelectedText()) {
+        separate(); //make it a separate undo/redo command
+        insert(clip);
+        separate();
+    }
 }
 
 #endif // !QT_NO_CLIPBOARD
@@ -161,12 +198,10 @@ void QLineControl::backspace()
             --m_cursor;
             if (m_maskData)
                 m_cursor = prevMaskBlank(m_cursor);
-            QChar uc = m_text.at(m_cursor);
-            if (m_cursor > 0 && uc.unicode() >= 0xdc00 && uc.unicode() < 0xe000) {
+            if (m_cursor > 0 && m_text.at(m_cursor).isLowSurrogate()) {
                 // second half of a surrogate, check if we have the first half as well,
                 // if yes delete both at once
-                uc = m_text.at(m_cursor - 1);
-                if (uc.unicode() >= 0xd800 && uc.unicode() < 0xdc00) {
+                if (m_text.at(m_cursor - 1).isHighSurrogate()) {
                     internalDelete(true);
                     --m_cursor;
                 }
@@ -249,12 +284,20 @@ void QLineControl::setSelection(int start, int length)
         m_selstart = start;
         m_selend = qMin(start + length, (int)m_text.length());
         m_cursor = m_selend;
-    } else {
+    } else if (length < 0){
         if (start == m_selend && start + length == m_selstart)
             return;
         m_selstart = qMax(start + length, 0);
         m_selend = start;
         m_cursor = m_selstart;
+    } else if (m_selstart != m_selend) {
+        m_selstart = 0;
+        m_selend = 0;
+        m_cursor = start;
+    } else {
+        m_cursor = start;
+        emitCursorPositionChanged();
+        return;
     }
     emit selectionChanged();
     emitCursorPositionChanged();
@@ -298,6 +341,7 @@ void QLineControl::init(const QString &txt)
 */
 void QLineControl::updatePasswordEchoEditing(bool editing)
 {
+    cancelPasswordEchoTimer();
     m_passwordEchoEditing = editing;
     updateDisplayText();
 }
@@ -401,6 +445,8 @@ void QLineControl::moveCursor(int pos, bool mark)
 void QLineControl::processInputMethodEvent(QInputMethodEvent *event)
 {
     int priorState = 0;
+    int originalSelectionStart = m_selstart;
+    int originalSelectionEnd = m_selend;
     bool isGettingInput = !event->commitString().isEmpty()
             || event->preeditString() != preeditAreaText()
             || event->replacementLength() > 0;
@@ -409,15 +455,21 @@ void QLineControl::processInputMethodEvent(QInputMethodEvent *event)
     if (isGettingInput) {
         // If any text is being input, remove selected text.
         priorState = m_undoState;
+        if (echoMode() == QLineEdit::PasswordEchoOnEdit && !passwordEchoEditing()) {
+            updatePasswordEchoEditing(true);
+            m_selstart = 0;
+            m_selend = m_text.length();
+        }
         removeSelectedText();
     }
 
-
     int c = m_cursor; // cursor position after insertion of commit string
     if (event->replacementStart() <= 0)
-        c += event->commitString().length() + qMin(-event->replacementStart(), event->replacementLength());
+        c += event->commitString().length() - qMin(-event->replacementStart(), event->replacementLength());
 
     m_cursor += event->replacementStart();
+    if (m_cursor < 0)
+        m_cursor = 0;
 
     // insert commit string
     if (event->replacementLength()) {
@@ -426,11 +478,11 @@ void QLineControl::processInputMethodEvent(QInputMethodEvent *event)
         removeSelectedText();
     }
     if (!event->commitString().isEmpty()) {
-        insert(event->commitString());
+        internalInsert(event->commitString());
         cursorPositionChanged = true;
     }
 
-    m_cursor = qMin(c, m_text.length());
+    m_cursor = qBound(0, c, m_text.length());
 
     for (int i = 0; i < event->attributes().size(); ++i) {
         const QInputMethodEvent::Attribute &a = event->attributes().at(i);
@@ -451,6 +503,7 @@ void QLineControl::processInputMethodEvent(QInputMethodEvent *event)
 #ifndef QT_NO_IM
     setPreeditArea(m_cursor, event->preeditString());
 #endif //QT_NO_IM
+    const int oldPreeditCursor = m_preeditCursor;
     m_preeditCursor = event->preeditString().length();
     m_hideCursor = false;
     QList<QTextLayout::FormatRange> formats;
@@ -471,9 +524,13 @@ void QLineControl::processInputMethodEvent(QInputMethodEvent *event)
         }
     }
     m_textLayout.setAdditionalFormats(formats);
-    updateDisplayText();
+    updateDisplayText(/*force*/ true);
+    if (originalSelectionStart != m_selstart || originalSelectionEnd != m_selend)
+        emit selectionChanged();
     if (cursorPositionChanged)
         emitCursorPositionChanged();
+    else if (m_preeditCursor != oldPreeditCursor)
+        emit updateMicroFocus();
     if (isGettingInput)
         finishChange(priorState);
 }
@@ -481,7 +538,7 @@ void QLineControl::processInputMethodEvent(QInputMethodEvent *event)
 /*!
     \internal
 
-    Draws the display text for the line control using the given 
+    Draws the display text for the line control using the given
     \a painter, \a clip, and \a offset.  Which aspects of the display text
     are drawn is specified by the given \a flags.
 
@@ -505,10 +562,12 @@ void QLineControl::draw(QPainter *painter, const QPoint &offset, const QRect &cl
             o.format.setForeground(m_palette.brush(QPalette::HighlightedText));
         } else {
             // mask selection
-            o.start = m_cursor;
-            o.length = 1;
-            o.format.setBackground(m_palette.brush(QPalette::Text));
-            o.format.setForeground(m_palette.brush(QPalette::Window));
+            if(!m_blinkPeriod || m_blinkStatus){
+                o.start = m_cursor;
+                o.length = 1;
+                o.format.setBackground(m_palette.brush(QPalette::Text));
+                o.format.setForeground(m_palette.brush(QPalette::Window));
+            }
         }
         selections.append(o);
     }
@@ -517,8 +576,11 @@ void QLineControl::draw(QPainter *painter, const QPoint &offset, const QRect &cl
         m_textLayout.draw(painter, offset, selections, clip);
 
     if (flags & DrawCursor){
-        if(!m_blinkPeriod || m_blinkStatus)
-            m_textLayout.drawCursor(painter, offset, m_cursor, m_cursorWidth);
+        int cursor = m_cursor;
+        if (m_preeditCursor != -1)
+            cursor += m_preeditCursor;
+        if (!m_hideCursor && (!m_blinkPeriod || m_blinkStatus))
+            m_textLayout.drawCursor(painter, offset, cursor, m_cursorWidth);
     }
 }
 
@@ -526,15 +588,18 @@ void QLineControl::draw(QPainter *painter, const QPoint &offset, const QRect &cl
     \internal
 
     Sets the selection to cover the word at the given cursor position.
-    The word boundries is defined by the behavior of QTextLayout::SkipWords
+    The word boundaries are defined by the behavior of QTextLayout::SkipWords
     cursor mode.
 */
 void QLineControl::selectWordAtPos(int cursor)
 {
-    int c = m_textLayout.previousCursorPosition(cursor, QTextLayout::SkipWords);
+    int next = cursor + 1;
+    if(next > end())
+        --next;
+    int c = m_textLayout.previousCursorPosition(next, QTextLayout::SkipWords);
     moveCursor(c, false);
     // ## text layout should support end of words.
-    int end = m_textLayout.nextCursorPosition(cursor, QTextLayout::SkipWords);
+    int end = m_textLayout.nextCursorPosition(c, QTextLayout::SkipWords);
     while (end > cursor && m_text[end-1].isSpace())
         --end;
     moveCursor(end, true);
@@ -610,6 +675,7 @@ bool QLineControl::finishChange(int validateFromState, bool update, bool edited)
 */
 void QLineControl::internalSetText(const QString &txt, int pos, bool edited)
 {
+    cancelPasswordEchoTimer();
     internalDeselect();
     emit resetInputContext();
     QString oldText = m_text;
@@ -623,7 +689,12 @@ void QLineControl::internalSetText(const QString &txt, int pos, bool edited)
     m_modifiedState =  m_undoState = 0;
     m_cursor = (pos < 0 || pos > m_text.length()) ? m_text.length() : pos;
     m_textDirty = (oldText != m_text);
-    finishChange(-1, true, edited);
+    bool changed = finishChange(-1, true, edited);
+
+#ifndef QT_NO_ACCESSIBILITY
+    if (changed)
+        QAccessible::updateAccessibility(parent(), 0, QAccessible::TextUpdated);
+#endif
 }
 
 
@@ -657,6 +728,13 @@ void QLineControl::addCommand(const Command &cmd)
 */
 void QLineControl::internalInsert(const QString &s)
 {
+#ifdef QT_GUI_PASSWORD_ECHO_DELAY
+    if (m_echoMode == QLineEdit::Password) {
+        if (m_passwordEchoTimer != 0)
+            killTimer(m_passwordEchoTimer);
+        m_passwordEchoTimer = startTimer(qt_passwordEchoDelay);
+    }
+#endif
     if (hasSelectedText())
         addCommand(Command(SetSelection, m_cursor, 0, m_selstart, m_selend));
     if (m_maskData) {
@@ -694,6 +772,7 @@ void QLineControl::internalInsert(const QString &s)
 void QLineControl::internalDelete(bool wasBackspace)
 {
     if (m_cursor < (int) m_text.length()) {
+        cancelPasswordEchoTimer();
         if (hasSelectedText())
             addCommand(Command(SetSelection, m_cursor, 0, m_selstart, m_selend));
         addCommand(Command((CommandType)((m_maskData ? 2 : 0) + (wasBackspace ? Remove : Delete)),
@@ -720,6 +799,7 @@ void QLineControl::internalDelete(bool wasBackspace)
 void QLineControl::removeSelectedText()
 {
     if (m_selstart < m_selend && m_selend <= (int) m_text.length()) {
+        cancelPasswordEchoTimer();
         separate();
         int i ;
         addCommand(Command(SetSelection, m_cursor, 0, m_selstart, m_selend));
@@ -1118,6 +1198,7 @@ void QLineControl::internalUndo(int until)
 {
     if (!isUndoAvailable())
         return;
+    cancelPasswordEchoTimer();
     internalDeselect();
     while (m_undoState && m_undoState > until) {
         Command& cmd = m_history[--m_undoState];
@@ -1201,7 +1282,7 @@ void QLineControl::internalRedo()
 /*!
     \internal
 
-    If the current cursor position differs from the last emited cursor
+    If the current cursor position differs from the last emitted cursor
     position, emits cursorPositionChanged().
 */
 void QLineControl::emitCursorPositionChanged()
@@ -1210,6 +1291,9 @@ void QLineControl::emitCursorPositionChanged()
         const int oldLast = m_lastCursorPos;
         m_lastCursorPos = m_cursor;
         cursorPositionChanged(oldLast, m_cursor);
+#ifndef QT_NO_ACCESSIBILITY
+        QAccessible::updateAccessibility(parent(), 0, QAccessible::TextCaretMoved);
+#endif
     }
 }
 
@@ -1298,6 +1382,15 @@ void QLineControl::setCursorBlinkPeriod(int msec)
     m_blinkPeriod = msec;
 }
 
+void QLineControl::resetCursorBlinkTimer()
+{
+    if (m_blinkPeriod == 0 || m_blinkTimer == 0)
+        return;
+    killTimer(m_blinkTimer);
+    m_blinkTimer = startTimer(m_blinkPeriod / 2);
+    m_blinkStatus = 1;
+}
+
 void QLineControl::timerEvent(QTimerEvent *event)
 {
     if (event->timerId() == m_blinkTimer) {
@@ -1310,6 +1403,12 @@ void QLineControl::timerEvent(QTimerEvent *event)
     } else if (event->timerId() == m_tripleClickTimer) {
         killTimer(m_tripleClickTimer);
         m_tripleClickTimer = 0;
+#ifdef QT_GUI_PASSWORD_ECHO_DELAY
+    } else if (event->timerId() == m_passwordEchoTimer) {
+        killTimer(m_passwordEchoTimer);
+        m_passwordEchoTimer = 0;
+        updateDisplayText();
+#endif
     }
 }
 
@@ -1340,13 +1439,14 @@ bool QLineControl::processEvent(QEvent* ev)
 #endif
     switch(ev->type()){
 #ifndef QT_NO_GRAPHICSVIEW
+        case QEvent::GraphicsSceneMouseDoubleClick:
         case QEvent::GraphicsSceneMouseMove:
         case QEvent::GraphicsSceneMouseRelease:
         case QEvent::GraphicsSceneMousePress:{
                QGraphicsSceneMouseEvent *gvEv = static_cast<QGraphicsSceneMouseEvent*>(ev);
-               QMouseEvent* mouse = new QMouseEvent(ev->type(),
+               QMouseEvent mouse(ev->type(),
                     gvEv->pos().toPoint(), gvEv->button(), gvEv->buttons(), gvEv->modifiers());
-               processMouseEvent(mouse); break;
+               processMouseEvent(&mouse); break;
         }
 #endif
         case QEvent::MouseButtonPress:
@@ -1361,6 +1461,8 @@ bool QLineControl::processEvent(QEvent* ev)
             processInputMethodEvent(static_cast<QInputMethodEvent*>(ev)); break;
 #ifndef QT_NO_SHORTCUT
         case QEvent::ShortcutOverride:{
+            if (isReadOnly())
+                return false;
             QKeyEvent* ke = static_cast<QKeyEvent*>(ev);
             if (ke == QKeySequence::Copy
                 || ke == QKeySequence::Paste
@@ -1427,6 +1529,7 @@ void QLineControl::processMouseEvent(QMouseEvent* ev)
             moveCursor(cursor, mark);
             break;
         }
+        case QEvent::GraphicsSceneMouseDoubleClick:
         case QEvent::MouseButtonDblClick:
             if (ev->button() == Qt::LeftButton) {
                 selectWordAtPos(xToPos(ev->pos().x()));
@@ -1544,6 +1647,7 @@ void QLineControl::processKeyEvent(QKeyEvent* event)
     }
 
     bool unknown = false;
+    bool visual = cursorMoveStyle() == Qt::VisualMoveStyle;
 
     if (false) {
     }
@@ -1564,8 +1668,14 @@ void QLineControl::processKeyEvent(QKeyEvent* event)
         copy();
     }
     else if (event == QKeySequence::Paste) {
-        if (!isReadOnly())
-            paste();
+        if (!isReadOnly()) {
+            QClipboard::Mode mode = QClipboard::Clipboard;
+#ifdef Q_WS_X11
+            if (event->modifiers() == (Qt::CTRL | Qt::SHIFT) && event->key() == Qt::Key_Insert)
+                mode = QClipboard::Selection;
+#endif
+            paste(mode);
+        }
     }
     else if (event == QKeySequence::Cut) {
         if (!isReadOnly()) {
@@ -1581,16 +1691,16 @@ void QLineControl::processKeyEvent(QKeyEvent* event)
         }
     }
 #endif //QT_NO_CLIPBOARD
-    else if (event == QKeySequence::MoveToStartOfLine) {
+    else if (event == QKeySequence::MoveToStartOfLine || event == QKeySequence::MoveToStartOfBlock) {
         home(0);
     }
-    else if (event == QKeySequence::MoveToEndOfLine) {
+    else if (event == QKeySequence::MoveToEndOfLine || event == QKeySequence::MoveToEndOfBlock) {
         end(0);
     }
-    else if (event == QKeySequence::SelectStartOfLine) {
+    else if (event == QKeySequence::SelectStartOfLine || event == QKeySequence::SelectStartOfBlock) {
         home(1);
     }
-    else if (event == QKeySequence::SelectEndOfLine) {
+    else if (event == QKeySequence::SelectEndOfLine || event == QKeySequence::SelectEndOfBlock) {
         end(1);
     }
     else if (event == QKeySequence::MoveToNextChar) {
@@ -1602,11 +1712,11 @@ void QLineControl::processKeyEvent(QKeyEvent* event)
 #endif
             moveCursor(selectionEnd(), false);
         } else {
-            cursorForward(0, layoutDirection() == Qt::LeftToRight ? 1 : -1);
+            cursorForward(0, visual ? 1 : (layoutDirection() == Qt::LeftToRight ? 1 : -1));
         }
     }
     else if (event == QKeySequence::SelectNextChar) {
-        cursorForward(1, layoutDirection() == Qt::LeftToRight ? 1 : -1);
+        cursorForward(1, visual ? 1 : (layoutDirection() == Qt::LeftToRight ? 1 : -1));
     }
     else if (event == QKeySequence::MoveToPreviousChar) {
 #if !defined(Q_WS_WIN) || defined(QT_NO_COMPLETER)
@@ -1617,11 +1727,11 @@ void QLineControl::processKeyEvent(QKeyEvent* event)
 #endif
             moveCursor(selectionStart(), false);
         } else {
-            cursorForward(0, layoutDirection() == Qt::LeftToRight ? -1 : 1);
+            cursorForward(0, visual ? -1 : (layoutDirection() == Qt::LeftToRight ? -1 : 1));
         }
     }
     else if (event == QKeySequence::SelectPreviousChar) {
-        cursorForward(1, layoutDirection() == Qt::LeftToRight ? -1 : 1);
+        cursorForward(1, visual ? -1 : (layoutDirection() == Qt::LeftToRight ? -1 : 1));
     }
     else if (event == QKeySequence::MoveToNextWord) {
         if (echoMode() == QLineEdit::Normal)
@@ -1666,6 +1776,7 @@ void QLineControl::processKeyEvent(QKeyEvent* event)
     }
 #endif // QT_NO_SHORTCUT
     else {
+        bool handled = false;
 #ifdef Q_WS_MAC
         if (event->key() == Qt::Key_Up || event->key() == Qt::Key_Down) {
             Qt::KeyboardModifiers myModifiers = (event->modifiers() & ~Qt::KeypadModifier);
@@ -1683,6 +1794,7 @@ void QLineControl::processKeyEvent(QKeyEvent* event)
                     event->key() == Qt::Key_Up ? home(0) : end(0);
                 }
             }
+            handled = true;
         }
 #endif
         if (event->modifiers() & Qt::ControlModifier) {
@@ -1715,7 +1827,8 @@ void QLineControl::processKeyEvent(QKeyEvent* event)
             break;
 #endif
             default:
-                unknown = true;
+                if (!handled)
+                    unknown = true;
             }
         } else { // ### check for *no* modifier
             switch (event->key()) {
@@ -1746,9 +1859,9 @@ void QLineControl::processKeyEvent(QKeyEvent* event)
                 }
                 break;
 #endif
-
             default:
-                unknown = true;
+                if (!handled)
+                    unknown = true;
             }
         }
     }
