@@ -1,40 +1,40 @@
 /****************************************************************************
 **
-** Copyright (C) 2010 Nokia Corporation and/or its subsidiary(-ies).
+** Copyright (C) 2011 Nokia Corporation and/or its subsidiary(-ies).
 ** All rights reserved.
 ** Contact: Nokia Corporation (qt-info@nokia.com)
 **
 ** This file is part of the QtGui module of the Qt Toolkit.
 **
 ** $QT_BEGIN_LICENSE:LGPL$
-** Commercial Usage
-** Licensees holding valid Qt Commercial licenses may use this file in
-** accordance with the Qt Commercial License Agreement provided with the
-** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and Nokia.
-**
 ** GNU Lesser General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU Lesser
-** General Public License version 2.1 as published by the Free Software
-** Foundation and appearing in the file LICENSE.LGPL included in the
-** packaging of this file.  Please review the following information to
-** ensure the GNU Lesser General Public License version 2.1 requirements
-** will be met: http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
+** This file may be used under the terms of the GNU Lesser General Public
+** License version 2.1 as published by the Free Software Foundation and
+** appearing in the file LICENSE.LGPL included in the packaging of this
+** file. Please review the following information to ensure the GNU Lesser
+** General Public License version 2.1 requirements will be met:
+** http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
 **
 ** In addition, as a special exception, Nokia gives you certain additional
-** rights.  These rights are described in the Nokia Qt LGPL Exception
+** rights. These rights are described in the Nokia Qt LGPL Exception
 ** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
 **
 ** GNU General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU
-** General Public License version 3.0 as published by the Free Software
-** Foundation and appearing in the file LICENSE.GPL included in the
-** packaging of this file.  Please review the following information to
-** ensure the GNU General Public License version 3.0 requirements will be
-** met: http://www.gnu.org/copyleft/gpl.html.
+** Alternatively, this file may be used under the terms of the GNU General
+** Public License version 3.0 as published by the Free Software Foundation
+** and appearing in the file LICENSE.GPL included in the packaging of this
+** file. Please review the following information to ensure the GNU General
+** Public License version 3.0 requirements will be met:
+** http://www.gnu.org/copyleft/gpl.html.
+**
+** Other Usage
+** Alternatively, this file may be used in accordance with the terms and
+** conditions contained in a signed written agreement between you and Nokia.
+**
+**
+**
+**
 **
-** If you have questions regarding the use of this file, please contact
-** Nokia at qt-info@nokia.com.
 ** $QT_END_LICENSE$
 **
 ****************************************************************************/
@@ -502,7 +502,7 @@ void QMainWindow::setToolButtonStyle(Qt::ToolButtonStyle toolButtonStyle)
 */
 QMenuBar *QMainWindow::menuBar() const
 {
-    QMenuBar *menuBar = qobject_cast<QMenuBar *>(d_func()->layout->menuBar());
+    QMenuBar *menuBar = qobject_cast<QMenuBar *>(layout()->menuBar());
     if (!menuBar) {
         QMainWindow *self = const_cast<QMainWindow *>(this);
         menuBar = new QMenuBar(self);
@@ -1501,8 +1501,6 @@ bool QMainWindow::event(QEvent *event)
     \i Before Qt 4.5, if you called showFullScreen() on the main window, the QToolbar would
         disappear since it is considered to be part of the title bar. Qt 4.5 and up will now work around this by pulling
         the toolbars out and back into the regular toolbar and vice versa when you swap out.
-        However, a good practice would be that turning off the unified toolbar before you call
-        showFullScreen() and restoring it after you call showNormal().
     \endlist
 
     Setting this back to false will remove these restrictions.
@@ -1516,15 +1514,28 @@ void QMainWindow::setUnifiedTitleAndToolBarOnMac(bool set)
     if (!isWindow() || d->useHIToolBar == set || QSysInfo::MacintoshVersion < QSysInfo::MV_10_3)
         return;
 
-    // ### Disable the unified toolbar when using anything but the native graphics system.
-    // ### Disable when using alien widgets as well
-    if (windowSurface() || testAttribute(Qt::WA_NativeWindow) == false)
-        return;
-
     d->useHIToolBar = set;
     createWinId(); // We need the hiview for down below.
 
+#ifdef QT_MAC_USE_COCOA
+    // Activate the unified toolbar with the raster engine.
+    if (windowSurface() && set) {
+        d->layout->unifiedSurface = new QUnifiedToolbarSurface(this);
+    }
+#endif // QT_MAC_USE_COCOA
+
     d->layout->updateHIToolBarStatus();
+
+#ifdef QT_MAC_USE_COCOA
+    // Deactivate the unified toolbar with the raster engine.
+    if (windowSurface() && !set) {
+        if (d->layout->unifiedSurface) {
+            delete d->layout->unifiedSurface;
+            d->layout->unifiedSurface = 0;
+        }
+    }
+#endif // QT_MAC_USE_COCOA
+
     // Enabling the unified toolbar clears the opaque size grip setting, update it.
     d->macUpdateOpaqueSizeGrip();
 #else
@@ -1634,7 +1645,7 @@ QMenu *QMainWindow::createPopupMenu()
     Q_D(QMainWindow);
     QMenu *menu = 0;
 #ifndef QT_NO_DOCKWIDGET
-    QList<QDockWidget *> dockwidgets = qFindChildren<QDockWidget *>(this);
+    QList<QDockWidget *> dockwidgets = findChildren<QDockWidget *>();
     if (dockwidgets.size()) {
         menu = new QMenu(this);
         for (int i = 0; i < dockwidgets.size(); ++i) {
@@ -1648,7 +1659,7 @@ QMenu *QMainWindow::createPopupMenu()
     }
 #endif // QT_NO_DOCKWIDGET
 #ifndef QT_NO_TOOLBAR
-    QList<QToolBar *> toolbars = qFindChildren<QToolBar *>(this);
+    QList<QToolBar *> toolbars = findChildren<QToolBar *>();
     if (toolbars.size()) {
         if (!menu)
             menu = new QMenu(this);
