@@ -1,40 +1,40 @@
 /****************************************************************************
 **
-** Copyright (C) 2009 Nokia Corporation and/or its subsidiary(-ies).
+** Copyright (C) 2011 Nokia Corporation and/or its subsidiary(-ies).
 ** All rights reserved.
 ** Contact: Nokia Corporation (qt-info@nokia.com)
 **
 ** This file is part of the QtGui module of the Qt Toolkit.
 **
 ** $QT_BEGIN_LICENSE:LGPL$
-** Commercial Usage
-** Licensees holding valid Qt Commercial licenses may use this file in
-** accordance with the Qt Commercial License Agreement provided with the
-** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and Nokia.
-**
 ** GNU Lesser General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU Lesser
-** General Public License version 2.1 as published by the Free Software
-** Foundation and appearing in the file LICENSE.LGPL included in the
-** packaging of this file.  Please review the following information to
-** ensure the GNU Lesser General Public License version 2.1 requirements
-** will be met: http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
+** This file may be used under the terms of the GNU Lesser General Public
+** License version 2.1 as published by the Free Software Foundation and
+** appearing in the file LICENSE.LGPL included in the packaging of this
+** file. Please review the following information to ensure the GNU Lesser
+** General Public License version 2.1 requirements will be met:
+** http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
 **
 ** In addition, as a special exception, Nokia gives you certain additional
-** rights.  These rights are described in the Nokia Qt LGPL Exception
+** rights. These rights are described in the Nokia Qt LGPL Exception
 ** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
 **
 ** GNU General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU
-** General Public License version 3.0 as published by the Free Software
-** Foundation and appearing in the file LICENSE.GPL included in the
-** packaging of this file.  Please review the following information to
-** ensure the GNU General Public License version 3.0 requirements will be
-** met: http://www.gnu.org/copyleft/gpl.html.
+** Alternatively, this file may be used under the terms of the GNU General
+** Public License version 3.0 as published by the Free Software Foundation
+** and appearing in the file LICENSE.GPL included in the packaging of this
+** file. Please review the following information to ensure the GNU General
+** Public License version 3.0 requirements will be met:
+** http://www.gnu.org/copyleft/gpl.html.
+**
+** Other Usage
+** Alternatively, this file may be used in accordance with the terms and
+** conditions contained in a signed written agreement between you and Nokia.
+**
+**
+**
+**
 **
-** If you have questions regarding the use of this file, please contact
-** Nokia at qt-info@nokia.com.
 ** $QT_END_LICENSE$
 **
 ****************************************************************************/
@@ -195,6 +195,7 @@ public:
     void _q_removeTab(int);
     void _q_tabMoved(int from, int to);
     void init();
+    bool hasHeightForWidth() const;
 
     QTabBar *tabs;
     QStackedWidget *stack;
@@ -246,6 +247,15 @@ void QTabWidgetPrivate::init()
 
 }
 
+bool QTabWidgetPrivate::hasHeightForWidth() const
+{
+    bool has = size_policy.hasHeightForWidth();
+    if (!has && stack)
+        has = qt_widget_private(stack)->hasHeightForWidth();
+    return has;
+}
+
+
 /*!
     Initialize \a option with the values from this QTabWidget. This method is useful
     for subclasses when they need a QStyleOptionTabWidgetFrame, but don't want to fill
@@ -542,6 +552,8 @@ void QTabWidget::setTabEnabled(int index, bool enable)
 {
     Q_D(QTabWidget);
     d->tabs->setTabEnabled(index, enable);
+    if (QWidget *widget = d->stack->widget(index))
+        widget->setEnabled(enable);
 }
 
 /*!
@@ -699,8 +711,8 @@ void QTabWidget::setTabBar(QTabBar* tb)
     setFocusProxy(d->tabs);
     connect(d->tabs, SIGNAL(currentChanged(int)),
             this, SLOT(_q_showTab(int)));
-    connect(d->tabs, SIGNAL(tabMoved(int, int)),
-            this, SLOT(_q_tabMoved(int, int)));
+    connect(d->tabs, SIGNAL(tabMoved(int,int)),
+            this, SLOT(_q_tabMoved(int,int)));
     if (d->tabs->tabsClosable())
         connect(d->tabs, SIGNAL(tabCloseRequested(int)),
                 this, SIGNAL(tabCloseRequested(int)));
@@ -814,8 +826,8 @@ QSize QTabWidget::sizeHint() const
 {
     Q_D(const QTabWidget);
     QSize lc(0, 0), rc(0, 0);
-    QStyleOption opt(0);
-    opt.init(this);
+    QStyleOptionTabWidgetFrameV2 opt;
+    initStyleOption(&opt);
     opt.state = QStyle::State_None;
 
     if (d->leftCornerWidget)
@@ -863,14 +875,59 @@ QSize QTabWidget::minimumSizeHint() const
 
     QSize sz = basicSize(d->pos == North || d->pos == South, lc, rc, s, t);
 
-    QStyleOption opt(0);
-    opt.rect = rect();
+    QStyleOptionTabWidgetFrameV2 opt;
+    initStyleOption(&opt);
     opt.palette = palette();
     opt.state = QStyle::State_None;
     return style()->sizeFromContents(QStyle::CT_TabWidget, &opt, sz, this)
                     .expandedTo(QApplication::globalStrut());
 }
 
+/*!
+    \reimp
+    \since 4.8
+*/
+int QTabWidget::heightForWidth(int width) const
+{
+    Q_D(const QTabWidget);
+    QStyleOptionTabWidgetFrameV2 opt;
+    initStyleOption(&opt);
+    opt.state = QStyle::State_None;
+
+    QSize zero(0,0);
+    const QSize padding = style()->sizeFromContents(QStyle::CT_TabWidget, &opt, zero, this)
+                                  .expandedTo(QApplication::globalStrut());
+
+    QSize lc(0, 0), rc(0, 0);
+    if (d->leftCornerWidget)
+        lc = d->leftCornerWidget->sizeHint();
+    if(d->rightCornerWidget)
+        rc = d->rightCornerWidget->sizeHint();
+    if (!d->dirty) {
+        QTabWidget *that = (QTabWidget*)this;
+        that->setUpLayout(true);
+    }
+    QSize t(d->tabs->sizeHint());
+
+    if(usesScrollButtons())
+        t = t.boundedTo(QSize(200,200));
+    else
+        t = t.boundedTo(QApplication::desktop()->size());
+
+    const bool tabIsHorizontal = (d->pos == North || d->pos == South);
+    const int contentsWidth = width - padding.width();
+    int stackWidth = contentsWidth;
+    if (!tabIsHorizontal)
+        stackWidth -= qMax(t.width(), qMax(lc.width(), rc.width()));
+
+    int stackHeight = d->stack->heightForWidth(stackWidth);
+    QSize s(stackWidth, stackHeight);
+
+    QSize contentSize = basicSize(tabIsHorizontal, lc, rc, s, t);
+    return (contentSize + padding).expandedTo(QApplication::globalStrut()).height();
+}
+
+
 /*!
     \reimp
  */
