@@ -1,40 +1,40 @@
 /****************************************************************************
 **
-** Copyright (C) 2009 Nokia Corporation and/or its subsidiary(-ies).
+** Copyright (C) 2011 Nokia Corporation and/or its subsidiary(-ies).
 ** All rights reserved.
 ** Contact: Nokia Corporation (qt-info@nokia.com)
 **
 ** This file is part of the QtNetwork module of the Qt Toolkit.
 **
 ** $QT_BEGIN_LICENSE:LGPL$
-** Commercial Usage
-** Licensees holding valid Qt Commercial licenses may use this file in
-** accordance with the Qt Commercial License Agreement provided with the
-** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and Nokia.
-**
 ** GNU Lesser General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU Lesser
-** General Public License version 2.1 as published by the Free Software
-** Foundation and appearing in the file LICENSE.LGPL included in the
-** packaging of this file.  Please review the following information to
-** ensure the GNU Lesser General Public License version 2.1 requirements
-** will be met: http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
+** This file may be used under the terms of the GNU Lesser General Public
+** License version 2.1 as published by the Free Software Foundation and
+** appearing in the file LICENSE.LGPL included in the packaging of this
+** file. Please review the following information to ensure the GNU Lesser
+** General Public License version 2.1 requirements will be met:
+** http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
 **
 ** In addition, as a special exception, Nokia gives you certain additional
-** rights.  These rights are described in the Nokia Qt LGPL Exception
+** rights. These rights are described in the Nokia Qt LGPL Exception
 ** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
 **
 ** GNU General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU
-** General Public License version 3.0 as published by the Free Software
-** Foundation and appearing in the file LICENSE.GPL included in the
-** packaging of this file.  Please review the following information to
-** ensure the GNU General Public License version 3.0 requirements will be
-** met: http://www.gnu.org/copyleft/gpl.html.
+** Alternatively, this file may be used under the terms of the GNU General
+** Public License version 3.0 as published by the Free Software Foundation
+** and appearing in the file LICENSE.GPL included in the packaging of this
+** file. Please review the following information to ensure the GNU General
+** Public License version 3.0 requirements will be met:
+** http://www.gnu.org/copyleft/gpl.html.
+**
+** Other Usage
+** Alternatively, this file may be used in accordance with the terms and
+** conditions contained in a signed written agreement between you and Nokia.
+**
+**
+**
+**
 **
-** If you have questions regarding the use of this file, please contact
-** Nokia at qt-info@nokia.com.
 ** $QT_END_LICENSE$
 **
 ****************************************************************************/
@@ -119,6 +119,7 @@ void QHttpNetworkReply::setRequest(const QHttpNetworkRequest &request)
 {
     Q_D(QHttpNetworkReply);
     d->request = request;
+    d->ssl = request.isSsl();
 }
 
 int QHttpNetworkReply::statusCode() const
@@ -176,12 +177,56 @@ qint64 QHttpNetworkReply::bytesAvailableNextBlock() const
         return -1;
 }
 
+bool QHttpNetworkReply::readAnyAvailable() const
+{
+    Q_D(const QHttpNetworkReply);
+    return (d->responseData.bufferCount() > 0);
+}
+
 QByteArray QHttpNetworkReply::readAny()
 {
     Q_D(QHttpNetworkReply);
+    if (d->responseData.bufferCount() == 0)
+        return QByteArray();
+
+    // we'll take the last buffer, so schedule another read from http
+    if (d->downstreamLimited && d->responseData.bufferCount() == 1)
+        d->connection->d_func()->readMoreLater(this);
     return d->responseData.read();
 }
 
+QByteArray QHttpNetworkReply::readAll()
+{
+    Q_D(QHttpNetworkReply);
+    return d->responseData.readAll();
+}
+
+void QHttpNetworkReply::setDownstreamLimited(bool dsl)
+{
+    Q_D(QHttpNetworkReply);
+    d->downstreamLimited = dsl;
+    d->connection->d_func()->readMoreLater(this);
+}
+
+bool QHttpNetworkReply::supportsUserProvidedDownloadBuffer()
+{
+    Q_D(QHttpNetworkReply);
+    return (!d->isChunked() && !d->autoDecompress && d->bodyLength > 0);
+}
+
+void QHttpNetworkReply::setUserProvidedDownloadBuffer(char* b)
+{
+    Q_D(QHttpNetworkReply);
+    if (supportsUserProvidedDownloadBuffer())
+        d->userProvidedDownloadBuffer = b;
+}
+
+char* QHttpNetworkReply::userProvidedDownloadBuffer()
+{
+    Q_D(QHttpNetworkReply);
+    return d->userProvidedDownloadBuffer;
+}
+
 bool QHttpNetworkReply::isFinished() const
 {
     return d_func()->state == QHttpNetworkReplyPrivate::AllDoneState;
@@ -192,16 +237,25 @@ bool QHttpNetworkReply::isPipeliningUsed() const
     return d_func()->pipeliningUsed;
 }
 
+QHttpNetworkConnection* QHttpNetworkReply::connection()
+{
+    return d_func()->connection;
+}
+
 
 QHttpNetworkReplyPrivate::QHttpNetworkReplyPrivate(const QUrl &newUrl)
-    : QHttpNetworkHeaderPrivate(newUrl), state(NothingDoneState), statusCode(100),
+    : QHttpNetworkHeaderPrivate(newUrl)
+    , state(NothingDoneState)
+    , ssl(false)
+    , statusCode(100),
       majorVersion(0), minorVersion(0), bodyLength(0), contentRead(0), totalProgress(0),
       chunkedTransferEncoding(false),
       connectionCloseEnabled(true),
       forceConnectionCloseEnabled(false),
       currentChunkSize(0), currentChunkRead(0), connection(0), initInflate(false),
       autoDecompress(false), responseData(), requestIsPrepared(false)
-      ,pipeliningUsed(false)
+      ,pipeliningUsed(false), downstreamLimited(false)
+      ,userProvidedDownloadBuffer(0)
 {
 }
 
@@ -209,7 +263,7 @@ QHttpNetworkReplyPrivate::~QHttpNetworkReplyPrivate()
 {
 }
 
-void QHttpNetworkReplyPrivate::clear()
+void QHttpNetworkReplyPrivate::clearHttpLayerInformation()
 {
     state = NothingDoneState;
     statusCode = 100;
@@ -219,17 +273,24 @@ void QHttpNetworkReplyPrivate::clear()
     currentChunkSize = 0;
     currentChunkRead = 0;
     connectionCloseEnabled = true;
-    connection = 0;
 #ifndef QT_NO_COMPRESS
     if (initInflate)
         inflateEnd(&inflateStrm);
 #endif
     initInflate = false;
     streamEnd = false;
-    autoDecompress = false;
     fields.clear();
 }
 
+// TODO: Isn't everything HTTP layer related? We don't need to set connection and connectionChannel to 0 at all
+void QHttpNetworkReplyPrivate::clear()
+{
+    connection = 0;
+    connectionChannel = 0;
+    autoDecompress = false;
+    clearHttpLayerInformation();
+}
+
 // QHttpNetworkReplyPrivate
 qint64 QHttpNetworkReplyPrivate::bytesAvailable() const
 {
@@ -239,7 +300,7 @@ qint64 QHttpNetworkReplyPrivate::bytesAvailable() const
 bool QHttpNetworkReplyPrivate::isGzipped()
 {
     QByteArray encoding = headerField("content-encoding");
-    return encoding.toLower() == "gzip";
+    return qstricmp(encoding.constData(), "gzip") == 0;
 }
 
 void QHttpNetworkReplyPrivate::removeAutoDecompressHeader()
@@ -247,11 +308,10 @@ void QHttpNetworkReplyPrivate::removeAutoDecompressHeader()
     // The header "Content-Encoding  = gzip" is retained.
     // Content-Length is removed since the actual one send by the server is for compressed data
     QByteArray name("content-length");
-    QByteArray lowerName = name.toLower();
     QList<QPair<QByteArray, QByteArray> >::Iterator it = fields.begin(),
                                                    end = fields.end();
     while (it != end) {
-        if (name == it->first.toLower()) {
+        if (qstricmp(name.constData(), it->first.constData()) == 0) {
             fields.erase(it);
             break;
         }
@@ -269,6 +329,7 @@ bool QHttpNetworkReplyPrivate::findChallenge(bool forProxy, QByteArray &challeng
     QList<QByteArray> challenges = headerFieldValues(header);
     for (int i = 0; i<challenges.size(); i++) {
         QByteArray line = challenges.at(i);
+        // todo use qstrincmp
         if (!line.toLower().startsWith("negotiate"))
             challenge = line;
     }
@@ -400,26 +461,49 @@ int QHttpNetworkReplyPrivate::gunzipBodyPartially(QByteArray &compressed, QByteA
         }
         have = sizeof(out) - inflateStrm.avail_out;
         inflated.append(QByteArray((const char *)out, have));
-     } while (inflateStrm.avail_out == 0);
+     } while (inflateStrm.avail_out == 0 && inflateStrm.avail_in > 0);
     // clean up and return
     if (ret <= Z_ERRNO || ret == Z_STREAM_END) {
-        inflateEnd(&inflateStrm);
-        initInflate = false;
+        gunzipBodyPartiallyEnd();
     }
     streamEnd = (ret == Z_STREAM_END);
     return ret;
 }
+
+void QHttpNetworkReplyPrivate::gunzipBodyPartiallyEnd()
+{
+    if (initInflate) {
+        inflateEnd(&inflateStrm);
+        initInflate = false;
+    }
+}
+
 #endif
 
 qint64 QHttpNetworkReplyPrivate::readStatus(QAbstractSocket *socket)
 {
+    if (fragment.isEmpty()) {
+        // reserve bytes for the status line. This is better than always append() which reallocs the byte array
+        fragment.reserve(32);
+    }
+
     qint64 bytes = 0;
     char c;
+    qint64 haveRead = 0;
+
+    do {
+        haveRead = socket->read(&c, 1);
+        if (haveRead == -1)
+            return -1; // unexpected EOF
+        else if (haveRead == 0)
+            break; // read more later
+        else if (haveRead == 1 && bytes == 0 && (c == 11 || c == '\n' || c == '\r' || c == ' ' || c == 31))
+            continue; // Ignore all whitespace that was trailing froma previous request on that socket
+
+        bytes++;
 
-    while (socket->bytesAvailable()) {
         // allow both CRLF & LF (only) line endings
-        if (socket->peek(&c, 1) == 1 && c == '\n') {
-            bytes += socket->read(&c, 1); // read the "n"
+        if (c == '\n') {
             // remove the CR at the end
             if (fragment.endsWith('\r')) {
                 fragment.truncate(fragment.length()-1);
@@ -432,11 +516,6 @@ qint64 QHttpNetworkReplyPrivate::readStatus(QAbstractSocket *socket)
             }
             break;
         } else {
-            c = 0;
-            int haveRead = socket->read(&c, 1);
-            if (haveRead == -1)
-                return -1;
-            bytes += haveRead;
             fragment.append(c);
         }
 
@@ -446,8 +525,7 @@ qint64 QHttpNetworkReplyPrivate::readStatus(QAbstractSocket *socket)
             fragment.clear();
             return -1;
         }
-
-    }
+    } while (haveRead == 1);
 
     return bytes;
 }
@@ -490,20 +568,46 @@ bool QHttpNetworkReplyPrivate::parseStatus(const QByteArray &status)
 
 qint64 QHttpNetworkReplyPrivate::readHeader(QAbstractSocket *socket)
 {
+    if (fragment.isEmpty()) {
+        // according to http://dev.opera.com/articles/view/mama-http-headers/ the average size of the header
+        // block is 381 bytes.
+        // reserve bytes. This is better than always append() which reallocs the byte array.
+        fragment.reserve(512);
+    }
+
     qint64 bytes = 0;
     char c = 0;
     bool allHeaders = false;
-    while (!allHeaders && socket->bytesAvailable()) {
-        if (socket->peek(&c, 1) == 1 && c == '\n') {
-            // check for possible header endings. As per HTTP rfc,
-            // the header endings will be marked by CRLFCRLF. But
-            // we will allow CRLFLF, LFLF & CRLFCRLF
-            if (fragment.endsWith("\n\r") || fragment.endsWith('\n'))
-                allHeaders = true;
+    qint64 haveRead = 0;
+    do {
+        haveRead = socket->read(&c, 1);
+        if (haveRead == 0) {
+            // read more later
+            break;
+        } else if (haveRead == -1) {
+            // connection broke down
+            return -1;
+        } else {
+            fragment.append(c);
+            bytes++;
+
+            if (c == '\n') {
+                // check for possible header endings. As per HTTP rfc,
+                // the header endings will be marked by CRLFCRLF. But
+                // we will allow CRLFCRLF, CRLFLF, LFLF
+                if (fragment.endsWith("\r\n\r\n")
+                    || fragment.endsWith("\r\n\n")
+                    || fragment.endsWith("\n\n"))
+                    allHeaders = true;
+
+                // there is another case: We have no headers. Then the fragment equals just the line ending
+                if ((fragment.length() == 2 && fragment.endsWith("\r\n"))
+                    || (fragment.length() == 1 && fragment.endsWith("\n")))
+                    allHeaders = true;
+            }
         }
-        bytes += socket->read(&c, 1);
-        fragment.append(c);
-    }
+    } while (!allHeaders && haveRead > 0);
+
     // we received all headers now parse them
     if (allHeaders) {
         parseHeader(fragment);
@@ -568,14 +672,34 @@ bool QHttpNetworkReplyPrivate::isConnectionCloseEnabled()
     return connectionCloseEnabled || forceConnectionCloseEnabled;
 }
 
+// note this function can only be used for non-chunked, non-compressed with
+// known content length
+qint64 QHttpNetworkReplyPrivate::readBodyVeryFast(QAbstractSocket *socket, char *b)
+{
+    // This first read is to flush the buffer inside the socket
+    qint64 haveRead = 0;
+    haveRead = socket->read(b, bodyLength - contentRead);
+    if (haveRead == -1) {
+        return 0; // ### error checking here;
+    }
+    contentRead += haveRead;
+
+    if (contentRead == bodyLength) {
+        state = AllDoneState;
+    }
+
+    return haveRead;
+}
+
 // note this function can only be used for non-chunked, non-compressed with
 // known content length
 qint64 QHttpNetworkReplyPrivate::readBodyFast(QAbstractSocket *socket, QByteDataBuffer *rb)
 {
+
     qint64 toBeRead = qMin(socket->bytesAvailable(), bodyLength - contentRead);
     QByteArray bd;
     bd.resize(toBeRead);
-    qint64 haveRead = socket->read(bd.data(), bd.size());
+    qint64 haveRead = socket->read(bd.data(), toBeRead);
     if (haveRead == -1) {
         bd.clear();
         return 0; // ### error checking here;
@@ -597,29 +721,34 @@ qint64 QHttpNetworkReplyPrivate::readBody(QAbstractSocket *socket, QByteDataBuff
 {
     qint64 bytes = 0;
     if (isChunked()) {
-        bytes += readReplyBodyChunked(socket, out); // chunked transfer encoding (rfc 2616, sec 3.6)
-    } else if (bodyLength > 0) { // we have a Content-Length
+        // chunked transfer encoding (rfc 2616, sec 3.6)
+        bytes += readReplyBodyChunked(socket, out);
+    } else if (bodyLength > 0) {
+        // we have a Content-Length
         bytes += readReplyBodyRaw(socket, out, bodyLength - contentRead);
         if (contentRead + bytes == bodyLength)
             state = AllDoneState;
     } else {
+        // no content length. just read what's possible
         bytes += readReplyBodyRaw(socket, out, socket->bytesAvailable());
     }
     contentRead += bytes;
     return bytes;
 }
 
-qint64 QHttpNetworkReplyPrivate::readReplyBodyRaw(QIODevice *in, QByteDataBuffer *out, qint64 size)
+qint64 QHttpNetworkReplyPrivate::readReplyBodyRaw(QAbstractSocket *socket, QByteDataBuffer *out, qint64 size)
 {
+    // FIXME get rid of this function and just use readBodyFast and give it socket->bytesAvailable()
     qint64 bytes = 0;
-    Q_ASSERT(in);
+    Q_ASSERT(socket);
     Q_ASSERT(out);
 
-    int toBeRead = qMin<qint64>(128*1024, qMin<qint64>(size, in->bytesAvailable()));
+    int toBeRead = qMin<qint64>(128*1024, qMin<qint64>(size, socket->bytesAvailable()));
+
     while (toBeRead > 0) {
         QByteArray byteData;
         byteData.resize(toBeRead);
-        qint64 haveRead = in->read(byteData.data(), byteData.size());
+        qint64 haveRead = socket->read(byteData.data(), byteData.size());
         if (haveRead <= 0) {
             // ### error checking here
             byteData.clear();
@@ -631,25 +760,35 @@ qint64 QHttpNetworkReplyPrivate::readReplyBodyRaw(QIODevice *in, QByteDataBuffer
         bytes += haveRead;
         size -= haveRead;
 
-        toBeRead = qMin<qint64>(128*1024, qMin<qint64>(size, in->bytesAvailable()));
+        toBeRead = qMin<qint64>(128*1024, qMin<qint64>(size, socket->bytesAvailable()));
     }
     return bytes;
 
 }
 
-qint64 QHttpNetworkReplyPrivate::readReplyBodyChunked(QIODevice *in, QByteDataBuffer *out)
+qint64 QHttpNetworkReplyPrivate::readReplyBodyChunked(QAbstractSocket *socket, QByteDataBuffer *out)
 {
     qint64 bytes = 0;
-    while (in->bytesAvailable()) { // while we can read from input
-        // if we are done with the current chunk, get the size of the new chunk
+    while (socket->bytesAvailable()) {
         if (currentChunkRead >= currentChunkSize) {
+            // For the first chunk and when we're done with a chunk
             currentChunkSize = 0;
             currentChunkRead = 0;
             if (bytes) {
+                // After a chunk
                 char crlf[2];
-                bytes += in->read(crlf, 2); // read the "\r\n" after the chunk
+                // read the "\r\n" after the chunk
+                qint64 haveRead = socket->read(crlf, 2);
+                // FIXME: This code is slightly broken and not optimal. What if the 2 bytes are not available yet?!
+                // For nice reasons (the toLong in getChunkSize accepting \n at the beginning
+                // it right now still works, but we should definitely fix this.
+
+                if (haveRead != 2)
+                    return bytes; // FIXME
+                bytes += haveRead;
             }
-            bytes += getChunkSize(in, &currentChunkSize);
+            // Note that chunk size gets stored in currentChunkSize, what is returned is the bytes read
+            bytes += getChunkSize(socket, &currentChunkSize);
             if (currentChunkSize == -1)
                 break;
         }
@@ -659,8 +798,8 @@ qint64 QHttpNetworkReplyPrivate::readReplyBodyChunked(QIODevice *in, QByteDataBu
             break;
         }
 
-        // otherwise, try to read what is missing for this chunk
-        qint64 haveRead = readReplyBodyRaw (in, out, currentChunkSize - currentChunkRead);
+        // otherwise, try to begin reading this chunk / to read what is missing for this chunk
+        qint64 haveRead = readReplyBodyRaw (socket, out, currentChunkSize - currentChunkRead);
         currentChunkRead += haveRead;
         bytes += haveRead;
 
@@ -670,22 +809,25 @@ qint64 QHttpNetworkReplyPrivate::readReplyBodyChunked(QIODevice *in, QByteDataBu
     return bytes;
 }
 
-qint64 QHttpNetworkReplyPrivate::getChunkSize(QIODevice *in, qint64 *chunkSize)
+qint64 QHttpNetworkReplyPrivate::getChunkSize(QAbstractSocket *socket, qint64 *chunkSize)
 {
     qint64 bytes = 0;
     char crlf[2];
     *chunkSize = -1;
-    int bytesAvailable = in->bytesAvailable();
+
+    int bytesAvailable = socket->bytesAvailable();
+    // FIXME rewrite to permanent loop without bytesAvailable
     while (bytesAvailable > bytes) {
-        qint64 sniffedBytes =  in->peek(crlf, 2);
+        qint64 sniffedBytes = socket->peek(crlf, 2);
         int fragmentSize = fragment.size();
+
         // check the next two bytes for a "\r\n", skip blank lines
         if ((fragmentSize && sniffedBytes == 2 && crlf[0] == '\r' && crlf[1] == '\n')
            ||(fragmentSize > 1 && fragment.endsWith('\r')  && crlf[0] == '\n'))
         {
-            bytes += in->read(crlf, 1);     // read the \r or \n
+            bytes += socket->read(crlf, 1);     // read the \r or \n
             if (crlf[0] == '\r')
-                bytes += in->read(crlf, 1); // read the \n
+                bytes += socket->read(crlf, 1); // read the \n
             bool ok = false;
             // ignore the chunk-extension
             fragment = fragment.mid(0, fragment.indexOf(';')).trimmed();
@@ -695,10 +837,15 @@ qint64 QHttpNetworkReplyPrivate::getChunkSize(QIODevice *in, qint64 *chunkSize)
         } else {
             // read the fragment to the buffer
             char c = 0;
-            bytes += in->read(&c, 1);
+            qint64 haveRead = socket->read(&c, 1);
+            if (haveRead < 0) {
+                return -1; // FIXME
+            }
+            bytes += haveRead;
             fragment.append(c);
         }
     }
+
     return bytes;
 }
 
@@ -746,9 +893,15 @@ bool QHttpNetworkReplyPrivate::expectContent()
         || statusCode == 204 || statusCode == 304)
         return false;
     if (request.operation() == QHttpNetworkRequest::Head)
-        return !shouldEmitSignals();
-    if (contentLength() == 0)
+        return false; // no body expected for HEAD request
+    qint64 expectedContentLength = contentLength();
+    if (expectedContentLength == 0)
         return false;
+    if (expectedContentLength == -1 && bodyLength == 0) {
+        // The content-length header was stripped, but its value was 0.
+        // This would be the case for an explicitly zero-length compressed response.
+        return false;
+    }
     return true;
 }
 
@@ -765,9 +918,15 @@ void QHttpNetworkReplyPrivate::eraseData()
 QSslConfiguration QHttpNetworkReply::sslConfiguration() const
 {
     Q_D(const QHttpNetworkReply);
-    if (d->connection)
-        return d->connection->d_func()->sslConfiguration(*this);
-    return QSslConfiguration();
+
+    if (!d->connectionChannel)
+        return QSslConfiguration();
+
+    QSslSocket *sslSocket = qobject_cast<QSslSocket*>(d->connectionChannel->socket);
+    if (!sslSocket)
+        return QSslConfiguration();
+
+    return sslSocket->sslConfiguration();
 }
 
 void QHttpNetworkReply::setSslConfiguration(const QSslConfiguration &config)
