@@ -1,40 +1,40 @@
 /****************************************************************************
 **
-** Copyright (C) 2009 Nokia Corporation and/or its subsidiary(-ies).
+** Copyright (C) 2011 Nokia Corporation and/or its subsidiary(-ies).
 ** All rights reserved.
 ** Contact: Nokia Corporation (qt-info@nokia.com)
 **
 ** This file is part of the QtNetwork module of the Qt Toolkit.
 **
 ** $QT_BEGIN_LICENSE:LGPL$
-** Commercial Usage
-** Licensees holding valid Qt Commercial licenses may use this file in
-** accordance with the Qt Commercial License Agreement provided with the
-** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and Nokia.
-**
 ** GNU Lesser General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU Lesser
-** General Public License version 2.1 as published by the Free Software
-** Foundation and appearing in the file LICENSE.LGPL included in the
-** packaging of this file.  Please review the following information to
-** ensure the GNU Lesser General Public License version 2.1 requirements
-** will be met: http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
+** This file may be used under the terms of the GNU Lesser General Public
+** License version 2.1 as published by the Free Software Foundation and
+** appearing in the file LICENSE.LGPL included in the packaging of this
+** file. Please review the following information to ensure the GNU Lesser
+** General Public License version 2.1 requirements will be met:
+** http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
 **
 ** In addition, as a special exception, Nokia gives you certain additional
-** rights.  These rights are described in the Nokia Qt LGPL Exception
+** rights. These rights are described in the Nokia Qt LGPL Exception
 ** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
 **
 ** GNU General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU
-** General Public License version 3.0 as published by the Free Software
-** Foundation and appearing in the file LICENSE.GPL included in the
-** packaging of this file.  Please review the following information to
-** ensure the GNU General Public License version 3.0 requirements will be
-** met: http://www.gnu.org/copyleft/gpl.html.
+** Alternatively, this file may be used under the terms of the GNU General
+** Public License version 3.0 as published by the Free Software Foundation
+** and appearing in the file LICENSE.GPL included in the packaging of this
+** file. Please review the following information to ensure the GNU General
+** Public License version 3.0 requirements will be met:
+** http://www.gnu.org/copyleft/gpl.html.
+**
+** Other Usage
+** Alternatively, this file may be used in accordance with the terms and
+** conditions contained in a signed written agreement between you and Nokia.
+**
+**
+**
+**
 **
-** If you have questions regarding the use of this file, please contact
-** Nokia at qt-info@nokia.com.
 ** $QT_END_LICENSE$
 **
 ****************************************************************************/
@@ -428,9 +428,9 @@ void QNetmaskAddress::setPrefixLength(QAbstractSocket::NetworkLayerProtocol prot
     QHostAddress is normally used with the QTcpSocket, QTcpServer,
     and QUdpSocket to connect to a host or to set up a server.
 
-    A host address is set with setAddress(), checked for its type
-    using isIPv4Address() or isIPv6Address(), and retrieved with
-    toIPv4Address(), toIPv6Address(), or toString().
+    A host address is set with setAddress(), and retrieved with
+    toIPv4Address(), toIPv6Address(), or toString(). You can check the
+    type with protocol().
 
     \note Please note that QHostAddress does not do DNS lookups.
     QHostInfo is needed for that.
@@ -679,7 +679,8 @@ void QHostAddress::setAddress(const struct sockaddr *sockaddr)
     For example, if the address is 127.0.0.1, the returned value is
     2130706433 (i.e. 0x7f000001).
 
-    This value is only valid if isIp4Addr() returns true.
+    This value is only valid if the Protocol() is
+    \l{QAbstractSocket::}{IPv4Protocol}.
 
     \sa toString()
 */
@@ -704,7 +705,8 @@ QAbstractSocket::NetworkLayerProtocol QHostAddress::protocol() const
 
     \snippet doc/src/snippets/code/src_network_kernel_qhostaddress.cpp 0
 
-    This value is only valid if isIPv6Address() returns true.
+    This value is only valid if the protocol() is
+    \l{QAbstractSocket::}{IPv6Protocol}.
 
     \sa toString()
 */
@@ -718,7 +720,8 @@ Q_IPV6ADDR QHostAddress::toIPv6Address() const
     Returns the address as a string.
 
     For example, if the address is the IPv4 address 127.0.0.1, the
-    returned string is "127.0.0.1".
+    returned string is "127.0.0.1". For IPv6 the string format will 
+    follow the RFC5952 recommendation.
 
     \sa toIPv4Address()
 */
@@ -739,8 +742,32 @@ QString QHostAddress::toString() const
             ugle[i] = (quint16(d->a6[2*i]) << 8) | quint16(d->a6[2*i+1]);
         }
         QString s;
-        s.sprintf("%X:%X:%X:%X:%X:%X:%X:%X",
-                  ugle[0], ugle[1], ugle[2], ugle[3], ugle[4], ugle[5], ugle[6], ugle[7]);
+        QString temp;
+        bool zeroDetected = false;
+        bool zeroShortened = false;
+        for (int i = 0; i < 8; i++) {
+            if ((ugle[i] != 0) || zeroShortened) {
+                temp.sprintf("%X", ugle[i]);
+                s.append(temp);
+                if (zeroDetected)
+                    zeroShortened = true;
+            } else {
+                if (!zeroDetected) {
+                    if (i<7 && (ugle[i+1] == 0)) {
+                        s.append(QLatin1Char(':'));
+                        zeroDetected = true;
+                    } else {
+                        temp.sprintf("%X", ugle[i]);
+                        s.append(temp);
+                        if (i<7)
+                            s.append(QLatin1Char(':'));
+                    }
+                }
+            }
+            if (i<7 && ((ugle[i] != 0) || zeroShortened || (i==0 && zeroDetected)))
+                s.append(QLatin1Char(':'));
+        }
+
         if (!d->scopeId.isEmpty())
             s.append(QLatin1Char('%') + d->scopeId);
         return s;
@@ -1097,7 +1124,7 @@ uint qHash(const QHostAddress &key)
     Writes host address \a address to the stream \a out and returns a reference
     to the stream.
 
-    \sa {Format of the QDataStream operators}
+    \sa {Serializing Qt Data Types}
 */
 QDataStream &operator<<(QDataStream &out, const QHostAddress &address)
 {
@@ -1127,7 +1154,7 @@ QDataStream &operator<<(QDataStream &out, const QHostAddress &address)
     Reads a host address into \a address from the stream \a in and returns a
     reference to the stream.
 
-    \sa {Format of the QDataStream operators}
+    \sa {Serializing Qt Data Types}
 */
 QDataStream &operator>>(QDataStream &in, QHostAddress &address)
 {
