@@ -1,40 +1,40 @@
 /****************************************************************************
 **
-** Copyright (C) 2009 Nokia Corporation and/or its subsidiary(-ies).
+** Copyright (C) 2011 Nokia Corporation and/or its subsidiary(-ies).
 ** All rights reserved.
 ** Contact: Nokia Corporation (qt-info@nokia.com)
 **
 ** This file is part of the QtNetwork module of the Qt Toolkit.
 **
 ** $QT_BEGIN_LICENSE:LGPL$
-** Commercial Usage
-** Licensees holding valid Qt Commercial licenses may use this file in
-** accordance with the Qt Commercial License Agreement provided with the
-** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and Nokia.
-**
 ** GNU Lesser General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU Lesser
-** General Public License version 2.1 as published by the Free Software
-** Foundation and appearing in the file LICENSE.LGPL included in the
-** packaging of this file.  Please review the following information to
-** ensure the GNU Lesser General Public License version 2.1 requirements
-** will be met: http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
+** This file may be used under the terms of the GNU Lesser General Public
+** License version 2.1 as published by the Free Software Foundation and
+** appearing in the file LICENSE.LGPL included in the packaging of this
+** file. Please review the following information to ensure the GNU Lesser
+** General Public License version 2.1 requirements will be met:
+** http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
 **
 ** In addition, as a special exception, Nokia gives you certain additional
-** rights.  These rights are described in the Nokia Qt LGPL Exception
+** rights. These rights are described in the Nokia Qt LGPL Exception
 ** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
 **
 ** GNU General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU
-** General Public License version 3.0 as published by the Free Software
-** Foundation and appearing in the file LICENSE.GPL included in the
-** packaging of this file.  Please review the following information to
-** ensure the GNU General Public License version 3.0 requirements will be
-** met: http://www.gnu.org/copyleft/gpl.html.
+** Alternatively, this file may be used under the terms of the GNU General
+** Public License version 3.0 as published by the Free Software Foundation
+** and appearing in the file LICENSE.GPL included in the packaging of this
+** file. Please review the following information to ensure the GNU General
+** Public License version 3.0 requirements will be met:
+** http://www.gnu.org/copyleft/gpl.html.
+**
+** Other Usage
+** Alternatively, this file may be used in accordance with the terms and
+** conditions contained in a signed written agreement between you and Nokia.
+**
+**
+**
+**
 **
-** If you have questions regarding the use of this file, please contact
-** Nokia at qt-info@nokia.com.
 ** $QT_END_LICENSE$
 **
 ****************************************************************************/
@@ -42,9 +42,10 @@
 #include "qhttpsocketengine_p.h"
 #include "qtcpsocket.h"
 #include "qhostaddress.h"
-#include "qdatetime.h"
 #include "qurl.h"
 #include "qhttp.h"
+#include "qelapsedtimer.h"
+#include "qnetworkinterface.h"
 
 #if !defined(QT_NO_NETWORKPROXY) && !defined(QT_NO_HTTP)
 #include <qdebug.h>
@@ -71,6 +72,9 @@ bool QHttpSocketEngine::initialize(QAbstractSocket::SocketType type, QAbstractSo
     setProtocol(protocol);
     setSocketType(type);
     d->socket = new QTcpSocket(this);
+#ifndef QT_NO_BEARERMANAGEMENT
+    d->socket->setProperty("_q_networkSession", property("_q_networkSession"));
+#endif
 
     // Explicitly disable proxying on the proxy socket itself to avoid
     // unwanted recursion.
@@ -145,6 +149,8 @@ bool QHttpSocketEngine::connectInternal()
     // Handshake isn't done. If unconnected, start connecting.
     if (d->state == None && d->socket->state() == QAbstractSocket::UnconnectedState) {
         setState(QAbstractSocket::ConnectingState);
+        //limit buffer in internal socket, data is buffered in the external socket under application control
+        d->socket->setReadBufferSize(65536);
         d->socket->connectToHost(d->proxy.hostName(), d->proxy.port());
     }
 
@@ -212,28 +218,14 @@ qint64 QHttpSocketEngine::bytesAvailable() const
 qint64 QHttpSocketEngine::read(char *data, qint64 maxlen)
 {
     Q_D(QHttpSocketEngine);
-    qint64 bytesRead = 0;
-
-    if (!d->readBuffer.isEmpty()) {
-        // Read as much from the buffer as we can.
-        bytesRead = qMin((qint64)d->readBuffer.size(), maxlen);
-        memcpy(data, d->readBuffer.constData(), bytesRead);
-        data += bytesRead;
-        maxlen -= bytesRead;
-        d->readBuffer = d->readBuffer.mid(bytesRead);
-    }
-
-    qint64 bytesReadFromSocket = d->socket->read(data, maxlen);
+    qint64 bytesRead = d->socket->read(data, maxlen);
 
     if (d->socket->state() == QAbstractSocket::UnconnectedState
         && d->socket->bytesAvailable() == 0) {
         emitReadNotification();
     }
 
-    if (bytesReadFromSocket > 0) {
-        // Add to what we read so far.
-        bytesRead += bytesReadFromSocket;
-    } else if (bytesRead == 0 && bytesReadFromSocket == -1) {
+    if (bytesRead == -1) {
         // If nothing has been read so far, and the direct socket read
         // failed, return the socket's error. Otherwise, fall through and
         // return as much as we read so far.
@@ -253,6 +245,36 @@ qint64 QHttpSocketEngine::write(const char *data, qint64 len)
 }
 
 #ifndef QT_NO_UDPSOCKET
+#ifndef QT_NO_NETWORKINTERFACE
+bool QHttpSocketEngine::joinMulticastGroup(const QHostAddress &,
+                                           const QNetworkInterface &)
+{
+    setError(QAbstractSocket::UnsupportedSocketOperationError,
+             QLatin1String("Operation on socket is not supported"));
+    return false;
+}
+
+bool QHttpSocketEngine::leaveMulticastGroup(const QHostAddress &,
+                                            const QNetworkInterface &)
+{
+    setError(QAbstractSocket::UnsupportedSocketOperationError,
+             QLatin1String("Operation on socket is not supported"));
+    return false;
+}
+
+QNetworkInterface QHttpSocketEngine::multicastInterface() const
+{
+    return QNetworkInterface();
+}
+
+bool QHttpSocketEngine::setMulticastInterface(const QNetworkInterface &)
+{
+    setError(QAbstractSocket::UnsupportedSocketOperationError,
+             QLatin1String("Operation on socket is not supported"));
+    return false;
+}
+#endif // QT_NO_NETWORKINTERFACE
+
 qint64 QHttpSocketEngine::readDatagram(char *, qint64, QHostAddress *,
                                        quint16 *)
 {
@@ -333,7 +355,7 @@ bool QHttpSocketEngine::waitForRead(int msecs, bool *timedOut)
     if (!d->socket || d->socket->state() == QAbstractSocket::UnconnectedState)
         return false;
 
-    QTime stopWatch;
+    QElapsedTimer stopWatch;
     stopWatch.start();
 
     // Wait for more data if nothing is available.
@@ -380,7 +402,7 @@ bool QHttpSocketEngine::waitForWrite(int msecs, bool *timedOut)
         return true;
     }
 
-    QTime stopWatch;
+    QElapsedTimer stopWatch;
     stopWatch.start();
 
     // If we're not connected yet, wait until we are, and until bytes have
@@ -481,7 +503,13 @@ void QHttpSocketEngine::slotSocketConnected()
     data += path;
     data += " HTTP/1.1\r\n";
     data += "Proxy-Connection: keep-alive\r\n"
-            "User-Agent: Mozilla/5.0\r\n"
+            "User-Agent: ";
+    QVariant v = property("_q_user-agent");
+    if (v.isValid())
+        data += v.toByteArray();
+    else
+        data += "Mozilla/5.0";
+    data += "\r\n"
             "Host: " + peerAddress + "\r\n";
     QAuthenticatorPrivate *priv = QAuthenticatorPrivate::getPrivate(d->authenticator);
     //qDebug() << "slotSocketConnected: priv=" << priv << (priv ? (int)priv->method : -1);
@@ -560,7 +588,7 @@ void QHttpSocketEngine::slotSocketReadNotification()
     }
 
     QHttpResponseHeader responseHeader(QString::fromLatin1(d->readBuffer));
-    d->readBuffer.clear();
+    d->readBuffer.clear(); // we parsed the proxy protocol response. from now on direct socket reading will be done
 
     int statusCode = responseHeader.statusCode();
     if (statusCode == 200) {
@@ -586,6 +614,10 @@ void QHttpSocketEngine::slotSocketReadNotification()
 
         bool willClose;
         QString proxyConnectionHeader = responseHeader.value(QLatin1String("Proxy-Connection"));
+        // Although most proxies use the unofficial Proxy-Connection header, the Connection header
+        // from http spec is also allowed.
+        if (proxyConnectionHeader.isEmpty())
+            proxyConnectionHeader = responseHeader.value(QLatin1String("Connection"));
         proxyConnectionHeader = proxyConnectionHeader.toLower();
         if (proxyConnectionHeader == QLatin1String("close")) {
             willClose = true;
@@ -689,11 +721,10 @@ void QHttpSocketEngine::slotSocketError(QAbstractSocket::SocketError error)
 
     d->state = None;
     setError(error, d->socket->errorString());
-    if (error == QAbstractSocket::RemoteHostClosedError) {
-        emitReadNotification();
-    } else {
+    if (error != QAbstractSocket::RemoteHostClosedError)
         qDebug() << "QHttpSocketEngine::slotSocketError: got weird error =" << error;
-    }
+    //read notification needs to always be emitted, otherwise the higher layer doesn't get the disconnected signal
+    emitReadNotification();
 }
 
 void QHttpSocketEngine::slotSocketStateChanged(QAbstractSocket::SocketState state)
@@ -728,7 +759,10 @@ void QHttpSocketEngine::emitReadNotification()
 {
     Q_D(QHttpSocketEngine);
     d->readNotificationActivated = true;
-    if (d->readNotificationEnabled && !d->readNotificationPending) {
+    // if there is a connection notification pending we have to emit the readNotification
+    // incase there is connection error. This is only needed for Windows, but it does not
+    // hurt in other cases.
+    if ((d->readNotificationEnabled && !d->readNotificationPending) || d->connectionNotificationPending) {
         d->readNotificationPending = true;
         QMetaObject::invokeMethod(this, "emitPendingReadNotification", Qt::QueuedConnection);
     }
