@@ -1,40 +1,40 @@
 /****************************************************************************
 **
-** Copyright (C) 2009 Nokia Corporation and/or its subsidiary(-ies).
+** Copyright (C) 2011 Nokia Corporation and/or its subsidiary(-ies).
 ** All rights reserved.
 ** Contact: Nokia Corporation (qt-info@nokia.com)
 **
 ** This file is part of the QtNetwork module of the Qt Toolkit.
 **
 ** $QT_BEGIN_LICENSE:LGPL$
-** Commercial Usage
-** Licensees holding valid Qt Commercial licenses may use this file in
-** accordance with the Qt Commercial License Agreement provided with the
-** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and Nokia.
-**
 ** GNU Lesser General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU Lesser
-** General Public License version 2.1 as published by the Free Software
-** Foundation and appearing in the file LICENSE.LGPL included in the
-** packaging of this file.  Please review the following information to
-** ensure the GNU Lesser General Public License version 2.1 requirements
-** will be met: http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
+** This file may be used under the terms of the GNU Lesser General Public
+** License version 2.1 as published by the Free Software Foundation and
+** appearing in the file LICENSE.LGPL included in the packaging of this
+** file. Please review the following information to ensure the GNU Lesser
+** General Public License version 2.1 requirements will be met:
+** http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
 **
 ** In addition, as a special exception, Nokia gives you certain additional
-** rights.  These rights are described in the Nokia Qt LGPL Exception
+** rights. These rights are described in the Nokia Qt LGPL Exception
 ** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
 **
 ** GNU General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU
-** General Public License version 3.0 as published by the Free Software
-** Foundation and appearing in the file LICENSE.GPL included in the
-** packaging of this file.  Please review the following information to
-** ensure the GNU General Public License version 3.0 requirements will be
-** met: http://www.gnu.org/copyleft/gpl.html.
+** Alternatively, this file may be used under the terms of the GNU General
+** Public License version 3.0 as published by the Free Software Foundation
+** and appearing in the file LICENSE.GPL included in the packaging of this
+** file. Please review the following information to ensure the GNU General
+** Public License version 3.0 requirements will be met:
+** http://www.gnu.org/copyleft/gpl.html.
+**
+** Other Usage
+** Alternatively, this file may be used in accordance with the terms and
+** conditions contained in a signed written agreement between you and Nokia.
+**
+**
+**
+**
 **
-** If you have questions regarding the use of this file, please contact
-** Nokia at qt-info@nokia.com.
 ** $QT_END_LICENSE$
 **
 ****************************************************************************/
@@ -49,13 +49,14 @@
 #include "qdebug.h"
 #include "qhash.h"
 #include "qqueue.h"
-#include "qdatetime.h"
+#include "qelapsedtimer.h"
 #include "qmutex.h"
 #include "qthread.h"
 #include "qcoreapplication.h"
 #include "qurl.h"
 #include "qauthenticator.h"
 #include <qendian.h>
+#include <qnetworkinterface.h>
 
 QT_BEGIN_NAMESPACE
 
@@ -308,7 +309,7 @@ struct QSocks5BindData : public QSocks5Data
     quint16 localPort;
     QHostAddress peerAddress;
     quint16 peerPort;
-    QDateTime timeStamp;
+    QElapsedTimer timeStamp;
 };
 
 struct QSocks5RevivedDatagram
@@ -369,7 +370,7 @@ void QSocks5BindStore::add(int socketDescriptor, QSocks5BindData *bindData)
     if (store.contains(socketDescriptor)) {
         // qDebug() << "delete it";
     }
-    bindData->timeStamp = QDateTime::currentDateTime();
+    bindData->timeStamp.start();
     store.insert(socketDescriptor, bindData);
     // start sweep timer if not started
     if (sweepTimerId == -1)
@@ -412,7 +413,7 @@ void QSocks5BindStore::timerEvent(QTimerEvent * event)
         QMutableHashIterator<int, QSocks5BindData *> it(store);
         while (it.hasNext()) {
             it.next();
-            if (it.value()->timeStamp.secsTo(QDateTime::currentDateTime()) > 350) {
+            if (it.value()->timeStamp.hasExpired(350000)) {
                 QSOCKS5_DEBUG << "QSocks5BindStore removing JJJJ";
                 it.remove();
             }
@@ -555,6 +556,9 @@ void QSocks5SocketEnginePrivate::initialize(Socks5Mode socks5Mode)
         udpData = new QSocks5UdpAssociateData;
         data = udpData;
         udpData->udpSocket = new QUdpSocket(q);
+#ifndef QT_NO_BEARERMANAGEMENT
+        udpData->udpSocket->setProperty("_q_networksession", q->property("_q_networksession"));
+#endif
         udpData->udpSocket->setProxy(QNetworkProxy::NoProxy);
         QObject::connect(udpData->udpSocket, SIGNAL(readyRead()),
                          q, SLOT(_q_udpSocketReadNotification()),
@@ -566,6 +570,9 @@ void QSocks5SocketEnginePrivate::initialize(Socks5Mode socks5Mode)
     }
 
     data->controlSocket = new QTcpSocket(q);
+#ifndef QT_NO_BEARERMANAGEMENT
+    data->controlSocket->setProperty("_q_networksession", q->property("_q_networksession"));
+#endif
     data->controlSocket->setProxy(QNetworkProxy::NoProxy);
     QObject::connect(data->controlSocket, SIGNAL(connected()), q, SLOT(_q_controlSocketConnected()),
                      Qt::DirectConnection);
@@ -1119,6 +1126,8 @@ bool QSocks5SocketEngine::connectInternal()
     if (d->socks5State == QSocks5SocketEnginePrivate::Uninitialized
         && d->socketState != QAbstractSocket::ConnectingState) {
         setState(QAbstractSocket::ConnectingState);
+        //limit buffer in internal socket, data is buffered in the external socket under application control
+        d->data->controlSocket->setReadBufferSize(65536);
         d->data->controlSocket->connectToHost(d->proxyInfo.hostName(), d->proxyInfo.port());
         return false;
     }
@@ -1355,7 +1364,7 @@ bool QSocks5SocketEngine::bind(const QHostAddress &address, quint16 port)
     }
 
     int msecs = SOCKS5_BLOCKING_BIND_TIMEOUT;
-    QTime stopWatch;
+    QElapsedTimer stopWatch;
     stopWatch.start();
     d->data->controlSocket->connectToHost(d->proxyInfo.hostName(), d->proxyInfo.port());
     if (!d->waitForConnected(msecs, 0) ||
@@ -1375,6 +1384,9 @@ bool QSocks5SocketEngine::bind(const QHostAddress &address, quint16 port)
         d->udpData->associatePort = d->localPort;
         d->localPort = 0;
         QUdpSocket dummy;
+#ifndef QT_NO_BEARERMANAGEMENT
+        dummy.setProperty("_q_networksession", property("_q_networksession"));
+#endif
         dummy.setProxy(QNetworkProxy::NoProxy);
         if (!dummy.bind()
             || writeDatagram(0,0, d->data->controlSocket->localAddress(), dummy.localPort()) != 0
@@ -1455,7 +1467,7 @@ void QSocks5SocketEngine::close()
     if (d->data && d->data->controlSocket) {
         if (d->data->controlSocket->state() == QAbstractSocket::ConnectedState) {
             int msecs = 100;
-            QTime stopWatch;
+            QElapsedTimer stopWatch;
             stopWatch.start();
             while (!d->data->controlSocket->bytesToWrite()) {
                if (!d->data->controlSocket->waitForBytesWritten(qt_timeout_value(msecs, stopWatch.elapsed())))
@@ -1530,8 +1542,13 @@ qint64 QSocks5SocketEngine::write(const char *data, qint64 len)
             // ### Handle this error.
         }
 
-        d->data->controlSocket->write(sealedBuf);
+        qint64 written = d->data->controlSocket->write(sealedBuf);
+        if (written <= 0) {
+            QSOCKS5_Q_DEBUG << "native write returned" << written;
+            return written;
+        }
         d->data->controlSocket->waitForBytesWritten(0);
+        //NB: returning len rather than written for the OK case, because the "sealing" may increase the length
         return len;
 #ifndef QT_NO_UDPSOCKET
     } else if (d->mode == QSocks5SocketEnginePrivate::UdpAssociateMode) {
@@ -1544,6 +1561,37 @@ qint64 QSocks5SocketEngine::write(const char *data, qint64 len)
 }
 
 #ifndef QT_NO_UDPSOCKET
+#ifndef QT_NO_NETWORKINTERFACE
+bool QSocks5SocketEngine::joinMulticastGroup(const QHostAddress &,
+                                             const QNetworkInterface &)
+{
+    setError(QAbstractSocket::UnsupportedSocketOperationError,
+             QLatin1String("Operation on socket is not supported"));
+    return false;
+}
+
+bool QSocks5SocketEngine::leaveMulticastGroup(const QHostAddress &,
+                                              const QNetworkInterface &)
+{
+    setError(QAbstractSocket::UnsupportedSocketOperationError,
+             QLatin1String("Operation on socket is not supported"));
+    return false;
+}
+
+
+QNetworkInterface QSocks5SocketEngine::multicastInterface() const
+{
+    return QNetworkInterface();
+}
+
+bool QSocks5SocketEngine::setMulticastInterface(const QNetworkInterface &)
+{
+    setError(QAbstractSocket::UnsupportedSocketOperationError,
+             QLatin1String("Operation on socket is not supported"));
+    return false;
+}
+#endif // QT_NO_NETWORKINTERFACE
+
 qint64 QSocks5SocketEngine::readDatagram(char *data, qint64 maxlen, QHostAddress *addr,
                                         quint16 *port)
 {
@@ -1674,7 +1722,7 @@ bool QSocks5SocketEnginePrivate::waitForConnected(int msecs, bool *timedOut)
         mode == BindMode ? BindSuccess :
         UdpAssociateSuccess;
 
-    QTime stopWatch;
+    QElapsedTimer stopWatch;
     stopWatch.start();
 
     while (socks5State != wantedState) {
@@ -1699,7 +1747,7 @@ bool QSocks5SocketEngine::waitForRead(int msecs, bool *timedOut)
 
     d->readNotificationActivated = false;
 
-    QTime stopWatch;
+    QElapsedTimer stopWatch;
     stopWatch.start();
 
     // are we connected yet?
@@ -1749,7 +1797,7 @@ bool QSocks5SocketEngine::waitForWrite(int msecs, bool *timedOut)
     Q_D(QSocks5SocketEngine);
     QSOCKS5_DEBUG << "waitForWrite" << msecs;
 
-    QTime stopWatch;
+    QElapsedTimer stopWatch;
     stopWatch.start();
 
     // are we connected yet?
