@@ -1,40 +1,40 @@
 /****************************************************************************
 **
-** Copyright (C) 2010 Nokia Corporation and/or its subsidiary(-ies).
+** Copyright (C) 2011 Nokia Corporation and/or its subsidiary(-ies).
 ** All rights reserved.
 ** Contact: Nokia Corporation (qt-info@nokia.com)
 **
 ** This file is part of the QtOpenGL module of the Qt Toolkit.
 **
 ** $QT_BEGIN_LICENSE:LGPL$
-** Commercial Usage
-** Licensees holding valid Qt Commercial licenses may use this file in
-** accordance with the Qt Commercial License Agreement provided with the
-** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and Nokia.
-**
 ** GNU Lesser General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU Lesser
-** General Public License version 2.1 as published by the Free Software
-** Foundation and appearing in the file LICENSE.LGPL included in the
-** packaging of this file.  Please review the following information to
-** ensure the GNU Lesser General Public License version 2.1 requirements
-** will be met: http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
+** This file may be used under the terms of the GNU Lesser General Public
+** License version 2.1 as published by the Free Software Foundation and
+** appearing in the file LICENSE.LGPL included in the packaging of this
+** file. Please review the following information to ensure the GNU Lesser
+** General Public License version 2.1 requirements will be met:
+** http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
 **
 ** In addition, as a special exception, Nokia gives you certain additional
-** rights.  These rights are described in the Nokia Qt LGPL Exception
+** rights. These rights are described in the Nokia Qt LGPL Exception
 ** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
 **
 ** GNU General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU
-** General Public License version 3.0 as published by the Free Software
-** Foundation and appearing in the file LICENSE.GPL included in the
-** packaging of this file.  Please review the following information to
-** ensure the GNU General Public License version 3.0 requirements will be
-** met: http://www.gnu.org/copyleft/gpl.html.
+** Alternatively, this file may be used under the terms of the GNU General
+** Public License version 3.0 as published by the Free Software Foundation
+** and appearing in the file LICENSE.GPL included in the packaging of this
+** file. Please review the following information to ensure the GNU General
+** Public License version 3.0 requirements will be met:
+** http://www.gnu.org/copyleft/gpl.html.
+**
+** Other Usage
+** Alternatively, this file may be used in accordance with the terms and
+** conditions contained in a signed written agreement between you and Nokia.
+**
+**
+**
+**
 **
-** If you have questions regarding the use of this file, please contact
-** Nokia at qt-info@nokia.com.
 ** $QT_END_LICENSE$
 **
 ****************************************************************************/
@@ -44,6 +44,7 @@
 
 #include <qdebug.h>
 #include <private/qgl_p.h>
+#include <private/qfont_p.h>
 #if !defined(QT_OPENGL_ES_1)
 #include <private/qpaintengineex_opengl2_p.h>
 #endif
@@ -201,6 +202,38 @@ int QGLFramebufferObjectFormat::samples() const
     return d->samples;
 }
 
+/*!
+    \since 4.8
+
+    Enables mipmapping if \a enabled is true; otherwise disables it.
+
+    Mipmapping is disabled by default.
+
+    If mipmapping is enabled, additional memory will be allocated for
+    the mipmap levels. The mipmap levels can be updated by binding the
+    texture and calling glGenerateMipmap(). Mipmapping cannot be enabled
+    for multisampled framebuffer objects.
+
+    \sa mipmap(), QGLFramebufferObject::texture()
+*/
+void QGLFramebufferObjectFormat::setMipmap(bool enabled)
+{
+    detach();
+    d->mipmap = enabled;
+}
+
+/*!
+    \since 4.8
+
+    Returns true if mipmapping is enabled.
+
+    \sa setMipmap()
+*/
+bool QGLFramebufferObjectFormat::mipmap() const
+{
+    return d->mipmap;
+}
+
 /*!
     Sets the attachment configuration of a framebuffer object to \a attachment.
 
@@ -324,6 +357,10 @@ void QGLFBOGLPaintDevice::setFBO(QGLFramebufferObject* f,
         fboFormat.setStencil(true);
     } else if (attachment == QGLFramebufferObject::Depth) {
         fboFormat.setDepth(true);
+        fboFormat.setStencil(false);
+    } else {
+        fboFormat.setDepth(false);
+        fboFormat.setStencil(false);
     }
 
     GLenum format = f->format().internalTextureFormat();
@@ -394,7 +431,8 @@ bool QGLFramebufferObjectPrivate::checkFramebufferStatus() const
 
 void QGLFramebufferObjectPrivate::init(QGLFramebufferObject *q, const QSize &sz,
                                        QGLFramebufferObject::Attachment attachment,
-                                       GLenum texture_target, GLenum internal_format, GLint samples)
+                                       GLenum texture_target, GLenum internal_format,
+                                       GLint samples, bool mipmap)
 {
     QGLContext *ctx = const_cast<QGLContext *>(QGLContext::currentContext());
     fbo_guard.setContext(ctx);
@@ -422,6 +460,8 @@ void QGLFramebufferObjectPrivate::init(QGLFramebufferObject *q, const QSize &sz,
         glBindTexture(target, texture);
         glTexImage2D(target, 0, internal_format, size.width(), size.height(), 0,
                 GL_RGBA, GL_UNSIGNED_BYTE, NULL);
+        if (mipmap)
+            glGenerateMipmap(GL_TEXTURE_2D);
 #ifndef QT_OPENGL_ES
         glTexParameteri(target, GL_TEXTURE_MIN_FILTER, GL_NEAREST);
         glTexParameteri(target, GL_TEXTURE_MAG_FILTER, GL_NEAREST);
@@ -442,6 +482,7 @@ void QGLFramebufferObjectPrivate::init(QGLFramebufferObject *q, const QSize &sz,
 
         color_buffer = 0;
     } else {
+        mipmap = false;
         GLint maxSamples;
         glGetIntegerv(GL_MAX_SAMPLES_EXT, &maxSamples);
 
@@ -468,13 +509,17 @@ void QGLFramebufferObjectPrivate::init(QGLFramebufferObject *q, const QSize &sz,
             glGetRenderbufferParameteriv(GL_RENDERBUFFER_EXT, GL_RENDERBUFFER_SAMPLES_EXT, &samples);
     }
 
+    // In practice, a combined depth-stencil buffer is supported by all desktop platforms, while a
+    // separate stencil buffer is not. On embedded devices however, a combined depth-stencil buffer
+    // might not be supported while separate buffers are, according to QTBUG-12861.
+
     if (attachment == QGLFramebufferObject::CombinedDepthStencil
         && (QGLExtensions::glExtensions() & QGLExtensions::PackedDepthStencil)) {
         // depth and stencil buffer needs another extension
-        glGenRenderbuffers(1, &depth_stencil_buffer);
-        Q_ASSERT(!glIsRenderbuffer(depth_stencil_buffer));
-        glBindRenderbuffer(GL_RENDERBUFFER_EXT, depth_stencil_buffer);
-        Q_ASSERT(glIsRenderbuffer(depth_stencil_buffer));
+        glGenRenderbuffers(1, &depth_buffer);
+        Q_ASSERT(!glIsRenderbuffer(depth_buffer));
+        glBindRenderbuffer(GL_RENDERBUFFER_EXT, depth_buffer);
+        Q_ASSERT(glIsRenderbuffer(depth_buffer));
         if (samples != 0 && glRenderbufferStorageMultisampleEXT)
             glRenderbufferStorageMultisampleEXT(GL_RENDERBUFFER_EXT, samples,
                 GL_DEPTH24_STENCIL8_EXT, size.width(), size.height());
@@ -482,49 +527,99 @@ void QGLFramebufferObjectPrivate::init(QGLFramebufferObject *q, const QSize &sz,
             glRenderbufferStorage(GL_RENDERBUFFER_EXT,
                 GL_DEPTH24_STENCIL8_EXT, size.width(), size.height());
 
-        GLint i = 0;
-        glGetRenderbufferParameteriv(GL_RENDERBUFFER_EXT, GL_RENDERBUFFER_DEPTH_SIZE_EXT, &i);
+        stencil_buffer = depth_buffer;
         glFramebufferRenderbuffer(GL_FRAMEBUFFER_EXT, GL_DEPTH_ATTACHMENT_EXT,
-                                     GL_RENDERBUFFER_EXT, depth_stencil_buffer);
+                                     GL_RENDERBUFFER_EXT, depth_buffer);
         glFramebufferRenderbuffer(GL_FRAMEBUFFER_EXT, GL_STENCIL_ATTACHMENT_EXT,
-                                     GL_RENDERBUFFER_EXT, depth_stencil_buffer);
-        fbo_attachment = QGLFramebufferObject::CombinedDepthStencil;
+                                     GL_RENDERBUFFER_EXT, stencil_buffer);
 
         valid = checkFramebufferStatus();
-        if (!valid)
-            glDeleteRenderbuffers(1, &depth_stencil_buffer);
-    } else if (attachment == QGLFramebufferObject::Depth
-               || attachment == QGLFramebufferObject::CombinedDepthStencil)
+        if (!valid) {
+            glDeleteRenderbuffers(1, &depth_buffer);
+            stencil_buffer = depth_buffer = 0;
+        }
+    }
+
+    if (depth_buffer == 0 && (attachment == QGLFramebufferObject::CombinedDepthStencil
+        || (attachment == QGLFramebufferObject::Depth)))
     {
-        glGenRenderbuffers(1, &depth_stencil_buffer);
-        Q_ASSERT(!glIsRenderbuffer(depth_stencil_buffer));
-        glBindRenderbuffer(GL_RENDERBUFFER_EXT, depth_stencil_buffer);
-        Q_ASSERT(glIsRenderbuffer(depth_stencil_buffer));
+        glGenRenderbuffers(1, &depth_buffer);
+        Q_ASSERT(!glIsRenderbuffer(depth_buffer));
+        glBindRenderbuffer(GL_RENDERBUFFER_EXT, depth_buffer);
+        Q_ASSERT(glIsRenderbuffer(depth_buffer));
         if (samples != 0 && glRenderbufferStorageMultisampleEXT) {
 #ifdef QT_OPENGL_ES
-#define GL_DEPTH_COMPONENT16 0x81A5
-            glRenderbufferStorageMultisampleEXT(GL_RENDERBUFFER_EXT, samples,
-                GL_DEPTH_COMPONENT16, size.width(), size.height());
+            if (QGLExtensions::glExtensions() & QGLExtensions::Depth24) {
+                glRenderbufferStorageMultisampleEXT(GL_RENDERBUFFER_EXT, samples,
+                    GL_DEPTH_COMPONENT24_OES, size.width(), size.height());
+            } else {
+                glRenderbufferStorageMultisampleEXT(GL_RENDERBUFFER_EXT, samples,
+                    GL_DEPTH_COMPONENT16, size.width(), size.height());
+            }
 #else
             glRenderbufferStorageMultisampleEXT(GL_RENDERBUFFER_EXT, samples,
                 GL_DEPTH_COMPONENT, size.width(), size.height());
 #endif
         } else {
 #ifdef QT_OPENGL_ES
-#define GL_DEPTH_COMPONENT16 0x81A5
-            glRenderbufferStorage(GL_RENDERBUFFER_EXT, GL_DEPTH_COMPONENT16, size.width(), size.height());
+            if (QGLExtensions::glExtensions() & QGLExtensions::Depth24) {
+                glRenderbufferStorage(GL_RENDERBUFFER_EXT, GL_DEPTH_COMPONENT24_OES, 
+                                        size.width(), size.height());
+            } else {
+                glRenderbufferStorage(GL_RENDERBUFFER_EXT, GL_DEPTH_COMPONENT16, 
+                                        size.width(), size.height());
+            }
 #else
             glRenderbufferStorage(GL_RENDERBUFFER_EXT, GL_DEPTH_COMPONENT, size.width(), size.height());
 #endif
         }
-        GLint i = 0;
-        glGetRenderbufferParameteriv(GL_RENDERBUFFER_EXT, GL_RENDERBUFFER_DEPTH_SIZE_EXT, &i);
         glFramebufferRenderbuffer(GL_FRAMEBUFFER_EXT, GL_DEPTH_ATTACHMENT_EXT,
-                                     GL_RENDERBUFFER_EXT, depth_stencil_buffer);
-        fbo_attachment = QGLFramebufferObject::Depth;
+                                     GL_RENDERBUFFER_EXT, depth_buffer);
         valid = checkFramebufferStatus();
-        if (!valid)
-            glDeleteRenderbuffers(1, &depth_stencil_buffer);
+        if (!valid) {
+            glDeleteRenderbuffers(1, &depth_buffer);
+            depth_buffer = 0;
+        }
+    }
+
+    if (stencil_buffer == 0 && (attachment == QGLFramebufferObject::CombinedDepthStencil)) {
+        glGenRenderbuffers(1, &stencil_buffer);
+        Q_ASSERT(!glIsRenderbuffer(stencil_buffer));
+        glBindRenderbuffer(GL_RENDERBUFFER_EXT, stencil_buffer);
+        Q_ASSERT(glIsRenderbuffer(stencil_buffer));
+        if (samples != 0 && glRenderbufferStorageMultisampleEXT) {
+#ifdef QT_OPENGL_ES
+            glRenderbufferStorageMultisampleEXT(GL_RENDERBUFFER_EXT, samples,
+                GL_STENCIL_INDEX8_EXT, size.width(), size.height());
+#else
+            glRenderbufferStorageMultisampleEXT(GL_RENDERBUFFER_EXT, samples,
+                GL_STENCIL_INDEX, size.width(), size.height());
+#endif
+        } else {
+#ifdef QT_OPENGL_ES
+            glRenderbufferStorage(GL_RENDERBUFFER_EXT, GL_STENCIL_INDEX8_EXT,
+                                  size.width(), size.height());
+#else
+            glRenderbufferStorage(GL_RENDERBUFFER_EXT, GL_STENCIL_INDEX,
+                                  size.width(), size.height());
+#endif
+        }
+        glFramebufferRenderbuffer(GL_FRAMEBUFFER_EXT, GL_STENCIL_ATTACHMENT_EXT,
+                                  GL_RENDERBUFFER_EXT, stencil_buffer);
+        valid = checkFramebufferStatus();
+        if (!valid) {
+            glDeleteRenderbuffers(1, &stencil_buffer);
+            stencil_buffer = 0;
+        }
+    }
+
+    // The FBO might have become valid after removing the depth or stencil buffer.
+    valid = checkFramebufferStatus();
+
+    if (depth_buffer && stencil_buffer) {
+        fbo_attachment = QGLFramebufferObject::CombinedDepthStencil;
+    } else if (depth_buffer) {
+        fbo_attachment = QGLFramebufferObject::Depth;
     } else {
         fbo_attachment = QGLFramebufferObject::NoAttachment;
     }
@@ -535,6 +630,10 @@ void QGLFramebufferObjectPrivate::init(QGLFramebufferObject *q, const QSize &sz,
             glDeleteRenderbuffers(1, &color_buffer);
         else
             glDeleteTextures(1, &texture);
+        if (depth_buffer)
+            glDeleteRenderbuffers(1, &depth_buffer);
+        if (stencil_buffer && depth_buffer != stencil_buffer)
+            glDeleteRenderbuffers(1, &stencil_buffer);
         glDeleteFramebuffers(1, &fbo);
         fbo_guard.setId(0);
     }
@@ -544,6 +643,7 @@ void QGLFramebufferObjectPrivate::init(QGLFramebufferObject *q, const QSize &sz,
     format.setSamples(int(samples));
     format.setAttachment(fbo_attachment);
     format.setInternalTextureFormat(internal_format);
+    format.setMipmap(mipmap);
 }
 
 /*!
@@ -616,6 +716,13 @@ void QGLFramebufferObjectPrivate::init(QGLFramebufferObject *q, const QSize &sz,
     as a texture, you first need to copy from it to a regular framebuffer
     object using QGLContext::blitFramebuffer().
 
+    \section1 Threading
+
+    As of Qt 4.8, it's possible to draw into a QGLFramebufferObject
+    using a QPainter in a separate thread. Note that OpenGL 2.0 or
+    OpenGL ES 2.0 is required for this to work. Also, under X11, it's
+    necessary to set the Qt::AA_X11InitThreads application attribute.
+
     \sa {Framebuffer Object Example}
 */
 
@@ -708,7 +815,7 @@ QGLFramebufferObject::QGLFramebufferObject(const QSize &size, const QGLFramebuff
 {
     Q_D(QGLFramebufferObject);
     d->init(this, size, format.attachment(), format.textureTarget(), format.internalTextureFormat(),
-            format.samples());
+            format.samples(), format.mipmap());
 }
 
 /*! \overload
@@ -722,7 +829,7 @@ QGLFramebufferObject::QGLFramebufferObject(int width, int height, const QGLFrame
 {
     Q_D(QGLFramebufferObject);
     d->init(this, QSize(width, height), format.attachment(), format.textureTarget(),
-            format.internalTextureFormat(), format.samples());
+            format.internalTextureFormat(), format.samples(), format.mipmap());
 }
 
 #ifdef Q_MAC_COMPAT_GL_FUNCTIONS
@@ -817,8 +924,10 @@ QGLFramebufferObject::~QGLFramebufferObject()
             glDeleteTextures(1, &d->texture);
         if (d->color_buffer)
             glDeleteRenderbuffers(1, &d->color_buffer);
-        if (d->depth_stencil_buffer)
-            glDeleteRenderbuffers(1, &d->depth_stencil_buffer);
+        if (d->depth_buffer)
+            glDeleteRenderbuffers(1, &d->depth_buffer);
+        if (d->stencil_buffer && d->stencil_buffer != d->depth_buffer)
+            glDeleteRenderbuffers(1, &d->stencil_buffer);
         GLuint fbo = d->fbo();
         glDeleteFramebuffers(1, &fbo);
     }
@@ -984,11 +1093,11 @@ QImage QGLFramebufferObject::toImage() const
 }
 
 #if !defined(QT_OPENGL_ES_1)
-Q_GLOBAL_STATIC(QGL2PaintEngineEx, qt_buffer_2_engine)
+Q_GLOBAL_STATIC(QGLEngineThreadStorage<QGL2PaintEngineEx>, qt_buffer_2_engine)
 #endif
 
 #ifndef QT_OPENGL_ES_2
-Q_GLOBAL_STATIC(QOpenGLPaintEngine, qt_buffer_engine)
+Q_GLOBAL_STATIC(QGLEngineThreadStorage<QOpenGLPaintEngine>, qt_buffer_engine)
 #endif
 
 /*! \reimp */
@@ -1002,7 +1111,7 @@ QPaintEngine *QGLFramebufferObject::paintEngine() const
 #if !defined (QT_OPENGL_ES_2)
     if (qt_gl_preferGL2Engine()) {
 #endif
-        QPaintEngine *engine = qt_buffer_2_engine();
+        QPaintEngine *engine = qt_buffer_2_engine()->engine();
         if (engine->isActive() && engine->paintDevice() != this) {
             d->engine = new QGL2PaintEngineEx;
             return d->engine;
@@ -1014,7 +1123,7 @@ QPaintEngine *QGLFramebufferObject::paintEngine() const
 #endif
 
 #if !defined(QT_OPENGL_ES_2)
-    QPaintEngine *engine = qt_buffer_engine();
+    QPaintEngine *engine = qt_buffer_engine()->engine();
     if (engine->isActive() && engine->paintDevice() != this) {
         d->engine = new QOpenGLPaintEngine;
         return d->engine;
@@ -1111,9 +1220,6 @@ void QGLFramebufferObject::drawTexture(const QPointF &point, QMacCompatGLuint te
 }
 #endif
 
-extern int qt_defaultDpiX();
-extern int qt_defaultDpiY();
-
 /*! \reimp */
 int QGLFramebufferObject::metric(PaintDeviceMetric metric) const
 {
