@@ -1,40 +1,40 @@
 /****************************************************************************
 **
-** Copyright (C) 2009 Nokia Corporation and/or its subsidiary(-ies).
+** Copyright (C) 2011 Nokia Corporation and/or its subsidiary(-ies).
 ** All rights reserved.
 ** Contact: Nokia Corporation (qt-info@nokia.com)
 **
 ** This file is part of the plugins of the Qt Toolkit.
 **
 ** $QT_BEGIN_LICENSE:LGPL$
-** Commercial Usage
-** Licensees holding valid Qt Commercial licenses may use this file in
-** accordance with the Qt Commercial License Agreement provided with the
-** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and Nokia.
-**
 ** GNU Lesser General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU Lesser
-** General Public License version 2.1 as published by the Free Software
-** Foundation and appearing in the file LICENSE.LGPL included in the
-** packaging of this file.  Please review the following information to
-** ensure the GNU Lesser General Public License version 2.1 requirements
-** will be met: http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
+** This file may be used under the terms of the GNU Lesser General Public
+** License version 2.1 as published by the Free Software Foundation and
+** appearing in the file LICENSE.LGPL included in the packaging of this
+** file. Please review the following information to ensure the GNU Lesser
+** General Public License version 2.1 requirements will be met:
+** http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
 **
 ** In addition, as a special exception, Nokia gives you certain additional
-** rights.  These rights are described in the Nokia Qt LGPL Exception
+** rights. These rights are described in the Nokia Qt LGPL Exception
 ** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
 **
 ** GNU General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU
-** General Public License version 3.0 as published by the Free Software
-** Foundation and appearing in the file LICENSE.GPL included in the
-** packaging of this file.  Please review the following information to
-** ensure the GNU General Public License version 3.0 requirements will be
-** met: http://www.gnu.org/copyleft/gpl.html.
+** Alternatively, this file may be used under the terms of the GNU General
+** Public License version 3.0 as published by the Free Software Foundation
+** and appearing in the file LICENSE.GPL included in the packaging of this
+** file. Please review the following information to ensure the GNU General
+** Public License version 3.0 requirements will be met:
+** http://www.gnu.org/copyleft/gpl.html.
+**
+** Other Usage
+** Alternatively, this file may be used in accordance with the terms and
+** conditions contained in a signed written agreement between you and Nokia.
+**
+**
+**
+**
 **
-** If you have questions regarding the use of this file, please contact
-** Nokia at qt-info@nokia.com.
 ** $QT_END_LICENSE$
 **
 ****************************************************************************/
@@ -93,7 +93,7 @@ int QAccessibleMenu::childAt(int x, int y) const
 QString QAccessibleMenu::text(Text t, int child) const
 {
     QString tx = QAccessibleWidgetEx::text(t, child);
-    if (tx.size())
+    if (!child && tx.size())
         return tx;
 
     switch (t) {
@@ -433,6 +433,8 @@ QString QAccessibleMenuItem::actionText(int action, Text text, int child ) const
         switch (action) {
         case Press:
         case DefaultAction:
+            if (m_action->menu())
+                return QMenu::tr("Open");
             return QMenu::tr("Execute");
             break;
         default:
@@ -444,11 +446,41 @@ QString QAccessibleMenuItem::actionText(int action, Text text, int child ) const
 
 bool QAccessibleMenuItem::doAction(int action, int child, const QVariantList & /*params = QVariantList()*/ )
 {
-    if ((action == Press || action == DefaultAction) && child == 0) {
-        m_action->trigger();
-        return true;
+    if ((child) || ((action != DefaultAction) && (action != Press)))
+        return false;
+
+    // if the action has a menu, expand/hide it
+    if (m_action->menu()) {
+        if (m_action->menu()->isVisible()) {
+            m_action->menu()->hide();
+            return true;
+        } else {
+            if (QMenuBar *bar = qobject_cast<QMenuBar*>(owner())) {
+                bar->setActiveAction(m_action);
+                return true;
+            } else if (QMenu *menu = qobject_cast<QMenu*>(owner())){
+                menu->setActiveAction(m_action);
+                return true;
+            }
+        }
     }
-    return false;
+    // no menu
+    m_action->trigger();
+    return true;
+}
+
+// action interface
+int QAccessibleMenuItem::actionCount()
+{
+    return 1;
+}
+
+void QAccessibleMenuItem::doAction(int actionIndex)
+{
+    if (actionIndex)
+        return;
+
+    doAction(DefaultAction, 0);
 }
 
 int QAccessibleMenuItem::indexOfChild( const QAccessibleInterface * child ) const
@@ -618,7 +650,7 @@ QAccessible::State QAccessibleMenuItem::state(int child ) const
             delete iface;
         }
     }
-    return s;
+    return s | HasInvokeExtension;;
 }
 
 QString QAccessibleMenuItem::text ( Text t, int child ) const
@@ -654,17 +686,50 @@ QString QAccessibleMenuItem::text ( Text t, int child ) const
     return str;
 }
 
+// action interface
 int QAccessibleMenuItem::userActionCount ( int /*child*/ ) const
 {
     return 0;
 }
 
-
 QAction *QAccessibleMenuItem::action() const
 {
     return m_action;
 }
 
+QString QAccessibleMenuItem::description(int)
+{
+    return text(QAccessible::Description, 0);
+}
+
+QString QAccessibleMenuItem::name(int)
+{
+    return actionText(DefaultAction, QAccessible::Name, 0);
+}
+
+QString QAccessibleMenuItem::localizedName(int)
+{
+    return text(QAccessible::Name, 0);
+}
+
+QStringList QAccessibleMenuItem::keyBindings(int)
+{
+    QStringList keys;
+#ifndef QT_NO_SHORTCUT
+    QKeySequence key = m_action->shortcut();
+    if (!key.isEmpty()) {
+        keys.append(key.toString());
+    }
+#endif
+    return keys;
+}
+
+
+QVariant QAccessibleMenuItem::invokeMethodEx(Method, int, const QVariantList &)
+{
+    return QVariant();
+}
+
 QWidget *QAccessibleMenuItem::owner() const
 {
     return m_owner;
