@@ -1,40 +1,40 @@
 /****************************************************************************
 **
-** Copyright (C) 2009 Nokia Corporation and/or its subsidiary(-ies).
+** Copyright (C) 2011 Nokia Corporation and/or its subsidiary(-ies).
 ** All rights reserved.
 ** Contact: Nokia Corporation (qt-info@nokia.com)
 **
 ** This file is part of the plugins of the Qt Toolkit.
 **
 ** $QT_BEGIN_LICENSE:LGPL$
-** Commercial Usage
-** Licensees holding valid Qt Commercial licenses may use this file in
-** accordance with the Qt Commercial License Agreement provided with the
-** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and Nokia.
-**
 ** GNU Lesser General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU Lesser
-** General Public License version 2.1 as published by the Free Software
-** Foundation and appearing in the file LICENSE.LGPL included in the
-** packaging of this file.  Please review the following information to
-** ensure the GNU Lesser General Public License version 2.1 requirements
-** will be met: http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
+** This file may be used under the terms of the GNU Lesser General Public
+** License version 2.1 as published by the Free Software Foundation and
+** appearing in the file LICENSE.LGPL included in the packaging of this
+** file. Please review the following information to ensure the GNU Lesser
+** General Public License version 2.1 requirements will be met:
+** http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
 **
 ** In addition, as a special exception, Nokia gives you certain additional
-** rights.  These rights are described in the Nokia Qt LGPL Exception
+** rights. These rights are described in the Nokia Qt LGPL Exception
 ** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
 **
 ** GNU General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU
-** General Public License version 3.0 as published by the Free Software
-** Foundation and appearing in the file LICENSE.GPL included in the
-** packaging of this file.  Please review the following information to
-** ensure the GNU General Public License version 3.0 requirements will be
-** met: http://www.gnu.org/copyleft/gpl.html.
+** Alternatively, this file may be used under the terms of the GNU General
+** Public License version 3.0 as published by the Free Software Foundation
+** and appearing in the file LICENSE.GPL included in the packaging of this
+** file. Please review the following information to ensure the GNU General
+** Public License version 3.0 requirements will be met:
+** http://www.gnu.org/copyleft/gpl.html.
+**
+** Other Usage
+** Alternatively, this file may be used in accordance with the terms and
+** conditions contained in a signed written agreement between you and Nokia.
+**
+**
+**
+**
 **
-** If you have questions regarding the use of this file, please contact
-** Nokia at qt-info@nokia.com.
 ** $QT_END_LICENSE$
 **
 ****************************************************************************/
@@ -83,8 +83,6 @@ QAbstractSpinBox *QAccessibleAbstractSpinBox::abstractSpinBox() const
 /*! \reimp */
 int QAccessibleAbstractSpinBox::childCount() const
 {
-    if (!abstractSpinBox()->isVisible())
-        return 0;
     return ValueDown;
 }
 
@@ -214,7 +212,7 @@ QVariant QAccessibleAbstractSpinBox::currentValue()
 
 void QAccessibleAbstractSpinBox::setCurrentValue(const QVariant &value)
 {
-    abstractSpinBox()->setProperty("setValue", value);
+    abstractSpinBox()->setProperty("value", value);
 }
 
 QVariant QAccessibleAbstractSpinBox::maximumValue()
@@ -233,7 +231,7 @@ QVariant QAccessibleAbstractSpinBox::invokeMethodEx(Method method, int child, co
     case ListSupportedMethods: {
         QSet<QAccessible::Method> set;
         set << ListSupportedMethods;
-        return qVariantFromValue(set | qvariant_cast<QSet<QAccessible::Method> >(
+        return QVariant::fromValue(set | qvariant_cast<QSet<QAccessible::Method> >(
                     QAccessibleWidgetEx::invokeMethodEx(method, child, params)));
     }
     default:
@@ -344,8 +342,6 @@ QDoubleSpinBox *QAccessibleDoubleSpinBox::doubleSpinBox() const
 /*! \reimp */
 int QAccessibleDoubleSpinBox::childCount() const
 {
-    if (!doubleSpinBox()->isVisible())
-        return 0;
     return ValueDown;
 }
 
@@ -410,8 +406,6 @@ QVariant QAccessibleDoubleSpinBox::invokeMethodEx(QAccessible::Method, int, cons
 /*! \reimp */
 QString QAccessibleDoubleSpinBox::text(Text textType, int child) const
 {
-    if (!doubleSpinBox()->isVisible())
-        return QString();
     switch (textType) {
     case Name:
         if (child == ValueUp)
@@ -540,16 +534,12 @@ QRect QAccessibleScrollBar::rect(int child) const
 /*! \reimp */
 int QAccessibleScrollBar::childCount() const
 {
-    if (!scrollBar()->isVisible())
-        return 0;
     return LineDown;
 }
 
 /*! \reimp */
 QString QAccessibleScrollBar::text(Text t, int child) const
 {
-    if (!scrollBar()->isVisible())
-        return QString();
     switch (t) {
     case Value:
         if (!child || child == Position)
@@ -698,16 +688,12 @@ QRect QAccessibleSlider::rect(int child) const
 /*! \reimp */
 int QAccessibleSlider::childCount() const
 {
-    if (!slider()->isVisible())
-        return 0;
     return PageRight;
 }
 
 /*! \reimp */
 QString QAccessibleSlider::text(Text t, int child) const
 {
-    if (!slider()->isVisible())
-        return QString();
     switch (t) {
     case Value:
         if (!child || child == 2)
@@ -814,7 +800,7 @@ QVariant QAccessibleAbstractSlider::invokeMethodEx(Method method, int child, con
     case ListSupportedMethods: {
         QSet<QAccessible::Method> set;
         set << ListSupportedMethods;
-        return qVariantFromValue(set | qvariant_cast<QSet<QAccessible::Method> >(
+        return QVariant::fromValue(set | qvariant_cast<QSet<QAccessible::Method> >(
                     QAccessibleWidgetEx::invokeMethodEx(method, child, params)));
     }
     default:
@@ -932,15 +918,11 @@ QRect QAccessibleDial::rect(int child) const
 
 int QAccessibleDial::childCount() const
 {
-    if (!dial()->isVisible())
-        return 0;
     return SliderHandle;
 }
 
 QString QAccessibleDial::text(Text textType, int child) const
 {
-    if (!dial()->isVisible())
-        return QString();
     if (textType == Value && child >= Self && child <= SliderHandle)
         return QString::number(dial()->value());
     if (textType == Name) {
