@@ -1,40 +1,40 @@
 /****************************************************************************
 **
-** Copyright (C) 2009 Nokia Corporation and/or its subsidiary(-ies).
+** Copyright (C) 2011 Nokia Corporation and/or its subsidiary(-ies).
 ** All rights reserved.
 ** Contact: Nokia Corporation (qt-info@nokia.com)
 **
 ** This file is part of the Qt Designer of the Qt Toolkit.
 **
 ** $QT_BEGIN_LICENSE:LGPL$
-** Commercial Usage
-** Licensees holding valid Qt Commercial licenses may use this file in
-** accordance with the Qt Commercial License Agreement provided with the
-** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and Nokia.
-**
 ** GNU Lesser General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU Lesser
-** General Public License version 2.1 as published by the Free Software
-** Foundation and appearing in the file LICENSE.LGPL included in the
-** packaging of this file.  Please review the following information to
-** ensure the GNU Lesser General Public License version 2.1 requirements
-** will be met: http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
+** This file may be used under the terms of the GNU Lesser General Public
+** License version 2.1 as published by the Free Software Foundation and
+** appearing in the file LICENSE.LGPL included in the packaging of this
+** file. Please review the following information to ensure the GNU Lesser
+** General Public License version 2.1 requirements will be met:
+** http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
 **
 ** In addition, as a special exception, Nokia gives you certain additional
-** rights.  These rights are described in the Nokia Qt LGPL Exception
+** rights. These rights are described in the Nokia Qt LGPL Exception
 ** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
 **
 ** GNU General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU
-** General Public License version 3.0 as published by the Free Software
-** Foundation and appearing in the file LICENSE.GPL included in the
-** packaging of this file.  Please review the following information to
-** ensure the GNU General Public License version 3.0 requirements will be
-** met: http://www.gnu.org/copyleft/gpl.html.
+** Alternatively, this file may be used under the terms of the GNU General
+** Public License version 3.0 as published by the Free Software Foundation
+** and appearing in the file LICENSE.GPL included in the packaging of this
+** file. Please review the following information to ensure the GNU General
+** Public License version 3.0 requirements will be met:
+** http://www.gnu.org/copyleft/gpl.html.
+**
+** Other Usage
+** Alternatively, this file may be used in accordance with the terms and
+** conditions contained in a signed written agreement between you and Nokia.
+**
+**
+**
+**
 **
-** If you have questions regarding the use of this file, please contact
-** Nokia at qt-info@nokia.com.
 ** $QT_END_LICENSE$
 **
 ****************************************************************************/
@@ -377,7 +377,8 @@ void QtResourceViewPrivate::createPaths()
     if (!m_resourceModel)
         return;
 
-    const QString root(QLatin1Char(':'));
+    // Resource root up until 4.6 was ':', changed to ":/" as of 4.7
+    const QString root(QLatin1String(":/"));
 
     QMap<QString, QString> contents = m_resourceModel->contents();
     QMapIterator<QString, QString> itContents(contents);
@@ -420,7 +421,7 @@ void QtResourceViewPrivate::filterOutResources()
     // 3) we hide these items which has pathToVisible value false.
 
     const bool matchAll = m_filterPattern.isEmpty();
-    const QString root(QLatin1Char(':'));
+    const QString root(QLatin1String(":/"));
 
     QQueue<QString> pathQueue;
     pathQueue.enqueue(root);
@@ -625,12 +626,12 @@ QtResourceView::QtResourceView(QDesignerFormEditorInterface *core, QWidget *pare
     d_ptr->m_listWidget->setIconSize(QSize(48, 48));
     d_ptr->m_listWidget->setGridSize(QSize(64, 64));
 
-    connect(d_ptr->m_treeWidget, SIGNAL(currentItemChanged(QTreeWidgetItem *, QTreeWidgetItem *)),
-                    this, SLOT(slotCurrentPathChanged(QTreeWidgetItem *)));
-    connect(d_ptr->m_listWidget, SIGNAL(currentItemChanged(QListWidgetItem *, QListWidgetItem *)),
-                    this, SLOT(slotCurrentResourceChanged(QListWidgetItem *)));
-    connect(d_ptr->m_listWidget, SIGNAL(itemActivated(QListWidgetItem *)),
-                    this, SLOT(slotResourceActivated(QListWidgetItem *)));
+    connect(d_ptr->m_treeWidget, SIGNAL(currentItemChanged(QTreeWidgetItem*,QTreeWidgetItem*)),
+                    this, SLOT(slotCurrentPathChanged(QTreeWidgetItem*)));
+    connect(d_ptr->m_listWidget, SIGNAL(currentItemChanged(QListWidgetItem*,QListWidgetItem*)),
+                    this, SLOT(slotCurrentResourceChanged(QListWidgetItem*)));
+    connect(d_ptr->m_listWidget, SIGNAL(itemActivated(QListWidgetItem*)),
+                    this, SLOT(slotResourceActivated(QListWidgetItem*)));
     d_ptr->m_listWidget->setContextMenuPolicy(Qt::CustomContextMenu);
     connect(d_ptr->m_listWidget, SIGNAL(customContextMenuRequested(QPoint)),
                 this, SLOT(slotListWidgetContextMenuRequested(QPoint)));
@@ -664,6 +665,8 @@ QString QtResourceView::selectedResource() const
 
 void QtResourceView::selectResource(const QString &resource)
 {
+    if (resource.isEmpty())
+        return;
     QFileInfo fi(resource);
     QDir dir = fi.absoluteDir();
     if (fi.isDir())
@@ -710,8 +713,8 @@ void QtResourceView::setSettingsKey(const QString &key)
 void QtResourceView::setResourceModel(QtResourceModel *model)
 {
     if (d_ptr->m_resourceModel) {
-        disconnect(d_ptr->m_resourceModel, SIGNAL(resourceSetActivated(QtResourceSet *, bool)),
-                    this, SLOT(slotResourceSetActivated(QtResourceSet *)));
+        disconnect(d_ptr->m_resourceModel, SIGNAL(resourceSetActivated(QtResourceSet*,bool)),
+                    this, SLOT(slotResourceSetActivated(QtResourceSet*)));
     }
 
     // clear here
@@ -723,8 +726,8 @@ void QtResourceView::setResourceModel(QtResourceModel *model)
     if (!d_ptr->m_resourceModel)
         return;
 
-    connect(d_ptr->m_resourceModel, SIGNAL(resourceSetActivated(QtResourceSet *, bool)),
-            this, SLOT(slotResourceSetActivated(QtResourceSet *)));
+    connect(d_ptr->m_resourceModel, SIGNAL(resourceSetActivated(QtResourceSet*,bool)),
+            this, SLOT(slotResourceSetActivated(QtResourceSet*)));
 
     // fill new here
     d_ptr->slotResourceSetActivated(d_ptr->m_resourceModel->currentResourceSet());
