@@ -1,40 +1,40 @@
 /****************************************************************************
 **
-** Copyright (C) 2009 Nokia Corporation and/or its subsidiary(-ies).
+** Copyright (C) 2011 Nokia Corporation and/or its subsidiary(-ies).
 ** All rights reserved.
 ** Contact: Nokia Corporation (qt-info@nokia.com)
 **
 ** This file is part of the Qt Linguist of the Qt Toolkit.
 **
 ** $QT_BEGIN_LICENSE:LGPL$
-** Commercial Usage
-** Licensees holding valid Qt Commercial licenses may use this file in
-** accordance with the Qt Commercial License Agreement provided with the
-** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and Nokia.
-**
 ** GNU Lesser General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU Lesser
-** General Public License version 2.1 as published by the Free Software
-** Foundation and appearing in the file LICENSE.LGPL included in the
-** packaging of this file.  Please review the following information to
-** ensure the GNU Lesser General Public License version 2.1 requirements
-** will be met: http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
+** This file may be used under the terms of the GNU Lesser General Public
+** License version 2.1 as published by the Free Software Foundation and
+** appearing in the file LICENSE.LGPL included in the packaging of this
+** file. Please review the following information to ensure the GNU Lesser
+** General Public License version 2.1 requirements will be met:
+** http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
 **
 ** In addition, as a special exception, Nokia gives you certain additional
-** rights.  These rights are described in the Nokia Qt LGPL Exception
+** rights. These rights are described in the Nokia Qt LGPL Exception
 ** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
 **
 ** GNU General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU
-** General Public License version 3.0 as published by the Free Software
-** Foundation and appearing in the file LICENSE.GPL included in the
-** packaging of this file.  Please review the following information to
-** ensure the GNU General Public License version 3.0 requirements will be
-** met: http://www.gnu.org/copyleft/gpl.html.
+** Alternatively, this file may be used under the terms of the GNU General
+** Public License version 3.0 as published by the Free Software Foundation
+** and appearing in the file LICENSE.GPL included in the packaging of this
+** file. Please review the following information to ensure the GNU General
+** Public License version 3.0 requirements will be met:
+** http://www.gnu.org/copyleft/gpl.html.
+**
+** Other Usage
+** Alternatively, this file may be used in accordance with the terms and
+** conditions contained in a signed written agreement between you and Nokia.
+**
+**
+**
+**
 **
-** If you have questions regarding the use of this file, please contact
-** Nokia at qt-info@nokia.com.
 ** $QT_END_LICENSE$
 **
 ****************************************************************************/
@@ -98,14 +98,8 @@ MessageEditor::MessageEditor(MultiDataModel *dataModel, QMainWindow *parent)
 {
     setObjectName(QLatin1String("scroll area"));
 
-    // Use white explicitly as the background color for the editor page.
     QPalette p;
-    p.setColor(QPalette::Active,   QPalette::Base,   Qt::white);
-    p.setColor(QPalette::Inactive, QPalette::Base,   Qt::white);
-    p.setColor(QPalette::Disabled, QPalette::Base,   Qt::white);
-    p.setColor(QPalette::Active,   QPalette::Window, Qt::white);
-    p.setColor(QPalette::Inactive, QPalette::Window, Qt::white);
-    p.setColor(QPalette::Disabled, QPalette::Window, Qt::white);
+    p.setBrush(QPalette::Window, p.brush(QPalette::Active, QPalette::Base));
     setPalette(p);
 
     setupEditorPage();
@@ -135,27 +129,19 @@ MessageEditor::MessageEditor(MultiDataModel *dataModel, QMainWindow *parent)
 void MessageEditor::setupEditorPage()
 {
     QFrame *editorPage = new QFrame;
-    editorPage->setObjectName(QLatin1String("editorPage"));
-
-    editorPage->setStyleSheet(QLatin1String(
-            "QFrame#editorPage { border-image: url(:/images/transbox.png) 12 16 16 12 repeat;"
-            "                    border-width: 12px 16px 16px 12px; }"
-            "QFrame#editorPage { background-color: white; }"
-            "QLabel { font-weight: bold; }"
-            ));
     editorPage->setSizePolicy(QSizePolicy(QSizePolicy::Expanding, QSizePolicy::Fixed));
 
     m_source = new FormWidget(tr("Source text"), false);
     m_source->setHideWhenEmpty(true);
     m_source->setWhatsThis(tr("This area shows the source text."));
-    connect(m_source, SIGNAL(selectionChanged(QTextEdit *)),
-            SLOT(selectionChanged(QTextEdit *)));
+    connect(m_source, SIGNAL(selectionChanged(QTextEdit*)),
+            SLOT(selectionChanged(QTextEdit*)));
 
     m_pluralSource = new FormWidget(tr("Source text (Plural)"), false);
     m_pluralSource->setHideWhenEmpty(true);
     m_pluralSource->setWhatsThis(tr("This area shows the plural form of the source text."));
-    connect(m_pluralSource, SIGNAL(selectionChanged(QTextEdit *)),
-            SLOT(selectionChanged(QTextEdit *)));
+    connect(m_pluralSource, SIGNAL(selectionChanged(QTextEdit*)),
+            SLOT(selectionChanged(QTextEdit*)));
 
     m_commentText = new FormWidget(tr("Developer comments"), false);
     m_commentText->setHideWhenEmpty(true);
@@ -222,11 +208,11 @@ void MessageEditor::messageModelAppended()
     ed.transCommentText->setWhatsThis(tr("Here you can enter comments for your own use."
                         " They have no effect on the translated applications.") );
     ed.transCommentText->getEditor()->installEventFilter(this);
-    connect(ed.transCommentText, SIGNAL(selectionChanged(QTextEdit *)),
-            SLOT(selectionChanged(QTextEdit *)));
-    connect(ed.transCommentText, SIGNAL(textChanged(QTextEdit *)),
-            SLOT(emitTranslatorCommentChanged(QTextEdit *)));
-    connect(ed.transCommentText, SIGNAL(textChanged(QTextEdit *)), SLOT(resetHoverSelection()));
+    connect(ed.transCommentText, SIGNAL(selectionChanged(QTextEdit*)),
+            SLOT(selectionChanged(QTextEdit*)));
+    connect(ed.transCommentText, SIGNAL(textChanged(QTextEdit*)),
+            SLOT(emitTranslatorCommentChanged(QTextEdit*)));
+    connect(ed.transCommentText, SIGNAL(textChanged(QTextEdit*)), SLOT(resetHoverSelection()));
     connect(ed.transCommentText, SIGNAL(cursorPositionChanged()), SLOT(resetHoverSelection()));
     fixTabOrder();
     QBoxLayout *box = new QVBoxLayout(ed.container);
@@ -275,7 +261,7 @@ void MessageEditor::messageModelDeleted(int model)
 void MessageEditor::addPluralForm(int model, const QString &label, bool writable)
 {
     FormMultiWidget *transEditor = new FormMultiWidget(label);
-    connect(transEditor, SIGNAL(editorCreated(QTextEdit *)), SLOT(editorCreated(QTextEdit *)));
+    connect(transEditor, SIGNAL(editorCreated(QTextEdit*)), SLOT(editorCreated(QTextEdit*)));
     transEditor->setEditingEnabled(writable);
     transEditor->setHideWhenEmpty(!writable);
     if (!m_editors[model].transTexts.isEmpty())
@@ -284,11 +270,11 @@ void MessageEditor::addPluralForm(int model, const QString &label, bool writable
     static_cast<QBoxLayout *>(m_editors[model].container->layout())->insertWidget(
         m_editors[model].transTexts.count(), transEditor);
 
-    connect(transEditor, SIGNAL(selectionChanged(QTextEdit *)),
-            SLOT(selectionChanged(QTextEdit *)));
-    connect(transEditor, SIGNAL(textChanged(QTextEdit *)),
-            SLOT(emitTranslationChanged(QTextEdit *)));
-    connect(transEditor, SIGNAL(textChanged(QTextEdit *)), SLOT(resetHoverSelection()));
+    connect(transEditor, SIGNAL(selectionChanged(QTextEdit*)),
+            SLOT(selectionChanged(QTextEdit*)));
+    connect(transEditor, SIGNAL(textChanged(QTextEdit*)),
+            SLOT(emitTranslationChanged(QTextEdit*)));
+    connect(transEditor, SIGNAL(textChanged(QTextEdit*)), SLOT(resetHoverSelection()));
     connect(transEditor, SIGNAL(cursorPositionChanged()), SLOT(resetHoverSelection()));
 
     m_editors[model].transTexts << transEditor;
@@ -312,6 +298,12 @@ void MessageEditor::editorCreated(QTextEdit *te)
     }
 }
 
+void MessageEditor::editorDestroyed()
+{
+    if (m_selectionHolder == sender())
+        resetSelection();
+}
+
 void MessageEditor::fixTabOrder()
 {
     m_tabOrderTimer.start(0);
@@ -359,9 +351,13 @@ static void clearSelection(QTextEdit *t)
 void MessageEditor::selectionChanged(QTextEdit *te)
 {
     if (te != m_selectionHolder) {
-        if (m_selectionHolder)
+        if (m_selectionHolder) {
             clearSelection(m_selectionHolder);
+            disconnect(this, SLOT(editorDestroyed()));
+        }
         m_selectionHolder = (te->textCursor().hasSelection() ? te : 0);
+        if (FormatTextEdit *fte = qobject_cast<FormatTextEdit*>(m_selectionHolder))
+            connect(fte, SIGNAL(editorDestroyed()), SLOT(editorDestroyed()));
         updateCanCutCopy();
     }
 }
@@ -378,6 +374,7 @@ void MessageEditor::resetSelection()
 {
     if (m_selectionHolder) {
         clearSelection(m_selectionHolder);
+        disconnect(this, SLOT(editorDestroyed()));
         m_selectionHolder = 0;
         updateCanCutCopy();
     }
