@@ -1,40 +1,40 @@
 /****************************************************************************
 **
-** Copyright (C) 2009 Nokia Corporation and/or its subsidiary(-ies).
+** Copyright (C) 2011 Nokia Corporation and/or its subsidiary(-ies).
 ** All rights reserved.
 ** Contact: Nokia Corporation (qt-info@nokia.com)
 **
 ** This file is part of the Qt Linguist of the Qt Toolkit.
 **
 ** $QT_BEGIN_LICENSE:LGPL$
-** Commercial Usage
-** Licensees holding valid Qt Commercial licenses may use this file in
-** accordance with the Qt Commercial License Agreement provided with the
-** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and Nokia.
-**
 ** GNU Lesser General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU Lesser
-** General Public License version 2.1 as published by the Free Software
-** Foundation and appearing in the file LICENSE.LGPL included in the
-** packaging of this file.  Please review the following information to
-** ensure the GNU Lesser General Public License version 2.1 requirements
-** will be met: http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
+** This file may be used under the terms of the GNU Lesser General Public
+** License version 2.1 as published by the Free Software Foundation and
+** appearing in the file LICENSE.LGPL included in the packaging of this
+** file. Please review the following information to ensure the GNU Lesser
+** General Public License version 2.1 requirements will be met:
+** http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
 **
 ** In addition, as a special exception, Nokia gives you certain additional
-** rights.  These rights are described in the Nokia Qt LGPL Exception
+** rights. These rights are described in the Nokia Qt LGPL Exception
 ** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
 **
 ** GNU General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU
-** General Public License version 3.0 as published by the Free Software
-** Foundation and appearing in the file LICENSE.GPL included in the
-** packaging of this file.  Please review the following information to
-** ensure the GNU General Public License version 3.0 requirements will be
-** met: http://www.gnu.org/copyleft/gpl.html.
+** Alternatively, this file may be used under the terms of the GNU General
+** Public License version 3.0 as published by the Free Software Foundation
+** and appearing in the file LICENSE.GPL included in the packaging of this
+** file. Please review the following information to ensure the GNU General
+** Public License version 3.0 requirements will be met:
+** http://www.gnu.org/copyleft/gpl.html.
+**
+** Other Usage
+** Alternatively, this file may be used in accordance with the terms and
+** conditions contained in a signed written agreement between you and Nokia.
+**
+**
+**
+**
 **
-** If you have questions regarding the use of this file, please contact
-** Nokia at qt-info@nokia.com.
 ** $QT_END_LICENSE$
 **
 ****************************************************************************/
@@ -209,6 +209,13 @@ bool DataModel::load(const QString &fileName, bool *langGuessed, QWidget *parent
         return false;
     }
 
+    if (!tor.messageCount()) {
+        QMessageBox::warning(parent, QObject::tr("Qt Linguist"),
+                             tr("The translation file '%1' will not be loaded because it is empty.")
+                             .arg(Qt::escape(fileName)));
+        return false;
+    }
+
     Translator::Duplicates dupes = tor.resolveDuplicates();
     if (!dupes.byId.isEmpty() || !dupes.byContents.isEmpty()) {
         QString err = tr("<qt>Duplicate messages found in '%1':").arg(Qt::escape(fileName));
@@ -368,7 +375,7 @@ bool DataModel::release(const QString &fileName, bool verbose, bool ignoreUnfini
     cd.m_verbose = verbose;
     cd.m_ignoreUnfinished = ignoreUnfinished;
     cd.m_saveMode = mode;
-    bool ok = tor.release(&file, cd);
+    bool ok = saveQM(tor, file, cd);
     if (!ok)
         QMessageBox::warning(parent, QObject::tr("Qt Linguist"), cd.error());
     return ok;
@@ -402,7 +409,7 @@ bool DataModel::setLanguageAndCountry(QLocale::Language lang, QLocale::Country c
     if (lang == QLocale::C || uint(lang) > uint(QLocale::LastLanguage)) // XXX does this make any sense?
         lang = QLocale::English;
     QByteArray rules;
-    bool ok = getNumerusInfo(lang, country, &rules, &m_numerusForms);
+    bool ok = getNumerusInfo(lang, country, &rules, &m_numerusForms, 0);
     m_localizedLanguage = QCoreApplication::translate("MessageEditor", QLocale::languageToString(lang).toAscii());
     m_countRefNeeds.clear();
     for (int i = 0; i < rules.size(); ++i) {
@@ -584,12 +591,16 @@ void MultiContextItem::putMessageItem(int pos, MessageItem *m)
     m_messageLists.last()[pos] = m;
 }
 
-void MultiContextItem::appendMessageItem(MessageItem *m)
+void MultiContextItem::appendMessageItems(const QList<MessageItem *> &m)
 {
+    QList<MessageItem *> nullItems = m; // Basically, just a reservation
+    for (int i = 0; i < nullItems.count(); ++i)
+        nullItems[i] = 0;
     for (int i = 0; i < m_messageLists.count() - 1; ++i)
-        m_messageLists[i].append(0);
-    m_messageLists.last().append(m);
-    m_multiMessageList.append(MultiMessageItem(m));
+        m_messageLists[i] += nullItems;
+    m_messageLists.last() += m;
+    foreach (MessageItem *mi, m)
+        m_multiMessageList.append(MultiMessageItem(mi));
 }
 
 void MultiContextItem::removeMultiMessageItem(int pos)
@@ -710,33 +721,43 @@ void MultiDataModel::append(DataModel *dm, bool readWrite)
         m_msgModel->endInsertColumns();
     }
     m_msgModel->endInsertColumns();
+    int appendedContexts = 0;
     for (int i = 0; i < dm->contextCount(); ++i) {
         ContextItem *c = dm->contextItem(i);
         int mcx = findContextIndex(c->context());
         if (mcx >= 0) {
             MultiContextItem *mc = multiContextItem(mcx);
             mc->assignLastModel(c, readWrite);
+            QList<MessageItem *> appendItems;
             for (int j = 0; j < c->messageCount(); ++j) {
                 MessageItem *m = c->messageItem(j);
                 int msgIdx = mc->findMessage(m->text(), m->comment());
-                if (msgIdx >= 0) {
+                if (msgIdx >= 0)
                     mc->putMessageItem(msgIdx, m);
-                } else {
-                    int msgCnt = mc->messageCount();
-                    m_msgModel->beginInsertRows(m_msgModel->createIndex(mcx, 0, 0), msgCnt, msgCnt);
-                    mc->appendMessageItem(m);
-                    m_msgModel->endInsertRows();
-                    ++m_numMessages;
-                }
+                else
+                    appendItems << m;
+            }
+            if (!appendItems.isEmpty()) {
+                int msgCnt = mc->messageCount();
+                m_msgModel->beginInsertRows(m_msgModel->createIndex(mcx, 0, 0),
+                                            msgCnt, msgCnt + appendItems.size() - 1);
+                mc->appendMessageItems(appendItems);
+                m_msgModel->endInsertRows();
+                m_numMessages += appendItems.size();
             }
         } else {
-            MultiContextItem item(modelCount() - 1, c, readWrite);
-            m_msgModel->beginInsertRows(QModelIndex(), contextCount(), contextCount());
-            m_multiContextList.append(item);
-            m_msgModel->endInsertRows();
-            m_numMessages += item.messageCount();
+            m_multiContextList << MultiContextItem(modelCount() - 1, c, readWrite);
+            m_numMessages += c->messageCount();
+            ++appendedContexts;
         }
     }
+    if (appendedContexts) {
+        // Do that en block to avoid itemview inefficiency. It doesn't hurt that we
+        // announce the availability of the data "long" after it was actually added.
+        m_msgModel->beginInsertRows(QModelIndex(),
+                                    contextCount() - appendedContexts, contextCount() - 1);
+        m_msgModel->endInsertRows();
+    }
     dm->setWritable(readWrite);
     updateCountsOnAdd(modelCount() - 1, readWrite);
     connect(dm, SIGNAL(modifiedChanged()), SLOT(onModifiedChanged()));
@@ -786,16 +807,9 @@ void MultiDataModel::closeAll()
     m_numFinished = 0;
     m_numEditable = 0;
     m_numMessages = 0;
-    int delCol = m_dataModels.count();
-    m_msgModel->beginRemoveColumns(QModelIndex(), 1, delCol);
-    for (int i = m_multiContextList.size(); --i >= 0;) {
-        m_msgModel->beginRemoveColumns(m_msgModel->createIndex(i, 0, 0), 1, delCol);
-        m_msgModel->endRemoveColumns();
-    }
     qDeleteAll(m_dataModels);
     m_dataModels.clear();
     m_multiContextList.clear();
-    m_msgModel->endRemoveColumns();
     m_msgModel->reset();
     emit allModelsDeleted();
     onModifiedChanged();
@@ -1224,17 +1238,17 @@ int MessageModel::columnCount(const QModelIndex &) const
 QVariant MessageModel::data(const QModelIndex &index, int role) const
 {
     static QVariant pxOn  =
-        qVariantFromValue(QPixmap(QLatin1String(":/images/s_check_on.png")));
+        QVariant::fromValue(QPixmap(QLatin1String(":/images/s_check_on.png")));
     static QVariant pxOff =
-        qVariantFromValue(QPixmap(QLatin1String(":/images/s_check_off.png")));
+        QVariant::fromValue(QPixmap(QLatin1String(":/images/s_check_off.png")));
     static QVariant pxObsolete =
-        qVariantFromValue(QPixmap(QLatin1String(":/images/s_check_obsolete.png")));
+        QVariant::fromValue(QPixmap(QLatin1String(":/images/s_check_obsolete.png")));
     static QVariant pxDanger =
-        qVariantFromValue(QPixmap(QLatin1String(":/images/s_check_danger.png")));
+        QVariant::fromValue(QPixmap(QLatin1String(":/images/s_check_danger.png")));
     static QVariant pxWarning =
-        qVariantFromValue(QPixmap(QLatin1String(":/images/s_check_warning.png")));
+        QVariant::fromValue(QPixmap(QLatin1String(":/images/s_check_warning.png")));
     static QVariant pxEmpty =
-        qVariantFromValue(QPixmap(QLatin1String(":/images/s_check_empty.png")));
+        QVariant::fromValue(QPixmap(QLatin1String(":/images/s_check_empty.png")));
 
     int row = index.row();
     int column = index.column() - 1;
