@@ -1,46 +1,45 @@
 /****************************************************************************
 **
-** Copyright (C) 2009 Nokia Corporation and/or its subsidiary(-ies).
+** Copyright (C) 2011 Nokia Corporation and/or its subsidiary(-ies).
 ** All rights reserved.
 ** Contact: Nokia Corporation (qt-info@nokia.com)
 **
 ** This file is part of the tools applications of the Qt Toolkit.
 **
 ** $QT_BEGIN_LICENSE:LGPL$
-** Commercial Usage
-** Licensees holding valid Qt Commercial licenses may use this file in
-** accordance with the Qt Commercial License Agreement provided with the
-** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and Nokia.
-**
 ** GNU Lesser General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU Lesser
-** General Public License version 2.1 as published by the Free Software
-** Foundation and appearing in the file LICENSE.LGPL included in the
-** packaging of this file.  Please review the following information to
-** ensure the GNU Lesser General Public License version 2.1 requirements
-** will be met: http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
+** This file may be used under the terms of the GNU Lesser General Public
+** License version 2.1 as published by the Free Software Foundation and
+** appearing in the file LICENSE.LGPL included in the packaging of this
+** file. Please review the following information to ensure the GNU Lesser
+** General Public License version 2.1 requirements will be met:
+** http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
 **
 ** In addition, as a special exception, Nokia gives you certain additional
-** rights.  These rights are described in the Nokia Qt LGPL Exception
+** rights. These rights are described in the Nokia Qt LGPL Exception
 ** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
 **
 ** GNU General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU
-** General Public License version 3.0 as published by the Free Software
-** Foundation and appearing in the file LICENSE.GPL included in the
-** packaging of this file.  Please review the following information to
-** ensure the GNU General Public License version 3.0 requirements will be
-** met: http://www.gnu.org/copyleft/gpl.html.
+** Alternatively, this file may be used under the terms of the GNU General
+** Public License version 3.0 as published by the Free Software Foundation
+** and appearing in the file LICENSE.GPL included in the packaging of this
+** file. Please review the following information to ensure the GNU General
+** Public License version 3.0 requirements will be met:
+** http://www.gnu.org/copyleft/gpl.html.
+**
+** Other Usage
+** Alternatively, this file may be used in accordance with the terms and
+** conditions contained in a signed written agreement between you and Nokia.
+**
+**
+**
+**
 **
-** If you have questions regarding the use of this file, please contact
-** Nokia at qt-info@nokia.com.
 ** $QT_END_LICENSE$
 **
 ****************************************************************************/
 
 #include <QMetaObject>
-#include <QDebug>
 #include "codemarker.h"
 #include "config.h"
 #include "node.h"
@@ -60,7 +59,6 @@ QList<CodeMarker *> CodeMarker::markers;
   been read.
  */
 CodeMarker::CodeMarker()
-    : slow(false)
 {
     markers.prepend(this);
 }
@@ -75,14 +73,11 @@ CodeMarker::~CodeMarker()
 }
 
 /*!
-  The only thing a code market initializes is its \e{slow}
-  flag. The \e{slow} flag indicates whether the operations
-  that slow down qdoc are to be performed or not. It is
-  turned off by default. 
+  A code market performs no initialization by default. Marker-specific
+  initialization is performed in subclasses.
  */
-void CodeMarker::initializeMarker(const Config &config)
+void CodeMarker::initializeMarker(const Config& ) // config
 {
-    slow = config.getBool(QLatin1String(CONFIG_SLOW));
 }
 
 /*!
@@ -177,7 +172,7 @@ const Node *CodeMarker::nodeForString(const QString& string)
 QString CodeMarker::stringForNode(const Node *node)
 {
     if (sizeof(const Node *) == sizeof(ulong)) {
-        return QString::number(reinterpret_cast<ulong>(node));
+        return QString::number(reinterpret_cast<quintptr>(node));
     }
     else {
         return QString::number(reinterpret_cast<qulonglong>(node));
@@ -257,6 +252,7 @@ QString CodeMarker::typified(const QString &string)
 QString CodeMarker::taggedNode(const Node* node)
 {
     QString tag;
+    QString name = node->name();
 
     switch (node->type()) {
     case Node::Namespace:
@@ -277,11 +273,20 @@ QString CodeMarker::taggedNode(const Node* node)
     case Node::Property:
         tag = QLatin1String("@property");
         break;
+#ifdef QDOC_QML
+    case Node::Fake:
+        if (node->subType() == Node::QmlClass) {
+            if (node->name().startsWith(QLatin1String("QML:")))
+                name = name.mid(4);                 // remove the "QML:" prefix
+        }
+        tag = QLatin1String("@property");
+        break;
+#endif
     default:
         tag = QLatin1String("@unknown");
         break;
     }
-    return QLatin1Char('<') + tag + QLatin1Char('>') + protect(node->name())
+    return QLatin1Char('<') + tag + QLatin1Char('>') + protect(name)
         + QLatin1String("</") + tag + QLatin1Char('>');
 }
 
@@ -457,7 +462,7 @@ bool CodeMarker::insertReimpFunc(FastSection& fs, Node* node, Status status)
 void CodeMarker::append(QList<Section>& sectionList, const FastSection& fs)
 {
     if (!fs.isEmpty()) {
-	Section section(fs.name,fs.singularMember,fs.pluralMember);
+	Section section(fs.name,fs.divClass,fs.singularMember,fs.pluralMember);
 	section.members = fs.memberMap.values();
         section.reimpMembers = fs.reimpMemberMap.values();
 	section.inherited = fs.inherited;
@@ -486,7 +491,7 @@ static QString encode(const QString &string)
 #endif
 }
 
-QStringList CodeMarker::macRefsForNode(const Node *node)
+QStringList CodeMarker::macRefsForNode(Node *node)
 {
     QString result = QLatin1String("cpp/");
     switch (node->type()) {
@@ -578,7 +583,7 @@ QStringList CodeMarker::macRefsForNode(const Node *node)
          {
              NodeList list = static_cast<const PropertyNode*>(node)->functions();
              QStringList stringList;
-             foreach (const Node *node, list) {
+             foreach (Node* node, list) {
                 stringList += macRefsForNode(node);
              }
              return stringList;
@@ -614,10 +619,20 @@ QString CodeMarker::macName(const Node *node, const QString &name)
   Get the list of documentation sections for the children of
   the specified QmlClassNode.
  */
-QList<Section> CodeMarker::qmlSections(const QmlClassNode* , SynopsisStyle )
+QList<Section> CodeMarker::qmlSections(const QmlClassNode* ,
+                                       SynopsisStyle ,
+                                       const Tree* )
 {
     return QList<Section>();
 }
 #endif
 
+const Node* CodeMarker::resolveTarget(const QString& , 
+                                      const Tree* ,
+		                      const Node* ,
+                                      const Node* )
+{
+    return 0;
+}
+
 QT_END_NAMESPACE
