@@ -1,40 +1,40 @@
 /****************************************************************************
 **
-** Copyright (C) 2010 Nokia Corporation and/or its subsidiary(-ies).
+** Copyright (C) 2011 Nokia Corporation and/or its subsidiary(-ies).
 ** All rights reserved.
 ** Contact: Nokia Corporation (qt-info@nokia.com)
 **
 ** This file is part of the tools applications of the Qt Toolkit.
 **
 ** $QT_BEGIN_LICENSE:LGPL$
-** Commercial Usage
-** Licensees holding valid Qt Commercial licenses may use this file in
-** accordance with the Qt Commercial License Agreement provided with the
-** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and Nokia.
-**
 ** GNU Lesser General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU Lesser
-** General Public License version 2.1 as published by the Free Software
-** Foundation and appearing in the file LICENSE.LGPL included in the
-** packaging of this file.  Please review the following information to
-** ensure the GNU Lesser General Public License version 2.1 requirements
-** will be met: http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
+** This file may be used under the terms of the GNU Lesser General Public
+** License version 2.1 as published by the Free Software Foundation and
+** appearing in the file LICENSE.LGPL included in the packaging of this
+** file. Please review the following information to ensure the GNU Lesser
+** General Public License version 2.1 requirements will be met:
+** http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
 **
 ** In addition, as a special exception, Nokia gives you certain additional
-** rights.  These rights are described in the Nokia Qt LGPL Exception
+** rights. These rights are described in the Nokia Qt LGPL Exception
 ** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
 **
 ** GNU General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU
-** General Public License version 3.0 as published by the Free Software
-** Foundation and appearing in the file LICENSE.GPL included in the
-** packaging of this file.  Please review the following information to
-** ensure the GNU General Public License version 3.0 requirements will be
-** met: http://www.gnu.org/copyleft/gpl.html.
+** Alternatively, this file may be used under the terms of the GNU General
+** Public License version 3.0 as published by the Free Software Foundation
+** and appearing in the file LICENSE.GPL included in the packaging of this
+** file. Please review the following information to ensure the GNU General
+** Public License version 3.0 requirements will be met:
+** http://www.gnu.org/copyleft/gpl.html.
+**
+** Other Usage
+** Alternatively, this file may be used in accordance with the terms and
+** conditions contained in a signed written agreement between you and Nokia.
+**
+**
+**
+**
 **
-** If you have questions regarding the use of this file, please contact
-** Nokia at qt-info@nokia.com.
 ** $QT_END_LICENSE$
 **
 ****************************************************************************/
@@ -50,6 +50,7 @@
 #include <QDebug>
 #include <QMessageBox>
 #include <QAtomicInt>
+#include <QLibraryInfo>
 #include "qdeclarativetester.h"
 
 QT_USE_NAMESPACE
@@ -59,14 +60,45 @@ QtMsgHandler systemMsgOutput = 0;
 static QDeclarativeViewer *openFile(const QString &fileName);
 static void showViewer(QDeclarativeViewer *viewer);
 
+QString warnings;
+void exitApp(int i)
+{
+#ifdef Q_OS_WIN
+    // Debugging output is not visible by default on Windows -
+    // therefore show modal dialog with errors instead.
+    if (!warnings.isEmpty()) {
+        QMessageBox::warning(0, QApplication::translate("QDeclarativeViewer", "Qt QML Viewer"), warnings);
+    }
+#endif
+    exit(i);
+}
+
+QWeakPointer<LoggerWidget> logger;
+static QAtomicInt recursiveLock(0);
+
 #if defined (Q_OS_SYMBIAN)
 #include <unistd.h>
 #include <sys/types.h>
 #include <sys/stat.h>
 #include <fcntl.h>
+#endif
 
 void myMessageOutput(QtMsgType type, const char *msg)
 {
+    QString strMsg = QString::fromLatin1(msg);
+
+    if (!QCoreApplication::closingDown()) {
+        if (!logger.isNull()) {
+            if (recursiveLock.testAndSetOrdered(0, 1)) {
+                QMetaObject::invokeMethod(logger.data(), "append", Q_ARG(QString, strMsg));
+                recursiveLock = 0;
+            }
+        } else {
+            warnings += strMsg;
+            warnings += QLatin1Char('\n');
+        }
+    }
+#if defined (Q_OS_SYMBIAN)
     static int fd = -1;
     if (fd == -1)
         fd = ::open("E:\\qml.log", O_WRONLY | O_CREAT);
@@ -74,41 +106,13 @@ void myMessageOutput(QtMsgType type, const char *msg)
     ::write(fd, msg, strlen(msg));
     ::write(fd, "\n", 1);
     ::fsync(fd);
-
     switch (type) {
     case QtFatalMsg:
         abort();
     }
-}
+#endif
 
-#else // !defined (Q_OS_SYMBIAN)
-
-QWeakPointer<LoggerWidget> logger;
-
-QString warnings;
-void showWarnings()
-{
-    if (!warnings.isEmpty()) {
-        QMessageBox::warning(0, QApplication::tr("Qt QML Viewer"), warnings);
-    }
-}
-
-static QAtomicInt recursiveLock(0);
-
-void myMessageOutput(QtMsgType type, const char *msg)
-{
-    QString strMsg = QString::fromLatin1(msg);
-
-    if (!logger.isNull() && !QCoreApplication::closingDown()) {
-        if (recursiveLock.testAndSetOrdered(0, 1)) {
-            QMetaObject::invokeMethod(logger.data(), "append", Q_ARG(QString, strMsg));
-            recursiveLock = 0;
-        }
-    } else {
-        warnings += strMsg;
-        warnings += QLatin1Char('\n');
-    }
-    if (systemMsgOutput) { // Windows
+    if (systemMsgOutput) {
         systemMsgOutput(type, msg);
     } else { // Unix
         fprintf(stderr, "%s\n", msg);
@@ -116,12 +120,10 @@ void myMessageOutput(QtMsgType type, const char *msg)
     }
 }
 
-#endif
-
 static QDeclarativeViewer* globalViewer = 0;
 
 // The qml file that is shown if the user didn't specify a QML file
-QString initialFile = "qrc:/startup/startup.qml";
+QString initialFile = QLatin1String("qrc:/startup/startup.qml");
 
 void usage()
 {
@@ -152,13 +154,20 @@ void usage()
     qWarning("  -I <directory> ........................... prepend to the module import search path,");
     qWarning("                                             display path if <directory> is empty");
     qWarning("  -P <directory> ........................... prepend to the plugin search path");
+#if defined(Q_WS_MAC)
+    qWarning("  -no-opengl ............................... don't use a QGLWidget for the viewport");
+    qWarning("  -opengl .................................. use a QGLWidget for the viewport (default)");
+#else
+    qWarning("  -no-opengl ............................... don't use a QGLWidget for the viewport (default)");
     qWarning("  -opengl .................................. use a QGLWidget for the viewport");
+#endif
     qWarning("  -script <path> ........................... set the script to use");
     qWarning("  -scriptopts <options>|help ............... set the script options to use");
 
     qWarning(" ");
     qWarning(" Press F1 for interactive help");
-    exit(1);
+
+    exitApp(1);
 }
 
 void scriptOptsUsage()
@@ -169,6 +178,7 @@ void scriptOptsUsage()
     qWarning("  play ..................................... playback an existing script");
     qWarning("  testimages ............................... record images or compare images on playback");
     qWarning("  testerror ................................ test 'error' property of root item on playback");
+    qWarning("  testskip  ................................ test 'skip' property of root item on playback");
     qWarning("  snapshot ................................. file being recorded is static,");
     qWarning("                                             only one frame will be recorded or tested");
     qWarning("  exitoncomplete ........................... cleanly exit the viewer on script completion");
@@ -176,7 +186,8 @@ void scriptOptsUsage()
     qWarning("  saveonexit ............................... save recording on viewer exit");
     qWarning(" ");
     qWarning(" One of record, play or both must be specified.");
-    exit(1);
+
+    exitApp(1);
 }
 
 enum WarningsConfig { ShowWarnings, HideWarnings, DefaultWarnings };
@@ -188,7 +199,7 @@ struct ViewerOptions
           fps(0.0),
           autorecord_from(0),
           autorecord_to(0),
-          dither("none"),
+          dither(QLatin1String("none")),
           runScript(false),
           devkeys(false),
           cache(0),
@@ -302,6 +313,8 @@ static void parseScriptOptions()
             scriptOptions |= QDeclarativeViewer::TestImages;
         } else if (option == QLatin1String("testerror")) {
             scriptOptions |= QDeclarativeViewer::TestErrorProperty;
+        } else if (option == QLatin1String("testskip")) {
+            scriptOptions |= QDeclarativeViewer::TestSkipProperty;
         } else if (option == QLatin1String("exitoncomplete")) {
             scriptOptions |= QDeclarativeViewer::ExitOnComplete;
         } else if (option == QLatin1String("exitonfailure")) {
@@ -323,52 +336,54 @@ static void parseCommandLineOptions(const QStringList &arguments)
     for (int i = 1; i < arguments.count(); ++i) {
         bool lastArg = (i == arguments.count() - 1);
         QString arg = arguments.at(i);
-        if (arg == "-frameless") {
+        if (arg == QLatin1String("-frameless")) {
             opts.frameless = true;
-        } else if (arg == "-maximized") {
+        } else if (arg == QLatin1String("-maximized")) {
             opts.maximized = true;
-        } else if (arg == "-fullscreen") {
+        } else if (arg == QLatin1String("-fullscreen")) {
             opts.fullScreen = true;
-        } else if (arg == "-stayontop") {
+        } else if (arg == QLatin1String("-stayontop")) {
             opts.stayOnTop = true;
-        } else if (arg == "-netcache") {
+        } else if (arg == QLatin1String("-netcache")) {
             if (lastArg) usage();
             opts.cache = arguments.at(++i).toInt();
-        } else if (arg == "-recordrate") {
+        } else if (arg == QLatin1String("-recordrate")) {
             if (lastArg) usage();
             opts.fps = arguments.at(++i).toDouble();
-        } else if (arg == "-recordfile") {
+        } else if (arg == QLatin1String("-recordfile")) {
             if (lastArg) usage();
             opts.recordfile = arguments.at(++i);
-        } else if (arg == "-record") {
+        } else if (arg == QLatin1String("-record")) {
             if (lastArg) usage();
             opts.recordargs << arguments.at(++i);
-        } else if (arg == "-recorddither") {
+        } else if (arg == QLatin1String("-recorddither")) {
             if (lastArg) usage();
             opts.dither = arguments.at(++i);
-        } else if (arg == "-autorecord") {
+        } else if (arg == QLatin1String("-autorecord")) {
             if (lastArg) usage();
             QString range = arguments.at(++i);
-            int dash = range.indexOf('-');
+            int dash = range.indexOf(QLatin1Char('-'));
             if (dash > 0)
                 opts.autorecord_from = range.left(dash).toInt();
             opts.autorecord_to = range.mid(dash+1).toInt();
-        } else if (arg == "-devicekeys") {
+        } else if (arg == QLatin1String("-devicekeys")) {
             opts.devkeys = true;
-        } else if (arg == "-dragthreshold") {
+        } else if (arg == QLatin1String("-dragthreshold")) {
             if (lastArg) usage();
             qApp->setStartDragDistance(arguments.at(++i).toInt());
         } else if (arg == QLatin1String("-v") || arg == QLatin1String("-version")) {
             qWarning("Qt QML Viewer version %s", QT_VERSION_STR);
-            exit(0);
-        } else if (arg == "-translation") {
+            exitApp(0);
+        } else if (arg == QLatin1String("-translation")) {
             if (lastArg) usage();
             opts.translationFile = arguments.at(++i);
-        } else if (arg == "-opengl") {
+        } else if (arg == QLatin1String("-no-opengl")) {
+            opts.useGL = false;
+        } else if (arg == QLatin1String("-opengl")) {
             opts.useGL = true;
-        } else if (arg == "-qmlbrowser") {
+        } else if (arg == QLatin1String("-qmlbrowser")) {
             opts.useNativeFileBrowser = false;
-        } else if (arg == "-warnings") {
+        } else if (arg == QLatin1String("-warnings")) {
             if (lastArg) usage();
             QString warningsStr = arguments.at(++i);
             if (warningsStr == QLatin1String("show")) {
@@ -378,42 +393,42 @@ static void parseCommandLineOptions(const QStringList &arguments)
             } else {
                 usage();
             }
-        } else if (arg == "-I" || arg == "-L") {
-            if (arg == "-L")
+        } else if (arg == QLatin1String("-I") || arg == QLatin1String("-L")) {
+            if (arg == QLatin1String("-L"))
                 qWarning("-L option provided for compatibility only, use -I instead");
             if (lastArg) {
                 QDeclarativeEngine tmpEngine;
                 QString paths = tmpEngine.importPathList().join(QLatin1String(":"));
                 qWarning("Current search path: %s", paths.toLocal8Bit().constData());
-                exit(0);
+                exitApp(0);
             }
             opts.imports << arguments.at(++i);
-        } else if (arg == "-P") {
+        } else if (arg == QLatin1String("-P")) {
             if (lastArg) usage();
             opts.plugins << arguments.at(++i);
-        } else if (arg == "-script") {
+        } else if (arg == QLatin1String("-script")) {
             if (lastArg) usage();
             opts.script = arguments.at(++i);
-        } else if (arg == "-scriptopts") {
+        } else if (arg == QLatin1String("-scriptopts")) {
             if (lastArg) usage();
             opts.scriptopts = arguments.at(++i);
-        } else if (arg == "-savescript") {
+        } else if (arg == QLatin1String("-savescript")) {
             if (lastArg) usage();
             opts.script = arguments.at(++i);
             opts.runScript = false;
-        } else if (arg == "-playscript") {
+        } else if (arg == QLatin1String("-playscript")) {
             if (lastArg) usage();
             opts.script = arguments.at(++i);
             opts.runScript = true;
-        } else if (arg == "-sizeviewtorootobject") {
+        } else if (arg == QLatin1String("-sizeviewtorootobject")) {
             opts.sizeToView = false;
-        } else if (arg == "-sizerootobjecttoview") {
+        } else if (arg == QLatin1String("-sizerootobjecttoview")) {
             opts.sizeToView = true;
-        } else if (arg == "-experimentalgestures") {
+        } else if (arg == QLatin1String("-experimentalgestures")) {
             opts.experimentalGestures = true;
-        } else if (!arg.startsWith('-')) {
+        } else if (!arg.startsWith(QLatin1Char('-'))) {
             fileNames.append(arg);
-        } else if (true || arg == "-help") {
+        } else if (true || arg == QLatin1String("-help")) {
             usage();
         }
     }
@@ -448,7 +463,6 @@ static QDeclarativeViewer *createViewer()
         viewer->setScript(opts.script);
     }
 
-#if !defined(Q_OS_SYMBIAN)
     logger = viewer->warningsWidget();
     if (opts.warningsConfig == ShowWarnings) {
         logger.data()->setDefaultVisibility(LoggerWidget::ShowWarnings);
@@ -456,7 +470,6 @@ static QDeclarativeViewer *createViewer()
     } else if (opts.warningsConfig == HideWarnings){
         logger.data()->setDefaultVisibility(LoggerWidget::HideWarnings);
     }
-#endif
 
     if (opts.experimentalGestures)
         viewer->enableExperimentalGestures();
@@ -508,47 +521,52 @@ QDeclarativeViewer *openFile(const QString &fileName)
 
 int main(int argc, char ** argv)
 {
-#if defined (Q_OS_SYMBIAN)
-    qInstallMsgHandler(myMessageOutput);
-#else
     systemMsgOutput = qInstallMsgHandler(myMessageOutput);
-#endif
-
-#if defined (Q_OS_WIN)
-    // Debugging output is not visible by default on Windows -
-    // therefore show modal dialog with errors instad.
-    atexit(showWarnings);
-#endif
 
 #if defined (Q_WS_X11) || defined (Q_WS_MAC)
     //### default to using raster graphics backend for now
     bool gsSpecified = false;
     for (int i = 0; i < argc; ++i) {
-        QString arg = argv[i];
-        if (arg == "-graphicssystem") {
+        QString arg = QString::fromAscii(argv[i]);
+        if (arg == QLatin1String("-graphicssystem")) {
             gsSpecified = true;
             break;
         }
     }
 
     if (!gsSpecified)
-        QApplication::setGraphicsSystem("raster");
+        QApplication::setGraphicsSystem(QLatin1String("raster"));
 #endif
 
     Application app(argc, argv);
-    app.setApplicationName("QtQmlViewer");
-    app.setOrganizationName("Nokia");
-    app.setOrganizationDomain("nokia.com");
+    app.setApplicationName(QLatin1String("QtQmlViewer"));
+    app.setOrganizationName(QLatin1String("Nokia"));
+    app.setOrganizationDomain(QLatin1String("nokia.com"));
 
     QDeclarativeViewer::registerTypes();
     QDeclarativeTester::registerTypes();
 
     parseCommandLineOptions(app.arguments());
 
+    QTranslator translator;
+    QTranslator qtTranslator;
+    QString sysLocale = QLocale::system().name();
+    if (translator.load(QLatin1String("qmlviewer_") + sysLocale, QLibraryInfo::location(QLibraryInfo::TranslationsPath))) {
+        app.installTranslator(&translator);
+        if (qtTranslator.load(QLatin1String("qt_") + sysLocale, QLibraryInfo::location(QLibraryInfo::TranslationsPath))) {
+            app.installTranslator(&qtTranslator);
+        } else {
+            app.removeTranslator(&translator);
+        }
+    }
+
     QTranslator qmlTranslator;
     if (!opts.translationFile.isEmpty()) {
-        qmlTranslator.load(opts.translationFile);
-        app.installTranslator(&qmlTranslator);
+        if (qmlTranslator.load(opts.translationFile)) {
+            app.installTranslator(&qmlTranslator);
+        } else {
+            qWarning() << "Could not load the translation file" << opts.translationFile;
+        }
     }
 
     if (opts.fullScreen && opts.maximized)
