@@ -1,40 +1,40 @@
 /****************************************************************************
 **
-** Copyright (C) 2009 Nokia Corporation and/or its subsidiary(-ies).
+** Copyright (C) 2011 Nokia Corporation and/or its subsidiary(-ies).
 ** All rights reserved.
 ** Contact: Nokia Corporation (qt-info@nokia.com)
 **
 ** This file is part of the tools applications of the Qt Toolkit.
 **
 ** $QT_BEGIN_LICENSE:LGPL$
-** Commercial Usage
-** Licensees holding valid Qt Commercial licenses may use this file in
-** accordance with the Qt Commercial License Agreement provided with the
-** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and Nokia.
-**
 ** GNU Lesser General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU Lesser
-** General Public License version 2.1 as published by the Free Software
-** Foundation and appearing in the file LICENSE.LGPL included in the
-** packaging of this file.  Please review the following information to
-** ensure the GNU Lesser General Public License version 2.1 requirements
-** will be met: http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
+** This file may be used under the terms of the GNU Lesser General Public
+** License version 2.1 as published by the Free Software Foundation and
+** appearing in the file LICENSE.LGPL included in the packaging of this
+** file. Please review the following information to ensure the GNU Lesser
+** General Public License version 2.1 requirements will be met:
+** http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
 **
 ** In addition, as a special exception, Nokia gives you certain additional
-** rights.  These rights are described in the Nokia Qt LGPL Exception
+** rights. These rights are described in the Nokia Qt LGPL Exception
 ** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
 **
 ** GNU General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU
-** General Public License version 3.0 as published by the Free Software
-** Foundation and appearing in the file LICENSE.GPL included in the
-** packaging of this file.  Please review the following information to
-** ensure the GNU General Public License version 3.0 requirements will be
-** met: http://www.gnu.org/copyleft/gpl.html.
+** Alternatively, this file may be used under the terms of the GNU General
+** Public License version 3.0 as published by the Free Software Foundation
+** and appearing in the file LICENSE.GPL included in the packaging of this
+** file. Please review the following information to ensure the GNU General
+** Public License version 3.0 requirements will be met:
+** http://www.gnu.org/copyleft/gpl.html.
+**
+** Other Usage
+** Alternatively, this file may be used in accordance with the terms and
+** conditions contained in a signed written agreement between you and Nokia.
+**
+**
+**
+**
 **
-** If you have questions regarding the use of this file, please contact
-** Nokia at qt-info@nokia.com.
 ** $QT_END_LICENSE$
 **
 ****************************************************************************/
@@ -421,20 +421,20 @@ void QtGradientStopsWidget::setGradientStopsModel(QtGradientStopsModel *model)
         return;
 
     if (d_ptr->m_model) {
-        disconnect(d_ptr->m_model, SIGNAL(stopAdded(QtGradientStop *)),
-                    this, SLOT(slotStopAdded(QtGradientStop *)));
-        disconnect(d_ptr->m_model, SIGNAL(stopRemoved(QtGradientStop *)),
-                    this, SLOT(slotStopRemoved(QtGradientStop *)));
-        disconnect(d_ptr->m_model, SIGNAL(stopMoved(QtGradientStop *, qreal)),
-                    this, SLOT(slotStopMoved(QtGradientStop *, qreal)));
-        disconnect(d_ptr->m_model, SIGNAL(stopsSwapped(QtGradientStop *, QtGradientStop *)),
-                    this, SLOT(slotStopsSwapped(QtGradientStop *, QtGradientStop *)));
-        disconnect(d_ptr->m_model, SIGNAL(stopChanged(QtGradientStop *, const QColor &)),
-                    this, SLOT(slotStopChanged(QtGradientStop *, const QColor &)));
-        disconnect(d_ptr->m_model, SIGNAL(stopSelected(QtGradientStop *, bool)),
-                    this, SLOT(slotStopSelected(QtGradientStop *, bool)));
-        disconnect(d_ptr->m_model, SIGNAL(currentStopChanged(QtGradientStop *)),
-                    this, SLOT(slotCurrentStopChanged(QtGradientStop *)));
+        disconnect(d_ptr->m_model, SIGNAL(stopAdded(QtGradientStop*)),
+                    this, SLOT(slotStopAdded(QtGradientStop*)));
+        disconnect(d_ptr->m_model, SIGNAL(stopRemoved(QtGradientStop*)),
+                    this, SLOT(slotStopRemoved(QtGradientStop*)));
+        disconnect(d_ptr->m_model, SIGNAL(stopMoved(QtGradientStop*,qreal)),
+                    this, SLOT(slotStopMoved(QtGradientStop*,qreal)));
+        disconnect(d_ptr->m_model, SIGNAL(stopsSwapped(QtGradientStop*,QtGradientStop*)),
+                    this, SLOT(slotStopsSwapped(QtGradientStop*,QtGradientStop*)));
+        disconnect(d_ptr->m_model, SIGNAL(stopChanged(QtGradientStop*,QColor)),
+                    this, SLOT(slotStopChanged(QtGradientStop*,QColor)));
+        disconnect(d_ptr->m_model, SIGNAL(stopSelected(QtGradientStop*,bool)),
+                    this, SLOT(slotStopSelected(QtGradientStop*,bool)));
+        disconnect(d_ptr->m_model, SIGNAL(currentStopChanged(QtGradientStop*)),
+                    this, SLOT(slotCurrentStopChanged(QtGradientStop*)));
 
         d_ptr->m_stops.clear();
     }
@@ -442,20 +442,20 @@ void QtGradientStopsWidget::setGradientStopsModel(QtGradientStopsModel *model)
     d_ptr->m_model = model;
 
     if (d_ptr->m_model) {
-        connect(d_ptr->m_model, SIGNAL(stopAdded(QtGradientStop *)),
-                    this, SLOT(slotStopAdded(QtGradientStop *)));
-        connect(d_ptr->m_model, SIGNAL(stopRemoved(QtGradientStop *)),
-                    this, SLOT(slotStopRemoved(QtGradientStop *)));
-        connect(d_ptr->m_model, SIGNAL(stopMoved(QtGradientStop *, qreal)),
-                    this, SLOT(slotStopMoved(QtGradientStop *, qreal)));
-        connect(d_ptr->m_model, SIGNAL(stopsSwapped(QtGradientStop *, QtGradientStop *)),
-                    this, SLOT(slotStopsSwapped(QtGradientStop *, QtGradientStop *)));
-        connect(d_ptr->m_model, SIGNAL(stopChanged(QtGradientStop *, const QColor &)),
-                    this, SLOT(slotStopChanged(QtGradientStop *, const QColor &)));
-        connect(d_ptr->m_model, SIGNAL(stopSelected(QtGradientStop *, bool)),
-                    this, SLOT(slotStopSelected(QtGradientStop *, bool)));
-        connect(d_ptr->m_model, SIGNAL(currentStopChanged(QtGradientStop *)),
-                    this, SLOT(slotCurrentStopChanged(QtGradientStop *)));
+        connect(d_ptr->m_model, SIGNAL(stopAdded(QtGradientStop*)),
+                    this, SLOT(slotStopAdded(QtGradientStop*)));
+        connect(d_ptr->m_model, SIGNAL(stopRemoved(QtGradientStop*)),
+                    this, SLOT(slotStopRemoved(QtGradientStop*)));
+        connect(d_ptr->m_model, SIGNAL(stopMoved(QtGradientStop*,qreal)),
+                    this, SLOT(slotStopMoved(QtGradientStop*,qreal)));
+        connect(d_ptr->m_model, SIGNAL(stopsSwapped(QtGradientStop*,QtGradientStop*)),
+                    this, SLOT(slotStopsSwapped(QtGradientStop*,QtGradientStop*)));
+        connect(d_ptr->m_model, SIGNAL(stopChanged(QtGradientStop*,QColor)),
+                    this, SLOT(slotStopChanged(QtGradientStop*,QColor)));
+        connect(d_ptr->m_model, SIGNAL(stopSelected(QtGradientStop*,bool)),
+                    this, SLOT(slotStopSelected(QtGradientStop*,bool)));
+        connect(d_ptr->m_model, SIGNAL(currentStopChanged(QtGradientStop*)),
+                    this, SLOT(slotCurrentStopChanged(QtGradientStop*)));
 
         QList<QtGradientStop *> stops = d_ptr->m_model->stops().values();
         QListIterator<QtGradientStop *> itStop(stops);
