@@ -233,6 +233,11 @@ Privzeta bo edninska univerzalna oblika.</translation>
         <source>Cancel</source>
         <translation>Prekliči</translation>
     </message>
+    <message>
+        <source></source>
+        <comment>Choose Edit|Find from the menu bar or press Ctrl+F to pop up the Find dialog</comment>
+        <translation></translation>
+    </message>
 </context>
 <context>
     <name>FormMultiWidget</name>
@@ -263,6 +268,99 @@ Privzeta bo edninska univerzalna oblika.</translation>
         <translation>Ali izbrišem ne-prazno dolžinsko inačico?</translation>
     </message>
 </context>
+<context>
+    <name>LConvert</name>
+    <message>
+        <source>
+Usage:
+    lconvert [options] &lt;infile&gt; [&lt;infile&gt;...]
+
+lconvert is part of Qt&apos;s Linguist tool chain. It can be used as a
+stand-alone tool to convert and filter translation data files.
+The following file formats are supported:
+
+%1
+If multiple input files are specified, they are merged with
+translations from later files taking precedence.
+
+Options:
+    -h
+    --help  Display this information and exit.
+
+    -i &lt;infile&gt;
+    --input-file &lt;infile&gt;
+           Specify input file. Use if &lt;infile&gt; might start with a dash.
+           This option can be used several times to merge inputs.
+           May be &apos;-&apos; (standard input) for use in a pipe.
+
+    -o &lt;outfile&gt;
+    --output-file &lt;outfile&gt;
+           Specify output file. Default is &apos;-&apos; (standard output).
+
+    -if &lt;informat&gt;
+    --input-format &lt;format&gt;
+           Specify input format for subsequent &lt;infile&gt;s.
+           The format is auto-detected from the file name and defaults to &apos;ts&apos;.
+
+    -of &lt;outformat&gt;
+    --output-format &lt;outformat&gt;
+           Specify output format. See -if.
+
+    --input-codec &lt;codec&gt;
+           Specify encoding for QM and PO input files. Default is &apos;Latin1&apos;
+           for QM and &apos;UTF-8&apos; for PO files. UTF-8 is always tried as well for
+           QM, corresponding to the possible use of the trUtf8() function.
+
+    --output-codec &lt;codec&gt;
+           Specify encoding for PO output files. Default is &apos;UTF-8&apos;.
+
+    --drop-tags &lt;regexp&gt;
+           Drop named extra tags when writing TS or XLIFF files.
+           May be specified repeatedly.
+
+    --drop-translations
+           Drop existing translations and reset the status to &apos;unfinished&apos;.
+           Note: this implies --no-obsolete.
+
+    --source-language &lt;language&gt;[_&lt;region&gt;]
+           Specify/override the language of the source strings. Defaults to
+           POSIX if not specified and the file does not name it yet.
+
+    --target-language &lt;language&gt;[_&lt;region&gt;]
+           Specify/override the language of the translation.
+           The target language is guessed from the file name if this option
+           is not specified and the file contents name no language yet.
+
+    --no-obsolete
+           Drop obsolete messages.
+
+    --no-finished
+           Drop finished messages.
+
+    --sort-contexts
+           Sort contexts in output TS file alphabetically.
+
+    --locations {absolute|relative|none}
+           Override how source code references are saved in TS files.
+           Default is absolute.
+
+    --no-ui-lines
+           Drop line numbers from references to UI files.
+
+    --verbose
+           be a bit more verbose
+
+Long options can be specified with only one leading dash, too.
+
+Return value:
+    0 on success
+    1 on command line parse failures
+    2 on read failures
+    3 on write failures
+</source>
+        <translation type="unfinished"></translation>
+    </message>
+</context>
 <context>
     <name>LRelease</name>
     <message numerus="yes">
@@ -301,6 +399,590 @@ Privzeta bo edninska univerzalna oblika.</translation>
             <numerusform>    Prezrl %n neprevedenih izvornih besedil</numerusform>
         </translation>
     </message>
+    <message>
+        <source>Usage:
+    lrelease [options] project-file
+    lrelease [options] ts-files [-qm qm-file]
+
+lrelease is part of Qt&apos;s Linguist tool chain. It can be used as a
+stand-alone tool to convert XML-based translations files in the TS
+format into the &apos;compiled&apos; QM format used by QTranslator objects.
+
+Options:
+    -help  Display this information and exit
+    -idbased
+           Use IDs instead of source strings for message keying
+    -compress
+           Compress the QM files
+    -nounfinished
+           Do not include unfinished translations
+    -removeidentical
+           If the translated text is the same as
+           the source text, do not include the message
+    -markuntranslated &lt;prefix&gt;
+           If a message has no real translation, use the source text
+           prefixed with the given string instead
+    -silent
+           Do not explain what is being done
+    -version
+           Display the version of lrelease and exit
+</source>
+        <translation type="unfinished"></translation>
+    </message>
+    <message>
+        <source>lrelease error: %1</source>
+        <translation type="unfinished"></translation>
+    </message>
+    <message>
+        <source>Updating &apos;%1&apos;...
+</source>
+        <translation type="unfinished"></translation>
+    </message>
+    <message>
+        <source>Removing translations equal to source text in &apos;%1&apos;...
+</source>
+        <translation type="unfinished"></translation>
+    </message>
+    <message>
+        <source>lrelease error: cannot create &apos;%1&apos;: %2
+</source>
+        <translation type="unfinished"></translation>
+    </message>
+    <message>
+        <source>lrelease error: cannot save &apos;%1&apos;: %2</source>
+        <translation type="unfinished"></translation>
+    </message>
+    <message>
+        <source>lrelease version %1
+</source>
+        <translation type="unfinished"></translation>
+    </message>
+    <message>
+        <source>lrelease error: cannot read project file &apos;%1&apos;.
+</source>
+        <translation type="unfinished"></translation>
+    </message>
+    <message>
+        <source>lrelease error: cannot process project file &apos;%1&apos;.
+</source>
+        <translation type="unfinished"></translation>
+    </message>
+    <message>
+        <source>lrelease warning: Met no &apos;TRANSLATIONS&apos; entry in project file &apos;%1&apos;
+</source>
+        <translation type="unfinished"></translation>
+    </message>
+</context>
+<context>
+    <name>LUpdate</name>
+    <message>
+        <source>Parenthesis/bracket/brace mismatch between #if and #else branches; using #if branch
+</source>
+        <translation type="unfinished"></translation>
+    </message>
+    <message>
+        <source>Parenthesis/brace mismatch between #if and #else branches; using #if branch
+</source>
+        <translation type="unfinished"></translation>
+    </message>
+    <message>
+        <source>Unterminated C++ comment
+</source>
+        <translation type="unfinished"></translation>
+    </message>
+    <message>
+        <source>Unterminated C++ string
+</source>
+        <translation type="unfinished"></translation>
+    </message>
+    <message>
+        <source>Excess closing brace in C++ code (or abuse of the C++ preprocessor)
+</source>
+        <translation type="unfinished"></translation>
+    </message>
+    <message>
+        <source>Excess closing parenthesis in C++ code (or abuse of the C++ preprocessor)
+</source>
+        <translation type="unfinished"></translation>
+    </message>
+    <message>
+        <source>Excess closing bracket in C++ code (or abuse of the C++ preprocessor)
+</source>
+        <translation type="unfinished"></translation>
+    </message>
+    <message>
+        <source>circular inclusion of %1
+</source>
+        <translation type="unfinished"></translation>
+    </message>
+    <message>
+        <source>Cannot open %1: %2
+</source>
+        <translation type="unfinished"></translation>
+    </message>
+    <message>
+        <source>//% cannot be used with tr() / QT_TR_NOOP(). Ignoring
+</source>
+        <translation type="unfinished"></translation>
+    </message>
+    <message>
+        <source>Qualifying with unknown namespace/class %1::%2
+</source>
+        <translation type="unfinished"></translation>
+    </message>
+    <message>
+        <source>tr() cannot be called without context
+</source>
+        <translation type="unfinished"></translation>
+    </message>
+    <message>
+        <source>Class &apos;%1&apos; lacks Q_OBJECT macro
+</source>
+        <translation type="unfinished"></translation>
+    </message>
+    <message>
+        <source>It is not recommended to call tr() from within a constructor &apos;%1::%2&apos;
+</source>
+        <translation type="unfinished"></translation>
+    </message>
+    <message>
+        <source>//% cannot be used with translate() / QT_TRANSLATE_NOOP(). Ignoring
+</source>
+        <translation type="unfinished"></translation>
+    </message>
+    <message>
+        <source>//= cannot be used with qtTrId() / QT_TRID_NOOP(). Ignoring
+</source>
+        <translation type="unfinished"></translation>
+    </message>
+    <message>
+        <source>Unexpected character in meta string
+</source>
+        <translation type="unfinished"></translation>
+    </message>
+    <message>
+        <source>Unterminated meta string
+</source>
+        <translation type="unfinished"></translation>
+    </message>
+    <message>
+        <source>Cannot invoke tr() like this
+</source>
+        <translation type="unfinished"></translation>
+    </message>
+    <message>
+        <source>Discarding unconsumed meta data
+</source>
+        <translation type="unfinished"></translation>
+    </message>
+    <message>
+        <source>Unbalanced opening brace in C++ code (or abuse of the C++ preprocessor)
+</source>
+        <translation type="unfinished"></translation>
+    </message>
+    <message>
+        <source>Unbalanced opening parenthesis in C++ code (or abuse of the C++ preprocessor)
+</source>
+        <translation type="unfinished"></translation>
+    </message>
+    <message>
+        <source>Unbalanced opening bracket in C++ code (or abuse of the C++ preprocessor)
+</source>
+        <translation type="unfinished"></translation>
+    </message>
+    <message>
+        <source>Cannot open %1: %2</source>
+        <translation type="unfinished"></translation>
+    </message>
+    <message>
+        <source>Unterminated Java comment.
+</source>
+        <translation type="unfinished"></translation>
+    </message>
+    <message>
+        <source>Invalid Unicode value.
+</source>
+        <translation type="unfinished"></translation>
+    </message>
+    <message>
+        <source>Unterminated string.
+</source>
+        <translation type="unfinished"></translation>
+    </message>
+    <message>
+        <source>String used in translation can contain only literals concatenated with other literals, not expressions or numbers.
+</source>
+        <translation type="unfinished"></translation>
+    </message>
+    <message>
+        <source>&apos;class&apos; must be followed by a class name.
+</source>
+        <translation type="unfinished"></translation>
+    </message>
+    <message>
+        <source>Excess closing brace.
+</source>
+        <translation type="unfinished"></translation>
+    </message>
+    <message>
+        <source>&apos;package&apos; must be followed by package name.
+</source>
+        <translation type="unfinished"></translation>
+    </message>
+    <message>
+        <source>Unbalanced opening brace.
+</source>
+        <translation type="unfinished"></translation>
+    </message>
+    <message>
+        <source>Unbalanced opening parenthesis.
+</source>
+        <translation type="unfinished"></translation>
+    </message>
+    <message>
+        <source>Usage:
+    lupdate [options] [project-file]...
+    lupdate [options] [source-file|path|@lst-file]... -ts ts-files|@lst-file
+
+lupdate is part of Qt&apos;s Linguist tool chain. It extracts translatable
+messages from Qt UI files, C++, Java and JavaScript/QtScript source code.
+Extracted messages are stored in textual translation source files (typically
+Qt TS XML). New and modified messages can be merged into existing TS files.
+
+Options:
+    -help  Display this information and exit.
+    -no-obsolete
+           Drop all obsolete strings.
+    -extensions &lt;ext&gt;[,&lt;ext&gt;]...
+           Process files with the given extensions only.
+           The extension list must be separated with commas, not with whitespace.
+           Default: &apos;%1&apos;.
+    -pluralonly
+           Only include plural form messages.
+    -silent
+           Do not explain what is being done.
+    -no-sort
+           Do not sort contexts in TS files.
+    -no-recursive
+           Do not recursively scan the following directories.
+    -recursive
+           Recursively scan the following directories (default).
+    -I &lt;includepath&gt; or -I&lt;includepath&gt;
+           Additional location to look for include files.
+           May be specified multiple times.
+    -locations {absolute|relative|none}
+           Specify/override how source code references are saved in TS files.
+           Default is absolute.
+    -no-ui-lines
+           Do not record line numbers in references to UI files.
+    -disable-heuristic {sametext|similartext|number}
+           Disable the named merge heuristic. Can be specified multiple times.
+    -pro &lt;filename&gt;
+           Name of a .pro file. Useful for files with .pro file syntax but
+           different file suffix. Projects are recursed into and merged.
+    -source-language &lt;language&gt;[_&lt;region&gt;]
+           Specify the language of the source strings for new files.
+           Defaults to POSIX if not specified.
+    -target-language &lt;language&gt;[_&lt;region&gt;]
+           Specify the language of the translations for new files.
+           Guessed from the file name if not specified.
+    -ts &lt;ts-file&gt;...
+           Specify the output file(s). This will override the TRANSLATIONS
+           and nullify the CODECFORTR from possibly specified project files.
+    -codecfortr &lt;codec&gt;
+           Specify the codec assumed for tr() calls. Effective only with -ts.
+    -version
+           Display the version of lupdate and exit.
+    @lst-file
+           Read additional file names (one per line) from lst-file.
+</source>
+        <translation type="unfinished"></translation>
+    </message>
+    <message>
+        <source>lupdate warning: Codec for tr() &apos;%1&apos; disagrees with existing file&apos;s codec &apos;%2&apos;. Expect trouble.
+</source>
+        <translation type="unfinished"></translation>
+    </message>
+    <message>
+        <source>lupdate warning: Specified target language &apos;%1&apos; disagrees with existing file&apos;s language &apos;%2&apos;. Ignoring.
+</source>
+        <translation type="unfinished"></translation>
+    </message>
+    <message>
+        <source>lupdate warning: Specified source language &apos;%1&apos; disagrees with existing file&apos;s language &apos;%2&apos;. Ignoring.
+</source>
+        <translation type="unfinished"></translation>
+    </message>
+    <message>
+        <source>Updating &apos;%1&apos;...
+</source>
+        <translation type="unfinished"></translation>
+    </message>
+    <message>
+        <source>Stripping non plural forms in &apos;%1&apos;...
+</source>
+        <translation type="unfinished"></translation>
+    </message>
+    <message>
+        <source>lupdate warning: Codec for source &apos;%1&apos; is invalid. Falling back to codec for tr().
+</source>
+        <translation type="unfinished"></translation>
+    </message>
+    <message>
+        <source>lupdate warning: TS files from command line will override TRANSLATIONS in %1.
+</source>
+        <translation type="unfinished"></translation>
+    </message>
+    <message>
+        <source>lupdate warning: TS files from command line prevent recursing into %1.
+</source>
+        <translation type="unfinished"></translation>
+    </message>
+    <message>
+        <source>lupdate warning: no TS files specified. Only diagnostics will be produced for &apos;%1&apos;.
+</source>
+        <translation type="unfinished"></translation>
+    </message>
+    <message>
+        <source>The option -target-language requires a parameter.
+</source>
+        <translation type="unfinished"></translation>
+    </message>
+    <message>
+        <source>The option -source-language requires a parameter.
+</source>
+        <translation type="unfinished"></translation>
+    </message>
+    <message>
+        <source>The option -disable-heuristic requires a parameter.
+</source>
+        <translation type="unfinished"></translation>
+    </message>
+    <message>
+        <source>Invalid heuristic name passed to -disable-heuristic.
+</source>
+        <translation type="unfinished"></translation>
+    </message>
+    <message>
+        <source>The option -locations requires a parameter.
+</source>
+        <translation type="unfinished"></translation>
+    </message>
+    <message>
+        <source>Invalid parameter passed to -locations.
+</source>
+        <translation type="unfinished"></translation>
+    </message>
+    <message>
+        <source>The -codecfortr option should be followed by a codec name.
+</source>
+        <translation type="unfinished"></translation>
+    </message>
+    <message>
+        <source>The -extensions option should be followed by an extension list.
+</source>
+        <translation type="unfinished"></translation>
+    </message>
+    <message>
+        <source>The -pro option should be followed by a filename of .pro file.
+</source>
+        <translation type="unfinished"></translation>
+    </message>
+    <message>
+        <source>The -I option should be followed by a path.
+</source>
+        <translation type="unfinished"></translation>
+    </message>
+    <message>
+        <source>Unrecognized option &apos;%1&apos;.
+</source>
+        <translation type="unfinished"></translation>
+    </message>
+    <message>
+        <source>lupdate error: List file &apos;%1&apos; is not readable.
+</source>
+        <translation type="unfinished"></translation>
+    </message>
+    <message>
+        <source>lupdate warning: For some reason, &apos;%1&apos; is not writable.
+</source>
+        <translation type="unfinished"></translation>
+    </message>
+    <message>
+        <source>lupdate error: File &apos;%1&apos; has no recognized extension.
+</source>
+        <translation type="unfinished"></translation>
+    </message>
+    <message>
+        <source>lupdate error: File &apos;%1&apos; does not exist.
+</source>
+        <translation type="unfinished"></translation>
+    </message>
+    <message>
+        <source>Scanning directory &apos;%1&apos;...
+</source>
+        <translation type="unfinished"></translation>
+    </message>
+    <message>
+        <source>lupdate warning: -target-language usually only makes sense with exactly one TS file.
+</source>
+        <translation type="unfinished"></translation>
+    </message>
+    <message>
+        <source>lupdate warning: -codecfortr has no effect without -ts.
+</source>
+        <translation type="unfinished"></translation>
+    </message>
+    <message>
+        <source>lupdate warning: no TS files specified. Only diagnostics will be produced.
+</source>
+        <translation type="unfinished"></translation>
+    </message>
+    <message>
+        <source>lupdate error: Both project and source files / include paths specified.
+</source>
+        <translation type="unfinished"></translation>
+    </message>
+    <message numerus="yes">
+        <source>    Found %n source text(s) (%1 new and %2 already existing)
+</source>
+        <translation type="unfinished">
+            <numerusform></numerusform>
+            <numerusform></numerusform>
+            <numerusform></numerusform>
+            <numerusform></numerusform>
+        </translation>
+    </message>
+    <message numerus="yes">
+        <source>    Removed %n obsolete entries
+</source>
+        <translation type="unfinished">
+            <numerusform></numerusform>
+            <numerusform></numerusform>
+            <numerusform></numerusform>
+            <numerusform></numerusform>
+        </translation>
+    </message>
+    <message numerus="yes">
+        <source>    Kept %n obsolete entries
+</source>
+        <translation type="unfinished">
+            <numerusform></numerusform>
+            <numerusform></numerusform>
+            <numerusform></numerusform>
+            <numerusform></numerusform>
+        </translation>
+    </message>
+    <message numerus="yes">
+        <source>    Number heuristic provided %n translation(s)
+</source>
+        <translation type="unfinished">
+            <numerusform></numerusform>
+            <numerusform></numerusform>
+            <numerusform></numerusform>
+            <numerusform></numerusform>
+        </translation>
+    </message>
+    <message numerus="yes">
+        <source>    Same-text heuristic provided %n translation(s)
+</source>
+        <translation type="unfinished">
+            <numerusform></numerusform>
+            <numerusform></numerusform>
+            <numerusform></numerusform>
+            <numerusform></numerusform>
+        </translation>
+    </message>
+    <message numerus="yes">
+        <source>    Similar-text heuristic provided %n translation(s)
+</source>
+        <translation type="unfinished">
+            <numerusform></numerusform>
+            <numerusform></numerusform>
+            <numerusform></numerusform>
+            <numerusform></numerusform>
+        </translation>
+    </message>
+    <message>
+        <source>Illegal character</source>
+        <translation type="unfinished"></translation>
+    </message>
+    <message>
+        <source>Unclosed string at end of line</source>
+        <translation type="unfinished"></translation>
+    </message>
+    <message>
+        <source>Illegal escape sequence</source>
+        <translation type="unfinished"></translation>
+    </message>
+    <message>
+        <source>Illegal unicode escape sequence</source>
+        <translation type="unfinished"></translation>
+    </message>
+    <message>
+        <source>Unclosed comment at end of file</source>
+        <translation type="unfinished"></translation>
+    </message>
+    <message>
+        <source>Illegal syntax for exponential number</source>
+        <translation type="unfinished"></translation>
+    </message>
+    <message>
+        <source>Identifier cannot start with numeric literal</source>
+        <translation type="unfinished"></translation>
+    </message>
+    <message>
+        <source>Unterminated regular expression literal</source>
+        <translation type="unfinished"></translation>
+    </message>
+    <message>
+        <source>//% cannot be used with %1(). Ignoring
+</source>
+        <translation type="unfinished"></translation>
+    </message>
+    <message>
+        <source>%1() requires at least two arguments.
+</source>
+        <translation type="unfinished"></translation>
+    </message>
+    <message>
+        <source>%1(): both arguments must be literal strings.
+</source>
+        <translation type="unfinished"></translation>
+    </message>
+    <message>
+        <source>%1() requires at least one argument.
+</source>
+        <translation type="unfinished"></translation>
+    </message>
+    <message>
+        <source>%1(): text to translate must be a literal string.
+</source>
+        <translation type="unfinished"></translation>
+    </message>
+    <message>
+        <source>//= cannot be used with %1(). Ignoring
+</source>
+        <translation type="unfinished"></translation>
+    </message>
+    <message>
+        <source>%1(): identifier must be a literal string.
+</source>
+        <translation type="unfinished"></translation>
+    </message>
+    <message>
+        <source>Expected </source>
+        <extracomment>Beginning of the string that contains comma-separated list of expected tokens</extracomment>
+        <translation type="unfinished"></translation>
+    </message>
+    <message>
+        <source>XML error: Parse error at line %1, column %2 (%3).</source>
+        <translation type="unfinished"></translation>
+    </message>
+    <message>
+        <source>Parse error in UI file</source>
+        <translation type="unfinished"></translation>
+    </message>
 </context>
 <context>
     <name>MainWindow</name>
@@ -844,6 +1526,11 @@ Privzeta bo edninska univerzalna oblika.</translation>
         <source>Length Variants</source>
         <translation>Dolžinske inačice</translation>
     </message>
+    <message>
+        <source></source>
+        <comment>This is the application&apos;s main window.</comment>
+        <translation></translation>
+    </message>
     <message>
         <source>Source text</source>
         <translation>Izvorno besedilo</translation>
@@ -1061,8 +1748,8 @@ Vse datoteke (*)</translation>
         <translation>Različica %1</translation>
     </message>
     <message>
-        <source>&lt;center&gt;&lt;img src=&quot;:/images/splash.png&quot;/&gt;&lt;/img&gt;&lt;p&gt;%1&lt;/p&gt;&lt;/center&gt;&lt;p&gt;Qt Linguist is a tool for adding translations to Qt applications.&lt;/p&gt;&lt;p&gt;Copyright (C) 2010 Nokia Corporation and/or its subsidiary(-ies).</source>
-        <translation>&lt;center&gt;&lt;img src=&quot;:/images/splash.png&quot;/&gt;&lt;/img&gt;&lt;p&gt;%1&lt;/p&gt;&lt;/center&gt;&lt;p&gt;Qt Linguist je orodje za dodajanje prevodov programom Qt..&lt;/p&gt;&lt;p&gt;Avtorske pravice © 2010 Nokia Corporation in/ali njene podružnice.&lt;/p&gt;&lt;p&gt;Prevedel: &lt;a href=&quot;mailto:jlp@holodeck1.com&quot;&gt;Jure Repinc&lt;/a&gt;, &lt;a href=&quot;http://www.lugos.si/&quot;&gt;LUGOS&lt;/a&gt;&lt;/p&gt;</translation>
+        <source>&lt;center&gt;&lt;img src=&quot;:/images/splash.png&quot;/&gt;&lt;/img&gt;&lt;p&gt;%1&lt;/p&gt;&lt;/center&gt;&lt;p&gt;Qt Linguist is a tool for adding translations to Qt applications.&lt;/p&gt;&lt;p&gt;Copyright (C) 2011 Nokia Corporation and/or its subsidiary(-ies).</source>
+        <translation>&lt;center&gt;&lt;img src=&quot;:/images/splash.png&quot;/&gt;&lt;/img&gt;&lt;p&gt;%1&lt;/p&gt;&lt;/center&gt;&lt;p&gt;Qt Linguist je orodje za dodajanje prevodov programom Qt..&lt;/p&gt;&lt;p&gt;Avtorske pravice © 2011 Nokia Corporation in/ali njene podružnice.&lt;/p&gt;&lt;p&gt;Prevedel: &lt;a href=&quot;mailto:jlp@holodeck1.com&quot;&gt;Jure Repinc&lt;/a&gt;, &lt;a href=&quot;http://www.lugos.si/&quot;&gt;LUGOS&lt;/a&gt;&lt;/p&gt;</translation>
     </message>
     <message>
         <source>Do you want to save the modified files?</source>
@@ -1191,6 +1878,11 @@ Vse datoteke (*)</translation>
 </context>
 <context>
     <name>MessageEditor</name>
+    <message>
+        <source></source>
+        <comment>This is the right panel of the main window.</comment>
+        <translation></translation>
+    </message>
     <message>
         <source>Russian</source>
         <translation>Ruščina</translation>
@@ -1289,6 +1981,13 @@ Vrstica: %2</translation>
         <translation>&lt;neimenovan kontekst&gt;</translation>
     </message>
 </context>
+<context>
+    <name>PhraseBook</name>
+    <message>
+        <source>Parse error at line %1, column %2 (%3).</source>
+        <translation type="unfinished"></translation>
+    </message>
+</context>
 <context>
     <name>PhraseBookBox</name>
     <message>
@@ -1359,6 +2058,11 @@ Vrstica: %2</translation>
         <source>Close</source>
         <translation>Zapri</translation>
     </message>
+    <message>
+        <source></source>
+        <comment>Go to Phrase &gt; Edit Phrase Book... The dialog that pops up is a PhraseBookBox.</comment>
+        <translation></translation>
+    </message>
     <message>
         <source>(New Entry)</source>
         <translation>(Nov vnos)</translation>
@@ -1456,6 +2160,11 @@ Vrstica: %2</translation>
         <source>XLIFF localization files</source>
         <translation>Prevajalske datoteke XLIFF</translation>
     </message>
+    <message>
+        <source>lupdate version %1
+</source>
+        <translation type="unfinished"></translation>
+    </message>
 </context>
 <context>
     <name>SourceCodeView</name>
